@@ -8,7 +8,10 @@
 // color of saturated red was used.  (middle grey 
 
 
-
+#include "Cursors32/ClipStretchLeft.xpm"
+#include "Cursors32/ClipStretchRight.xpm"
+#include "Cursors32/ClipTrimLeft.xpm"
+#include "Cursors32/ClipTrimRight.xpm"
 #include "Cursors32/CrosshairCursor.xpm"
 #include "Cursors32/DisabledCursor.xpm"
 #include "Cursors32/RearrangeCursor.xpm"
