@@ -37,6 +37,7 @@ for shared and private configs - which need to move out.
 #include "ModuleManager.h"
 #include "PlatformCompatibility.h"
 #include "Base64.h"
+#include "Variant.h"
 
 ///////////////////////////////////////////////////////////////////////////////
 //
@@ -366,7 +367,7 @@ void PluginManager::InitializePlugins()
 // PluginManager implementation
 // ----------------------------------------------------------------------------
 
-static PluginManager::FileConfigFactory sFactory;
+static PluginManager::ConfigFactory sFactory;
 
 // ============================================================================
 //
@@ -385,7 +386,7 @@ PluginManager & PluginManager::Get()
    return *mInstance;
 }
 
-void PluginManager::Initialize(FileConfigFactory factory)
+void PluginManager::Initialize(ConfigFactory factory)
 {
    sFactory = move(factory);
 
@@ -525,6 +526,7 @@ bool PluginManager::DropFile(const wxString &fileName)
                   mRegisteredPlugins[id].SetEnabled(enable);
                // Make changes to enabled status persist:
                this->Save();
+               this->NotifyPluginsChanged();
             }
 
             return true;
@@ -549,7 +551,7 @@ void PluginManager::Load()
       // This DeleteAll affects pluginregistry.cfg only, not audacity.cfg
       // That is, the memory of on/off states of effect (and generator,
       // analyzer, and tool) plug-ins
-      registry.DeleteAll();
+      registry.Clear();
       registry.Flush();
       return;
    }
@@ -562,19 +564,14 @@ void PluginManager::Load()
       // Conversion code here, for when registry version changes.
 
       // We iterate through the effects, possibly updating their info.
-      wxString groupName;
-      long groupIndex;
       wxString group = GetPluginTypeString(PluginTypeEffect);
       wxString cfgPath = REGROOT + group + wxCONFIG_PATH_SEPARATOR;
       wxArrayString groupsToDelete;
 
-      registry.SetPath(cfgPath);
-      for (bool cont = registry.GetFirstGroup(groupName, groupIndex);
-         cont;
-         registry.SetPath(cfgPath),
-         cont = registry.GetNextGroup(groupName, groupIndex))
+      auto cfgGroup = registry.BeginGroup(cfgPath);
+      for(const auto& groupName : registry.GetChildGroups())
       {
-         registry.SetPath(groupName);
+         auto effectGroup = registry.BeginGroup(groupName);
          wxString effectSymbol = registry.Read(KEY_SYMBOL, "");
          wxString effectVersion = registry.Read(KEY_VERSION, "");
 
@@ -595,14 +592,12 @@ void PluginManager::Load()
                groupsToDelete.push_back(cfgPath + groupName);
             }
          }
-
       }
       // Doing the deletion within the search loop risked skipping some items,
       // hence the delayed delete.
       for (unsigned int i = 0; i < groupsToDelete.size(); i++) {
          registry.DeleteGroup(groupsToDelete[i]);
       }
-      registry.SetPath("");
       // Updates done.  Make sure we read the updated data later.
       registry.Flush();
    }
@@ -620,7 +615,7 @@ void PluginManager::Load()
    return;
 }
 
-void PluginManager::LoadGroup(FileConfig *pRegistry, PluginType type)
+void PluginManager::LoadGroup(audacity::BasicSettings *pRegistry, PluginType type)
 {
 #ifdef __WXMAC__
    // Bug 1590: On Mac, we should purge the registry of Nyquist plug-ins
@@ -658,22 +653,15 @@ void PluginManager::LoadGroup(FileConfig *pRegistry, PluginType type)
 
    wxString strVal;
    bool boolVal;
-   wxString groupName;
-   long groupIndex;
-   wxString group = GetPluginTypeString(type);
-   wxString cfgPath = REGROOT + group + wxCONFIG_PATH_SEPARATOR;
+   wxString cfgPath = REGROOT + GetPluginTypeString(type) + wxCONFIG_PATH_SEPARATOR;
 
-   pRegistry->SetPath(cfgPath);
-   for (bool cont = pRegistry->GetFirstGroup(groupName, groupIndex);
-        cont;
-        pRegistry->SetPath(cfgPath),
-        cont = pRegistry->GetNextGroup(groupName, groupIndex))
+   const auto cfgGroup = pRegistry->BeginGroup(cfgPath);
+   for(const auto& group : pRegistry->GetChildGroups())
    {
       PluginDescriptor plug;
+      const auto effectGroup = pRegistry->BeginGroup(group);
 
-      pRegistry->SetPath(groupName);
-
-      groupName = ConvertID(groupName);
+      auto groupName = ConvertID(group);
 
       // Bypass group if the ID is already in use
       if (mRegisteredPlugins.count(groupName))
@@ -684,7 +672,7 @@ void PluginManager::LoadGroup(FileConfig *pRegistry, PluginType type)
       plug.SetPluginType(type);
 
       // Get the provider ID and bypass group if not found
-      if (!pRegistry->Read(KEY_PROVIDERID, &strVal, wxEmptyString))
+      if (!pRegistry->Read(KEY_PROVIDERID, &strVal, {}))
       {
          // Bypass group if the provider isn't valid
          if (!strVal.empty() && !mRegisteredPlugins.count(strVal))
@@ -693,7 +681,7 @@ void PluginManager::LoadGroup(FileConfig *pRegistry, PluginType type)
       plug.SetProviderID(PluginID(strVal));
 
       // Get the path (optional)
-      pRegistry->Read(KEY_PATH, &strVal, wxEmptyString);
+      pRegistry->Read(KEY_PATH, &strVal, {});
       if (!AcceptPath(strVal))
          // Ignore the obsolete path in the config file, during session,
          // but don't remove it from the file.  Maybe you really want to
@@ -884,7 +872,7 @@ void PluginManager::Save()
    auto &registry = *pRegistry;
 
    // Clear pluginregistry.cfg (not audacity.cfg)
-   registry.DeleteAll();
+   registry.Clear();
 
    // Save the individual groups
    SaveGroup(&registry, PluginTypeEffect);
@@ -909,12 +897,17 @@ void PluginManager::Save()
    mRegver = REGVERCUR;
 }
 
+void PluginManager::NotifyPluginsChanged()
+{
+   Publisher<PluginsChangedMessage>::Publish({});
+}
+
 const PluginRegistryVersion &PluginManager::GetRegistryVersion() const
 {
    return mRegver;
 }
 
-void PluginManager::SaveGroup(FileConfig *pRegistry, PluginType type)
+void PluginManager::SaveGroup(audacity::BasicSettings *pRegistry, PluginType type)
 {
    wxString group = GetPluginTypeString(type);
    for (auto &pair : mRegisteredPlugins) {
@@ -925,7 +918,7 @@ void PluginManager::SaveGroup(FileConfig *pRegistry, PluginType type)
          continue;
       }
 
-      pRegistry->SetPath(REGROOT + group + wxCONFIG_PATH_SEPARATOR + ConvertID(plug.GetID()));
+      const auto pluginGroup = pRegistry->BeginGroup(REGROOT + group + wxCONFIG_PATH_SEPARATOR + ConvertID(plug.GetID()));
 
       pRegistry->Write(KEY_PATH, plug.GetPath());
 
@@ -1306,6 +1299,26 @@ wxString PluginManager::GetPluginTypeString(PluginType type)
    return str;
 }
 
+bool PluginManager::IsPluginAvailable(const PluginDescriptor& plug)
+{
+   const auto& providerID = plug.GetProviderID();
+   auto provider = ModuleManager::Get().CreateProviderInstance(providerID, wxEmptyString);
+
+   if (provider == nullptr)
+   {
+      wxLogWarning("Unable to find a provider for '%s'", providerID);
+      return false;
+   }
+
+   if (provider->CheckPluginExist(plug.GetPath()) == false)
+   {
+      wxLogWarning("Plugin '%s' does not exist", plug.GetID());
+      return false;
+   }
+
+   return true;
+}
+
 PluginDescriptor & PluginManager::CreatePlugin(const PluginID & id,
                                                ComponentInterface *ident,
                                                PluginType type)
@@ -1324,7 +1337,7 @@ PluginDescriptor & PluginManager::CreatePlugin(const PluginID & id,
    return plug;
 }
 
-FileConfig *PluginManager::GetSettings()
+audacity::BasicSettings *PluginManager::GetSettings()
 {
    if (!mSettings)
    {
@@ -1354,44 +1367,27 @@ FileConfig *PluginManager::GetSettings()
    return mSettings.get();
 }
 
-bool PluginManager::HasGroup(const RegistryPath & group)
+bool PluginManager::HasGroup(const RegistryPath & groupName)
 {
    auto settings = GetSettings();
 
-   bool res = settings->HasGroup(group);
-   if (res)
-   {
-      // The group exists, but empty groups aren't considered valid
-      wxString oldPath = settings->GetPath();
-      settings->SetPath(group);
-      res = settings->GetNumberOfEntries() || settings->GetNumberOfGroups();
-      settings->SetPath(oldPath);
-   }
-
-   return res;
+   if(!settings->HasGroup(groupName))
+      return false;
+   
+   auto group = settings->BeginGroup(groupName);
+   return !settings->GetChildGroups().empty() || !settings->GetChildKeys().empty();
 }
 
-bool PluginManager::GetSubgroups(const RegistryPath & group, RegistryPaths & subgroups)
+bool PluginManager::GetSubgroups(const RegistryPath & groupName, RegistryPaths & subgroups)
 {
-   if (group.empty() || !HasGroup(group))
+   if (groupName.empty() || !HasGroup(groupName))
    {
       return false;
    }
 
-   wxString path = GetSettings()->GetPath();
-   GetSettings()->SetPath(group);
-
-   wxString name;
-   long index = 0;
-   if (GetSettings()->GetFirstGroup(name, index))
-   {
-      do
-      {
-         subgroups.push_back(name);
-      } while (GetSettings()->GetNextGroup(name, index));
-   }
-
-   GetSettings()->SetPath(path);
+   auto group = GetSettings()->BeginGroup(groupName);
+   for(const auto& name : GetSettings()->GetChildGroups())
+      subgroups.push_back(name);
 
    return true;
 }
@@ -1401,9 +1397,12 @@ bool PluginManager::HasConfigValue(const RegistryPath & key)
    return GetSettings()->Exists(key);
 }
 
+template<typename T> class TD;
+
 bool PluginManager::GetConfigValue(
    const RegistryPath & key, ConfigReference var, ConfigConstReference defval)
 {
+   using namespace Variant;
    if (key.empty())
       return false;
    const auto visitor = [&](const auto var){
@@ -1412,7 +1411,9 @@ bool PluginManager::GetConfigValue(
       using Type = typename decltype(var)::type;
       const auto pDefval =
          std::get_if<std::reference_wrapper<const Type>>(&defval);
-      return GetSettings()->Read(key, pVar, *pDefval);
+      //TD<decltype(pDefval)> defType;
+      //return true;
+      return GetSettings()->Read(key, pVar, pDefval->get());
    };
    return Visit(visitor, var);
 }
@@ -1420,6 +1421,7 @@ bool PluginManager::GetConfigValue(
 bool PluginManager::SetConfigValue(
    const RegistryPath & key, ConfigConstReference value)
 {
+   using namespace Variant;
    if (key.empty())
       return false;
    const auto visitor = [&](const auto value){
