@@ -15,6 +15,7 @@ Paul Licameli -- split from SampleBlock.cpp and SampleBlock.h
 #include "DBConnection.h"
 #include "ProjectFileIO.h"
 #include "SampleFormat.h"
+#include "AudioSegmentSampleView.h"
 #include "XMLTagHandler.h"
 
 #include "SampleBlock.h" // to inherit
@@ -24,18 +25,26 @@ Paul Licameli -- split from SampleBlock.cpp and SampleBlock.h
 #include "SentryHelper.h"
 #include <wx/log.h>
 
+#include <mutex>
+
 class SqliteSampleBlockFactory;
 
 ///\brief Implementation of @ref SampleBlock using Sqlite database
 class SqliteSampleBlock final : public SampleBlock
 {
 public:
+   BlockSampleView GetFloatSampleView(bool mayThrow) override;
 
+private:
+   std::weak_ptr<std::vector<float>> mCache;
+   std::mutex mCacheMutex;
+
+public:
    explicit SqliteSampleBlock(
       const std::shared_ptr<SqliteSampleBlockFactory> &pFactory);
    ~SqliteSampleBlock() override;
 
-   void CloseLock() override;
+   void CloseLock() noexcept override;
 
    void SetSamples(
       constSamplePtr src, size_t numsamples, sampleFormat srcformat);
@@ -160,7 +169,7 @@ private:
    void OnEndPurge();
 
    friend SqliteSampleBlock;
-   
+
    AudacityProject &mProject;
    Observer::Subscription mUndoSubscription;
    std::optional<SampleBlock::DeletionCallback::Scope> mScope;
@@ -289,6 +298,39 @@ SampleBlockPtr SqliteSampleBlockFactory::DoCreateFromXML(
    return sb;
 }
 
+BlockSampleView SqliteSampleBlock::GetFloatSampleView(bool mayThrow)
+{
+   assert(mSampleCount > 0);
+
+   // Double-checked locking.
+   // `weak_ptr::lock()` guarantees atomicity, which is important to make this
+   // work without races.
+   auto cache = mCache.lock();
+   if (cache)
+      return cache;
+   std::lock_guard<std::mutex> lock(mCacheMutex);
+   cache = mCache.lock();
+   if (cache)
+      return cache;
+
+   const auto newCache =
+      std::make_shared<std::vector<float>>(mSampleCount);
+   try {
+      const auto cachedSize = DoGetSamples(
+         reinterpret_cast<samplePtr>(newCache->data()), floatSample, 0,
+         mSampleCount);
+      assert(cachedSize == mSampleCount);
+   }
+   catch (...)
+   {
+      if (mayThrow)
+         std::rethrow_exception(std::current_exception());
+      std::fill(newCache->begin(), newCache->end(), 0.f);
+   }
+   mCache = newCache;
+   return newCache;
+}
+
 SqliteSampleBlock::SqliteSampleBlock(
    const std::shared_ptr<SqliteSampleBlockFactory> &pFactory)
 :  mpFactory(pFactory)
@@ -348,7 +390,7 @@ DBConnection *SqliteSampleBlock::Conn() const
    return pConnection.get();
 }
 
-void SqliteSampleBlock::CloseLock()
+void SqliteSampleBlock::CloseLock() noexcept
 {
    mLocked = true;
 }
@@ -578,7 +620,7 @@ size_t SqliteSampleBlock::GetBlob(void *dest,
       // Just showing the user a simple message, not the library error too
       // which isn't internationalized
       // Actually this can lead to 'Could not read from file' error message
-      // but it can also lead to no error message at all and a flat line, 
+      // but it can also lead to no error message at all and a flat line,
       // depending on where GetBlob is called from.
       // The latter can happen when repainting the screen.
       // That possibly happens on a very slow machine.  Possibly that's the
@@ -602,13 +644,13 @@ size_t SqliteSampleBlock::GetBlob(void *dest,
    /*
     Will dithering happen in CopySamples?  Answering this as of 3.0.3 by
     examining all uses.
-    
+
     As this function is called from GetSummary, no, because destination format
     is float.
 
     There is only one other call to this function, in DoGetSamples.  At one
     call to that function, in DoGetMinMaxRMS, again format is float always.
-    
+
     There is only one other call to DoGetSamples, in SampleBlock::GetSamples().
     In one call to that function, in WaveformView.cpp, again format is float.
 
@@ -756,7 +798,7 @@ void SqliteSampleBlock::Commit(Sizes sizes)
 
       wxASSERT_MSG(false, wxT("Binding failed...bug!!!"));
    }
- 
+
    // Execute the statement
    rc = sqlite3_step(stmt);
    if (rc != SQLITE_DONE)
@@ -782,6 +824,10 @@ void SqliteSampleBlock::Commit(Sizes sizes)
    mSamples.reset();
    mSummary256.reset();
    mSummary64k.reset();
+   {
+      std::lock_guard<std::mutex> lock(mCacheMutex);
+      mCache.reset();
+   }
 
    // Clear statement bindings and rewind statement
    sqlite3_clear_bindings(stmt);
@@ -877,7 +923,7 @@ void SqliteSampleBlock::CalcSummary(Sizes sizes)
          samplebuffer.get(), mSampleCount);
       samples = samplebuffer.get();
    }
-   
+
    mSummary256.reinit(mSummary256Bytes);
    mSummary64k.reinit(mSummary64kBytes);
 
@@ -1051,7 +1097,7 @@ void SqliteSampleBlockFactory::OnBeginPurge(size_t begin, size_t end)
 {
    // Install a callback function that updates a progress indicator
    using namespace BasicUI;
-   
+
    //Avoid showing dialog to the user if purge operation
    //does not take much time, as it will resign focus from main window
    //but dialog itself may not be presented to the user at all.
