@@ -251,7 +251,7 @@ public:
 
 //-- Add functions.  These only add a widget or 2.
    void HandleOptionality(const TranslatableString &Prompt);
-   void AddPrompt(const TranslatableString &Prompt, int wrapWidth = 0);
+   wxStaticText* AddPrompt(const TranslatableString &Prompt, int wrapWidth = 0);
    void AddUnits(const TranslatableString &Prompt, int wrapWidth = 0);
    void AddTitle(const TranslatableString &Prompt, int wrapWidth = 0);
    wxWindow * AddWindow(wxWindow* pWindow, int PositionFlags = wxALIGN_CENTRE);
@@ -294,8 +294,8 @@ public:
       const TranslatableString &Caption,
       const wxString &Value, const int nChars);
    wxTextCtrl * AddNumericTextBox(
-      const TranslatableString &Caption,
-      const wxString &Value, const int nChars);
+      const TranslatableString &Caption, const wxString& Value,
+      const int nChars, bool acceptEnter = false);
    wxTextCtrl * AddTextWindow(const wxString &Value);
    wxListBox * AddListBox(const wxArrayStringEx &choices);
 
@@ -400,8 +400,12 @@ public:
    wxTextCtrl * TieTextBox(
       const TranslatableString &Prompt, double &Value, const int nChars=0);
 
-   wxTextCtrl * TieNumericTextBox( const TranslatableString &Prompt, int &Value, const int nChars=0);
-   wxTextCtrl * TieNumericTextBox( const TranslatableString &Prompt, double &Value, const int nChars=0);
+   wxTextCtrl* TieNumericTextBox(
+      const TranslatableString& Prompt, int& Value, const int nChars = 0,
+      bool acceptEnter = false);
+   wxTextCtrl* TieNumericTextBox(
+      const TranslatableString& Prompt, double& Value, const int nChars = 0,
+      bool acceptEnter = false);
 
    wxCheckBox * TieCheckBox( const TranslatableString &Prompt, bool & Var );
    wxCheckBox * TieCheckBoxOnRight( const TranslatableString & Prompt, bool & Var );
@@ -469,7 +473,7 @@ public:
    virtual wxTextCtrl * TieNumericTextBox(
       const TranslatableString & Prompt,
       const DoubleSetting &Setting,
-      const int nChars);
+      const int nChars, bool acceptEnter = false);
    virtual wxSlider * TieSlider(
       const TranslatableString & Prompt,
       const IntSetting &Setting,
@@ -557,7 +561,8 @@ private:
       const TranslatableString &Prompt,
       WrappedType &  WrappedRef, const int nChars);
    wxTextCtrl * DoTieNumericTextBox(
-      const TranslatableString &Prompt, WrappedType &  WrappedRef, const int nChars);
+      const TranslatableString& Prompt, WrappedType& WrappedRef,
+      const int nChars, bool acceptEnter = false);
    wxCheckBox * DoTieCheckBox( const TranslatableString &Prompt, WrappedType & WrappedRef );
    wxSlider * DoTieSlider(
       const TranslatableString &Prompt,
