@@ -43,6 +43,10 @@ deps=(
    mpg123
    wavpack
    wxwidgets-gtk3
+   vst3sdk
+   rapidjson
+   opusfile
+   opus
 )
 
 pacman -Syu --noconfirm \
