@@ -28,13 +28,9 @@ hold information about one contributor to Audacity.
 
 #include "AboutDialog.h"
 
-#include <wx/dialog.h>
 #include <wx/html/htmlwin.h>
-#include <wx/button.h>
 #include <wx/hyperlink.h>
-#include <wx/sizer.h>
 #include <wx/statbmp.h>
-#include <wx/intl.h>
 #include <wx/sstream.h>
 #include <wx/txtstrm.h>
 #include <wx/statbox.h>
@@ -43,8 +39,8 @@ hold information about one contributor to Audacity.
 #include "FileNames.h"
 #include "HelpText.h"
 #include "ShuttleGui.h"
-#include "widgets/HelpSystem.h"
-#include "ui/AccessibleLinksFormatter.h"
+#include "HelpSystem.h"
+#include "AccessibleLinksFormatter.h"
 
 #include "AllThemeResources.h"
 #include "Theme.h"
@@ -132,31 +128,24 @@ void AboutDialog::CreateCreditsList()
       XO("%s, graphics");
 
    // The Audacity Team: developers and support
-
-   AddCredit(wxT("Gonzalo Guzm\u00E1n"), documentationAndSupportFormat, roleTeamMember);
+   AddCredit(wxT("Matthieu Hodgkinson"), developerFormat, roleTeamMember);
    AddCredit(wxT("Peter Jonas"), developerFormat, roleTeamMember);
    AddCredit(wxT("Martin Keary"), roleTeamMember);
    AddCredit(wxT("Sergey Lapysh"), testerFormat, roleTeamMember);
+   AddCredit(wxT("Yana Larina"), roleTeamMember);
    AddCredit(wxT("Paul Licameli"), developerFormat, roleTeamMember);
-   AddCredit(wxT("Pietro Marcello"), developerFormat, roleTeamMember);
+   AddCredit(wxT("Dilson's Pickles"), designerFormat, roleTeamMember);
    AddCredit(wxT("K. Soze"), developerFormat, roleTeamMember);
    AddCredit(wxT("Anita Sudan"), roleTeamMember);
    AddCredit(wxT("Vitaly Sverchinsky"), developerFormat, roleTeamMember);
    AddCredit(wxT("Dmitry Vedenko"), developerFormat, roleTeamMember);
-   AddCredit(wxT("Leo Wattenberg"), documentationAndSupportFormat, roleTeamMember);
+   AddCredit(wxT("Leo Wattenberg"), designerFormat, roleTeamMember);
    AddCredit(wxT("Jessica Williamson"), designerFormat, roleTeamMember);
    
-   // Former Musers
-   AddCredit(wxT("Anton Gerasimov"), developerFormat, roleExMuse);
-   AddCredit(wxT("Jouni Helminen"), designerFormat, roleExMuse);
-   AddCredit(wxT("Pavel Penikov"), testerFormat, roleExMuse);
-   
    // Emeritus: people who were "lead developers" or made an
    // otherwise distinguished contribution, but who are no
    // longer active.
-   AddCredit(
-      wxT("[[https://wiki.audacityteam.org/wiki/User:Galeandrews|Gale Andrews]]"),
-      qualityAssuranceFormat, roleEmeritusTeam);
+   AddCredit(wxT("Gale Andrews"), qualityAssuranceFormat, roleEmeritusTeam);
    AddCredit(wxT("Richard Ash"), developerFormat, roleEmeritusTeam);
    AddCredit(wxT("Christian Brochec"),
       documentationAndSupportFrenchFormat, roleEmeritusTeam);
@@ -198,10 +187,13 @@ void AboutDialog::CreateCreditsList()
    AddCredit(wxT("Cory Cook"), developerFormat, roleContributor);
    AddCredit(wxT("Craig DeForest"), developerFormat, roleContributor);
    AddCredit(wxT("Edgar Franke (Edgar-RFT)"), developerFormat, roleContributor);
+   AddCredit(wxT("Anton Gerasimov"), developerFormat, roleContributor);
    AddCredit(wxT("Mitch Golden"), developerFormat, roleContributor);
    AddCredit(wxT("Brian Gunlogson"), developerFormat, roleContributor);
+   AddCredit(wxT("Gonzalo Guzm\u00E1n"), documentationAndSupportFormat, roleContributor);
    AddCredit(wxT("Andrew Hallendorff"), developerFormat, roleContributor);
    AddCredit(wxT("Robert H\u00E4nggi"), developerFormat, roleContributor);
+   AddCredit(wxT("Jouni Helminen"), designerFormat, roleContributor);
    AddCredit(wxT("Daniel Horgan"), developerFormat, roleContributor);
    AddCredit(wxT("David Hostetler"), developerFormat, roleContributor);
    AddCredit(wxT("Edward Hui"), developerFormat, roleContributor);
@@ -213,10 +205,13 @@ void AboutDialog::CreateCreditsList()
    AddCredit(wxT("Paul Livesey"), developerFormat, roleContributor);
    AddCredit(wxT("Harvey Lubin"), graphicArtistFormat, roleContributor);
    AddCredit(wxT("Max Maisel"), developerFormat, roleContributor);
+   AddCredit(wxT("Pietro Marcello"), developerFormat, roleContributor);
    AddCredit(wxT("Greg Mekkes"), developerFormat, roleContributor);
    AddCredit(wxT("Abe Milde"), developerFormat, roleContributor);
+   AddCredit(wxT("Ryan Miller"), testerFormat, roleContributor);
    AddCredit(wxT("Paul Nasca"), developerFormat, roleContributor);
    AddCredit(wxT("Clayton Otey"), developerFormat, roleContributor);
+   AddCredit(wxT("Pavel Penikov"), testerFormat, roleContributor);
    AddCredit(wxT("Mark Phillips"), developerFormat, roleContributor);
    AddCredit(wxT("Andr\u00E9 Pinto"), developerFormat, roleContributor);
    AddCredit(wxT("Pokechu22"), developerFormat, roleContributor);
@@ -386,12 +381,6 @@ void AboutDialog::PopulateAudacityPage( ShuttleGui & S )
       << XO("%s Team Members").Format( ProgramName )
       << wxT("</b><br>")
       << GetCreditsByRole(roleTeamMember)
-	  
-	  << wxT("<p><b>")
-	  /* i18n-hint: Musers are people working at Muse Group */
-      << XO("Former Musers")
-      << wxT("</b><br>")
-      << GetCreditsByRole(roleExMuse)
 
       << wxT("<p><b> ")
       << XO("Emeritus:")
@@ -442,7 +431,7 @@ void AboutDialog::PopulateAudacityPage( ShuttleGui & S )
       << wxT("<p><br>&nbsp; &nbsp; ")
       /* i18n-hint Audacity's name substitutes for first and third %s,
        and a "copyright" symbol for the second */
-      << XO("%s software is copyright %s 1999-2021 %s Team.")
+      << XO("%s software is copyright %s 1999-2023 %s Team.")
          .Format(
             Verbatim("<b>%s<sup>&reg;</sup></b>").Format( ProgramName ),
             wxT("&copy;"),
@@ -463,10 +452,8 @@ void AboutDialog::PopulateAudacityPage( ShuttleGui & S )
       //v For now, change to AudacityLogoWithName via old-fashioned way, not Theme.
       wxBitmap logo(AudacityLogoWithName_xpm); //v
 
-      // JKC: Resize to 50% of size.  Later we may use a smaller xpm as
-      // our source, but this allows us to tweak the size - if we want to.
-      // It also makes it easier to revert to full size if we decide to.
-      const float fScale = 0.5f;// smaller size.
+      //Setup to scale the logo larger and smaller as necessary
+      const float fScale = 1.0f;
       wxImage RescaledImage(logo.ConvertToImage());
       wxColour MainColour(
          RescaledImage.GetRed(1,1),
@@ -524,11 +511,6 @@ void AboutDialog::PopulateInformationPage( ShuttleGui & S )
 
    /* this builds up the list of information to go in the window in the string
     * informationStr */
-   informationStr
-      << wxT("<h2><center>")
-      << XO("Build Information")
-      << wxT("</center></h2>\n")
-      << VerCheckHtml();
  
    informationStr
       << wxT("<h3>")
@@ -624,14 +606,24 @@ void AboutDialog::PopulateInformationPage( ShuttleGui & S )
    informationStr
       << wxT("<table>");   // start table of file formats supported
 
-
-   #if defined(USE_LIBMAD)
-   /* i18n-hint: This is what the library (libmad) does - imports MP3 files */
-   AddBuildinfoRow(&informationStr, wxT("libmad"), XO("MP3 Importing"), enabled);
-   #elif defined(USE_LIBID3TAG)
-   AddBuildinfoRow(&informationStr, wxT("libmpg123"), XO("MP3 Importing"), enabled);
+   #if defined(USE_LIBMPG123)
+   AddBuildinfoRow(&informationStr, wxT("libmpg123"), XO("MP3 Import"), enabled);
    #else
-   AddBuildinfoRow(&informationStr, wxT("libmad"), XO("MP3 Importing"), disabled);
+   AddBuildinfoRow(&informationStr, wxT("libmpg123"), XO("MP3 Import"), disabled);
+   #endif
+
+   #if USE_LIBMP3LAME
+   AddBuildinfoRow(
+      &informationStr, wxT("libmp3lame"),
+      /* i18n-hint: LAME is the codec name. This name should not be translated
+       */
+      XO("MP3 Export"), enabled);
+   #else
+   AddBuildinfoRow(
+      &informationStr, wxT("libopus"),
+      /* i18n-hint: Opus is the codec name. This name should not be translated
+       */
+      XO("Opus Import and Export"), disabled);
    #endif
 
    #ifdef USE_LIBVORBIS
@@ -644,6 +636,19 @@ void AboutDialog::PopulateInformationPage( ShuttleGui & S )
          XO("Ogg Vorbis Import and Export"), disabled);
    #endif
 
+   #if USE_LIBVORBIS && USE_LIBOPUS && USE_OPUSFILE
+   AddBuildinfoRow(
+      &informationStr, wxT("libopus"),
+      /* i18n-hint: Opus is the codec name. This name should not be translated */
+      XO("Opus Import and Export"), enabled);
+   #else
+   AddBuildinfoRow(
+      &informationStr, wxT("libopus"),
+      /* i18n-hint: Opus is the codec name. This name should not be translated
+       */
+      XO("Opus Import and Export"), disabled);
+   #endif
+
    #ifdef USE_LIBID3TAG
    AddBuildinfoRow(&informationStr, wxT("libid3tag"), XO("ID3 tag support"),
          enabled);
@@ -670,26 +675,12 @@ void AboutDialog::PopulateInformationPage( ShuttleGui & S )
          disabled);
    # endif
 
-   # if USE_QUICKTIME
-   AddBuildinfoRow(&informationStr, wxT("QuickTime"), XO("Import via QuickTime"),
-         enabled);
-   # else
-   AddBuildinfoRow(&informationStr, wxT("QuickTime"), XO("Import via QuickTime"),
-         disabled);
-   # endif
-
    #ifdef USE_FFMPEG
    AddBuildinfoRow(&informationStr, wxT("ffmpeg"), XO("FFmpeg Import/Export"), enabled);
    #else
    AddBuildinfoRow(&informationStr, wxT("ffmpeg"), XO("FFmpeg Import/Export"), disabled);
    #endif
 
-   #ifdef USE_GSTREAMER
-   AddBuildinfoRow(&informationStr, wxT("gstreamer"), XO("Import via GStreamer"), enabled);
-   #else
-   AddBuildinfoRow(&informationStr, wxT("gstreamer"), XO("Import via GStreamer"), disabled);
-   #endif
-
    informationStr << wxT("</table>\n");  //end table of file formats supported
 
    informationStr
