@@ -29,16 +29,18 @@
 #include "AllThemeResources.h"
 #include "AudioIO.h"
 #include "widgets/BasicMenu.h"
+#include "Beats.h"
 #include "CellularPanel.h"
 #include "../images/Cursors.h"
 #include "HitTestResult.h"
-#include "Menus.h"
 #include "Prefs.h"
 #include "Project.h"
 #include "ProjectAudioIO.h"
 #include "ProjectAudioManager.h"
 #include "ProjectWindows.h"
 #include "ProjectStatus.h"
+#include "ProjectTimeRuler.h"
+#include "ProjectTimeSignature.h"
 #include "ProjectWindow.h"
 #include "RefreshCode.h"
 #include "SelectUtilities.h"
@@ -50,12 +52,16 @@
 #include "prefs/TracksPrefs.h"
 #include "prefs/ThemePrefs.h"
 #include "toolbars/ToolBar.h"
+#include "toolbars/ToolManager.h"
 #include "tracks/ui/Scrubbing.h"
-#include "tracks/ui/TrackView.h"
+#include "tracks/ui/ChannelView.h"
 #include "widgets/AButton.h"
-#include "widgets/AudacityMessageBox.h"
+#include "AudacityMessageBox.h"
 #include "widgets/Grabber.h"
-#include "widgets/wxWidgetsWindowPlacement.h"
+#include "widgets/LinearUpdater.h"
+#include "widgets/BeatsFormat.h"
+#include "widgets/TimeFormat.h"
+#include "wxWidgetsWindowPlacement.h"
 
 #include <wx/dcclient.h>
 #include <wx/menu.h>
@@ -206,6 +212,18 @@ protected:
 
 class AdornedRulerPanel::PlayRegionAdjustingHandle : public CommonRulerHandle {
 public:
+   PlayRegionAdjustingHandle(
+      AdornedRulerPanel *pParent, wxCoord xx, MenuChoice menuChoice,
+      wxCursor *cursor,
+      size_t numGuides = 1)
+   : CommonRulerHandle{ pParent, xx, menuChoice }
+   , mNumGuides{ numGuides }
+   {
+      if (cursor) {
+         mCursor = *cursor;
+      }
+   }
+
    PlayRegionAdjustingHandle(
       AdornedRulerPanel *pParent, wxCoord xx, MenuChoice menuChoice,
       wxCursor cursor,
@@ -215,6 +233,11 @@ public:
    , mNumGuides{ numGuides }
    {}
 
+   void SetCursor(wxCursor cursor)
+   {
+      mCursor = cursor;
+   }
+
    HitTestPreview Preview(
       const TrackPanelMouseState &state, AudacityProject *pProject)
    override
@@ -643,8 +666,8 @@ void AdornedRulerPanel::TrackPanelGuidelineOverlay::Draw(
       }
       pCellularPanel
          ->VisitCells( [&]( const wxRect &rect, TrackPanelCell &cell ) {
-            const auto pTrackView = dynamic_cast<TrackView*>(&cell);
-            if (!pTrackView)
+            const auto pChannelView = dynamic_cast<ChannelView*>(&cell);
+            if (!pChannelView)
                return;
 
             // Draw the NEW indicator in its NEW location
@@ -666,7 +689,9 @@ void AdornedRulerPanel::TrackPanelGuidelineOverlay::Draw(
 **********************************************************************/
 
 enum {
-   OnSyncQuickPlaySelID = 7000,
+   OnMinutesAndSecondsID = 7000,
+   OnBeatsAndMeasuresID,
+   OnSyncQuickPlaySelID,
    OnAutoScrollID,
    OnTogglePlayRegionID,
    OnClearPlayRegionID,
@@ -681,6 +706,8 @@ BEGIN_EVENT_TABLE(AdornedRulerPanel, CellularPanel)
    EVT_LEAVE_WINDOW(AdornedRulerPanel::OnLeave)
 
    // Context menu commands
+   EVT_MENU(OnMinutesAndSecondsID, AdornedRulerPanel::OnTimelineFormatChange)
+   EVT_MENU(OnBeatsAndMeasuresID, AdornedRulerPanel::OnTimelineFormatChange)
    EVT_MENU(OnSyncQuickPlaySelID, AdornedRulerPanel::OnSyncSelToQuickPlay)
    EVT_MENU(OnAutoScrollID, AdornedRulerPanel::OnAutoScroll)
    EVT_MENU(OnTogglePlayRegionID, AdornedRulerPanel::OnTogglePlayRegion)
@@ -767,14 +794,17 @@ private:
 };
 #endif
 
-static auto handOpenCursor =
-    MakeCursor(wxCURSOR_HAND, RearrangeCursorXpm, 16, 16);
+static std::unique_ptr<wxCursor> handOpenCursor = nullptr;
 
 class AdornedRulerPanel::MovePlayRegionHandle final : public PlayRegionAdjustingHandle {
 public:
    MovePlayRegionHandle( AdornedRulerPanel *pParent, wxCoord xx )
-   : PlayRegionAdjustingHandle( pParent, xx, MenuChoice::QuickPlay, *handOpenCursor, 2 )
+   : PlayRegionAdjustingHandle( pParent, xx, MenuChoice::QuickPlay, handOpenCursor.get(), 2 )
    {
+      if (!handOpenCursor) {
+         handOpenCursor = MakeCursor(wxCURSOR_HAND, RearrangeCursorXpm, 16, 16);
+         SetCursor(*handOpenCursor);
+      }
    }
 
    ~MovePlayRegionHandle()
@@ -1255,9 +1285,11 @@ AdornedRulerPanel::AdornedRulerPanel(AudacityProject* project,
    wxWindowID id,
    const wxPoint& pos,
    const wxSize& size,
-   ViewInfo *viewinfo)
-:  CellularPanel(parent, id, pos, size, viewinfo)
-, mProject(project)
+   ViewInfo *viewinfo
+)  : CellularPanel(parent, id, pos, size, viewinfo)
+   , mProject { project }
+   , mUpdater { ProjectTimeRuler::Get(*project).GetUpdater() }
+   , mRuler { ProjectTimeRuler::Get(*project).GetRuler() }   
 {
    SetLayoutDirection(wxLayout_LeftToRight);
 
@@ -1280,9 +1312,11 @@ AdornedRulerPanel::AdornedRulerPanel(AudacityProject* project,
 
    mOuter = GetClientRect();
 
-   mRuler.SetUseZoomInfo(mLeftOffset, mViewInfo);
+   mTimeDisplayMode = TimeDisplayModePreference.ReadEnum();
+
+   mUpdater.SetData(mViewInfo, mLeftOffset);
+
    mRuler.SetLabelEdges( false );
-   mRuler.SetFormat( Ruler::TimeFormat );
 
    mTracks = &TrackList::Get( *project );
 
@@ -1308,6 +1342,9 @@ AdornedRulerPanel::AdornedRulerPanel(AudacityProject* project,
    mPlayRegionSubscription = mViewInfo->selectedRegion.Subscribe(
       *this, &AdornedRulerPanel::OnSelectionChange);
 
+   mRulerInvalidatedSubscription =
+      mRuler.Subscribe([this](auto) { Refresh(); });
+   
    // And call it once to initialize it
    DoSelectionChange( mViewInfo->selectedRegion );
 }
@@ -1345,6 +1382,10 @@ void AdornedRulerPanel::UpdatePrefs()
    }
 #endif
 #endif
+
+   mTimeDisplayMode = TimeDisplayModePreference.ReadEnum();
+   Refresh();
+   // Update();
 }
 
 void AdornedRulerPanel::ReCreateButtons()
@@ -1363,14 +1404,14 @@ void AdornedRulerPanel::ReCreateButtons()
    }
 
    size_t iButton = 0;
-   // Make the short row of time ruler pushbottons.
+   // Make the short row of time ruler push buttons.
    // Don't bother with sizers.  Their sizes and positions are fixed.
    // Add a grabber converted to a spacer.
    // This makes it visually clearer that the button is a button.
 
    wxPoint position( 1, 0 );
 
-   Grabber * pGrabber = safenew Grabber(this, this->GetId());
+   Grabber * pGrabber = safenew Grabber(this, {});
    pGrabber->SetAsSpacer( true );
    //pGrabber->SetSize( 10, 27 ); // default is 10,27
    pGrabber->SetPosition( position );
@@ -1979,7 +2020,7 @@ auto AdornedRulerPanel::QPHandle::Release(
 
 void AdornedRulerPanel::HandleQPRelease(wxMouseEvent &evt)
 {
-   auto &viewInfo = ViewInfo::Get( *GetProject() );
+   auto &viewInfo = ViewInfo::Get(*GetProject());
    auto &playRegion = viewInfo.playRegion;
    playRegion.Order();
 
@@ -2093,7 +2134,7 @@ void AdornedRulerPanel::StartQPPlay(
       newDefault = (loopEnabled && newDefault);
       if (newDefault)
          cutPreview = false;
-      auto options = DefaultPlayOptions( *mProject, newDefault );
+      auto options = ProjectAudioIO::GetDefaultOptions(*mProject, newDefault);
 
       if (!cutPreview) {
          if (pStartTime)
@@ -2230,7 +2271,21 @@ void AdornedRulerPanel::ShowMenu(const wxPoint & pos)
    const auto &playRegion = viewInfo.playRegion;
    wxMenu rulerMenu;
 
-   auto pDrag = rulerMenu.AppendCheckItem(OnSyncQuickPlaySelID, _("Enable dragging selection"));
+   {
+      auto item = rulerMenu.AppendRadioItem(OnMinutesAndSecondsID,
+         _("Minutes and Seconds"));
+      item->Check(mTimeDisplayMode == TimeDisplayMode::MinutesAndSeconds);
+   }
+
+   {
+      auto item = rulerMenu.AppendRadioItem(OnBeatsAndMeasuresID,
+         _("Beats and Measures"));
+      item->Check(mTimeDisplayMode == TimeDisplayMode::BeatsAndMeasures);
+   }
+
+   rulerMenu.AppendSeparator();
+
+   auto pDrag = rulerMenu.AppendCheckItem(OnSyncQuickPlaySelID, _("Setting a loop region also makes an audio selection"));
    pDrag->Check(mPlayRegionDragsSelection && playRegion.Active());
    pDrag->Enable(playRegion.Active());
 
@@ -2278,13 +2333,6 @@ void AdornedRulerPanel::ShowScrubMenu(const wxPoint & pos)
    );
 }
 
-void AdornedRulerPanel::OnSyncSelToQuickPlay(wxCommandEvent&)
-{
-   mPlayRegionDragsSelection = (mPlayRegionDragsSelection)? false : true;
-   gPrefs->Write(wxT("/QuickPlay/DragSelection"), mPlayRegionDragsSelection);
-   gPrefs->Flush();
-}
-
 void AdornedRulerPanel::DragSelection(AudacityProject &project)
 {
    auto &viewInfo = ViewInfo::Get( project );
@@ -2310,6 +2358,27 @@ void AdornedRulerPanel::HandleSnapping(size_t index)
    mIsSnapped[index] = results.Snapped();
 }
 
+void AdornedRulerPanel::OnTimelineFormatChange(wxCommandEvent& event)
+{
+   int id = event.GetId();
+   TimeDisplayMode changeFlag = mTimeDisplayMode;
+   wxASSERT(id == OnMinutesAndSecondsID || id == OnBeatsAndMeasuresID);
+   mTimeDisplayMode = id == OnBeatsAndMeasuresID ? TimeDisplayMode::BeatsAndMeasures :
+                                             TimeDisplayMode::MinutesAndSeconds;
+   
+   TimeDisplayModePreference.WriteEnum(mTimeDisplayMode);
+
+   if (changeFlag != mTimeDisplayMode)
+      Refresh();
+}
+
+void AdornedRulerPanel::OnSyncSelToQuickPlay(wxCommandEvent&)
+{
+   mPlayRegionDragsSelection = (mPlayRegionDragsSelection) ? false : true;
+   gPrefs->Write(wxT("/QuickPlay/DragSelection"), mPlayRegionDragsSelection);
+   gPrefs->Flush();
+}
+
 #if 0
 void AdornedRulerPanel::OnTimelineToolTips(wxCommandEvent&)
 {
@@ -2319,6 +2388,7 @@ void AdornedRulerPanel::OnTimelineToolTips(wxCommandEvent&)
 }
 #endif
 
+
 void AdornedRulerPanel::OnAutoScroll(wxCommandEvent&)
 {
    if (mViewInfo->bUpdateTrackIndicator)
@@ -2452,6 +2522,14 @@ void AdornedRulerPanel::DoDrawMarks(wxDC * dc, bool /*text */ )
 
    mRuler.SetTickColour( theTheme.Colour( TimelineTextColor() ) );
    mRuler.SetRange( min, max, hiddenMin, hiddenMax );
+   if (mTimeDisplayMode == TimeDisplayMode::BeatsAndMeasures)
+   {
+      mRuler.SetTickLengths({ 5, 3, 1 });
+   }
+   else if (mTimeDisplayMode == TimeDisplayMode::MinutesAndSeconds)
+   {
+      mRuler.SetTickLengths({ 4, 2, 2 });
+   }
    mRuler.Draw( *dc );
 }
 
@@ -2496,12 +2574,13 @@ void AdornedRulerPanel::DoDrawPlayRegion(
    wxDC * dc, const wxRect &rectP, const wxRect &rectL, const wxRect &rectR)
 {
    const auto &viewInfo = ViewInfo::Get(*mProject);
-   const auto &playRegion = viewInfo.playRegion;
-   if (playRegion.IsLastActiveRegionClear())
-      return;
+   const auto& playRegion = viewInfo.playRegion;
 
    const bool isActive = (mLastPlayRegionActive = playRegion.Active());
 
+   if (playRegion.IsLastActiveRegionClear())
+      return;
+
    // Paint the selected region bolder if independently varying, else dim
    const auto color = TimelineLoopRegionColor(isActive);
    dc->SetBrush( wxBrush( theTheme.Colour( color )) );
@@ -2576,8 +2655,11 @@ int AdornedRulerPanel::GetRulerHeight(bool showScrubBar)
 
 void AdornedRulerPanel::SetLeftOffset(int offset)
 {
-   mLeftOffset = offset;
-   mRuler.SetUseZoomInfo(offset, mViewInfo);
+   if (mLeftOffset != offset) {
+      mLeftOffset = offset;
+      mUpdater.SetData(mViewInfo, offset);
+      mRuler.Invalidate();
+   }
 }
 
 // Draws the scrubbing/seeking indicator.
@@ -2662,7 +2744,7 @@ void AdornedRulerPanel::ClearPlayRegion()
 
    auto &viewInfo = ViewInfo::Get( *GetProject() );
    auto &playRegion = viewInfo.playRegion;
-   playRegion.SetTimes( -1, -1 );
+   playRegion.Clear();
 
    Refresh();
 }
@@ -2796,7 +2878,7 @@ void AdornedRulerPanel::TogglePinnedHead()
 {
    bool value = !TracksPrefs::GetPinnedHeadPreference();
    TracksPrefs::SetPinnedHeadPreference(value, true);
-   MenuManager::ModifyAllProjectToolbarMenus();
+   ToolManager::ModifyAllProjectToolbarMenus();
 
    auto &project = *mProject;
    // Update button image
@@ -2806,3 +2888,44 @@ void AdornedRulerPanel::TogglePinnedHead()
    if (scrubber.HasMark())
       scrubber.SetScrollScrubbing(value);
 }
+
+TimeDisplayMode AdornedRulerPanel::GetTimeDisplayMode() const
+{
+   return mTimeDisplayMode;
+}
+
+void AdornedRulerPanel::SetTimeDisplayMode(TimeDisplayMode type)
+{
+   if (mTimeDisplayMode == type)
+      return;
+
+   mTimeDisplayMode = type;
+   TimeDisplayModePreference.WriteEnum(mTimeDisplayMode);
+   Refresh();
+}
+
+// Attach menu item
+
+#include "commands/CommandContext.h"
+#include "commands/CommandManager.h"
+#include "CommonCommandFlags.h"
+
+namespace {
+void OnTogglePinnedHead(const CommandContext &context)
+{
+   AdornedRulerPanel::Get( context.project ).TogglePinnedHead();
+}
+
+using namespace MenuTable;
+using Options = CommandManager::Options;
+AttachedItem sAttachment{
+   { wxT("Transport/Other/Options/Part2"), { OrderingHint::Begin, {} } },
+   Command( wxT("PinnedHead"), XXO("Enable pinned play &head"),
+      OnTogglePinnedHead,
+      // Switching of scrolling on and off is permitted
+      // even during transport
+      AlwaysEnabledFlag,
+      Options{}.CheckTest([](const AudacityProject&){
+         return TracksPrefs::GetPinnedHeadPreference(); } ) )
+};
+}
