@@ -11,7 +11,6 @@
 #ifndef __AUDACITY_ADORNED_RULER_PANEL__
 #define __AUDACITY_ADORNED_RULER_PANEL__
 
-#include "widgets/BeatsFormat.h"
 #include "CellularPanel.h"
 #include "widgets/Ruler.h" // member variable
 #include "widgets/LinearUpdater.h"
@@ -19,6 +18,7 @@
 #include "Observer.h"
 #include "Prefs.h"
 #include "ViewInfo.h" // for PlayRegion
+#include "TimeDisplayMode.h"
 
 class AudacityProject;
 struct AudioIOEvent;
@@ -135,16 +135,11 @@ private:
    bool IsWithinMarker(int mousePosX, double markerTime);
 
 private:
+   AudacityProject* const mProject;
+   
+   LinearUpdater& mUpdater;
+   Ruler& mRuler;
 
-   // Stateless formatter object, but not used by default...
-   BeatsFormat mBeatsFormat;
-
-   LinearUpdater mUpdater;
-
-   // ... Time formatter used by default instead
-   Ruler mRuler{ mUpdater, TimeFormat::Instance() };
-
-   AudacityProject *const mProject;
    TrackList *mTracks;
 
    wxRect mOuter;
@@ -174,8 +169,6 @@ private:
    void ShowScrubMenu(const wxPoint & pos);
    static void DragSelection(AudacityProject &project);
    void HandleSnapping(size_t index);
-   void UpdateBeatsAndMeasuresFormat();
-   void RefreshTimelineFormat();
    void OnTimelineFormatChange(wxCommandEvent& evt);
    void OnSyncSelToQuickPlay(wxCommandEvent &evt);
    //void OnTimelineToolTips(wxCommandEvent &evt);
@@ -254,7 +247,7 @@ private:
    Observer::Subscription mAudioIOSubscription,
       mPlayRegionSubscription,
       mThemeChangeSubscription,
-      mProjectTimeSignatureChangedSubscription;
+      mRulerInvalidatedSubscription;
 
    // classes implementing subdivision for CellularPanel
    struct Subgroup;
@@ -267,22 +260,12 @@ private:
    double mLastDrawnZoom{};
 
 public:
-
-   enum RulerTypeValues : int {
-      stMinutesAndSeconds,
-      stBeatsAndMeasures,
-
-      stNumRulerTypes,
-   };
-
-   RulerTypeValues GetRulerType() const;
-   void SetRulerType(RulerTypeValues rulerType);
+   TimeDisplayMode GetTimeDisplayMode() const;
+   void SetTimeDisplayMode(TimeDisplayMode rulerType);
 
 private:
 
-   RulerTypeValues mRulerType;
+   TimeDisplayMode mTimeDisplayMode;
 };
 
-extern EnumSetting<AdornedRulerPanel::RulerTypeValues> RulerPanelViewPreference;
-
 #endif //define __AUDACITY_ADORNED_RULER_PANEL__
