@@ -38,6 +38,13 @@ class AudacityApp final : public wxApp {
  public:
    AudacityApp();
    ~AudacityApp();
+   
+   bool Initialize(int& argc, wxChar** argv) override;
+#ifdef __WXMAC__
+   bool OSXIsGUIApplication() override;
+#endif
+   
+   void CleanUp() override;
    bool OnInit() override;
    bool InitPart2();
    int OnRun() override;
