@@ -28,7 +28,6 @@ processing.  See also MacrosWindow and ApplyMacroDialog.
 #include <wx/time.h>
 
 #include "Project.h"
-#include "ProjectAudioManager.h"
 #include "ProjectHistory.h"
 #include "ProjectSettings.h"
 #include "ProjectWindow.h"
@@ -41,20 +40,19 @@ processing.  See also MacrosWindow and ApplyMacroDialog.
 #include "Prefs.h"
 #include "SelectFile.h"
 #include "SelectUtilities.h"
-#include "Shuttle.h"
+#include "SettingsVisitor.h"
 #include "Track.h"
 #include "UndoManager.h"
 
 #include "AllThemeResources.h"
 
-#include "widgets/AudacityMessageBox.h"
+#include "AudacityMessageBox.h"
 
 #include "commands/CommandContext.h"
 #include "commands/CommandDispatch.h"
 
 MacroCommands::MacroCommands( AudacityProject &project )
 : mProject{ project }
-, mExporter{ project }
 {
    ResetMacro();
 
@@ -482,49 +480,6 @@ wxString MacroCommands::PromptForPresetFor(const CommandID & command, const wxSt
    return preset;
 }
 
-/// DoAudacityCommand() takes a PluginID and executes the associated command.
-///
-/// At the moment flags are used only to indicate whether to prompt for
-/// parameters
-bool MacroCommands::DoAudacityCommand(
-   const PluginID & ID, const CommandContext & context, unsigned flags )
-{
-   auto &project = context.project;
-   auto &window = ProjectWindow::Get( project );
-   const PluginDescriptor *plug = PluginManager::Get().GetPlugin(ID);
-   if (!plug)
-      return false;
-
-   if (flags & EffectManager::kConfigured)
-   {
-      ProjectAudioManager::Get( project ).Stop();
-//    SelectAllIfNone();
-   }
-
-   EffectManager & em = EffectManager::Get();
-   bool success = em.DoAudacityCommand(ID, 
-      context,
-      &window,
-      (flags & EffectManager::kConfigured) == 0);
-
-   if (!success)
-      return false;
-
-/*
-   if (em.GetSkipStateFlag())
-      flags = flags | OnEffectFlags::kSkipState;
-
-   if (!(flags & OnEffectFlags::kSkipState))
-   {
-      wxString shortDesc = em.GetCommandName(ID);
-      wxString longDesc = em.GetCommandDescription(ID);
-      PushState(longDesc, shortDesc);
-   }
-*/
-   window.RedrawProject();
-   return true;
-}
-
 bool MacroCommands::ApplyEffectCommand(
    const PluginID & ID, const TranslatableString &friendlyCommand,
    const CommandID & command, const wxString & params,
@@ -565,7 +520,7 @@ bool MacroCommands::ApplyEffectCommand(
    {
       if( plug->GetPluginType() == PluginTypeAudacityCommand )
          // and apply the effect...
-         res = DoAudacityCommand(ID,
+         res = CommandDispatch::DoAudacityCommand(ID,
             Context,
             EffectManager::kConfigured |
             EffectManager::kSkipState |
@@ -602,7 +557,7 @@ bool MacroCommands::ApplyCommand( const TranslatableString &friendlyCommand,
    AudacityProject *project = &mProject;
    auto &manager = CommandManager::Get( *project );
    if( pContext ){
-      if( ::HandleTextualCommand(
+      if( CommandDispatch::HandleTextualCommand(
          manager, command, *pContext, AlwaysEnabledFlag, true ) )
          return true;
       pContext->Status( wxString::Format(
@@ -612,7 +567,7 @@ bool MacroCommands::ApplyCommand( const TranslatableString &friendlyCommand,
    else
    {
       const CommandContext context(  mProject );
-      if( HandleTextualCommand(
+      if( CommandDispatch::HandleTextualCommand(
          manager, command, context, AlwaysEnabledFlag, true ) )
          return true;
    }
@@ -634,11 +589,9 @@ bool MacroCommands::ApplyCommandInBatchMode(
    // Recalc flags and enable items that may have become enabled.
    MenuManager::Get(*project).UpdateMenus(false);
    // enter batch mode...
-   bool prevShowMode = settings.GetShowId3Dialog();
    project->mBatchMode++;
    auto cleanup = finally( [&] {
       // exit batch mode...
-      settings.SetShowId3Dialog(prevShowMode);
       project->mBatchMode--;
    } );
 
@@ -818,7 +771,7 @@ bool MacroCommands::ReportAndSkip(
    const TranslatableString & friendlyCommand, const wxString & params)
 {
    int bDebug;
-   gPrefs->Read(wxT("/Batch/Debug"), &bDebug, false);
+   gPrefs->Read(wxT("/Batch/Debug"), &bDebug, 0);
    if( bDebug == 0 )
       return false;
 
