@@ -26,14 +26,11 @@
 #include <wx/defs.h>
 #include <wx/checkbox.h>
 #include <wx/choice.h>
-#include <wx/intl.h>
 #include <wx/log.h>
-#include <wx/sizer.h>
 #include <wx/statbox.h>
 #include <wx/stattext.h>
 #include <wx/textctrl.h>
 #include <wx/listctrl.h>
-#include <wx/radiobut.h>
 #include <wx/button.h>
 #include <wx/imaglist.h>
 #include <wx/settings.h>
@@ -50,22 +47,23 @@
 #include "SelectUtilities.h"
 #include "Track.h"
 #include "commands/CommandManager.h"
-#include "effects/Effect.h"
+#include "Effect.h"
+#include "effects/EffectUI.h"
 #include "../images/Arrow.xpm"
 #include "../images/Empty9x16.xpm"
 #include "UndoManager.h"
 
 #include "AllThemeResources.h"
 
-#include "widgets/FileDialog/FileDialog.h"
+#include "FileDialog/FileDialog.h"
 #include "FileNames.h"
-#include "import/Import.h"
-#include "widgets/AudacityMessageBox.h"
-#include "widgets/AudacityTextEntryDialog.h"
-#include "widgets/HelpSystem.h"
+#include "Import.h"
+#include "AudacityMessageBox.h"
+#include "AudacityTextEntryDialog.h"
+#include "HelpSystem.h"
 
 #if wxUSE_ACCESSIBILITY
-#include "widgets/WindowAccessible.h"
+#include "WindowAccessible.h"
 #endif
 
 #define MacrosPaletteTitle XO("Macros Palette")
@@ -443,8 +441,6 @@ void ApplyMacroDialog::OnApplyToFiles(wxCommandEvent & WXUNUSED(event))
 
    mMacroCommands.ReadMacro(name); 
    {
-      // Move global clipboard contents aside temporarily
-      Clipboard tempClipboard;
       auto &globalClipboard = Clipboard::Get();
 
       // DV: Macro invocation on file will reset the project to the
@@ -453,10 +449,8 @@ void ApplyMacroDialog::OnApplyToFiles(wxCommandEvent & WXUNUSED(event))
       if (globalClipboard.Project().lock().get() == project)
          globalClipboard.Clear();
 
-      globalClipboard.Swap(tempClipboard);
-      auto cleanup = finally([&]{
-         globalClipboard.Swap(tempClipboard);
-      });
+      // Move global clipboard contents aside temporarily
+      Clipboard::Scope scope;
 
       wxWindowDisabler wd(&activityWin);
       for (i = 0; i < (int)files.size(); i++) {
@@ -1385,3 +1379,230 @@ void MacrosWindow::UpdatePrefs()
 {
    UpdateDisplay(mbExpanded);
 }
+
+// The rest of this file installs hooks
+
+#include "CommonCommandFlags.h"
+#include "commands/CommandContext.h"
+#include "effects/EffectManager.h"
+#include "ProjectWindows.h"
+namespace {
+
+AttachedWindows::RegisteredFactory sMacrosWindowKey{
+   []( AudacityProject &parent ) -> wxWeakRef< wxWindow > {
+      auto &window = ProjectWindow::Get( parent );
+      return safenew MacrosWindow(
+         &window, parent, true
+      );
+   }
+};
+
+void OnApplyMacroDirectlyByName(
+   const CommandContext& context, const MacroID& Name);
+
+void OnRepeatLastTool(const CommandContext& context)
+{
+   auto& menuManager = MenuManager::Get(context.project);
+   switch (menuManager.mLastToolRegistration) {
+     case MenuCreator::repeattypeplugin:
+     {
+        auto lastEffect = menuManager.mLastTool;
+        if (!lastEffect.empty())
+        {
+           EffectUI::DoEffect(
+              lastEffect, context, menuManager.mRepeatToolFlags);
+        }
+     }
+       break;
+     case MenuCreator::repeattypeunique:
+        CommandManager::Get(context.project).DoRepeatProcess(context,
+           menuManager.mLastToolRegisteredId);
+        break;
+     case MenuCreator::repeattypeapplymacro:
+        OnApplyMacroDirectlyByName(context, menuManager.mLastTool);
+        break;
+   }
+}
+
+void OnManageMacros(const CommandContext &context )
+{
+   auto &project = context.project;
+   CommandManager::Get(project).RegisterLastTool(context);  //Register Macros as Last Tool
+   auto macrosWindow = &GetAttachedWindows(project)
+      .AttachedWindows::Get< MacrosWindow >( sMacrosWindowKey );
+   if (macrosWindow) {
+      macrosWindow->Show();
+      macrosWindow->Raise();
+      macrosWindow->UpdateDisplay( true );
+   }
+}
+
+void OnApplyMacrosPalette(const CommandContext &context )
+{
+   auto &project = context.project;
+   CommandManager::Get(project).RegisterLastTool(context);  //Register Palette as Last Tool
+   auto macrosWindow = &GetAttachedWindows(project)
+      .AttachedWindows::Get< MacrosWindow >( sMacrosWindowKey );
+   if (macrosWindow) {
+      macrosWindow->Show();
+      macrosWindow->Raise();
+      macrosWindow->UpdateDisplay( false );
+   }
+}
+
+void OnApplyMacroDirectly(const CommandContext &context )
+{
+   const MacroID& Name = context.parameter.GET();
+   OnApplyMacroDirectlyByName(context, Name);
+}
+
+void OnApplyMacroDirectlyByName(const CommandContext& context, const MacroID& Name)
+
+{
+   auto &project = context.project;
+   auto &window = ProjectWindow::Get( project );
+   //wxLogDebug( "Macro was: %s", context.parameter);
+   ApplyMacroDialog dlg( &window, project );
+   //const auto &Name = context.parameter;
+
+// We used numbers previously, but macros could get renumbered, making
+// macros containing macros unpredictable.
+#ifdef MACROS_BY_NUMBERS
+   long item=0;
+   // Take last three letters (of e.g. Macro007) and convert to a number.
+   Name.Mid( Name.length() - 3 ).ToLong( &item, 10 );
+   dlg.ApplyMacroToProject( item, false );
+#else
+   dlg.ApplyMacroToProject( Name, false );
+#endif
+   /* i18n-hint: %s will be the name of the macro which will be
+    * repeated if this menu item is chosen */
+   MenuManager::ModifyUndoMenuItems( project );
+
+   TranslatableString desc;
+   EffectManager& em = EffectManager::Get();
+   auto shortDesc = em.GetCommandName(Name);
+   auto& undoManager = UndoManager::Get(project);
+   auto& commandManager = CommandManager::Get(project);
+   int cur = undoManager.GetCurrentState();
+   if (undoManager.UndoAvailable()) {
+       undoManager.GetShortDescription(cur, &desc);
+       commandManager.Modify(wxT("RepeatLastTool"), XXO("&Repeat %s")
+          .Format(desc));
+       auto& menuManager = MenuManager::Get(project);
+       menuManager.mLastTool = Name;
+       menuManager.mLastToolRegistration = MenuCreator::repeattypeapplymacro;
+   }
+
+}
+
+void PopulateMacrosMenu(MenuTable::MenuItems &items, CommandFlag flags)
+{
+   auto names = MacroCommands::GetNames(); // these names come from filenames
+   for (const auto &name : names) {
+      auto MacroID = ApplyMacroDialog::MacroIdOfName(name);
+      items.push_back(MenuTable::Command(MacroID,
+         Verbatim(name), // file name verbatim
+         OnApplyMacroDirectly,
+         flags,
+         CommandManager::Options{}.AllowInMacros()
+      ));
+   }
+}
+
+const ReservedCommandFlag&
+   HasLastToolFlag() { static ReservedCommandFlag flag{
+      [](const AudacityProject &project) {
+      auto& menuManager = MenuManager::Get(project);
+         if (menuManager.mLastToolRegistration == MenuCreator::repeattypeunique) return true;
+         return !menuManager.mLastTool.empty();
+      }
+   }; return flag;
+}
+}
+
+using namespace MenuTable;
+
+BaseItemSharedPtr PluginMenuItems()
+{
+   using Options = CommandManager::Options;
+   static BaseItemSharedPtr items{
+   Items( "Macros",
+      Section( "RepeatLast",
+         // Delayed evaluation:
+         [](AudacityProject &project)
+         {
+            const auto &lastTool = MenuManager::Get(project).mLastTool;
+            TranslatableString buildMenuLabel;
+            if (!lastTool.empty())
+               buildMenuLabel = XO("Repeat %s")
+                  .Format( EffectManager::Get().GetCommandName(lastTool) );
+            else
+               buildMenuLabel = XO("Repeat Last Tool");
+
+            return Command( wxT("RepeatLastTool"), buildMenuLabel,
+               OnRepeatLastTool,
+               AudioIONotBusyFlag() |
+                  HasLastToolFlag(),
+               Options{}.IsGlobal() );
+         }
+      ),
+
+      Command( wxT("ManageMacros"), XXO("&Macro Manager"),
+         OnManageMacros, AudioIONotBusyFlag() ),
+
+      Menu( wxT("Macros"), XXO("&Apply Macro"),
+         // Palette has no access key to ensure first letter navigation of
+         // sub menu
+         Section( "",
+            Command( wxT("ApplyMacrosPalette"), XXO("Palette..."),
+               OnApplyMacrosPalette, AudioIONotBusyFlag() )
+         ),
+
+         Section( "",
+            // Delayed evaluation:
+            [](AudacityProject&) {
+               auto result = Items("");
+               PopulateMacrosMenu(*result, AudioIONotBusyFlag());
+               return result;
+            }
+         )
+      )
+   ) };
+   return items;
+}
+
+AttachedItem sAttachment1{
+   wxT("Tools/Manage"),
+   Indirect(PluginMenuItems())
+};
+
+BaseItemSharedPtr ExtraScriptablesIMenu()
+{
+   // These are the more useful to VI user Scriptables.
+   static BaseItemSharedPtr menu{
+   // i18n-hint: Scriptables are commands normally used from Python, Perl etc.
+   Menu( wxT("Scriptables1"), XXO("Script&ables I") )
+   };
+   return menu;
+}
+
+AttachedItem sAttachment2{
+   wxT("Optional/Extra/Part2"),
+   Indirect(ExtraScriptablesIMenu())
+};
+
+BaseItemSharedPtr ExtraScriptablesIIMenu()
+{
+   // Less useful to VI users.
+   static BaseItemSharedPtr menu{
+   // i18n-hint: Scriptables are commands normally used from Python, Perl etc.
+   Menu( wxT("Scriptables2"), XXO("Scripta&bles II") )
+   };
+   return menu;
+}
+
+AttachedItem sAttachment3{
+   wxT("Optional/Extra/Part2"),
+   Indirect(ExtraScriptablesIIMenu())
+};
