@@ -27,7 +27,7 @@
 
 *//*****************************************************************/
 
-#include "Audacity.h"
+
 #include "CellularPanel.h"
 
 #include <wx/eventfilter.h>
@@ -60,20 +60,42 @@ struct CellularPanel::Filter : wxEventFilter
 
    int FilterEvent( wxEvent &event ) override
    {
-      if ( spActivePanel &&
-         event.GetEventType() == wxEVT_KEY_DOWN &&
-         static_cast< wxKeyEvent& >( event ).GetKeyCode() == WXK_ESCAPE ) {
-         spActivePanel->HandleEscapeKey( true );
-         return Event_Processed;
+      const auto type = event.GetEventType();
+      if (type == wxEVT_KEY_DOWN &&
+          static_cast< wxKeyEvent& >( event ).GetKeyCode() == WXK_ESCAPE ) {
+         bool eatEvent = false;
+         for (const auto &pPanel: {spClickedPanel, spEnteredPanel}) {
+            if (pPanel) {
+               eatEvent = true;
+               // Handle escape either in the clicked panel to abort a drag, or
+               // to switch among hit test candidates before button down in the
+               // entered (but not yet clicked) panel
+               pPanel->HandleEscapeKey( true );
+            }
+         }
+         if (eatEvent)
+            return Event_Processed;
       }
-      else
-         return Event_Skip;
+      else if ((type == wxEVT_LEFT_DOWN ||
+                type == wxEVT_RIGHT_DOWN ||
+                type == wxEVT_MIDDLE_DOWN)) {
+         if ( spClickedPanel &&
+             spClickedPanel != event.GetEventObject() ) {
+            // Clicking away from the panel doesn't necessarily change wxWidgets
+            // focus, so we use this global filter instead
+            spClickedPanel->DoKillFocus();
+            // Don't eat the event
+         }
+      }
+      return Event_Skip;
    }
 
-   static wxWeakRef< CellularPanel > spActivePanel;
+   static wxWeakRef< CellularPanel > spClickedPanel;
+   static wxWeakRef< CellularPanel > spEnteredPanel;
 };
 
-wxWeakRef< CellularPanel > CellularPanel::Filter::spActivePanel = nullptr;
+wxWeakRef< CellularPanel > CellularPanel::Filter::spClickedPanel = nullptr;
+wxWeakRef< CellularPanel > CellularPanel::Filter::spEnteredPanel = nullptr;
 
 struct CellularPanel::State
 {
@@ -152,7 +174,7 @@ void CellularPanel::Uncapture(bool escaping, wxMouseState *pState)
    HandleMotion( *pState );
  
    if ( escaping || !AcceptsFocus() )
-      Filter::spActivePanel = nullptr;
+      Filter::spClickedPanel = nullptr;
 }
 
 bool CellularPanel::CancelDragging( bool escaping )
@@ -183,7 +205,8 @@ bool CellularPanel::HandleEscapeKey(bool down)
 
    {
       auto target = Target();
-      if (target && target->HasEscape() && target->Escape(GetProject())) {
+      const auto pProject = GetProject();
+      if (target && target->HasEscape(pProject) && target->Escape(pProject)) {
          HandleCursorForPresentMouseState(false);
          return true;
       }
@@ -325,9 +348,17 @@ void CellularPanel::HandleMotion
 
       state.mLastCell = newCell;
 
+      // These lines caused P2 Bug 2617, repeated refreshing using all CPU.
+      // Disabling them might be causing something to not refresh,
+      // but so far I have not found a downside to disabling them.  JKC
+
+      // VS: https://github.com/audacity/audacity/issues/1363
+      // Extensive refresh request fixed by using std::move on 
+      // new envelope handle instance
       if (!oldCell && oldHandle != handle)
-         // Did not move cell to cell, but did change the target
-         refreshCode = updateFlags;
+          // Did not move cell to cell, but did change the target
+          refreshCode = updateFlags;
+
 
       if (handle && handle != oldHandle)
          handle->Enter(true, GetProject());
@@ -419,7 +450,7 @@ bool CellularPanel::HasEscape()
    auto &state = *mState;
   if (state.mTarget + 1 == state.mTargets.size() &&
        Target() &&
-       !Target()->HasEscape())
+       !Target()->HasEscape(GetProject()))
        return false;
 
    return state.mTargets.size() > 0;
@@ -723,8 +754,14 @@ try
       GetParent()->GetEventHandler()->ProcessEvent(e);
    }
 
-   if (event.Leaving())
+   if (event.Entering())
    {
+      Filter::spEnteredPanel = this;
+   }
+   else if (event.Leaving())
+   {
+      if (Filter::spEnteredPanel == this)
+         Filter::spEnteredPanel = nullptr;
       Leave();
 
       auto buttons =
@@ -733,7 +770,7 @@ try
          // event.ButtonIsDown(wxMOUSE_BTN_ANY);
          ::wxGetMouseState().ButtonIsDown(wxMOUSE_BTN_ANY);
 
-      if(!buttons) {
+      if (!buttons) {
          CancelDragging( false );
 
 #if defined(__WXMAC__)
@@ -813,6 +850,61 @@ catch( ... )
    throw;
 }
 
+namespace {
+
+class DefaultRightButtonHandler : public UIHandle {
+public:
+   explicit DefaultRightButtonHandler(
+      const std::shared_ptr<TrackPanelCell> &pCell )
+      : mwCell{ pCell }
+   {}
+
+   ~DefaultRightButtonHandler() override;
+
+   virtual Result Click
+      (const TrackPanelMouseEvent &event, AudacityProject *pProject) override
+   {
+      return RefreshCode::RefreshNone;
+   }
+
+   virtual Result Drag
+      (const TrackPanelMouseEvent &event, AudacityProject *pProject) override
+   {
+      return RefreshCode::RefreshNone;
+   }
+
+   virtual HitTestPreview Preview
+      (const TrackPanelMouseState &state, AudacityProject *pProject) override
+   {
+      return {};
+   }
+
+   virtual Result Release
+      (const TrackPanelMouseEvent &event, AudacityProject *pProject,
+       wxWindow *pParent) override
+   {
+      if (auto pCell = mwCell.lock()) {
+         auto point = event.event.GetPosition();
+         return pCell->DoContextMenu(event.rect, pParent, &point, pProject);
+      }
+      return RefreshCode::RefreshNone;
+   }
+
+   virtual Result Cancel(AudacityProject *pProject) override
+   {
+      return RefreshCode::RefreshNone;
+   }
+
+private:
+   std::weak_ptr<TrackPanelCell> mwCell;
+};
+
+DefaultRightButtonHandler::~DefaultRightButtonHandler()
+{
+}
+
+}
+
 void CellularPanel::HandleClick( const TrackPanelMouseEvent &tpmEvent )
 {
    auto pCell = tpmEvent.pCell;
@@ -829,6 +921,11 @@ void CellularPanel::HandleClick( const TrackPanelMouseEvent &tpmEvent )
 
    auto &state = *mState;
    state.mUIHandle = Target();
+   if (tpmEvent.event.RightDown() &&
+       !(state.mUIHandle && state.mUIHandle->HandlesRightClick())) {
+      if (auto pCell = state.mLastCell.lock())
+         state.mUIHandle = std::make_shared<DefaultRightButtonHandler>(pCell);
+   }
 
    if (state.mUIHandle) {
       // UIHANDLE CLICK
@@ -840,7 +937,7 @@ void CellularPanel::HandleClick( const TrackPanelMouseEvent &tpmEvent )
       if (refreshResult & RefreshCode::Cancelled)
          state.mUIHandle.reset(), handle.reset(), ClearTargets();
       else {
-         Filter::spActivePanel = this;
+         Filter::spClickedPanel = this;
 
 #if wxUSE_TOOLTIPS
          // Remove any outstanding tooltip
@@ -893,13 +990,25 @@ void CellularPanel::OnSetFocus(wxFocusEvent &event)
    Refresh( false);
 }
 
+void CellularPanel::DoKillFocus()
+{
+   if (auto pCell = GetFocusedCell()) {
+      auto refreshResult = pCell->LoseFocus(GetProject());
+      auto &state = *mState;
+      auto pClickedCell = state.mpClickedCell.lock();
+      if (pClickedCell)
+         ProcessUIHandleResult( pClickedCell.get(), {}, refreshResult );
+   }
+   Refresh( false);
+}
+
 void CellularPanel::OnKillFocus(wxFocusEvent & WXUNUSED(event))
 {
+   DoKillFocus();
    if (KeyboardCapture::IsHandler(this))
    {
       KeyboardCapture::Release(this);
    }
-   Refresh( false);
 }
 
 // Empty out-of-line default functions to fill Visitor's vtable
