@@ -14,17 +14,17 @@
 
 
 #include <memory>
-#include <wx/event.h> // to inherit wxEvtHandler
+
+#include "Observer.h"
 
 class AudacityProject;
 class TrackList;
 
-// An event emitted by the clipboard whenever its contents change.
-wxDECLARE_EXPORTED_EVENT( AUDACITY_DLL_API,
-                          EVT_CLIPBOARD_CHANGE, wxCommandEvent );
+//! Message is sent during idle time by the global clipboard
+struct ClipboardChangeMessage {};
 
 class AUDACITY_DLL_API Clipboard final
-   : public wxEvtHandler
+   : public Observer::Publisher<ClipboardChangeMessage>
 {
 public:
    static Clipboard &Get();
@@ -39,21 +39,39 @@ public:
 
    void Clear();
    
+   /*!
+    @pre `!newContents.GetOwner()`
+    */
    void Assign(
      TrackList && newContents, double t0, double t1,
      const std::weak_ptr<AudacityProject> &pProject );
 
-   Clipboard();
    ~Clipboard();
 
    void Swap( Clipboard &other );
 
-private:
+   struct AUDACITY_DLL_API Scope;
 
+private:
+   Clipboard();
+   Clipboard(const Clipboard &) = delete;
+   Clipboard &operator=(const Clipboard &) = delete;
+
+   /*!
+    @invariant `!mTracks->GetOwner()`
+    */
    std::shared_ptr<TrackList> mTracks;
    std::weak_ptr<AudacityProject> mProject{};
    double mT0{ 0 };
    double mT1{ 0 };
 };
 
+//! Empty the clipboard at start of scope; restore its contents after
+struct Clipboard::Scope {
+   Scope();
+   ~Scope();
+private:
+   Clipboard mTempClipboard;
+};
+
 #endif
