@@ -12,17 +12,24 @@
 #ifndef __AUDACITY_CRASH_REPORT__
 #define __AUDACITY_CRASH_REPORT__
 
-#include "Audacity.h"
-#include "Experimental.h"
+#undef HAS_CRASH_REPORT
+
+
+
 
 #if defined(EXPERIMENTAL_CRASH_REPORT)
 
-#include <wx/debugrpt.h>
+#include <wx/setup.h> // for wxUSE* macros
+#if defined(wxUSE_DEBUGREPORT) && wxUSE_DEBUGREPORT
+   #define HAS_CRASH_REPORT
+   #include <wx/debugrpt.h>
 
-namespace CrashReport
-{
-   void Generate(wxDebugReport::Context ctx);
-}
+   namespace CrashReport
+   {
+      AUDACITY_DLL_API
+      void Generate(wxDebugReport::Context ctx);
+   }
+#endif
 
 #endif
 
