@@ -44,11 +44,8 @@ the mouse around.
 #include <wx/dcclient.h>
 #include <wx/dcmemory.h>
 #include <wx/font.h>
-#include <wx/image.h>
 #include <wx/file.h>
-#include <wx/intl.h>
 #include <wx/scrolbar.h>
-#include <wx/sizer.h>
 #include <wx/slider.h>
 #include <wx/statbmp.h>
 #include <wx/stattext.h>
@@ -62,31 +59,36 @@ the mouse around.
 
 #include <math.h>
 
-#include "SelectFile.h"
-#include "ShuttleGui.h"
 #include "AColor.h"
+#include "AllThemeResources.h"
+#include "BasicUI.h"
 #include "CommonCommandFlags.h"
+#include "Decibels.h"
 #include "FFT.h"
 #include "PitchName.h"
-#include "Decibels.h"
 #include "Prefs.h"
 #include "Project.h"
 #include "ProjectWindow.h"
+#include "SelectFile.h"
+#include "ShuttleGui.h"
 #include "Theme.h"
 #include "ViewInfo.h"
-#include "AllThemeResources.h"
 
 #include "FileNames.h"
 
 #include "WaveTrack.h"
 
-#include "./widgets/HelpSystem.h"
-#include "widgets/AudacityMessageBox.h"
-#include "widgets/Ruler.h"
+#include "HelpSystem.h"
+#include "AudacityMessageBox.h"
+#include "widgets/RulerPanel.h"
+#include "widgets/LinearUpdater.h"
+#include "widgets/LogarithmicUpdater.h"
+#include "widgets/LinearDBFormat.h"
+#include "widgets/RealFormat.h"
 #include "widgets/VetoDialogHook.h"
 
 #if wxUSE_ACCESSIBILITY
-#include "widgets/WindowAccessible.h"
+#include "WindowAccessible.h"
 #endif
 
 #define FrequencyAnalysisTitle XO("Frequency Analysis")
@@ -293,7 +295,7 @@ void FrequencyPlotDialog::Populate()
             S.GetParent(), wxID_ANY, wxVERTICAL,
             wxSize{ 100, 100 }, // Ruler can't handle small sizes
             RulerPanel::Range{ 0.0, -dBRange },
-            Ruler::LinearDBFormat,
+            LinearDBFormat::Instance(),
             XO("dB"),
             RulerPanel::Options{}
                .LabelEdges(true)
@@ -376,7 +378,7 @@ void FrequencyPlotDialog::Populate()
             S.GetParent(), wxID_ANY, wxHORIZONTAL,
             wxSize{ 100, 100 }, // Ruler can't handle small sizes
             RulerPanel::Range{ 10, 20000 },
-            Ruler::RealFormat,
+            RealFormat::LinearInstance(),
             XO("Hz"),
             RulerPanel::Options{}
                .Log(true)
@@ -440,7 +442,7 @@ void FrequencyPlotDialog::Populate()
    // -------------------------------------------------------------------
    // ROW 5: Spacer
    // -------------------------------------------------------------------
-   
+
    S.AddSpace(5);
 
    S.SetBorder(2);
@@ -566,7 +568,8 @@ bool FrequencyPlotDialog::Show(bool show)
       dBRange = DecibelScaleCutoff.Read();
       if(dBRange < 90.)
          dBRange = 90.;
-      GetAudio();
+      if (!GetAudio())
+         return false;
       // Don't send an event.  We need the recalc right away.
       // so that mAnalyst is valid when we paint.
       //SendRecalcEvent();
@@ -578,18 +581,20 @@ bool FrequencyPlotDialog::Show(bool show)
    return res;
 }
 
-void FrequencyPlotDialog::GetAudio()
+bool FrequencyPlotDialog::GetAudio()
 {
    mData.reset();
    mDataLen = 0;
 
    int selcount = 0;
    bool warning = false;
-   for (auto track : TrackList::Get( *mProject ).Selected< const WaveTrack >()) {
-      auto &selectedRegion = ViewInfo::Get( *mProject ).selectedRegion;
-      if (selcount==0) {
+   for (auto track :
+      TrackList::Get(*mProject).Selected<const WaveTrack>()
+   ) {
+      auto &selectedRegion = ViewInfo::Get(*mProject).selectedRegion;
+      auto start = track->TimeToLongSamples(selectedRegion.t0());
+      if (selcount == 0) {
          mRate = track->GetRate();
-         auto start = track->TimeToLongSamples(selectedRegion.t0());
          auto end = track->TimeToLongSamples(selectedRegion.t1());
          auto dataLen = end - start;
          // Permit approximately 46.60 minutes of selected samples at
@@ -599,36 +604,54 @@ void FrequencyPlotDialog::GetAudio()
             warning = true;
             mDataLen = maxDataLen;
          }
-         else {
+         else
             mDataLen = dataLen.as_size_t();
-         }
          mData = Floats{ mDataLen };
-         // Don't allow throw for bad reads
-         track->GetFloats(mData.get(), start, mDataLen,
-                    fillZero, false);
       }
-      else {
-         if (track->GetRate() != mRate) {
-            AudacityMessageBox(
-               XO(
-"To plot the spectrum, all selected tracks must be the same sample rate.") );
-            mData.reset();
-            mDataLen = 0;
-            return;
-         }
-         auto start = track->TimeToLongSamples(selectedRegion.t0());
-         Floats buffer2{ mDataLen };
-         // Again, stop exceptions
-         track->GetFloats(buffer2.get(), start, mDataLen,
-                    fillZero, false);
+      const auto nChannels = track->NChannels();
+      if (track->GetRate() != mRate) {
+         using namespace BasicUI;
+         ShowMessageBox(
+            XO("To plot the spectrum, all selected tracks must have the same sample rate."),
+            MessageBoxOptions {}.Caption(XO("Error")).IconStyle(Icon::Error));
+         mData.reset();
+         mDataLen = 0;
+         return false;
+      }
+      Floats buffer1{ mDataLen };
+      Floats buffer2{ mDataLen };
+      float *const buffers[]{ buffer1.get(), buffer2.get() };
+      // Don't allow throw for bad reads
+      if (!track->GetFloats(
+             0, nChannels, buffers, start, mDataLen, false,
+             FillFormat::fillZero, false))
+      {
+         using namespace BasicUI;
+         ShowMessageBox(
+            XO("Audio could not be analyzed. This may be due to a stretched clip.\nTry resetting any stretched clips, or mixing and rendering the tracks before analyzing"),
+            MessageBoxOptions {}.Caption(XO("Error")).IconStyle(Icon::Error));
+         mData.reset();
+         mDataLen = 0;
+         return false;
+      }
+      size_t iChannel = 0;
+      if (selcount == 0) {
+         // First channel -- assign into mData
          for (size_t i = 0; i < mDataLen; i++)
-            mData[i] += buffer2[i];
+            mData[i] = buffers[0][i];
+         ++iChannel;
       }
-      selcount++;
+      // Later channels -- accumulate
+      for (; iChannel < nChannels; ++iChannel) {
+         const auto buffer = buffers[iChannel];
+         for (size_t i = 0; i < mDataLen; i++)
+            mData[i] += buffer[i];
+      }
+      ++selcount;
    }
 
    if (selcount == 0)
-      return;
+      return false;
 
    if (warning) {
       auto msg = XO(
@@ -636,6 +659,7 @@ void FrequencyPlotDialog::GetAudio()
          .Format(mDataLen / mRate);
       AudacityMessageBox( msg );
    }
+   return true;
 }
 
 void FrequencyPlotDialog::OnSize(wxSizeEvent & WXUNUSED(event))
@@ -674,10 +698,10 @@ void FrequencyPlotDialog::DrawPlot()
    if (!mData || mDataLen < mWindowSize || mAnalyst->GetProcessedSize() == 0) {
       wxMemoryDC memDC;
 
-      vRuler->ruler.SetLog(false);
+      vRuler->ruler.SetUpdater(&LinearUpdater::Instance());
       vRuler->ruler.SetRange(0.0, -dBRange);
 
-      hRuler->ruler.SetLog(false);
+      hRuler->ruler.SetUpdater(&LinearUpdater::Instance());
       hRuler->ruler.SetRange(0, 1);
 
       DrawBackground(memDC);
@@ -691,7 +715,7 @@ void FrequencyPlotDialog::DrawPlot()
       }
 
       memDC.SelectObject(wxNullBitmap);
-      
+
       mFreqPlot->Refresh();
 
       Refresh();
@@ -714,10 +738,10 @@ void FrequencyPlotDialog::DrawPlot()
 
    if (mAlg == SpectrumAnalyst::Spectrum) {
       vRuler->ruler.SetUnits(XO("dB"));
-      vRuler->ruler.SetFormat(Ruler::LinearDBFormat);
+      vRuler->ruler.SetFormat(&LinearDBFormat::Instance());
    } else {
       vRuler->ruler.SetUnits({});
-      vRuler->ruler.SetFormat(Ruler::RealFormat);
+      vRuler->ruler.SetFormat(&RealFormat::LinearInstance());
    }
    int w1, w2, h;
    vRuler->ruler.GetMaxSize(&w1, &h);
@@ -752,19 +776,19 @@ void FrequencyPlotDialog::DrawPlot()
       if (mLogAxis)
       {
          xStep = pow(2.0f, (log(xRatio) / log(2.0f)) / width);
-         hRuler->ruler.SetLog(true);
+         hRuler->ruler.SetUpdater(&LogarithmicUpdater::Instance());
       }
       else
       {
          xStep = (xMax - xMin) / width;
-         hRuler->ruler.SetLog(false);
+         hRuler->ruler.SetUpdater(&LinearUpdater::Instance());
       }
       hRuler->ruler.SetUnits(XO("Hz"));
    } else {
       xMin = 0;
       xMax = mAnalyst->GetProcessedSize() / mRate;
       xStep = (xMax - xMin) / width;
-      hRuler->ruler.SetLog(false);
+      hRuler->ruler.SetUpdater(&LinearUpdater::Instance());
       /* i18n-hint: short form of 'seconds'.*/
       hRuler->ruler.SetUnits(XO("s"));
    }
@@ -917,7 +941,7 @@ void FrequencyPlotDialog::PlotPaint(wxPaintEvent & event)
    float xPos = xMin;
 
    // Find the peak nearest the cursor and plot it
-   if ( r.Contains(mMouseX, mMouseY) & (mMouseX!=0) & (mMouseX!=r.width-1) ) {
+   if ( r.Contains(mMouseX, mMouseY) && (mMouseX!=0) && (mMouseX!=r.width-1) ) {
       if (mLogAxis)
          xPos = xMin * pow(xStep, mMouseX - (r.x + 1));
       else
@@ -1212,37 +1236,27 @@ AttachedWindows::RegisteredFactory sFrequencyWindowKey{
 };
 
 // Define our extra menu item that invokes that factory
-struct Handler : CommandHandlerObject {
-   void OnPlotSpectrum(const CommandContext &context)
-   {
-      auto &project = context.project;
-      CommandManager::Get(project).RegisterLastAnalyzer(context);  //Register Plot Spectrum as Last Analyzer
-      auto freqWindow = &GetAttachedWindows(project)
-         .Get< FrequencyPlotDialog >( sFrequencyWindowKey );
+void OnPlotSpectrum(const CommandContext &context)
+{
+   auto &project = context.project;
+   CommandManager::Get(project).RegisterLastAnalyzer(context);  //Register Plot Spectrum as Last Analyzer
+   auto freqWindow = &GetAttachedWindows(project)
+      .Get< FrequencyPlotDialog >( sFrequencyWindowKey );
 
-      if( VetoDialogHook::Call( freqWindow ) )
-         return;
-      freqWindow->Show(true);
-      freqWindow->Raise();
-      freqWindow->SetFocus();
-   }
-};
-
-CommandHandlerObject &findCommandHandler(AudacityProject &) {
-   // Handler is not stateful.  Doesn't need a factory registered with
-   // AudacityProject.
-   static Handler instance;
-   return instance;
+   if( VetoDialogHook::Call( freqWindow ) )
+      return;
+   freqWindow->Show(true);
+   freqWindow->Raise();
+   freqWindow->SetFocus();
 }
 
 // Register that menu item
 
 using namespace MenuTable;
 AttachedItem sAttachment{ wxT("Analyze/Analyzers/Windows"),
-   ( FinderScope{ findCommandHandler },
-      Command( wxT("PlotSpectrum"), XXO("Plot Spectrum..."),
-         &Handler::OnPlotSpectrum,
-         AudioIONotBusyFlag() | WaveTracksSelectedFlag() | TimeSelectedFlag() ) )
+   Command( wxT("PlotSpectrum"), XXO("Plot Spectrum..."),
+      OnPlotSpectrum,
+      AudioIONotBusyFlag() | WaveTracksSelectedFlag() | TimeSelectedFlag() )
 };
 
 }
