@@ -12,11 +12,17 @@
 
 #include <vector>
 #include <wx/string.h>
-#include "widgets/wxPanelWrapper.h"
+#include "wxPanelWrapper.h"
 
 class wxStaticText;
 class wxTextCtrl;
 
+enum class ScanType
+{
+   Startup,
+   Manual
+};
+
 ///Used to display a list of found plugins that aren't compatible
 ///with Audacity
 class IncompatiblePluginsDialog final : public wxDialogWrapper
@@ -25,7 +31,7 @@ class IncompatiblePluginsDialog final : public wxDialogWrapper
    wxTextCtrl* mPluginList{nullptr};
 public:
 
-   IncompatiblePluginsDialog(wxWindow *parent, wxWindowID id,
+   IncompatiblePluginsDialog(wxWindow *parent, wxWindowID id, ScanType scanType,
              const std::vector<wxString>& plugins = { },
              const wxPoint& pos = wxDefaultPosition,
              const wxSize& size = wxDefaultSize);
@@ -35,4 +41,6 @@ public:
 private:
    void OnPluginManagerClicked(wxCommandEvent&);
    void OnContinueClicked(wxCommandEvent&);
+
+   ScanType m_scanType;
 };
