@@ -45,6 +45,16 @@ for drawing different aspects of the label and its text box.
 #include "TimeWarper.h"
 #include "AudacityMessageBox.h"
 
+LabelTrack::Interval::~Interval() = default;
+
+std::shared_ptr<ChannelInterval>
+LabelTrack::Interval::DoGetChannel(size_t iChannel)
+{
+   if (iChannel == 0)
+      return std::make_shared<ChannelInterval>();
+   return {};
+}
+
 static ProjectFileIORegistry::ObjectReaderEntry readerEntry{
    "labeltrack",
    LabelTrack::New
@@ -76,16 +86,16 @@ LabelTrack* LabelTrack::Create(TrackList& trackList)
    return Create(trackList, trackList.MakeUniqueTrackName(GetDefaultName()));
 }
 
-LabelTrack::LabelTrack():
-   Track(),
-   mClipLen(0.0),
-   miLastLabel(-1)
+LabelTrack::LabelTrack()
+   : UniqueChannelTrack{}
+   , mClipLen{ 0.0 }
+   , miLastLabel{ -1 }
 {
 }
 
 LabelTrack::LabelTrack(const LabelTrack &orig, ProtectedCreationArg &&a)
-   : Track(orig, std::move(a))
-   , mClipLen(0.0)
+   : UniqueChannelTrack{ orig, std::move(a) }
+   , mClipLen{ 0.0 }
 {
    for (auto &original: orig.mLabels) {
       LabelStruct l { original.selectedRegion, original.title };
@@ -110,49 +120,34 @@ auto LabelTrack::ClassTypeInfo() -> const TypeInfo &
    return typeInfo();
 }
 
-Track::Holder LabelTrack::PasteInto( AudacityProject & ) const
+Track::Holder LabelTrack::PasteInto(AudacityProject &, TrackList &list) const
 {
+   assert(IsLeader());
    auto pNewTrack = std::make_shared<LabelTrack>();
    pNewTrack->Init(*this);
-   pNewTrack->Paste(0.0, this);
+   pNewTrack->Paste(0.0, *this);
+   list.Add(pNewTrack);
    return pNewTrack;
 }
 
-template<typename IntervalType>
-static IntervalType DoMakeInterval(const LabelStruct &label, size_t index)
+size_t LabelTrack::NIntervals() const
 {
-   return {
-      label.getT0(), label.getT1(),
-      std::make_unique<LabelTrack::IntervalData>( index ) };
+   return mLabels.size();
 }
 
-auto LabelTrack::MakeInterval( size_t index ) const -> ConstInterval
+auto LabelTrack::MakeInterval(size_t index) -> std::shared_ptr<Interval>
 {
-   return DoMakeInterval<ConstInterval>(mLabels[index], index);
+   if (index >= mLabels.size())
+      return {};
+   auto &label = mLabels[index];
+   return std::make_shared<Interval>(
+      *this, label.getT0(), label.getT1(), index);
 }
 
-auto LabelTrack::MakeInterval( size_t index ) -> Interval
+std::shared_ptr<WideChannelGroupInterval>
+LabelTrack::DoGetInterval(size_t iInterval)
 {
-   return DoMakeInterval<Interval>(mLabels[index], index);
-}
-
-template< typename Container, typename LabelTrack >
-static Container DoMakeIntervals(LabelTrack &track)
-{
-   Container result;
-   for (size_t ii = 0, nn = track.GetNumLabels(); ii < nn; ++ii)
-      result.emplace_back( track.MakeInterval( ii ) );
-   return result;
-}
-
-auto LabelTrack::GetIntervals() const -> ConstIntervals
-{
-   return DoMakeIntervals<ConstIntervals>(*this);
-}
-
-auto LabelTrack::GetIntervals() -> Intervals
-{
-   return DoMakeIntervals<Intervals>(*this);
+   return MakeInterval(iInterval);
 }
 
 void LabelTrack::SetLabel( size_t iLabel, const LabelStruct &newLabel )
@@ -168,14 +163,31 @@ LabelTrack::~LabelTrack()
 {
 }
 
-void LabelTrack::SetOffset(double dOffset)
+void LabelTrack::MoveTo(double origin)
 {
-   for (auto &labelStruct: mLabels)
-      labelStruct.selectedRegion.move(dOffset);
+   if (!mLabels.empty()) {
+      const auto offset = origin - mLabels[0].selectedRegion.t0();
+      for (auto &labelStruct: mLabels) {
+         labelStruct.selectedRegion.move(offset);
+      }
+   }
+}
+
+void LabelTrack::DoOnProjectTempoChange(
+   const std::optional<double>& oldTempo, double newTempo)
+{
+   assert(IsLeader());
+   if (!oldTempo.has_value())
+      return;
+   const auto ratio = *oldTempo / newTempo;
+   for (auto& label : mLabels)
+      label.selectedRegion.setTimes(
+         label.getT0() * ratio, label.getT1() * ratio);
 }
 
 void LabelTrack::Clear(double b, double e)
 {
+   assert(IsLeader());
    // May DELETE labels, so use subscripts to iterate
    for (size_t i = 0; i < mLabels.size(); ++i) {
       auto &labelStruct = mLabels[i];
@@ -330,40 +342,12 @@ void LabelTrack::SetSelected( bool s )
          this->SharedPointer<LabelTrack>(), {}, -1, -1 });
 }
 
-double LabelTrack::GetOffset() const
-{
-   return mOffset;
-}
-
-double LabelTrack::GetStartTime() const
-{
-   if (mLabels.empty())
-      return 0.0;
-   else
-      return mLabels[0].getT0();
-}
-
-double LabelTrack::GetEndTime() const
-{
-   //we need to scan through all the labels, because the last
-   //label might not have the right-most end (if there is overlap).
-   if (mLabels.empty())
-      return 0.0;
-
-   double end = 0.0;
-   for (auto &labelStruct: mLabels) {
-      const double t1 = labelStruct.getT1();
-      if(t1 > end)
-         end = t1;
-   }
-   return end;
-}
-
-Track::Holder LabelTrack::Clone() const
+TrackListHolder LabelTrack::Clone() const
 {
+   assert(IsLeader());
    auto result = std::make_shared<LabelTrack>(*this, ProtectedCreationArg{});
    result->Init(*this);
-   return result;
+   return TrackList::Temporary(nullptr, result, nullptr);
 }
 
 // Adjust label's left or right boundary, depending which is requested.
@@ -424,7 +408,7 @@ LabelStruct LabelStruct::Import(wxTextFile &file, int &index)
          token = toker.GetNextToken();
 
       sr.setTimes( t0, t1 );
-      
+
       title = token;
    }
 
@@ -662,6 +646,7 @@ XMLTagHandler *LabelTrack::HandleXMLChild(const std::string_view& tag)
 void LabelTrack::WriteXML(XMLWriter &xmlFile) const
 // may throw
 {
+   assert(IsLeader());
    int len = mLabels.size();
 
    xmlFile.StartTag(wxT("labeltrack"));
@@ -680,12 +665,11 @@ void LabelTrack::WriteXML(XMLWriter &xmlFile) const
    xmlFile.EndTag(wxT("labeltrack"));
 }
 
-Track::Holder LabelTrack::Cut(double t0, double t1)
+TrackListHolder LabelTrack::Cut(double t0, double t1)
 {
+   assert(IsLeader());
    auto tmp = Copy(t0, t1);
-
    Clear(t0, t1);
-
    return tmp;
 }
 
@@ -703,7 +687,7 @@ Track::Holder LabelTrack::SplitCut(double t0, double t1)
 }
 #endif
 
-Track::Holder LabelTrack::Copy(double t0, double t1, bool) const
+TrackListHolder LabelTrack::Copy(double t0, double t1, bool) const
 {
    auto tmp = std::make_shared<LabelTrack>();
    tmp->Init(*this);
@@ -751,20 +735,20 @@ Track::Holder LabelTrack::Copy(double t0, double t1, bool) const
    }
    lt->mClipLen = (t1 - t0);
 
-   return tmp;
+   return TrackList::Temporary(nullptr, tmp, nullptr);
 }
 
 
-bool LabelTrack::PasteOver(double t, const Track * src)
+bool LabelTrack::PasteOver(double t, const Track &src)
 {
-   auto result = src->TypeSwitch< bool >( [&](const LabelTrack *sl) {
+   auto result = src.TypeSwitch<bool>([&](const LabelTrack &sl) {
       int len = mLabels.size();
       int pos = 0;
 
       while (pos < len && mLabels[pos].getT0() < t)
          pos++;
 
-      for (auto &labelStruct: sl->mLabels) {
+      for (auto &labelStruct: sl.mLabels) {
          LabelStruct l {
             labelStruct.selectedRegion,
             labelStruct.getT0() + t,
@@ -775,27 +759,27 @@ bool LabelTrack::PasteOver(double t, const Track * src)
       }
 
       return true;
-   } );
+   });
 
-   if (! result )
+   if (!result)
       // THROW_INCONSISTENCY_EXCEPTION; // ?
       (void)0;// intentionally do nothing
 
    return result;
 }
 
-void LabelTrack::Paste(double t, const Track *src)
+void LabelTrack::Paste(double t, const Track &src)
 {
-   bool bOk = src->TypeSwitch< bool >( [&](const LabelTrack *lt) {
-      double shiftAmt = lt->mClipLen > 0.0 ? lt->mClipLen : lt->GetEndTime();
+   bool bOk = src.TypeSwitch<bool>([&](const LabelTrack &lt) {
+      double shiftAmt = lt.mClipLen > 0.0 ? lt.mClipLen : lt.GetEndTime();
 
       ShiftLabelsOnInsert(shiftAmt, t);
       PasteOver(t, src);
 
       return true;
-   } );
+   });
 
-   if ( !bOk )
+   if (!bOk)
       // THROW_INCONSISTENCY_EXCEPTION; // ?
       (void)0;// intentionally do nothing
 }
@@ -855,6 +839,7 @@ bool LabelTrack::Repeat(double t0, double t1, int n)
 
 void LabelTrack::SyncLockAdjust(double oldT1, double newT1)
 {
+   assert(IsLeader());
    if (newT1 > oldT1) {
       // Insert space within the track
 
@@ -870,9 +855,9 @@ void LabelTrack::SyncLockAdjust(double oldT1, double newT1)
    }
 }
 
-
-void LabelTrack::Silence(double t0, double t1)
+void LabelTrack::Silence(double t0, double t1, ProgressReporter)
 {
+   assert(IsLeader());
    int len = mLabels.size();
 
    // mLabels may resize as we iterate, so use subscripting
@@ -919,6 +904,7 @@ void LabelTrack::Silence(double t0, double t1)
 
 void LabelTrack::InsertSilence(double t, double len)
 {
+   assert(IsLeader());
    for (auto &labelStruct: mLabels) {
       double t0 = labelStruct.getT0();
       double t1 = labelStruct.getT1();
