@@ -551,16 +551,16 @@ MIDIPlay::~MIDIPlay()
    Pm_Terminate();
 }
 
-bool MIDIPlay::StartOtherStream(const TransportTracks &tracks,
+bool MIDIPlay::StartOtherStream(const TransportSequences &tracks,
    const PaStreamInfo* info, double, double rate)
 {
    mMidiPlaybackTracks.clear();
-   for (const auto &pTrack : tracks.otherPlayableTracks) {
-      pTrack->TypeSwitch( [&](const NoteTrack *pNoteTrack){
+   for (const auto &pSequence : tracks.otherPlayableSequences)
+      if (const auto pNoteTrack =
+         dynamic_cast<const NoteTrack *>(pSequence.get())
+      )
          mMidiPlaybackTracks.push_back(
             pNoteTrack->SharedPointer<const NoteTrack>());
-      } );
-   }
 
    streamStartTime = 0;
    streamStartTime = SystemTime(mUsingAlsa);
@@ -640,7 +640,7 @@ Iterator::Iterator(
       const void *cookie = t.get();
       it.begin_seq(seq,
          // casting away const, but allegro just uses the pointer opaquely
-         const_cast<void*>(cookie), t->GetOffset() + offset);
+         const_cast<void*>(cookie), t->GetStartTime() + offset);
    }
    Prime(send, startTime + offset);
 }
@@ -1190,7 +1190,7 @@ void MIDIPlay::ComputeOtherTimings(double rate, bool paused,
       mMidiPaused = false;
 }
 
-unsigned MIDIPlay::CountOtherSoloTracks() const
+unsigned MIDIPlay::CountOtherSolo() const
 {
    return std::count_if(
       mMidiPlaybackTracks.begin(), mMidiPlaybackTracks.end(),
