@@ -23,7 +23,6 @@
 #include <wx/dc.h>
 #include <wx/brush.h>
 #include <wx/pen.h>
-#include <wx/intl.h>
 
 #if defined(USE_MIDI)
 #include "../lib-src/header-substitutes/allegro.h"
@@ -105,7 +104,15 @@ SONFNS(AutoSave)
 
 #endif
 
+NoteTrack::Interval::~Interval() = default;
 
+std::shared_ptr<ChannelInterval>
+NoteTrack::Interval::DoGetChannel(size_t iChannel)
+{
+   if (iChannel == 0)
+      return std::make_shared<ChannelInterval>();
+   return {};
+}
 
 static ProjectFileIORegistry::ObjectReaderEntry readerEntry{
    "notetrack",
@@ -121,7 +128,7 @@ NoteTrack *NoteTrack::New( AudacityProject &project )
 }
 
 NoteTrack::NoteTrack()
-   : NoteTrackBase()
+   : UniqueChannelTrack{}
 {
    SetName(_("Note Track"));
 
@@ -158,7 +165,7 @@ Alg_seq &NoteTrack::GetSeq() const
    return *mSeq;
 }
 
-Track::Holder NoteTrack::Clone() const
+TrackListHolder NoteTrack::Clone() const
 {
    auto duplicate = std::make_shared<NoteTrack>();
    duplicate->Init(*this);
@@ -191,34 +198,35 @@ Track::Holder NoteTrack::Clone() const
    duplicate->SetBottomNote(mBottomNote);
    duplicate->SetTopNote(mTopNote);
    duplicate->SetVisibleChannels(GetVisibleChannels());
-   duplicate->SetOffset(GetOffset());
+   duplicate->MoveTo(mOrigin);
 #ifdef EXPERIMENTAL_MIDI_OUT
    duplicate->SetVelocity(GetVelocity());
 #endif
-   return duplicate;
+   return TrackList::Temporary(nullptr, duplicate, nullptr);
 }
 
 
-double NoteTrack::GetOffset() const
+void NoteTrack::DoOnProjectTempoChange(
+   const std::optional<double>& oldTempo, double newTempo)
 {
-   return mOffset;
-}
-
-double NoteTrack::GetStartTime() const
-{
-   return GetOffset();
-}
-
-double NoteTrack::GetEndTime() const
-{
-   return GetStartTime() + GetSeq().get_real_dur();
+   assert(IsLeader());
+   if (!oldTempo.has_value())
+      return;
+   const auto ratio = *oldTempo / newTempo;
+   auto& seq = GetSeq();
+   seq.convert_to_beats();
+   const auto b1 = seq.get_dur();
+   seq.convert_to_seconds();
+   const auto newDuration = seq.get_dur() * ratio;
+   seq.stretch_region(0, b1, newDuration);
+   seq.set_real_dur(newDuration);
 }
 
 void NoteTrack::WarpAndTransposeNotes(double t0, double t1,
                                       const TimeWarper &warper,
                                       double semitones)
 {
-   double offset = this->GetOffset(); // track is shifted this amount
+   double offset = this->mOrigin; // track is shifted this amount
    auto &seq = GetSeq();
    seq.convert_to_seconds(); // make sure time units are right
    t1 -= offset; // adjust time range to compensate for track offset
@@ -441,8 +449,9 @@ void NoteTrack::PrintSequence()
    fclose(debugOutput);
 }
 
-Track::Holder NoteTrack::Cut(double t0, double t1)
+TrackListHolder NoteTrack::Cut(double t0, double t1)
 {
+   assert(IsLeader());
    if (t1 < t0)
       THROW_INCONSISTENCY_EXCEPTION;
 
@@ -457,8 +466,8 @@ Track::Holder NoteTrack::Cut(double t0, double t1)
 
    auto &seq = GetSeq();
    seq.convert_to_seconds();
-   newTrack->mSeq.reset(seq.cut(t0 - GetOffset(), len, false));
-   newTrack->SetOffset(0);
+   newTrack->mSeq.reset(seq.cut(t0 - mOrigin, len, false));
+   newTrack->MoveTo(0);
 
    // Not needed
    // Alg_seq::cut seems to handle this
@@ -468,10 +477,10 @@ Track::Holder NoteTrack::Cut(double t0, double t1)
    //(mBottomNote,
    // mSerializationBuffer, mSerializationLength, mVisibleChannels)
 
-   return newTrack;
+   return TrackList::Temporary(nullptr, newTrack, nullptr);
 }
 
-Track::Holder NoteTrack::Copy(double t0, double t1, bool) const
+TrackListHolder NoteTrack::Copy(double t0, double t1, bool) const
 {
    if (t1 < t0)
       THROW_INCONSISTENCY_EXCEPTION;
@@ -484,14 +493,14 @@ Track::Holder NoteTrack::Copy(double t0, double t1, bool) const
 
    auto &seq = GetSeq();
    seq.convert_to_seconds();
-   newTrack->mSeq.reset(seq.copy(t0 - GetOffset(), len, false));
-   newTrack->SetOffset(0);
+   newTrack->mSeq.reset(seq.copy(t0 - mOrigin, len, false));
+   newTrack->MoveTo(0);
 
    // What should be done with the rest of newTrack's members?
    // (mBottomNote, mSerializationBuffer,
    // mSerializationLength, mVisibleChannels)
 
-   return newTrack;
+   return TrackList::Temporary(nullptr, newTrack, nullptr);
 }
 
 bool NoteTrack::Trim(double t0, double t1)
@@ -505,11 +514,11 @@ bool NoteTrack::Trim(double t0, double t1)
    //);
    seq.convert_to_seconds();
    // DELETE way beyond duration just in case something is out there:
-   seq.clear(t1 - GetOffset(), seq.get_dur() + 10000.0, false);
+   seq.clear(t1 - mOrigin, seq.get_dur() + 10000.0, false);
    // Now that stuff beyond selection is cleared, clear before selection:
-   seq.clear(0.0, t0 - GetOffset(), false);
+   seq.clear(0.0, t0 - mOrigin, false);
    // want starting time to be t0
-   SetOffset(t0);
+   MoveTo(t0);
 
    // Not needed
    // Alg_seq::clear seems to handle this
@@ -520,6 +529,7 @@ bool NoteTrack::Trim(double t0, double t1)
 
 void NoteTrack::Clear(double t0, double t1)
 {
+   assert(IsLeader());
    if (t1 < t0)
       THROW_INCONSISTENCY_EXCEPTION;
 
@@ -527,17 +537,17 @@ void NoteTrack::Clear(double t0, double t1)
 
    auto &seq = GetSeq();
 
-   auto offset = GetOffset();
+   auto offset = mOrigin;
    auto start = t0 - offset;
    if (start < 0.0) {
       // AlgSeq::clear will shift the cleared interval, not changing len, if
       // start is negative.  That's not what we want to happen.
       if (len > -start) {
          seq.clear(0, len + start, false);
-         SetOffset(t0);
+         MoveTo(t0);
       }
       else
-         SetOffset(offset - len);
+         MoveTo(offset - len);
    }
    else {
       //auto delta = -(
@@ -551,7 +561,7 @@ void NoteTrack::Clear(double t0, double t1)
    }
 }
 
-void NoteTrack::Paste(double t, const Track *src)
+void NoteTrack::Paste(double t, const Track &src)
 {
    // Paste inserts src at time t. If src has a positive offset,
    // the offset is treated as silence which is also inserted. If
@@ -562,47 +572,48 @@ void NoteTrack::Paste(double t, const Track *src)
    // the destination track).
 
    //Check that src is a non-NULL NoteTrack
-   bool bOk = src && src->TypeSwitch< bool >( [&](const NoteTrack *other) {
+   bool bOk = src.TypeSwitch<bool>( [&](const NoteTrack &other) {
 
-      auto myOffset = this->GetOffset();
+      auto myOffset = this->mOrigin;
       if (t < myOffset) {
          // workaround strange behavior described at
          // http://bugzilla.audacityteam.org/show_bug.cgi?id=1735#c3
-         SetOffset(t);
+         MoveTo(t);
          InsertSilence(t, myOffset - t);
       }
 
       double delta = 0.0;
       auto &seq = GetSeq();
-      auto offset = other->GetOffset();
-      if ( offset > 0 ) {
+      auto offset = other.mOrigin;
+      if (offset > 0) {
          seq.convert_to_seconds();
-         seq.insert_silence( t - GetOffset(), offset );
+         seq.insert_silence(t - mOrigin, offset);
          t += offset;
          // Is this needed or does Alg_seq::insert_silence take care of it?
          //delta += offset;
       }
 
       // This seems to be needed:
-      delta += std::max( 0.0, t - GetEndTime() );
+      delta += std::max(0.0, t - GetEndTime());
 
       // This, not:
-      //delta += other->GetSeq().get_real_dur();
+      //delta += other.GetSeq().get_real_dur();
 
-      seq.paste(t - GetOffset(), &other->GetSeq());
+      seq.paste(t - mOrigin, &other.GetSeq());
 
-      AddToDuration( delta );
+      AddToDuration(delta);
 
       return true;
    });
 
-   if ( !bOk )
+   if (!bOk)
       // THROW_INCONSISTENCY_EXCEPTION; // ?
       (void)0;// intentionally do nothing
 }
 
-void NoteTrack::Silence(double t0, double t1)
+void NoteTrack::Silence(double t0, double t1, ProgressReporter)
 {
+   assert(IsLeader());
    if (t1 < t0)
       THROW_INCONSISTENCY_EXCEPTION;
 
@@ -613,17 +624,18 @@ void NoteTrack::Silence(double t0, double t1)
    // XXX: do we want to set the all param?
    // If it's set, then it seems like notes are silenced if they start or end in the range,
    // otherwise only if they start in the range. --Poke
-   seq.silence(t0 - GetOffset(), len, false);
+   seq.silence(t0 - mOrigin, len, false);
 }
 
 void NoteTrack::InsertSilence(double t, double len)
 {
+   assert(IsLeader());
    if (len < 0)
       THROW_INCONSISTENCY_EXCEPTION;
 
    auto &seq = GetSeq();
    seq.convert_to_seconds();
-   seq.insert_silence(t - GetOffset(), len);
+   seq.insert_silence(t - mOrigin, len);
 
    // is this needed?
    // AddToDuration( len );
@@ -634,7 +646,7 @@ void NoteTrack::SetVelocity(float velocity)
 {
    if (GetVelocity() != velocity) {
       DoSetVelocity(velocity);
-      Notify();
+      Notify(false);
    }
 }
 
@@ -676,12 +688,12 @@ bool NoteTrack::Shift(double t) // t is always seconds
 QuantizedTimeAndBeat NoteTrack::NearestBeatTime( double time ) const
 {
    // Alg_seq knows nothing about offset, so remove offset time
-   double seq_time = time - GetOffset();
+   double seq_time = time - mOrigin;
    double beat;
    auto &seq = GetSeq();
    seq_time = seq.nearest_beat_time(seq_time, &beat);
    // add the offset back in to get "actual" audacity track time
-   return { seq_time + GetOffset(), beat };
+   return { seq_time + mOrigin, beat };
 }
 
 static const Track::TypeInfo &typeInfo()
@@ -702,26 +714,31 @@ auto NoteTrack::ClassTypeInfo() -> const TypeInfo &
    return typeInfo();
 }
 
-Track::Holder NoteTrack::PasteInto( AudacityProject & ) const
+Track::Holder NoteTrack::PasteInto(AudacityProject &, TrackList &list) const
 {
+   assert(IsLeader());
    auto pNewTrack = std::make_shared<NoteTrack>();
    pNewTrack->Init(*this);
-   pNewTrack->Paste(0.0, this);
+   pNewTrack->Paste(0.0, *this);
+   list.Add(pNewTrack);
    return pNewTrack;
 }
 
-auto NoteTrack::GetIntervals() const -> ConstIntervals
+size_t NoteTrack::NIntervals() const
 {
-   ConstIntervals results;
-   results.emplace_back( GetStartTime(), GetEndTime() );
-   return results;
+   return 1;
 }
 
-auto NoteTrack::GetIntervals() -> Intervals
+std::shared_ptr<WideChannelGroupInterval>
+NoteTrack::DoGetInterval(size_t iInterval)
 {
-   Intervals results;
-   results.emplace_back( GetStartTime(), GetEndTime() );
-   return results;
+   if (iInterval == 0) {
+      // Just one, and no extra info in it!
+      const auto start = mOrigin;
+      const auto end = start + GetSeq().get_real_dur();
+      return std::make_shared<Interval>(*this, start, end);
+   }
+   return {};
 }
 
 void NoteTrack::AddToDuration( double delta )
@@ -761,7 +778,7 @@ namespace
 Alg_seq *NoteTrack::MakeExportableSeq(std::unique_ptr<Alg_seq> &cleanup) const
 {
    cleanup.reset();
-   double offset = GetOffset();
+   double offset = mOrigin;
    if (offset == 0)
       return &GetSeq();
    // make a copy, deleting events that are shifted before time 0
@@ -891,7 +908,7 @@ bool NoteTrack::ExportMIDI(const wxString &f) const
 
 bool NoteTrack::ExportAllegro(const wxString &f) const
 {
-   double offset = GetOffset();
+   double offset = mOrigin;
    auto in_seconds = ImportExportPrefs::AllegroStyleSetting.ReadEnum();
    auto &seq = GetSeq();
    if (in_seconds) {
@@ -925,7 +942,7 @@ bool NoteTrack::HandleXMLTag(const std::string_view& tag, const AttributesList &
          else if (this->NoteTrackBase::HandleXMLAttribute(attr, value))
          {}
          else if (attr == "offset" && value.TryGet(dblValue))
-            SetOffset(dblValue);
+            MoveTo(dblValue);
          else if (attr == "visiblechannels") {
              if (!value.TryGet(nValue) ||
                  !IsValidVisibleChannels(nValue))
@@ -959,20 +976,21 @@ XMLTagHandler *NoteTrack::HandleXMLChild(const std::string_view&  WXUNUSED(tag))
 void NoteTrack::WriteXML(XMLWriter &xmlFile) const
 // may throw
 {
+   assert(IsLeader());
    std::ostringstream data;
    Track::Holder holder;
    const NoteTrack *saveme = this;
    if (!mSeq) {
       // replace saveme with an (unserialized) duplicate, which is
       // destroyed at end of function.
-      holder = Clone();
+      holder = (*Clone()->begin())->SharedPointer();
       saveme = static_cast<NoteTrack*>(holder.get());
    }
    saveme->GetSeq().write(data, true);
    xmlFile.StartTag(wxT("notetrack"));
    saveme->Track::WriteCommonXMLAttributes( xmlFile );
    this->NoteTrackBase::WriteXMLAttributes(xmlFile);
-   xmlFile.WriteAttr(wxT("offset"), saveme->GetOffset());
+   xmlFile.WriteAttr(wxT("offset"), saveme->mOrigin);
    xmlFile.WriteAttr(wxT("visiblechannels"),
       static_cast<int>(saveme->GetVisibleChannels()));
 
