@@ -12,8 +12,9 @@
 #define __AUDACITY_NOTETRACK__
 
 #include <utility>
+#include "AudioIOSequences.h"
 #include "Prefs.h"
-#include "Track.h"
+#include "PlayableTrack.h"
 
 #if defined(USE_MIDI)
 
@@ -59,7 +60,8 @@ class StretchHandle;
 class TimeWarper;
 
 class AUDACITY_DLL_API NoteTrack final
-   : public NoteTrackBase
+   : public UniqueChannelTrack<NoteTrackBase>
+   , public OtherPlayableSequence
 {
 public:
    // Construct and also build all attachments
@@ -71,14 +73,12 @@ public:
    virtual ~NoteTrack();
 
    using Holder = std::shared_ptr<NoteTrack>;
-   
+
 private:
-   Track::Holder Clone() const override;
+   TrackListHolder Clone() const override;
 
 public:
-   double GetOffset() const override;
-   double GetStartTime() const override;
-   double GetEndTime() const override;
+   void MoveTo(double origin) override { mOrigin = origin; }
 
    Alg_seq &GetSeq() const;
 
@@ -99,12 +99,14 @@ public:
    bool ExportAllegro(const wxString &f) const;
 
    // High-level editing
-   Track::Holder Cut  (double t0, double t1) override;
-   Track::Holder Copy (double t0, double t1, bool forClipboard = true) const override;
+   TrackListHolder Cut(double t0, double t1) override;
+   TrackListHolder Copy(double t0, double t1, bool forClipboard = true)
+      const override;
    bool Trim (double t0, double t1) /* not override */;
    void Clear(double t0, double t1) override;
-   void Paste(double t, const Track *src) override;
-   void Silence(double t0, double t1) override;
+   void Paste(double t, const Track &src) override;
+   void
+   Silence(double t0, double t1, ProgressReporter reportProgress = {}) override;
    void InsertSilence(double t, double len) override;
    bool Shift(double t) /* not override */;
 
@@ -202,17 +204,28 @@ public:
    const TypeInfo &GetTypeInfo() const override;
    static const TypeInfo &ClassTypeInfo();
 
-   Track::Holder PasteInto( AudacityProject & ) const override;
+   Track::Holder PasteInto(AudacityProject &project, TrackList &list)
+      const override;
 
-   ConstIntervals GetIntervals() const override;
-   Intervals GetIntervals() override;
+   size_t NIntervals() const override;
+
+   struct Interval : WideChannelGroupInterval {
+      using WideChannelGroupInterval::WideChannelGroupInterval;
+      ~Interval() override;
+      std::shared_ptr<ChannelInterval> DoGetChannel(size_t iChannel) override;
+   };
+
+private:
+   std::shared_ptr<WideChannelGroupInterval> DoGetInterval(size_t iInterval)
+      override;
 
- private:
 #ifdef EXPERIMENTAL_MIDI_OUT
    void DoSetVelocity(float velocity);
 #endif
 
    void AddToDuration( double delta );
+   void DoOnProjectTempoChange(
+      const std::optional<double>& oldTempo, double newTempo) override;
 
    // These are mutable to allow NoteTrack to switch details of representation
    // in logically const methods
@@ -244,6 +257,7 @@ public:
    std::atomic<unsigned> mVisibleChannels{ ALL_CHANNELS };
 
    std::weak_ptr<StretchHandle> mStretchHandle;
+   double mOrigin{ 0.0 };
 };
 
 /// Data used to display a note track
