@@ -10,13 +10,15 @@
 #include "PluginRegistrationDialog.h"
 
 #include "EffectInterface.h"
+#include "IncompatiblePluginsDialog.h"
 #include "ModuleManager.h"
 #include "PluginManager.h"
 #include "PluginStartupRegistration.h"
 #include "ShuttleGui.h"
-#include "widgets/AudacityMessageBox.h"
-#include "widgets/ProgressDialog.h"
+#include "AudacityMessageBox.h"
+#include "ProgressDialog.h"
 
+#include <set>
 #include <wx/setup.h> // for wxUSE_* macros
 #include <wx/app.h>
 #include <wx/defs.h>
@@ -36,7 +38,7 @@
 //
 // ============================================================================
 #if wxUSE_ACCESSIBILITY
-#include "widgets/WindowAccessible.h"
+#include "WindowAccessible.h"
 
 class CheckListAx final : public WindowAccessible
 {
@@ -874,7 +876,9 @@ void PluginRegistrationDialog::OnRescan(wxCommandEvent& WXUNUSED(evt))
    mEffects->Update();
 
    wxTheApp->CallAfter([this] {
-      PluginPaths disabledPlugins;
+      std::set<PluginPath> disabledPlugins;
+      std::vector<wxString> failedPlugins;
+
       auto& pm = PluginManager::Get();
 
       // Record list of plugins that are currently disabled
@@ -885,7 +889,7 @@ void PluginRegistrationDialog::OnRescan(wxCommandEvent& WXUNUSED(evt))
             continue;
 
          if (!plug.IsEnabled())
-            disabledPlugins.push_back(plug.GetPath());
+            disabledPlugins.insert(plug.GetPath());
       }
 
       pm.ClearEffectPlugins();
@@ -895,6 +899,8 @@ void PluginRegistrationDialog::OnRescan(wxCommandEvent& WXUNUSED(evt))
       {
          PluginStartupRegistration reg(newPlugins);
          reg.Run();
+
+         failedPlugins = reg.GetFailedPluginsPaths();
       }
 
       // Disable all plugins which were previously disabled
@@ -905,11 +911,18 @@ void PluginRegistrationDialog::OnRescan(wxCommandEvent& WXUNUSED(evt))
             continue;
 
          const auto& path = plug.GetPath();
-         if (make_iterator_range(disabledPlugins).contains(path))
+         if (disabledPlugins.find(path) != disabledPlugins.end())
             plug.SetEnabled(false);
       }
 
       pm.Save();
+      pm.NotifyPluginsChanged();
+
+      if (!failedPlugins.empty())
+      {
+         auto dialog = safenew IncompatiblePluginsDialog(this, wxID_ANY, ScanType::Manual, failedPlugins);
+         dialog->ShowModal();
+      }
 
       PopulateItemsList(pm);
       RegenerateEffectsList(mFilter);
@@ -1046,6 +1059,7 @@ void PluginRegistrationDialog::OnOK(wxCommandEvent & WXUNUSED(evt))
       }
 
       pm.Save();
+      pm.NotifyPluginsChanged();
    }
 
    EndModal(wxID_OK);
