@@ -31,6 +31,7 @@ Paul Licameli split from AudacityProject.cpp
 #include "ProjectRate.h"
 #include "ProjectSettings.h"
 #include "ProjectStatus.h"
+#include "ProjectTimeSignature.h"
 #include "ProjectWindow.h"
 #include "SelectFile.h"
 #include "SelectUtilities.h"
@@ -41,11 +42,13 @@ Paul Licameli split from AudacityProject.cpp
 #include "TrackPanel.h"
 #include "UndoManager.h"
 #include "WaveTrack.h"
-#include "WaveClip.h"
 #include "wxFileNameWrapper.h"
-#include "export/Export.h"
-#include "import/Import.h"
+#include "Export.h"
+#include "Import.h"
+#include "ImportProgressListener.h"
+#include "ImportPlugin.h"
 #include "import/ImportMIDI.h"
+#include "import/ImportStreamDialog.h"
 #include "toolbars/SelectionBar.h"
 #include "AudacityMessageBox.h"
 #include "widgets/FileHistory.h"
@@ -58,6 +61,9 @@ Paul Licameli split from AudacityProject.cpp
 
 #include <optional>
 
+#include "RealtimeEffectList.h"
+#include "tracks/playabletrack/wavetrack/WaveTrackUtils.h"
+
 static const AudacityProject::AttachedObjects::RegisteredFactory sFileManagerKey{
    []( AudacityProject &parent ){
       auto result = std::make_shared< ProjectFileManager >( parent );
@@ -159,66 +165,83 @@ auto ProjectFileManager::ReadProjectFile(
    ///
    /// Parse project file
    ///
-   bool bParseSuccess = projectFileIO.LoadProject(fileName, discardAutosave);
+   auto parseResult = projectFileIO.LoadProject(fileName, discardAutosave);
+   const bool bParseSuccess = parseResult.has_value();
    
    bool err = false;
+   std::optional<TranslatableString> linkTypeChangeReason;
+
+   TranslatableString otherError;
 
    if (bParseSuccess)
    {
-      if (discardAutosave)
-         // REVIEW: Failure OK?
-         projectFileIO.AutoSaveDelete();
-      else if (projectFileIO.IsRecovered()) {
-         bool resaved = false;
+      auto& tracks = TrackList::Get(project);
+      FixTracks(
+         tracks,
+         // Keep at most one of the error messages
+         [&](const auto& errorMessage) { otherError = errorMessage; err = true; },
+         [&](const auto& unlinkReason) { linkTypeChangeReason = unlinkReason; });
 
-         if (!projectFileIO.IsTemporary())
+      if (!err) {
+         if(linkTypeChangeReason && !discardAutosave)
          {
-            // Re-save non-temporary project to its own path.  This
-            // might fail to update the document blob in the database.
-            resaved = projectFileIO.SaveProject(fileName, nullptr);
+            BasicUI::ShowMessageBox(XO(
+//i18n-hint: Text of the message dialog that may appear on attempt
+//to open a project created by Audacity version prior to 3.4.
+//%s will be replaced with an explanation of the actual reason of
+//project modification.
+"%s\n"
+"This feature is not supported in Audacity versions past 3.3.3.\n"
+"These stereo tracks have been split into mono tracks.\n"
+"As a result, some realtime effects may be missing.\n"
+"Please verify that everything works as intended before saving."
+            ).Format(linkTypeChangeReason->Translation()));
          }
 
-         AudacityMessageBox(
-            resaved
-               ? XO("This project was not saved properly the last time Audacity ran.\n\n"
-                    "It has been recovered to the last snapshot.")
-               : XO("This project was not saved properly the last time Audacity ran.\n\n"
-                    "It has been recovered to the last snapshot, but you must save it\n"
-                    "to preserve its contents."),
-            XO("Project Recovered"),
-            wxICON_WARNING,
-            &window);
-      }
+         parseResult->Commit();
+         if (discardAutosave)
+            // REVIEW: Failure OK?
+            projectFileIO.AutoSaveDelete();
+         else if (projectFileIO.IsRecovered()) {
+            bool resaved = false;
 
-      // By making a duplicate set of pointers to the existing blocks
-      // on disk, we add one to their reference count, guaranteeing
-      // that their reference counts will never reach zero and thus
-      // the version saved on disk will be preserved until the
-      // user selects Save().
-      mLastSavedTracks = TrackList::Create( nullptr );
+            if (!projectFileIO.IsTemporary() &&
+               !linkTypeChangeReason)
+            {
+               // Re-save non-temporary project to its own path.  This
+               // might fail to update the document blob in the database.
+               resaved = projectFileIO.SaveProject(fileName, nullptr);
+            }
 
-      auto &tracks = TrackList::Get( project );
-      for (auto t : tracks.Any())
-      {
-         if (t->GetErrorOpening())
-         {
-            wxLogWarning(
-               wxT("Track %s had error reading clip values from project file."),
-               t->GetName());
-            err = true;
+            AudacityMessageBox(
+               resaved
+                  ? XO(
+"This project was not saved properly the last time Audacity ran.\n\n"
+"It has been recovered to the last snapshot.")
+                  : XO(
+"This project was not saved properly the last time Audacity ran.\n\n"
+"It has been recovered to the last snapshot, but you must save it\n"
+"to preserve its contents."),
+               XO("Project Recovered"),
+               wxICON_WARNING,
+               &window);
          }
 
-         err = ( !t->LinkConsistencyFix() ) || err;
-
-         mLastSavedTracks->Add(t->Duplicate());
+         // By making a duplicate set of pointers to the existing blocks
+         // on disk, we add one to their reference count, guaranteeing
+         // that their reference counts will never reach zero and thus
+         // the version saved on disk will be preserved until the
+         // user selects Save().
+         mLastSavedTracks = TrackList::Create( nullptr );
+         for (auto t : tracks)
+            mLastSavedTracks->Append(std::move(*t->Duplicate()));
       }
    }
 
-   return
-   {
+   return {
       bParseSuccess,
       err,
-      projectFileIO.GetLastError(),
+      (bParseSuccess ? otherError : projectFileIO.GetLastError()),
       FindHelpUrl(projectFileIO.GetLibraryError())
    };
 }
@@ -285,7 +308,7 @@ bool ProjectFileManager::DoSave(const FilePath & fileName, const bool fromSaveAs
       }
 
       auto &tracks = TrackList::Get( proj );
-      if (!tracks.Any())
+      if (tracks.empty())
       {
          if (UndoManager::Get( proj ).UnsavedChanges() &&
                settings.EmptyCanBeDirty())
@@ -374,10 +397,8 @@ bool ProjectFileManager::DoSave(const FilePath & fileName, const bool fromSaveAs
    mLastSavedTracks = TrackList::Create(nullptr);
 
    auto &tracks = TrackList::Get(proj);
-   for (auto t : tracks.Any())
-   {
-      mLastSavedTracks->Add(t->Duplicate());
-   }
+   for (auto t : tracks)
+      mLastSavedTracks->Append(std::move(*t->Duplicate()));
 
    // If we get here, saving the project was successful, so we can DELETE
    // any backup project.
@@ -743,9 +764,7 @@ void ProjectFileManager::CompactProjectOnClose()
    if (mLastSavedTracks)
    {
       for (auto wt : mLastSavedTracks->Any<WaveTrack>())
-      {
          wt->CloseLock();
-      }
 
       // Attempt to compact the project
       projectFileIO.Compact( { mLastSavedTracks.get() } );
@@ -989,6 +1008,62 @@ AudacityProject *ProjectFileManager::OpenFile( const ProjectChooserFn &chooser,
    return Get(project).OpenProjectFile(fileName, addtohistory);
 }
 
+void ProjectFileManager::FixTracks(TrackList& tracks,
+   const std::function<void(const TranslatableString&)>& onError,
+   const std::function<void(const TranslatableString&)>& onUnlink)
+{
+   Track* unlinkedTrack {};
+   for (const auto t : tracks) {
+      const auto linkType = t->GetLinkType();
+      // Note, the next function may have an important upgrading side effect,
+      // and return no error; or it may find a real error and repair it, but
+      // that repaired track won't be used because opening will fail.
+      if (!t->LinkConsistencyFix()) {
+         onError(XO("A channel of a stereo track was missing."));
+         unlinkedTrack = nullptr;
+      }
+      if(unlinkedTrack != nullptr)
+      {
+         //Not an elegant way to deal with stereo wave track linking
+         //compatibility between versions
+         if(const auto left = dynamic_cast<WaveTrack*>(unlinkedTrack))
+         {
+            if(const auto right = dynamic_cast<WaveTrack*>(t))
+            {
+               left->SetPan(-1.0f);
+               right->SetPan(1.0f);
+               RealtimeEffectList::Get(*left).Clear();
+               RealtimeEffectList::Get(*right).Clear();
+
+               if(left->GetRate() != right->GetRate())
+                  //i18n-hint: explains why opened project was auto-modified 
+                  onUnlink(XO("This project contained stereo tracks with different sample rates per channel."));
+               if(left->GetSampleFormat() != right->GetSampleFormat())
+                  //i18n-hint: explains why opened project was auto-modified  
+                  onUnlink(XO("This project contained stereo tracks with different sample formats in channels."));
+               //i18n-hint: explains why opened project was auto-modified 
+               onUnlink(XO("This project contained stereo tracks with non-aligned content."));
+            }
+         }
+         unlinkedTrack = nullptr;
+      }
+
+      if(linkType != ChannelGroup::LinkType::None &&
+         t->GetLinkType() == ChannelGroup::LinkType::None)
+      {
+         //Wait when LinkConsistencyFix is called on the second track
+         unlinkedTrack = t;
+      }
+
+      if (const auto message = t->GetErrorOpening()) {
+         wxLogWarning(
+            wxT("Track %s had error reading clip values from project file."),
+            t->GetName());
+         onError(*message);
+      }
+   }
+}
+
 AudacityProject *ProjectFileManager::OpenProjectFile(
    const FilePath &fileName, bool addtohistory)
 {
@@ -1004,7 +1079,7 @@ AudacityProject *ProjectFileManager::OpenProjectFile(
    const auto &errorStr = results.errorString;
    const bool err = results.trackError;
 
-   if (bParseSuccess) {
+   if (bParseSuccess && !err) {
       auto &formats = ProjectNumericFormats::Get( project );
       auto &settings = ProjectSettings::Get( project );
       window.mbInitializingScrollbar = true;
@@ -1019,7 +1094,7 @@ AudacityProject *ProjectFileManager::OpenProjectFile(
          formats.GetBandwidthSelectionFormatName());
       
       ProjectHistory::Get( project ).InitialState();
-      TrackFocus::Get( project ).Set( *tracks.Any().begin() );
+      TrackFocus::Get(project).Set(*tracks.begin());
       window.HandleResize();
       trackPanel.Refresh(false);
 
@@ -1033,7 +1108,7 @@ AudacityProject *ProjectFileManager::OpenProjectFile(
          FileHistory::Global().Append(fileName);
    }
 
-   if (bParseSuccess) {
+   if (bParseSuccess && !err) {
       if (projectFileIO.IsRecovered())
       {
          // PushState calls AutoSave(), so no longer need to do so here.
@@ -1052,7 +1127,7 @@ AudacityProject *ProjectFileManager::OpenProjectFile(
       // may have spared the files at the expense of leaked memory).  But
       // here is a better way to accomplish the intent, doing like what happens
       // when the project closes:
-      for ( auto pTrack : tracks.Any< WaveTrack >() )
+      for (auto pTrack : tracks.Any<WaveTrack>())
          pTrack->CloseLock();
 
       tracks.Clear(); //tracks.Clear(true);
@@ -1070,14 +1145,14 @@ AudacityProject *ProjectFileManager::OpenProjectFile(
 
 void
 ProjectFileManager::AddImportedTracks(const FilePath &fileName,
-                                      TrackHolders &&newTracks)
+   TrackHolders &&newTracks)
 {
    auto &project = mProject;
    auto &history = ProjectHistory::Get( project );
    auto &projectFileIO = ProjectFileIO::Get( project );
    auto &tracks = TrackList::Get( project );
 
-   std::vector< std::shared_ptr< Track > > results;
+   std::vector<Track*> results;
 
    SelectUtilities::SelectNone( project );
 
@@ -1093,26 +1168,21 @@ ProjectFileManager::AddImportedTracks(const FilePath &fileName,
    // all newly imported tracks are muted.
    const bool projectHasSolo =
       !(tracks.Any<PlayableTrack>() + &PlayableTrack::GetSolo).empty();
-   if (projectHasSolo)
-   {
-      for (auto& track : newTracks)
-         for (auto& channel : track)
-            channel->SetMute(true);
+   if (projectHasSolo) {
+      for (auto &group : newTracks)
+         for (const auto pTrack : group->Any<WaveTrack>())
+            pTrack->SetMute(true);
    }
 
    // Must add all tracks first (before using Track::IsLeader)
    for (auto &group : newTracks) {
-      if (group.empty()) {
-         wxASSERT(false);
+      if (group->empty()) {
+         assert(false);
          continue;
       }
-      auto first = group.begin()->get();
-      auto nChannels = group.size();
-      for (auto &uNewTrack : group) {
-         auto newTrack = tracks.Add( uNewTrack );
-         results.push_back(newTrack->SharedPointer());
-      }
-      tracks.MakeMultiChannelTrack(*first, nChannels, true);
+      for (const auto pTrack : group->Any<WaveTrack>())
+         results.push_back(pTrack);
+      tracks.Append(std::move(*group));
    }
    newTracks.clear();
       
@@ -1120,40 +1190,27 @@ ProjectFileManager::AddImportedTracks(const FilePath &fileName,
 
    // Add numbers to track names only if there is more than one (mono or stereo)
    // track (not necessarily, more than one channel)
-   const bool useSuffix =
-      make_iterator_range( results.begin() + 1, results.end() )
-         .any_of( []( decltype(*results.begin()) &pTrack )
-            { return pTrack->IsLeader(); } );
+   const bool useSuffix = results.size() > 1;
 
    for (const auto &newTrack : results) {
-      if ( newTrack->IsLeader() )
-         // Count groups only
-         ++i;
-
+      ++i;
       newTrack->SetSelected(true);
-
-      
       if (useSuffix)
-          //i18n-hint Name default name assigned to a clip on track import
-          newTrack->SetName(XC("%s %d", "clip name template").Format(trackNameBase, i + 1).Translation());
+         //i18n-hint Name default name assigned to a clip on track import
+         newTrack->SetName(XC("%s %d", "clip name template")
+            .Format(trackNameBase, i + 1).Translation());
       else
-          newTrack->SetName(trackNameBase);
+         newTrack->SetName(trackNameBase);
 
-      newTrack->TypeSwitch([&](WaveTrack *wt) {
+      newTrack->TypeSwitch([&](WaveTrack &wt) {
          if (newRate == 0)
-            newRate = wt->GetRate();
-         auto trackName = wt->GetName();
-         for (auto& clip : wt->GetClips())
-            clip->SetName(trackName);
+            newRate = wt.GetRate();
+         const auto trackName = wt.GetName();
+         for(const auto& interval : wt.Intervals())
+            interval->SetName(trackName);
       });
    }
 
-   // Automatically assign rate of imported file to whole project,
-   // if this is the first file that is imported
-   if (initiallyEmpty && newRate > 0) {
-      ProjectRate::Get(project).SetRate( newRate );
-   }
-
    history.PushState(XO("Imported '%s'").Format( fileName ),
        XO("Import"));
 
@@ -1189,16 +1246,79 @@ bool ImportProject(AudacityProject &dest, const FilePath &fileName)
       return false;
    auto &srcTracks = TrackList::Get(project);
    auto &destTracks = TrackList::Get(dest);
-   for (const Track *pTrack : srcTracks.Any()) {
-      auto destTrack = pTrack->PasteInto(dest);
-      Track::FinishCopy(pTrack, destTrack.get());
-      if (destTrack.use_count() == 1)
-         destTracks.Add(destTrack);
-   }
+   for (const Track *pTrack : srcTracks)
+      pTrack->PasteInto(dest, destTracks);
    Tags::Get(dest).Merge(Tags::Get(project));
 
    return true;
 }
+
+class ImportProgress final
+   : public ImportProgressListener
+{
+   wxWeakRef<AudacityProject> mProject;
+public:
+   
+   ImportProgress(AudacityProject& project)
+      : mProject(&project)
+   {
+
+   }
+   
+   bool OnImportFileOpened(ImportFileHandle& importFileHandle) override
+   {
+      mImportFileHandle = &importFileHandle;
+      // File has more than one stream - display stream selector
+      if (importFileHandle.GetStreamCount() > 1)
+      {
+         ImportStreamDialog ImportDlg(&importFileHandle, NULL, -1, XO("Select stream(s) to import"));
+
+         if (ImportDlg.ShowModal() == wxID_CANCEL)
+            return false;
+      }
+      // One stream - import it by default
+      else
+         importFileHandle.SetStreamUsage(0,TRUE);
+      return true;
+   }
+   
+   void OnImportProgress(double progress) override
+   {
+      constexpr double ProgressSteps { 1000.0 };
+      if(!mProgressDialog)
+      {
+         wxFileName ff( mImportFileHandle->GetFilename() );
+         auto title = XO("Importing %s").Format(  mImportFileHandle->GetFileDescription() );
+         mProgressDialog = BasicUI::MakeProgress(title, Verbatim(ff.GetFullName()));
+      }
+      auto result = mProgressDialog->Poll(progress * ProgressSteps, ProgressSteps);
+      if(result == BasicUI::ProgressResult::Cancelled)
+         mImportFileHandle->Cancel();
+      else if(result == BasicUI::ProgressResult::Stopped)
+         mImportFileHandle->Stop();
+   }
+   
+   void OnImportResult(ImportResult result) override
+   {
+      mProgressDialog.reset();
+      if(result == ImportResult::Error)
+      {
+         auto message = mImportFileHandle->GetErrorMessage();
+         if(!message.empty())
+         {
+            AudacityMessageBox(message, XO("Import"), wxOK | wxCENTRE | wxICON_ERROR,
+                               mProject ? &GetProjectFrame(*mProject) : nullptr);
+         }
+      }
+   }
+   
+private:
+   
+   ImportFileHandle* mImportFileHandle {nullptr};
+   std::unique_ptr<BasicUI::ProgressDialog> mProgressDialog;
+};
+
+
 }
 
 // If pNewTrackList is passed in non-NULL, it gets filled with the pointers to NEW tracks.
@@ -1270,7 +1390,9 @@ bool ProjectFileManager::Import(
          return false;
       }
 #endif
+      ImportProgress importProgress(project);
       bool success = Importer::Get().Import(project, fileName,
+                                            &importProgress,
                                             &WaveTrackFactory::Get( project ),
                                             newTracks,
                                             newTags.get(),
@@ -1284,6 +1406,11 @@ bool ProjectFileManager::Import(
       if (!success)
          return false;
 
+      const auto projectTempo = ProjectTimeSignature::Get(project).GetTempo();
+      for (auto trackList : newTracks)
+         for (auto track : *trackList)
+            track->OnProjectTempoChange(projectTempo);
+
       if (addToHistory) {
          FileHistory::Global().Append(fileName);
       }
