@@ -24,9 +24,8 @@ class wxTimerEvent;
 class AudacityProject;
 struct AudioIOStartStreamOptions;
 
-struct ProjectStatusEvent;
-
 enum StatusBarField : int;
+enum class ProjectFileIOMessage : int;
 
 ///\brief Object associated with a project for high-level management of the
 /// project's lifetime, including creation, destruction, opening from file,
@@ -40,8 +39,8 @@ public:
    static const ProjectManager &Get( const AudacityProject &project );
 
    explicit ProjectManager( AudacityProject &project );
-   ProjectManager( const ProjectManager & ) PROHIBITED;
-   ProjectManager &operator=( const ProjectManager & ) PROHIBITED;
+   ProjectManager( const ProjectManager & ) = delete;
+   ProjectManager &operator=( const ProjectManager & ) = delete;
    ~ProjectManager() override;
 
    // This is the factory for projects:
@@ -75,7 +74,7 @@ public:
          , mReuseNonemptyProject{ reuseNonemptyProject }
       {}
       //! Don't copy.  Use std::ref to pass it to ProjectFileManager
-      ProjectChooser( const ProjectChooser& ) PROHIBITED;
+      ProjectChooser( const ProjectChooser& ) = delete;
       //! Destroy any fresh project, or rollback the existing project, unless committed
       ~ProjectChooser();
       //! May create a fresh project
@@ -117,10 +116,9 @@ public:
    static void SetClosingAll(bool closing);
 
 private:
-   void OnReconnectionFailure(wxCommandEvent & event);
+   void OnReconnectionFailure(ProjectFileIOMessage);
    void OnCloseWindow(wxCloseEvent & event);
    void OnTimer(wxTimerEvent & event);
-   void OnOpenAudioFile(wxCommandEvent & event);
    void OnStatusChange(StatusBarField field);
 
    void RestartTimer();
@@ -130,7 +128,8 @@ private:
 
    std::unique_ptr<wxTimer> mTimer;
 
-   Observer::Subscription mSubscription;
+   Observer::Subscription mProjectStatusSubscription,
+      mProjectFileIOSubscription;
 
    DECLARE_EVENT_TABLE()
 
