@@ -26,13 +26,6 @@ class AudacityProject;
 wxDECLARE_EXPORTED_EVENT(AUDACITY_DLL_API,
    EVT_PROJECT_SETTINGS_CHANGE, wxCommandEvent);
 
-enum
-{
-   SNAP_OFF,
-   SNAP_NEAREST,
-   SNAP_PRIOR
-};
-
 namespace ToolCodes {
 enum {
    // The buttons that are in the Tools toolbar must be in correspondence
@@ -63,29 +56,17 @@ public:
    
    // Values retrievable from GetInt() of the event for settings change
    enum EventCode : int {
-      ChangedSyncLock,
       ChangedTool,
    };
 
    explicit ProjectSettings( AudacityProject &project );
-   ProjectSettings( const ProjectSettings & ) PROHIBITED;
-   ProjectSettings &operator=( const ProjectSettings & ) PROHIBITED;
+   ProjectSettings( const ProjectSettings & ) = delete;
+   ProjectSettings &operator=( const ProjectSettings & ) = delete;
 
 
    bool GetTracksFitVerticallyZoomed() const { return mTracksFitVerticallyZoomed; } //lda
    void SetTracksFitVerticallyZoomed(bool flag) { mTracksFitVerticallyZoomed = flag; } //lda
 
-   bool GetShowId3Dialog() const { return mShowId3Dialog; } //lda
-   void SetShowId3Dialog(bool flag) { mShowId3Dialog = flag; } //lda
-
-   bool IsSyncLocked() const;
-   void SetSyncLock(bool flag);
-   
-   // Snap To
-
-   void SetSnapTo(int snap);
-   int GetSnapTo() const;
-
    // Current tool
 
    void SetTool(int tool);
@@ -100,24 +81,6 @@ public:
 
    void SetOvertones(bool isSelected) { mbOvertones = isSelected; }
    bool IsOvertones() const { return mbOvertones; }
-   
-   // Selection Format
-   void SetSelectionFormat(const NumericFormatSymbol & format);
-   const NumericFormatSymbol & GetSelectionFormat() const;
-
-   // AudioTime format
-   void SetAudioTimeFormat(const NumericFormatSymbol & format);
-   const NumericFormatSymbol & GetAudioTimeFormat() const;
-
-   // Spectral Selection Formats
-   void SetFrequencySelectionFormatName(const NumericFormatSymbol & format);
-   const NumericFormatSymbol & GetFrequencySelectionFormatName() const;
-
-   void SetBandwidthSelectionFormatName(const NumericFormatSymbol & format);
-   const NumericFormatSymbol & GetBandwidthSelectionFormatName() const;
-
-   bool IsSoloSimple() const { return mSoloPref == wxT("Simple"); }
-   bool IsSoloNone() const { return mSoloPref == wxT("None"); }
 
    bool EmptyCanBeDirty() const { return mEmptyCanBeDirty; }
 
@@ -128,15 +91,6 @@ private:
 
    AudacityProject &mProject;
 
-   NumericFormatSymbol mSelectionFormat;
-   NumericFormatSymbol mFrequencySelectionFormatName;
-   NumericFormatSymbol mBandwidthSelectionFormatName;
-   NumericFormatSymbol mAudioTimeFormat;
-
-   wxString mSoloPref;
-
-   int mSnapTo;
-
    int mCurrentTool;
    int mCurrentBrushRadius;
    int mCurrentBrushHop;
@@ -144,8 +98,6 @@ private:
    bool mbOvertones { false };
    
    bool mTracksFitVerticallyZoomed{ false };  //lda
-   bool mShowId3Dialog{ true }; //lda
-   bool mIsSyncLocked{ false };
    bool mEmptyCanBeDirty;
    bool mShowSplashScreen;
 };
