@@ -25,7 +25,4 @@
 
 class AudacityProject;
 
-AUDACITY_DLL_API void OpenScreenshotTools( AudacityProject &project );
-void CloseScreenshotTools();
-
 #endif // __AUDACITY_SCREENSHOT__
