@@ -15,13 +15,26 @@
 #include "SelectedRegion.h"
 #include "SyncLock.h"
 #include "Theme.h"
+#include "TimeAndPitchInterface.h"
 #include "Track.h"
 #include "TrackArtist.h"
 #include "TrackPanelDrawingContext.h"
 #include "ZoomInfo.h"
+#include "TimeDisplayMode.h"
+#include "ProjectTimeRuler.h"
+
+#include "ProjectTimeSignature.h"
+
+#include "widgets/BeatsFormat.h"
+
 #include <wx/app.h>
 #include <wx/dc.h>
 
+#include <cassert>
+#include <cstdint>
+#include <cmath>
+#include <utility>
+
 static constexpr int ClipSelectionStrokeSize{ 1 };//px
 
 namespace
@@ -43,9 +56,6 @@ namespace
 
 /// Takes a value between min and max and returns a value between
 /// height and 0
-/// \todo  Should this function move int GuiWaveTrack where it can
-/// then use the zoomMin, zoomMax and height values without having
-/// to have them passed in to it??
 int GetWaveYPos(float value, float min, float max,
                 int height, bool dB, bool outer,
                 float dBr, bool clip)
@@ -188,7 +198,7 @@ wxString TrackArt::TruncateText(wxDC& dc, const wxString& text, const int maxWid
    return wxEmptyString;
 }
 
-wxRect TrackArt::DrawClipAffordance(wxDC& dc, const wxRect& rect, const wxString& title, bool highlight, bool selected)
+wxRect TrackArt::DrawClipAffordance(wxDC& dc, const wxRect& rect, bool highlight, bool selected)
 {
    //To make sure that roundings do not overlap each other
    auto clipFrameRadius = std::min(ClipFrameRadius, rect.width / 2);
@@ -236,20 +246,119 @@ wxRect TrackArt::DrawClipAffordance(wxDC& dc, const wxRect& rect, const wxString
       GetAffordanceTitleRect(rect.Intersect(clipRect)) :
       GetAffordanceTitleRect(rect);
 
-   if (!title.empty())
-   {
-      auto truncatedTitle = TrackArt::TruncateText(dc, title, titleRect.GetWidth());
-      if (!truncatedTitle.empty())
-      {
-         auto hAlign = wxTheApp->GetLayoutDirection() == wxLayout_RightToLeft ? wxALIGN_RIGHT : wxALIGN_LEFT;
-         dc.DrawLabel(truncatedTitle, titleRect, hAlign | wxALIGN_CENTER_VERTICAL);
-      }
-      else
-         return { };
-   }
    return titleRect;
 }
 
+namespace{
+wxString GetPlaybackSpeedFullText(double clipStretchRatio)
+{
+   // clang-format off
+   // We reckon that most of the time, a rounded percentage value is sufficient.
+   // There are two exceptions:
+   // - The clip is only slightly stretched such that the rounded value is 100.
+   //   There should be an indicator if and only if the clip is stretched, this
+   //   must be reliable. Yet showing "100%" would be confusing. Hence in that
+   //   case we constrain the values to [99.1, ..., 99.9, 100.1,
+   //   ..., 100.9], i.e., omitting 100.0.
+   // - The clip is stretched so much that the playback speed is less than 1%.
+   //   Make sure in that case that we never show 0%, but always at least 0.1%.
+   // clang-format on
+
+   const auto playbackSpeed = 100 / clipStretchRatio;
+   wxString fullText;
+
+   // We compare with .95 rather than 1, since playback speeds within [100.95,
+   // 101) get rounded to 101.0 and (99, 99.05] to 99.0 by `wxString::Format`.
+   // Let these be processed by the integer-display branch of this if statement
+   // instead.
+   if (fabs(playbackSpeed - 100.) < .95)
+      // Never show 100.0%
+      fullText = wxString::Format(
+         "%.1f%% speed", playbackSpeed > 100 ?
+                              std::max(playbackSpeed, 100.1) :
+                              std::min(playbackSpeed, 99.9));
+   else if (playbackSpeed < 1)
+      // Never show 0.0%
+      fullText =
+         wxString::Format("%.1f%% speed", std::max(playbackSpeed, 0.1));
+   else {
+      const auto roundedPlaybackSpeed =
+         static_cast<int>(std::round(playbackSpeed));
+      fullText = wxString::Format("%d%% speed", roundedPlaybackSpeed);
+   }
+   return fullText;
+}
+
+enum class HAlign
+{
+   left,
+   right
+};
+
+struct ClipTitle {
+   const wxString text;
+   const HAlign alignment;
+};
+
+std::optional<ClipTitle> DoDrawAudioTitle(
+   wxDC& dc, const wxRect& titleRect, const wxString& title)
+{
+   if(titleRect.IsEmpty())
+      return std::nullopt;
+   const auto hAlign = wxTheApp->GetLayoutDirection() == wxLayout_RightToLeft ?
+                          HAlign::right :
+                          HAlign::left;
+   if(title.empty())
+      return ClipTitle { "", hAlign  };
+   auto truncatedTitle = TrackArt::TruncateText(dc, title, titleRect.GetWidth());
+   if (!truncatedTitle.empty())
+   {
+      dc.DrawLabel(
+         truncatedTitle, titleRect,
+         (hAlign == HAlign::left ? wxALIGN_LEFT : wxALIGN_RIGHT) |
+            wxALIGN_CENTER_VERTICAL);
+      return ClipTitle{ truncatedTitle, hAlign };
+   }
+   return std::nullopt;
+}
+
+}
+
+bool TrackArt::DrawClipTitle(
+   wxDC& dc, const wxRect& titleRect, const wxString& title)
+{
+   return DoDrawAudioTitle(dc, titleRect, title).has_value();
+}
+
+bool TrackArt::DrawAudioClipTitle(
+   wxDC& dc, const wxRect& titleRect, const wxString& title,
+   double clipStretchRatio)
+{
+   const auto clipTitle = DoDrawAudioTitle(dc, titleRect, title);
+   if (!clipTitle.has_value())
+      return false;
+   if (!TimeAndPitchInterface::IsPassThroughMode(clipStretchRatio))
+   {
+      const auto fullText = GetPlaybackSpeedFullText(clipStretchRatio);
+      constexpr auto minSpaceBetweenTitleAndSpeed = 12; // pixels
+      const auto remainingWidth = std::max(
+         titleRect.GetWidth() - dc.GetTextExtent(clipTitle->text).GetWidth() -
+            minSpaceBetweenTitleAndSpeed,
+         0);
+      const auto truncatedText =
+         TrackArt::TruncateText(dc, fullText, remainingWidth);
+      if (truncatedText.find('%') != std::string::npos)
+         // Only show if there is room for the % sign, or else it can be hard to
+         // interpret.
+         dc.DrawLabel(
+            TrackArt::TruncateText(dc, fullText, remainingWidth), titleRect,
+            (clipTitle->alignment == HAlign::left ? wxALIGN_RIGHT :
+                                                    wxALIGN_LEFT) |
+               wxALIGN_CENTER_VERTICAL);
+   }
+   return true;
+}
+
 void TrackArt::DrawClipEdges(wxDC& dc, const wxRect& clipRect, bool selected)
 {
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
@@ -416,6 +525,182 @@ void TrackArt::DrawSyncLockTiles(
    }
 }
 
+namespace
+{
+constexpr double minSubdivisionWidth = 12.0;
+
+struct BeatsGridlinePainter final
+{
+   const ZoomInfo& zoomInfo;
+   const bool enabled;
+
+   const BeatsFormat& beatsRulerFormat;
+
+   const BeatsFormat::Tick majorTick;
+   const BeatsFormat::Tick minorTick;
+
+   // Note duration in seconds
+   const double noteDuration;
+   // Note width in pixels
+   const double noteWidth;
+   // How many "notes" should be colored with a single color?
+   const int64_t notesInBeat;
+
+
+BeatsGridlinePainter(const ZoomInfo& zoomInfo, const Track& track) noexcept
+       : zoomInfo { zoomInfo }
+       , enabled { TimeDisplayModePreference.ReadEnum() ==
+                   TimeDisplayMode::BeatsAndMeasures }
+       , beatsRulerFormat { ProjectTimeRuler::Get(GetProject(track))
+                               .GetBeatsFormat() }
+       , majorTick { beatsRulerFormat.GetSubdivision().major }
+       , minorTick { GetMinorTick() }
+       , noteDuration { minorTick.duration }
+       , noteWidth { zoomInfo.TimeRangeToPixelWidth(noteDuration) }
+       , notesInBeat { CalculateNotesInBeat() }
+   {
+   }
+
+void DrawSeparators (
+   wxDC& dc, const wxRect& rect, const wxPen& beatSepearatorPen, const wxPen& barSeparatorPen) const
+{
+   dc.SetPen (beatSepearatorPen);
+
+   const auto majorTick = beatsRulerFormat.GetSubdivision().major;
+   const auto minorTick = GetMinorTick();
+
+   const auto [firstNote, lastNote] = GetBoundaries(
+      rect, rect, noteWidth);
+
+   for (auto noteIndex = firstNote; noteIndex < lastNote; ++noteIndex)
+   {
+      const auto position = GetPositionInRect (noteIndex, rect, noteDuration);
+
+      if (position < rect.GetLeft () || position >= rect.GetRight ())
+         continue;
+
+      dc.SetPen(IsFirstInMajorTick(noteIndex) ? barSeparatorPen : beatSepearatorPen);
+      dc.DrawLine (position, rect.GetTop (), position, rect.GetBottom () + 1);
+   }
+}
+
+void DrawBackground (
+   wxDC& dc, const wxRect& subRect, const wxRect& fullRect, const wxBrush& strongBeatBrush,
+   const wxBrush& weakBeatBrush) const
+{
+   if (!UseAlternatingColors ())
+   {
+      dc.SetBrush(strongBeatBrush);
+      dc.DrawRectangle(subRect);
+      return;
+   }
+
+   auto [firstIndex, lastIndex] =
+      GetBoundaries (subRect, fullRect, noteWidth);
+
+   // Make first index to be on notesInBeatBoundary
+   firstIndex = (firstIndex / notesInBeat) * notesInBeat;
+
+   const auto beatDuration = noteDuration;
+
+   const auto top = fullRect.GetTop ();
+   const auto height = fullRect.GetHeight ();
+
+   bool strongBeat = (firstIndex / notesInBeat) % 2 == 0;
+   for (auto index = firstIndex; index < lastIndex; index += notesInBeat, strongBeat = !strongBeat)
+   {
+      const auto left = std::max<int> (
+         GetPositionInRect(index, fullRect, beatDuration),
+         subRect.GetLeft());
+      const auto right = std::min<int> (
+         GetPositionInRect(index + notesInBeat, fullRect, beatDuration),
+         subRect.GetRight());
+
+      const auto& brush = strongBeat ? strongBeatBrush : weakBeatBrush;
+
+      dc.SetBrush (brush);
+      dc.DrawRectangle (left, top, right - left + 1, height);
+   }
+}
+
+private:
+   const AudacityProject& GetProject(const Track& track) const
+   {
+      // Track is expected to have owner
+      assert(track.GetOwner());
+      // TracList is expected to have owner
+      assert(track.GetOwner()->GetOwner());
+
+      return *track.GetOwner()->GetOwner();
+   }
+
+   int64_t CalculateNotesInBeat() const
+   {
+      if (UseAlternatingColors())
+         return minorTick.lower / 4;
+
+      return 1;
+   }
+
+   bool IsFirstInMajorTick(int64_t noteIndex) const
+   {
+      const auto notesInMajorTick =
+         minorTick.lower * majorTick.upper / majorTick.lower / minorTick.upper;
+
+      if (notesInMajorTick == 0)
+         return false;
+
+      return noteIndex % notesInMajorTick == 0;
+   }
+
+   bool UseAlternatingColors() const
+   {
+      return minorTick.lower >= 4 && minorTick.upper == 1;
+   }
+
+   double GetPositionInRect(int64_t index, const wxRect& rect, double duration) const
+   {
+      return zoomInfo.TimeToPosition(index * duration) + rect.x + 1;
+   }
+
+   std::pair<int64_t, int64_t> GetBoundaries(const wxRect& subRect, const wxRect& fullRect, double width) const
+   {
+      const auto offset = subRect.x - fullRect.x;
+
+      return { std::floor(zoomInfo.GetAbsoluteOffset(offset) / width),
+               std::ceil(
+                  zoomInfo.GetAbsoluteOffset(offset + subRect.GetWidth()) /
+                  width) };
+   }
+
+   BeatsFormat::Tick GetMinorTick() const
+   {
+      const auto& subdivision = beatsRulerFormat.GetSubdivision();
+
+      auto tick = subdivision.minorMinor;
+
+      auto minorMinorLength =
+         zoomInfo.TimeRangeToPixelWidth(subdivision.minorMinor.duration);
+
+      const auto nextSubdivision = subdivision.minor.duration == 0.0 ?
+                                subdivision.major :
+                                subdivision.minor;
+
+      while (minorMinorLength <= minSubdivisionWidth)
+      {
+         tick.lower /= 2;
+         tick.duration *= 2.0;
+         minorMinorLength *= 2.0;
+
+         if (nextSubdivision.lower >= tick.lower)
+            return nextSubdivision;
+      }
+
+      return tick;
+   }
+};
+}
+
 void TrackArt::DrawBackgroundWithSelection(
    TrackPanelDrawingContext &context, const wxRect &rect,
    const Track *track, const wxBrush &selBrush, const wxBrush &unselBrush,
@@ -424,13 +709,35 @@ void TrackArt::DrawBackgroundWithSelection(
    const auto dc = &context.dc;
    const auto artist = TrackArtist::Get( context );
    const auto &selectedRegion = *artist->pSelectedRegion;
-   const auto &zoomInfo = *artist->pZoomInfo;
+   const auto& zoomInfo = *artist->pZoomInfo;
+
 
    //MM: Draw background. We should optimize that a bit more.
    const double sel0 = useSelection ? selectedRegion.t0() : 0.0;
    const double sel1 = useSelection ? selectedRegion.t1() : 0.0;
 
+   BeatsGridlinePainter gridlinePainter(zoomInfo, *track);
+
    dc->SetPen(*wxTRANSPARENT_PEN);
+
+   auto drawBgRect = [dc, &gridlinePainter, artist, &rect](
+                        const wxBrush& regularBrush,
+                        const wxBrush& beatStrongBrush,
+                        const wxBrush& beatWeakBrush, const wxRect& subRect)
+   {
+      if (!gridlinePainter.enabled)
+      {
+         // Track not selected; just draw background
+         dc->SetBrush(regularBrush);
+         dc->DrawRectangle(subRect);
+      }
+      else
+      {
+         gridlinePainter.DrawBackground(
+            *dc, subRect, rect, beatStrongBrush, beatWeakBrush);
+      }
+   };
+
    if (SyncLock::IsSelectedOrSyncLockSelected(track))
    {
       // Rectangles before, within, after the selection
@@ -444,8 +751,7 @@ void TrackArt::DrawBackgroundWithSelection(
       }
 
       if (before.width > 0) {
-         dc->SetBrush(unselBrush);
-         dc->DrawRectangle(before);
+         drawBgRect(unselBrush, artist->beatStrongBrush, artist->beatWeakBrush, before);
 
          within.x = 1 + before.GetRight();
       }
@@ -458,19 +764,16 @@ void TrackArt::DrawBackgroundWithSelection(
       // Bug 2389 - Selection can disappear
       // This handles case where no waveform is visible.
       if (within.width < 1)
-      {
          within.width = 1;
-      }
 
       if (within.width > 0) {
          if (track->GetSelected()) {
-            dc->SetBrush(selBrush);
-            dc->DrawRectangle(within);
+            drawBgRect(selBrush, artist->beatStrongSelBrush, artist->beatWeakSelBrush, within);
          }
          else {
             // Per condition above, track must be sync-lock selected
-            dc->SetBrush(unselBrush);
-            dc->DrawRectangle(within);
+            drawBgRect(
+               unselBrush, artist->beatStrongBrush, artist->beatWeakBrush, within);
             DrawSyncLockTiles( context, within );
          }
 
@@ -482,17 +785,18 @@ void TrackArt::DrawBackgroundWithSelection(
       }
 
       after.width = 1 + rect.GetRight() - after.x;
-      if (after.width > 0) {
-         dc->SetBrush(unselBrush);
-         dc->DrawRectangle(after);
-      }
+      if (after.width > 0)
+         drawBgRect(
+            unselBrush, artist->beatStrongBrush, artist->beatWeakBrush, after);
    }
    else
    {
-      // Track not selected; just draw background
-      dc->SetBrush(unselBrush);
-      dc->DrawRectangle(rect);
+      drawBgRect(
+         unselBrush, artist->beatStrongBrush, artist->beatWeakBrush, rect);
    }
+
+   if (gridlinePainter.enabled)
+      gridlinePainter.DrawSeparators(*dc, rect, artist->beatSepearatorPen, artist->barSepearatorPen);
 }
 
 void TrackArt::DrawCursor(TrackPanelDrawingContext& context,
@@ -501,7 +805,7 @@ void TrackArt::DrawCursor(TrackPanelDrawingContext& context,
    const auto dc = &context.dc;
    const auto artist = TrackArtist::Get(context);
    const auto& selectedRegion = *artist->pSelectedRegion;
-   
+
    if (selectedRegion.isPoint())
    {
        const auto& zoomInfo = *artist->pZoomInfo;
