@@ -27,9 +27,6 @@
 #include <wx/wx.h>
 #endif
 
-
-#include <wx/intl.h>
-
 #include "BasicUI.h"
 #include "Project.h"
 #include "SyncLock.h"
@@ -76,7 +73,7 @@ std::shared_ptr<Track> TrackPanelAx::GetFocus()
       }
       if (!focusedTrack) {
          focusedTrack =
-            Track::SharedPointer( *GetTracks().Any().first );
+            Track::SharedPointer(*GetTracks().Any().first);
          // only call SetFocus if the focus has changed to avoid
          // unnecessary focus events
          if (focusedTrack) 
@@ -110,8 +107,8 @@ std::shared_ptr<Track> TrackPanelAx::SetFocus( std::shared_ptr<Track> track )
    }
 #endif
 
-   if( !track )
-      track = Track::SharedPointer( *GetTracks().Any().begin() );
+   if (!track)
+      track = Track::SharedPointer(*GetTracks().begin());
 
    if ( mFocusedTrack.lock() != track ) {
       mFocusedTrack = track;
@@ -154,32 +151,13 @@ std::shared_ptr<Track> TrackPanelAx::SetFocus( std::shared_ptr<Track> track )
    return track;
 }
 
-// Returns TRUE if passed track has the focus
-bool TrackPanelAx::IsFocused( const Track *track )
-{
-   auto focusedTrack = mFocusedTrack.lock();
-   if( !focusedTrack )
-      focusedTrack = SetFocus();
-
-   // Remap track pointer if there are outstanding pending updates
-   auto origTrack =
-      GetTracks().FindById( track->GetId() );
-   if (origTrack)
-      track = origTrack;
-
-   return focusedTrack
-      ? TrackList::Channels(focusedTrack.get()).contains(track)
-      : !track;
-}
-
 int TrackPanelAx::TrackNum( const std::shared_ptr<Track> &target )
 {
    // Find 1-based position of the target in the visible tracks, or 0 if not
    // found
    int ndx = 0;
 
-   for ( auto t : GetTracks().Leaders() )
-   {
+   for (auto t : GetTracks()) {
       ndx++;
       if( t == target.get() )
       {
@@ -194,8 +172,7 @@ std::shared_ptr<Track> TrackPanelAx::FindTrack( int num )
 {
    int ndx = 0;
 
-   for ( auto t : GetTracks().Leaders() )
-   {
+   for (auto t : GetTracks()) {
       ndx++;
       if( ndx == num )
          return t->SharedPointer();
@@ -271,7 +248,7 @@ wxAccStatus TrackPanelAx::GetChild( int childId, wxAccessible** child )
 // Gets the number of children.
 wxAccStatus TrackPanelAx::GetChildCount( int* childCount )
 {
-   *childCount = GetTracks().Leaders().size();
+   *childCount = GetTracks().Any().size();
    return wxACC_OK;
 }
 
@@ -355,7 +332,7 @@ wxAccStatus TrackPanelAx::GetName( int childId, wxString* name )
    {
       if( childId == wxACC_SELF )
       {
-         *name = _( "TrackView" );
+         *name = _("TrackView");
       }
       else
       {
@@ -558,7 +535,7 @@ wxAccStatus TrackPanelAx::GetValue( int WXUNUSED(childId), wxString* WXUNUSED(st
 #if defined(__WXMAC__)
    if( childId == wxACC_SELF )
    {
-      *strValue = _( "TrackView" );
+      *strValue = _("TrackView");
    }
    else
    {
@@ -760,18 +737,11 @@ Track *TrackFocus::Get()
 void TrackFocus::Set( Track *pTrack )
 {
    if (mAx) {
-      pTrack = *TrackList::Get( mProject ).FindLeader( pTrack );
+      pTrack = *TrackList::Get( mProject ).Find( pTrack );
       mAx->SetFocus( Track::SharedPointer( pTrack ) );
    }
 }
 
-bool TrackFocus::IsFocused( const Track *pTrack )
-{
-   if (mAx)
-      return mAx->IsFocused( pTrack );
-   return false;
-}
-
 void TrackFocus::SetAccessible(
    wxWindow &owner,
    std::unique_ptr< TrackPanelAx > pAx
