@@ -8,10 +8,11 @@ Paul Licameli split from TrackPanel.cpp
 
 **********************************************************************/
 
-#include "Audacity.h"
+
 #include "TrackPanelResizeHandle.h"
 
 #include <wx/cursor.h>
+#include <wx/event.h>
 #include <wx/translation.h>
 
 #include "HitTestResult.h"
@@ -19,7 +20,7 @@ Paul Licameli split from TrackPanel.cpp
 #include "RefreshCode.h"
 #include "Track.h"
 #include "TrackPanelMouseEvent.h"
-#include "tracks/ui/TrackView.h"
+#include "tracks/ui/ChannelView.h"
 
 HitTestPreview TrackPanelResizeHandle::HitPreview(bool bLinked)
 {
@@ -56,20 +57,22 @@ UIHandle::Result TrackPanelResizeHandle::Click(
    const TrackPanelMouseEvent &evt, AudacityProject *pProject )
 {
    using namespace RefreshCode;
-   if ( evt.event.LeftDClick() && mMode == IsResizingBetweenLinkedTracks ) {
-      auto &tracks = TrackList::Get( *pProject );
-      auto pTrack = tracks.Lock(mpTrack);
-      if (pTrack &&
-          !TrackView::Get(*pTrack).GetMinimized()) {
-         auto range = TrackList::Channels( pTrack.get() );
+   if (evt.event.LeftDClick() && mMode == IsResizingBetweenLinkedTracks) {
+      auto &tracks = TrackList::Get(*pProject);
+      auto theChannel = FindChannel();
+      if (!theChannel)
+         return RefreshNone;
+      auto &view = ChannelView::Get(*theChannel);
+      if (!view.GetMinimized()) {
+         auto range = GetTrack(*theChannel).Channels();
          auto size = range.size();
-         auto height = range.sum( [](const Track *pTrack){
-            return TrackView::Get(*pTrack).GetHeight(); } );
+         auto height = range.sum( [](auto pChannel){
+            return ChannelView::Get(*pChannel).GetHeight(); } );
          int ii = 1;
          int coord = 0;
-         for ( const auto channel : range ) {
+         for (const auto pChannel : range) {
             int newCoord = ((double)ii++ /size) * height;
-            TrackView::Get(*channel).SetHeight( newCoord - coord );
+            ChannelView::Get(*pChannel).SetExpandedHeight(newCoord - coord);
             coord = newCoord;
          }
          ProjectHistory::Get( *pProject ).ModifyState(false);
@@ -80,48 +83,67 @@ UIHandle::Result TrackPanelResizeHandle::Click(
    return RefreshNone;
 }
 
-TrackPanelResizeHandle::TrackPanelResizeHandle
-( const std::shared_ptr<Track> &track, int y )
-   : mpTrack{ track }
+TrackPanelResizeHandle::TrackPanelResizeHandle(
+   const std::shared_ptr<Channel> &pChannel, int y
+)  : mwChannel{ pChannel }
    , mMouseClickY( y )
 {
    // TODO: more-than-two-channels
 
    //STM:  Determine whether we should rescale one or two tracks
-   auto channels = TrackList::Channels(track.get());
+   auto channels = GetTrack(*pChannel).Channels();
    auto last = *channels.rbegin();
-   auto &lastView = TrackView::Get( *last );
+   auto &lastView = ChannelView::Get(*last);
    mInitialTrackHeight = lastView.GetHeight();
-   mInitialActualHeight = lastView.GetActualHeight();
+   mInitialExpandedHeight = lastView.GetExpandedHeight();
    mInitialMinimized = lastView.GetMinimized();
 
    if (channels.size() > 1) {
       auto first = *channels.begin();
-      auto &firstView = TrackView::Get( *first );
+      auto &firstView = ChannelView::Get(*first);
 
       mInitialUpperTrackHeight = firstView.GetHeight();
-      mInitialUpperActualHeight = firstView.GetActualHeight();
+      mInitialUpperExpandedHeight = firstView.GetExpandedHeight();
 
-      if (track.get() == *channels.rbegin())
-         // capturedTrack is the lowest track
+      if (pChannel == *channels.rbegin())
+         // pChannel is lowest among two or more,
+         // so there is a previous channel
          mMode = IsResizingBelowLinkedTracks;
       else
-         // capturedTrack is not the lowest track
+         // pChannel is not the lowest among two or more,
+         // so there is a next channel
          mMode = IsResizingBetweenLinkedTracks;
    }
    else
+      // Don't assume there is a next or previous channel
       mMode = IsResizing;
 }
 
+Channel *TrackPanelResizeHandle::PrevChannel(Channel &channel)
+{
+   // Assume channel is last of two
+   // TODO: more-than-two-channels
+   auto channels = GetTrack(channel).Channels();
+   return &**channels.begin();
+}
+
+Channel *TrackPanelResizeHandle::NextChannel(Channel &channel)
+{
+   // Assume channel is first of two
+   // TODO: more-than-two-channels
+   auto channels = GetTrack(channel).Channels();
+   return &**channels.rbegin();
+}
+
 UIHandle::Result TrackPanelResizeHandle::Drag
 (const TrackPanelMouseEvent &evt, AudacityProject *pProject)
 {
    auto &tracks = TrackList::Get( *pProject );
-   auto pTrack = tracks.Lock(mpTrack);
-   if ( !pTrack )
+   auto theChannel = FindChannel();
+   if (!theChannel)
       return RefreshCode::Cancelled;
 
-   auto &view = TrackView::Get( *pTrack );
+   auto &view = ChannelView::Get(*theChannel);
 
    const wxMouseEvent &event = evt.event;
 
@@ -132,29 +154,28 @@ UIHandle::Result TrackPanelResizeHandle::Drag
    //
    // This used to be in HandleResizeClick(), but simply clicking
    // on a resize border would switch the minimized state.
-   auto &data = TrackView::Get( *pTrack );
-   if (data.GetMinimized()) {
-      auto channels = TrackList::Channels( pTrack.get() );
-      for (auto channel : channels) {
-         auto &channelView = TrackView::Get( *channel );
-         channelView.SetHeight(channelView.GetHeight());
-         channelView.SetMinimized( false );
+   if (view.GetMinimized()) {
+      auto channels = GetTrack(*theChannel).Channels();
+      for (auto pChannel : channels) {
+         auto &channelView = ChannelView::Get(*pChannel);
+         channelView.SetExpandedHeight(channelView.GetHeight());
+         channelView.SetMinimized(false);
       }
 
       if (channels.size() > 1) {
          // Initial values must be reset since they weren't based on the
          // minimized heights.
-         auto &channelView = TrackView::Get( **channels.begin() );
+         auto &channelView = ChannelView::Get(**channels.begin());
          mInitialUpperTrackHeight = channelView.GetHeight();
          mInitialTrackHeight = channelView.GetHeight();
       }
    }
 
    // Common pieces of code for MONO_WAVE_PAN and otherwise.
-   auto doResizeBelow = [&] (Track *prev, bool WXUNUSED(vStereo)) {
+   auto doResizeBelow = [&] (Channel *prev) {
       // TODO: more-than-two-channels
       
-      auto &prevView = TrackView::Get( *prev );
+      auto &prevView = ChannelView::Get(*prev);
 
       double proportion = static_cast < double >(mInitialTrackHeight)
       / (mInitialTrackHeight + mInitialUpperTrackHeight);
@@ -171,14 +192,14 @@ UIHandle::Result TrackPanelResizeHandle::Drag
       if (newUpperTrackHeight < prevView.GetMinimizedHeight())
          newUpperTrackHeight = prevView.GetMinimizedHeight();
 
-      view.SetHeight(newTrackHeight);
-      prevView.SetHeight(newUpperTrackHeight);
+      view.SetExpandedHeight(newTrackHeight);
+      prevView.SetExpandedHeight(newUpperTrackHeight);
    };
 
-   auto doResizeBetween = [&] (Track *next, bool WXUNUSED(vStereo)) {
+   auto doResizeBetween = [&] (Channel *next) {
       // TODO: more-than-two-channels
 
-      auto &nextView = TrackView::Get( *next );
+      auto &nextView = ChannelView::Get(*next);
       int newUpperTrackHeight = mInitialUpperTrackHeight + delta;
       int newTrackHeight = mInitialTrackHeight - delta;
 
@@ -194,15 +215,15 @@ UIHandle::Result TrackPanelResizeHandle::Drag
          mInitialUpperTrackHeight + mInitialTrackHeight - view.GetMinimizedHeight();
       }
 
-      view.SetHeight(newUpperTrackHeight);
-      nextView.SetHeight(newTrackHeight);
+      view.SetExpandedHeight(newUpperTrackHeight);
+      nextView.SetExpandedHeight(newTrackHeight);
    };
 
    auto doResize = [&] {
       int newTrackHeight = mInitialTrackHeight + delta;
       if (newTrackHeight < view.GetMinimizedHeight())
          newTrackHeight = view.GetMinimizedHeight();
-      view.SetHeight(newTrackHeight);
+      view.SetExpandedHeight(newTrackHeight);
    };
 
    //STM: We may be dragging one or two (stereo) tracks.
@@ -213,14 +234,14 @@ UIHandle::Result TrackPanelResizeHandle::Drag
    {
       case IsResizingBelowLinkedTracks:
       {
-         auto prev = * -- tracks.Find(pTrack.get());
-         doResizeBelow(prev, false);
+         // Assume previous channel is present, see constructor
+         doResizeBelow(PrevChannel(*theChannel));
          break;
       }
       case IsResizingBetweenLinkedTracks:
       {
-         auto next = * ++ tracks.Find(pTrack.get());
-         doResizeBetween(next, false);
+         // Assume next channel is present, see constructor
+         doResizeBetween(NextChannel(*theChannel));
          break;
       }
       case IsResizing:
@@ -259,38 +280,40 @@ UIHandle::Result TrackPanelResizeHandle::Release
 UIHandle::Result TrackPanelResizeHandle::Cancel(AudacityProject *pProject)
 {
    auto &tracks = TrackList::Get( *pProject );
-   auto pTrack = tracks.Lock(mpTrack);
-   if ( !pTrack )
+   auto theChannel = FindChannel();
+   if (!theChannel)
       return RefreshCode::Cancelled;
 
 
    switch (mMode) {
    case IsResizing:
    {
-      auto &view = TrackView::Get( *pTrack );
-      view.SetHeight(mInitialActualHeight);
+      auto &view = ChannelView::Get(*theChannel);
+      view.SetExpandedHeight(mInitialExpandedHeight);
       view.SetMinimized( mInitialMinimized );
    }
    break;
    case IsResizingBetweenLinkedTracks:
    {
-      Track *const next = * ++ tracks.Find(pTrack.get());
-      auto
-         &view = TrackView::Get( *pTrack ), &nextView = TrackView::Get( *next );
-      view.SetHeight(mInitialUpperActualHeight);
+      // Assume next channel is present, see constructor
+      const auto next = NextChannel(*theChannel);
+      auto &view = ChannelView::Get(*theChannel),
+         &nextView = ChannelView::Get(*next);
+      view.SetExpandedHeight(mInitialUpperExpandedHeight);
       view.SetMinimized( mInitialMinimized );
-      nextView.SetHeight(mInitialActualHeight);
+      nextView.SetExpandedHeight(mInitialExpandedHeight);
       nextView.SetMinimized( mInitialMinimized );
    }
    break;
    case IsResizingBelowLinkedTracks:
    {
-      Track *const prev = * -- tracks.Find(pTrack.get());
-      auto
-         &view = TrackView::Get( *pTrack ), &prevView = TrackView::Get( *prev );
-      view.SetHeight(mInitialActualHeight);
+      // Assume previous channel is present, see constructor
+      const auto prev = PrevChannel(*theChannel);
+      auto &view = ChannelView::Get(*theChannel),
+         &prevView = ChannelView::Get(*prev);
+      view.SetExpandedHeight(mInitialExpandedHeight);
       view.SetMinimized( mInitialMinimized );
-      prevView.SetHeight(mInitialUpperActualHeight);
+      prevView.SetExpandedHeight(mInitialUpperExpandedHeight);
       prevView.SetMinimized(mInitialMinimized);
    }
    break;
@@ -298,3 +321,13 @@ UIHandle::Result TrackPanelResizeHandle::Cancel(AudacityProject *pProject)
 
    return RefreshCode::RefreshAll;
 }
+
+Track &TrackPanelResizeHandle::GetTrack(Channel &channel)
+{
+   // TODO wide wave tracks -- just return channel.GetTrack()
+   // But until then, Track::Channels() will not iterate all channels when
+   // given a right hand track
+   // So be sure to substitute the leader
+   const auto pTrack = static_cast<Track*>(&channel.GetChannelGroup());
+   return **TrackList::Channels(pTrack).begin();
+}
