@@ -13,14 +13,19 @@ Paul Licameli split from TrackPanel.cpp
 
 #include "UIHandle.h"
 
+class Channel;
 class Track;
 
+//! Constructed from one channel, but changes height of all channels in a track
 class TrackPanelResizeHandle final : public UIHandle
 {
    TrackPanelResizeHandle(const TrackPanelResizeHandle&) = delete;
 
 public:
-   explicit TrackPanelResizeHandle( const std::shared_ptr<Track> &pTrack, int y );
+   /*!
+    @pre `pChannel != nullptr`
+    */
+   TrackPanelResizeHandle(const std::shared_ptr<Channel> &pChannel, int y);
 
    TrackPanelResizeHandle &operator=(const TrackPanelResizeHandle&) = default;
 
@@ -28,7 +33,7 @@ public:
 
    virtual ~TrackPanelResizeHandle();
 
-   std::shared_ptr<Track> GetTrack() const { return mpTrack.lock(); }
+   std::shared_ptr<Channel> FindChannel() const { return mwChannel.lock(); }
 
    Result Click
       (const TrackPanelMouseEvent &event, AudacityProject *pProject) override;
@@ -47,6 +52,10 @@ public:
    Result Cancel(AudacityProject *pProject) override;
 
 private:
+   static Track &GetTrack(Channel &channel);
+   Channel *PrevChannel(Channel &channel);
+   Channel *NextChannel(Channel &channel);
+
    enum Mode {
       IsResizing,
       IsResizingBetweenLinkedTracks,
@@ -54,7 +63,7 @@ private:
    };
    Mode mMode{ IsResizing };
 
-   std::weak_ptr<Track> mpTrack;
+   std::weak_ptr<Channel> mwChannel;
 
    bool mInitialMinimized{};
    int mInitialTrackHeight{};
