@@ -87,20 +87,18 @@ CommandManager.  It holds the callback for one command.
 #include <wx/evtloop.h>
 #include <wx/frame.h>
 #include <wx/hash.h>
-#include <wx/intl.h>
 #include <wx/log.h>
 #include <wx/menu.h>
-#include <wx/tokenzr.h>
 
 #include "../ActiveProject.h"
-#include "../Journal.h"
-#include "../JournalOutput.h"
-#include "../JournalRegistry.h"
+#include "Journal.h"
+#include "JournalOutput.h"
+#include "JournalRegistry.h"
 #include "../Menus.h"
 #include "Project.h"
 #include "ProjectWindows.h"
-#include "../widgets/AudacityMessageBox.h"
-#include "../widgets/HelpSystem.h"
+#include "AudacityMessageBox.h"
+#include "HelpSystem.h"
 
 
 // On wxGTK, there may be many many many plugins, but the menus don't automatically
@@ -244,7 +242,7 @@ const std::vector<NormalizedKeyString> &CommandManager::ExcludedList()
       const char *const strings[] = {
          // "Ctrl+I",
          "Ctrl+Alt+I",
-         "Ctrl+J",
+         //"Ctrl+J",
          "Ctrl+Alt+J",
          "Ctrl+Alt+V",
          "Alt+X",
@@ -756,17 +754,17 @@ CommandListEntry *CommandManager::NewIdentifier(const CommandID & nameIn,
       // and the normal reduced list.
       if( std::binary_search( mMaxListOnly.begin(), mMaxListOnly.end(),
                               entry->key ) )
+      {
          entry->key = {};
-
-      // Key from preferences overrides the default key given
-      gPrefs->SetPath(wxT("/NewKeys"));
+      }
+      auto newKeysGroup = gPrefs->BeginGroup("/NewKeys");
       // using GET to interpret CommandID as a config path component
       const auto &path = entry->name.GET();
       if (gPrefs->HasEntry(path)) {
+         // Key from preferences overrides the default key given
          entry->key =
-            NormalizedKeyString{ gPrefs->ReadObject(path, entry->key) };
+            NormalizedKeyString{ gPrefs->Read(path, entry->key) };
       }
-      gPrefs->SetPath(wxT("/"));
 
       mCommandList.push_back(std::move(entry));
       // Don't use the variable entry eny more!
@@ -886,6 +884,14 @@ wxString CommandManager::FormatLabelWithDisabledAccel(const CommandListEntry *en
          if( key.StartsWith("NUMPAD_ENTER" )) break;
          if( key.StartsWith("Backspace" )) break;
          if( key.StartsWith("Delete" )) break;
+
+         // https://github.com/audacity/audacity/issues/4457
+         // This code was proposed by David Bailes to fix
+         // the decimal separator input in wxTextCtrls that
+         // are children of the main window. 
+         if( key.StartsWith(",") ) break;
+         if( key.StartsWith(".") ) break;
+
 #endif
          //wxLogDebug("Added Accel:[%s][%s]", entry->label, entry->key );
          // Normal accelerator.
@@ -1177,6 +1183,10 @@ bool CommandManager::FilterKeyEvent(AudacityProject *project, const wxKeyEvent &
          case '8':
          case '9':
             return false;
+         case ',':
+         case '.':
+            if (!evt.HasAnyModifiers())
+               return false;
          }
       }
    }
@@ -1264,8 +1274,13 @@ bool CommandManager::HandleCommandEntry(AudacityProject &project,
    CommandContext context{ project, evt, entry->index, entry->parameter };
    if (pGivenContext)
       context.temporarySelection = pGivenContext->temporarySelection;
-   auto &handler = entry->finder(project);
-   (handler.*(entry->callback))(context);
+   // Discriminate the union entry->callback by entry->finder
+   if (auto &finder = entry->finder) {
+      auto &handler = finder(project);
+      (handler.*(entry->callback.memberFn))(context);
+   }
+   else
+      (entry->callback.nonMemberFn)(context);
    mLastProcessId = 0;
    return true;
 }
@@ -1300,8 +1315,13 @@ void CommandManager::RegisterLastTool(const CommandContext& context) {
 void CommandManager::DoRepeatProcess(const CommandContext& context, int id) {
    mLastProcessId = 0;  //Don't Process this as repeat
    CommandListEntry* entry = mCommandNumericIDHash[id];
-   auto& handler = entry->finder(context.project);
-   (handler.*(entry->callback))(context);
+   // Discriminate the union entry->callback by entry->finder
+   if (auto &finder = entry->finder) {
+      auto &handler = finder(context.project);
+      (handler.*(entry->callback.memberFn))(context);
+   }
+   else
+      (entry->callback.nonMemberFn)(context);
 }
 
 
