@@ -14,6 +14,7 @@
 
 #include "Identifier.h"
 
+#include "Callable.h"
 #include "ClientData.h"
 #include "CommandFunctors.h"
 #include "CommandFlag.h"
@@ -22,6 +23,7 @@
 #include "Keyboard.h"
 
 #include "Prefs.h"
+#include "Project.h"
 #include "Registry.h"
 
 #include <vector>
@@ -76,8 +78,8 @@ class AUDACITY_DLL_API CommandManager final
    CommandManager();
    virtual ~CommandManager();
 
-   CommandManager(const CommandManager&) PROHIBITED;
-   CommandManager &operator= (const CommandManager&) PROHIBITED;
+   CommandManager(const CommandManager&) = delete;
+   CommandManager &operator= (const CommandManager&) = delete;
 
    void SetMaxList();
    void PurgeData();
@@ -275,8 +277,7 @@ class AUDACITY_DLL_API CommandManager final
    ///
    /// Formatting summaries that include shortcut keys
    ///
-   TranslatableString DescribeCommandsAndShortcuts
-   (
+   TranslatableString DescribeCommandsAndShortcuts(
        // If a shortcut key is defined for the command, then it is appended,
        // parenthesized, after the translated name.
        const ComponentInterfaceSymbol commands[], size_t nCommands) const;
@@ -389,14 +390,18 @@ private:
 
 struct AUDACITY_DLL_API MenuVisitor : Registry::Visitor
 {
+   ~MenuVisitor() override;
+
    // final overrides
-   void BeginGroup( Registry::GroupItem &item, const Path &path ) final;
-   void EndGroup( Registry::GroupItem &item, const Path& ) final;
+   void BeginGroup( Registry::GroupItemBase &item, const Path &path ) final;
+   void EndGroup( Registry::GroupItemBase &item, const Path& ) final;
    void Visit( Registry::SingleItem &item, const Path &path ) final;
 
    // added virtuals
-   virtual void DoBeginGroup( Registry::GroupItem &item, const Path &path );
-   virtual void DoEndGroup( Registry::GroupItem &item, const Path &path );
+   //! Groups of type MenuItems are excluded from this callback
+   virtual void DoBeginGroup( Registry::GroupItemBase &item, const Path &path );
+   //! Groups of type MenuItems are excluded from this callback
+   virtual void DoEndGroup( Registry::GroupItemBase &item, const Path &path );
    virtual void DoVisit( Registry::SingleItem &item, const Path &path );
    virtual void DoSeparator();
 
@@ -406,17 +411,22 @@ private:
    std::vector<bool> needSeparator;
 };
 
-struct ToolbarMenuVisitor : MenuVisitor
+struct ProjectMenuVisitor : MenuVisitor
 {
-   explicit ToolbarMenuVisitor( AudacityProject &p ) : project{ p } {}
-   operator AudacityProject & () const { return project; }
-   AudacityProject &project;
+   explicit ProjectMenuVisitor(AudacityProject &p) : mProject{ p } {}
+   ~ProjectMenuVisitor() override;
+   virtual void *GetComputedItemContext() override;
+   AudacityProject &mProject;
 };
 
 // Define items that populate tables that specifically describe menu trees
 namespace MenuTable {
    using namespace Registry;
 
+   struct Traits : Registry::DefaultTraits {
+      using ComputedItemContextType = AudacityProject;
+   };
+
    // These are found by dynamic_cast
    struct AUDACITY_DLL_API MenuSection {
       virtual ~MenuSection();
@@ -427,48 +437,35 @@ namespace MenuTable {
       bool extension;
    };
 
+   struct MenuItemData {
+      MenuItemData(TranslatableString title) : mTitle{ std::move(title) } {}
+      const TranslatableString mTitle;
+   };
+
    // Describes a main menu in the toolbar, or a sub-menu
    struct AUDACITY_DLL_API MenuItem final
-      : ConcreteGroupItem< false, ToolbarMenuVisitor >
-      , WholeMenu {
-      // Construction from an internal name and a previously built-up
-      // vector of pointers
-      MenuItem( const Identifier &internalName,
-         const TranslatableString &title_, BaseItemPtrs &&items_ );
-      // In-line, variadic constructor that doesn't require building a vector
-      template< typename... Args >
-         MenuItem( const Identifier &internalName,
-            const TranslatableString &title_, Args&&... args )
-            : ConcreteGroupItem< false, ToolbarMenuVisitor >{
-               internalName, std::forward<Args>(args)... }
-            , title{ title_ }
-         {}
+      : Composite::Extension<
+         GroupItem<Traits>, MenuItemData, const Identifier&
+      >
+      , WholeMenu
+   {
+      using Extension::Extension;
       ~MenuItem() override;
-
-      TranslatableString title;
+      const auto &GetTitle() const { return mTitle; }
    };
 
+   using Condition = std::function<bool()>;
+
    // Collects other items that are conditionally shown or hidden, but are
    // always available to macro programming
    struct ConditionalGroupItem final
-      : ConcreteGroupItem< false, ToolbarMenuVisitor > {
-      using Condition = std::function< bool() >;
-
-      // Construction from an internal name and a previously built-up
-      // vector of pointers
-      ConditionalGroupItem( const Identifier &internalName,
-         Condition condition_, BaseItemPtrs &&items_ );
-      // In-line, variadic constructor that doesn't require building a vector
-      template< typename... Args >
-         ConditionalGroupItem( const Identifier &internalName,
-            Condition condition_, Args&&... args )
-            : ConcreteGroupItem< false, ToolbarMenuVisitor >{
-               internalName, std::forward<Args>(args)... }
-            , condition{ condition_ }
-         {}
+      : Composite::Extension<
+         GroupItem<Traits>, Condition, const Identifier &
+      >
+   {
+      using Extension::Extension;
       ~ConditionalGroupItem() override;
-
-      Condition condition;
+      using Condition::operator();
    };
 
    // usage:
@@ -487,12 +484,14 @@ namespace MenuTable {
       static CommandHandlerFinder sFinder;
 
    public:
+      //! @post result: `result != nullptr`
       static CommandHandlerFinder DefaultFinder() { return sFinder; }
 
+      //! @pre `finder != nullptr`
       explicit
       FinderScope( CommandHandlerFinder finder )
          : ValueRestorer( sFinder, finder )
-      {}
+      { assert(finder); }
    };
 
    // Describes one command in a menu
@@ -506,18 +505,34 @@ namespace MenuTable {
 
       // Takes a pointer to member function directly, and delegates to the
       // previous constructor; useful within the lifetime of a FinderScope
+      /*!
+       @pre `finder != nullptr`
+       */
       template< typename Handler >
-      CommandItem(const CommandID &name_,
-               const TranslatableString &label_in_,
+      CommandItem(const CommandID &name,
+               const TranslatableString &label_in,
                void (Handler::*pmf)(const CommandContext&),
-               CommandFlag flags_,
-               const CommandManager::Options &options_,
+               CommandFlag flags,
+               const CommandManager::Options &options = {},
                CommandHandlerFinder finder = FinderScope::DefaultFinder())
-         : CommandItem(name_, label_in_,
-            static_cast<CommandFunctorPointer>(pmf),
-            flags_, options_, finder)
-      {}
+         : CommandItem(name, label_in,
+            CommandFunctorPointer{
+               static_cast<CommandFunctorPointer::MemberFn>(pmf) },
+            flags, options, finder)
+      { assert(finder); }
 
+      // Takes a pointer to nonmember function and delegates to the first
+      // constructor
+      CommandItem(const CommandID &name,
+               const TranslatableString &label_in,
+               CommandFunctorPointer::NonMemberFn callback,
+               CommandFlag flags,
+               const CommandManager::Options &options = {})
+         : CommandItem(name, label_in,
+            CommandFunctorPointer{ callback },
+            flags, options, nullptr)
+      {}
+   
       ~CommandItem() override;
 
       const TranslatableString label_in;
@@ -532,7 +547,7 @@ namespace MenuTable {
    // in the CommandContext identifying the command
    struct AUDACITY_DLL_API CommandGroupItem final : SingleItem {
       CommandGroupItem(const Identifier &name_,
-               std::vector< ComponentInterfaceSymbol > items_,
+               std::vector<ComponentInterfaceSymbol> items_,
                CommandFunctorPointer callback_,
                CommandFlag flags_,
                bool isEffect_,
@@ -540,16 +555,32 @@ namespace MenuTable {
 
       // Takes a pointer to member function directly, and delegates to the
       // previous constructor; useful within the lifetime of a FinderScope
+      /*!
+       @pre `finder != nullptr`
+       */
       template< typename Handler >
       CommandGroupItem(const Identifier &name_,
-               std::vector< ComponentInterfaceSymbol > items_,
+               std::vector<ComponentInterfaceSymbol> items_,
                void (Handler::*pmf)(const CommandContext&),
                CommandFlag flags_,
                bool isEffect_,
                CommandHandlerFinder finder = FinderScope::DefaultFinder())
-         : CommandGroupItem(name_, std::move(items_),
-            static_cast<CommandFunctorPointer>(pmf),
+         : CommandGroupItem(name_, move(items_),
+            CommandFunctorPointer{
+               static_cast<CommandFunctorPointer::MemberFn>(pmf) },
             flags_, isEffect_, finder)
+      { assert(finder); }
+
+      // Takes a pointer to nonmember function and delegates to the first
+      // constructor
+      CommandGroupItem(const CommandID &name,
+               std::vector< ComponentInterfaceSymbol > items,
+               CommandFunctorPointer::NonMemberFn fn,
+               CommandFlag flags,
+               bool isEffect = false)
+         : CommandGroupItem(name, move(items),
+            CommandFunctorPointer{ fn },
+            flags, isEffect, nullptr)
       {}
 
       ~CommandGroupItem() override;
@@ -576,136 +607,92 @@ namespace MenuTable {
       Appender fn;
    };
 
-   struct MenuPart : ConcreteGroupItem< false, ToolbarMenuVisitor >, MenuSection
+   //! Groups of this type are inlined in the menu tree organization.  They
+   //! (but not their contained items) are excluded from visitations using
+   //! MenuVisitor
+   struct MenuItems
+      : Composite::Extension<
+         GroupItem<Traits>, void, const Identifier &
+      >
    {
-      template< typename... Args >
-      explicit
-      MenuPart( const Identifier &internalName, Args&&... args )
-         : ConcreteGroupItem< false, ToolbarMenuVisitor >{
-            internalName, std::forward< Args >( args )... }
-      {}
+      using Extension::Extension;
+      ~MenuItems() override;
+      //! Anonymous if its name is empty, else weakly ordered
+      Ordering GetOrdering() const override;
    };
-   using MenuItems = ConcreteGroupItem< true, ToolbarMenuVisitor >;
 
-   // The following, and Shared(), are the functions to use directly
-   // in writing table definitions.
+   struct MenuPart
+      : Composite::Extension<
+         GroupItem<Traits>, void, const Identifier &
+      >
+      , MenuSection
+   {
+      using Extension::Extension;
+      ~MenuPart() override;
+   };
 
-   // Group items can be constructed two ways.
-   // Pointers to subordinate items are moved into the result.
-   // Null pointers are permitted, and ignored when building the menu.
-   // Items are spliced into the enclosing menu.
-   // The name is untranslated and may be empty, to make the group transparent
-   // in identification of items by path.  Otherwise try to keep the name
-   // stable across Audacity versions.
-   template< typename... Args >
-   inline std::unique_ptr< MenuItems > Items(
-      const Identifier &internalName, Args&&... args )
-         { return std::make_unique< MenuItems >(
-            internalName, std::forward<Args>(args)... ); }
+   /*! @name Factories
+      The following, and Registry::Indirect(), are the functions to use directly
+      to specify elements of menu groupings.
+    */
+   //! @{
 
-   // Like Items, but insert a menu separator between the menu section and
-   // any other items or sections before or after it in the same (innermost,
-   // enclosing) menu.
-   // It's not necessary that the sisters of sections be other sections, but it
-   // might clarify the logical groupings.
-   template< typename... Args >
-   inline std::unique_ptr< MenuPart > Section(
-      const Identifier &internalName, Args&&... args )
-         { return std::make_unique< MenuPart >(
-            internalName, std::forward<Args>(args)... ); }
+   //! Variadic constructor from pointers to subordinate items, which are moved
+   //! into the result.
+   /*!
+    Null pointers are permitted, and ignored when building the menu.
+    Items are spliced into the enclosing menu.
+    The name is untranslated and may be empty, to make the group transparent
+    in identification of items by path.  Otherwise try to keep the name
+    stable across Audacity versions.
+    */
+   constexpr auto Items = Callable::UniqueMaker<MenuItems>();
+
+   //! Like Items, but insert a menu separator between the menu section and
+   //! any other items or sections before or after it in the same (innermost,
+   //! enclosing) menu.
+   /*!
+    It's not necessary that the sisters of sections be other sections, but it
+    might clarify the logical groupings.
+    */
+   constexpr auto Section = Callable::UniqueMaker<MenuPart>();
    
-   // Menu items can be constructed two ways, as for group items
-   // Items will appear in a main toolbar menu or in a sub-menu.
-   // The name is untranslated.  Try to keep the name stable across Audacity
-   // versions.
-   // If the name of a menu is empty, then subordinate items cannot be located
-   // by path.
-   template< typename... Args >
-   inline std::unique_ptr<MenuItem> Menu(
-      const Identifier &internalName, const TranslatableString &title, Args&&... args )
-         { return std::make_unique<MenuItem>(
-            internalName, title, std::forward<Args>(args)... ); }
-   inline std::unique_ptr<MenuItem> Menu(
-      const Identifier &internalName, const TranslatableString &title, BaseItemPtrs &&items )
-         { return std::make_unique<MenuItem>(
-            internalName, title, std::move( items ) ); }
+   //! Items will appear in a main toolbar menu or in a sub-menu.
+   /*!
+    The name is untranslated.  Try to keep the name stable across Audacity
+    versions.
+    If the name of a menu is empty, then subordinate items cannot be located
+    by path.
+    */
+   constexpr auto Menu = Callable::UniqueMaker<MenuItem>();
 
-   // Conditional group items can be constructed two ways, as for group items
-   // These items register in the CommandManager but are not shown in menus
-   // if the condition evaluates false.
-   // The name is untranslated.  Try to keep the name stable across Audacity
-   // versions.
-   // Name for conditional group must be non-empty.
-   template< typename... Args >
-   inline std::unique_ptr<ConditionalGroupItem> ConditionalItems(
-      const Identifier &internalName,
-      ConditionalGroupItem::Condition condition, Args&&... args )
-         { return std::make_unique<ConditionalGroupItem>(
-            internalName, condition, std::forward<Args>(args)... ); }
-   inline std::unique_ptr<ConditionalGroupItem> ConditionalItems(
-      const Identifier &internalName, ConditionalGroupItem::Condition condition,
-      BaseItemPtrs &&items )
-         { return std::make_unique<ConditionalGroupItem>(
-            internalName, condition, std::move( items ) ); }
+   //! These items register in the CommandManager but are not shown in menus
+   //! if the condition evaluates false.
+   /*!
+    The name is untranslated.  Try to keep the name stable across Audacity
+    versions.
+    Name for conditional group must be non-empty.
+    */
+   constexpr auto ConditionalItems = Callable::UniqueMaker<ConditionalGroupItem>();
 
-   // Make either a menu item or just a group, depending on the nonemptiness
-   // of the title.
-   // The name is untranslated and may be empty, to make the group transparent
-   // in identification of items by path.  Otherwise try to keep the name
-   // stable across Audacity versions.
-   // If the name of a menu is empty, then subordinate items cannot be located
-   // by path.
-   template< typename... Args >
-   inline BaseItemPtr MenuOrItems(
-      const Identifier &internalName, const TranslatableString &title, Args&&... args )
-         {  if ( title.empty() )
-               return Items( internalName, std::forward<Args>(args)... );
-            else
-               return std::make_unique<MenuItem>(
-                  internalName, title, std::forward<Args>(args)... ); }
-   inline BaseItemPtr MenuOrItems(
-      const Identifier &internalName,
-      const TranslatableString &title, BaseItemPtrs &&items )
-         {  if ( title.empty() )
-               return Items( internalName, std::move( items ) );
-            else
-               return std::make_unique<MenuItem>(
-                  internalName, title, std::move( items ) ); }
+   constexpr auto Command = Callable::UniqueMaker<CommandItem>();
 
-   template< typename Handler >
-   inline std::unique_ptr<CommandItem> Command(
-      const CommandID &name,
-      const TranslatableString &label_in,
-      void (Handler::*pmf)(const CommandContext&),
-      CommandFlag flags, const CommandManager::Options &options = {},
-      CommandHandlerFinder finder = FinderScope::DefaultFinder())
-   {
-      return std::make_unique<CommandItem>(
-         name, label_in, pmf, flags, options, finder
-      );
-   }
+   constexpr auto CommandGroup = Callable::UniqueMaker<CommandGroupItem,
+      const Identifier &, std::vector<ComponentInterfaceSymbol>>();
 
-   template< typename Handler >
-   inline std::unique_ptr<CommandGroupItem> CommandGroup(
-      const Identifier &name,
-      std::vector< ComponentInterfaceSymbol > items,
-      void (Handler::*pmf)(const CommandContext&),
-      CommandFlag flags, bool isEffect = false,
-      CommandHandlerFinder finder = FinderScope::DefaultFinder())
-   {
-      return std::make_unique<CommandGroupItem>(
-         name, std::move(items), pmf, flags, isEffect, finder
-      );
-   }
+   constexpr auto Special = Callable::UniqueMaker<SpecialItem>();
 
-   inline std::unique_ptr<SpecialItem> Special(
-      const Identifier &name, const SpecialItem::Appender &fn )
-         { return std::make_unique<SpecialItem>( name, fn ); }
+   //! @}
+
+   struct ItemRegistry {
+      static GroupItemBase &Registry();
+   };
 
    // Typically you make a static object of this type in the .cpp file that
    // also defines the added menu actions.
    // pItem can be specified by an expression using the inline functions above.
    struct AUDACITY_DLL_API AttachedItem final
+      : public RegisteredItem<BaseItem, ItemRegistry>
    {
       AttachedItem( const Placement &placement, BaseItemPtr pItem );
 
@@ -714,9 +701,6 @@ namespace MenuTable {
          : AttachedItem( Placement{ path }, std::move( pItem ) )
       {}
    };
-
-   void DestroyRegistry();
-
 }
 
 #endif
