@@ -23,6 +23,9 @@ This class now lists
 
 #include "GetInfoCommand.h"
 
+#include "CommandDispatch.h"
+#include "CommandManager.h"
+#include "../CommonCommandFlags.h"
 #include "LoadCommands.h"
 #include "Project.h"
 #include "../ProjectWindows.h"
@@ -32,23 +35,24 @@ This class now lists
 #include "../widgets/Overlay.h"
 #include "../TrackPanelAx.h"
 #include "../TrackPanel.h"
-#include "../WaveClip.h"
+#include "WaveClip.h"
 #include "ViewInfo.h"
-#include "../WaveTrack.h"
+#include "WaveTrack.h"
+#include "prefs/WaveformSettings.h"
 #include "../LabelTrack.h"
 #include "../NoteTrack.h"
-#include "../TimeTrack.h"
+#include "TimeTrack.h"
 #include "Envelope.h"
 
 #include "SelectCommand.h"
-#include "../ShuttleGui.h"
+#include "ShuttleGui.h"
 #include "CommandContext.h"
 
 #include "../prefs/PrefsDialog.h"
-#include "../Shuttle.h"
+#include "SettingsVisitor.h"
 #include "PluginManager.h"
-#include "../tracks/ui/TrackView.h"
-#include "../ShuttleGui.h"
+#include "../tracks/ui/ChannelView.h"
+#include "ShuttleGui.h"
 
 #include <wx/frame.h>
 #include <wx/log.h>
@@ -239,7 +243,7 @@ public:
    wxTextCtrl * TieNumericTextBox(
       const TranslatableString & Prompt,
       const DoubleSetting &Setting,
-      const int nChars) override;
+      const int nChars, bool acceptEnter) override;
    wxSlider * TieSlider(
       const TranslatableString & Prompt,
       const IntSetting &Setting,
@@ -357,7 +361,7 @@ wxTextCtrl * ShuttleGuiGetDefinition::TieIntegerTextBox(
 wxTextCtrl * ShuttleGuiGetDefinition::TieNumericTextBox(
    const TranslatableString & Prompt,
    const DoubleSetting &Setting,
-   const int nChars)
+   const int nChars, bool acceptEnter)
 {
    StartStruct();
    AddItem( Setting.GetPath(), "id" );
@@ -365,7 +369,7 @@ wxTextCtrl * ShuttleGuiGetDefinition::TieNumericTextBox(
    AddItem( "number", "type" );
    AddItem( Setting.GetDefault(), "default"  );
    EndStruct();
-   return ShuttleGui::TieNumericTextBox( Prompt, Setting, nChars );
+   return ShuttleGui::TieNumericTextBox( Prompt, Setting, nChars, acceptEnter );
 }
 
 wxSlider * ShuttleGuiGetDefinition::TieSlider(
@@ -468,7 +472,7 @@ bool GetInfoCommand::SendTracks(const CommandContext & context)
 {
    auto &tracks = TrackList::Get( context.project );
    context.StartArray();
-   for (auto trk : tracks.Leaders())
+   for (auto trk : tracks)
    {
       auto &trackFocus = TrackFocus::Get( context.project );
       Track * fTrack = trackFocus.Get();
@@ -478,30 +482,30 @@ bool GetInfoCommand::SendTracks(const CommandContext & context)
       context.AddBool( (trk == fTrack), "focused");
       context.AddBool( trk->GetSelected(), "selected" );
       //JKC: Possibly add later...
-      //context.AddItem( TrackView::Get( *trk ).GetHeight(), "height" );
-      trk->TypeSwitch( [&] (const WaveTrack* t ) {
+      //context.AddItem(ChannelView::GetChannelGroupHeight(*trk), "height");
+      trk->TypeSwitch( [&] (const WaveTrack &t) {
          float vzmin, vzmax;
-         t->GetDisplayBounds(&vzmin, &vzmax);
+         WaveformScale::Get(t).GetDisplayBounds(vzmin, vzmax);
          context.AddItem( "wave", "kind" );
-         context.AddItem( t->GetStartTime(), "start" );
-         context.AddItem( t->GetEndTime(), "end" );
-         context.AddItem( t->GetPan() , "pan");
-         context.AddItem( t->GetGain() , "gain");
-         context.AddItem( TrackList::Channels(t).size(), "channels");
-         context.AddBool( t->GetSolo(), "solo" );
-         context.AddBool( t->GetMute(), "mute");
+         context.AddItem( t.GetStartTime(), "start" );
+         context.AddItem( t.GetEndTime(), "end" );
+         context.AddItem( t.GetPan() , "pan");
+         context.AddItem( t.GetGain() , "gain");
+         context.AddItem( TrackList::NChannels(t), "channels");
+         context.AddBool( t.GetSolo(), "solo" );
+         context.AddBool( t.GetMute(), "mute");
          context.AddItem( vzmin, "VZoomMin");
          context.AddItem( vzmax, "VZoomMax");
       },
 #if defined(USE_MIDI)
-      [&](const NoteTrack *) {
+      [&](const NoteTrack &) {
          context.AddItem( "note", "kind" );
       },
 #endif
-      [&](const LabelTrack *) {
+      [&](const LabelTrack &) {
          context.AddItem( "label", "kind" );
       },
-      [&](const TimeTrack *) {
+      [&](const TimeTrack &) {
          context.AddItem( "time", "kind" );
       }
       );
@@ -516,9 +520,9 @@ bool GetInfoCommand::SendClips(const CommandContext &context)
    auto &tracks = TrackList::Get( context.project );
    int i=0;
    context.StartArray();
-   for (auto t : tracks.Leaders()) {
-      t->TypeSwitch([&](WaveTrack *waveTrack) {
-         WaveClipPointers ptrs(waveTrack->SortedClipArray());
+   for (auto t : tracks) {
+      t->TypeSwitch([&](WaveTrack &waveTrack) {
+         WaveClipPointers ptrs(waveTrack.SortedClipArray());
          for (WaveClip * pClip : ptrs) {
             context.StartStruct();
             context.AddItem((double)i, "track");
@@ -542,9 +546,9 @@ bool GetInfoCommand::SendEnvelopes(const CommandContext &context)
    int i=0;
    int j=0;
    context.StartArray();
-   for (auto t : tracks.Leaders()) {
-      t->TypeSwitch([&](WaveTrack *waveTrack) {
-         WaveClipPointers ptrs(waveTrack->SortedClipArray());
+   for (auto t : tracks) {
+      t->TypeSwitch([&](WaveTrack &waveTrack) {
+         WaveClipPointers ptrs(waveTrack.SortedClipArray());
          j = 0;
          for (WaveClip * pClip : ptrs) {
             context.StartStruct();
@@ -583,8 +587,8 @@ bool GetInfoCommand::SendLabels(const CommandContext &context)
    auto &tracks = TrackList::Get( context.project );
    int i=0;
    context.StartArray();
-   for (auto t : tracks.Leaders()) {
-      t->TypeSwitch( [&](LabelTrack *labelTrack) {
+   for (auto t : tracks) {
+      t->TypeSwitch( [&](LabelTrack &labelTrack) {
 #ifdef VERBOSE_LABELS_FORMATTING
          for (int nn = 0; nn< (int)labelTrack->mLabels.size(); nn++) {
             const auto &label = labelTrack->mLabels[nn];
@@ -599,7 +603,7 @@ bool GetInfoCommand::SendLabels(const CommandContext &context)
          context.StartArray();
          context.AddItem( (double)i ); // Track number.
          context.StartArray();
-         for ( const auto &label : labelTrack->GetLabels() ) {
+         for ( const auto &label : labelTrack.GetLabels() ) {
             context.StartArray();
             context.AddItem( label.getT0() ); // start
             context.AddItem( label.getT1() ); // end
@@ -705,24 +709,26 @@ void GetInfoCommand::ExploreTrackPanel( const CommandContext &context,
    AudacityProject * pProj = &context.project;
    auto &tp = TrackPanel::Get( *pProj );
    wxRect panelRect{ {}, tp.GetSize() };
-   for ( auto t : TrackList::Get( *pProj ).Any() ) {
-      auto rulers = tp.FindRulerRects(t);
-      for (auto &R : rulers) {
-         if (!R.Intersects(panelRect))
-            continue;
-         R.SetPosition( R.GetPosition() + P );
-         context.StartStruct();
-         context.AddItem( depth, "depth" );
-         context.AddItem( "VRuler", "label" );
-         context.StartField("box");
-         context.StartArray();
-         context.AddItem( R.GetLeft() );
-         context.AddItem( R.GetTop() );
-         context.AddItem( R.GetRight() );
-         context.AddItem( R.GetBottom() );
-         context.EndArray();
-         context.EndField();
-         context.EndStruct();
+   for (auto leader : TrackList::Get(*pProj)) {
+      for (auto t : leader->Channels()) {
+         auto rulers = tp.FindRulerRects(*t);
+         for (auto &R : rulers) {
+            if (!R.Intersects(panelRect))
+               continue;
+            R.SetPosition( R.GetPosition() + P );
+            context.StartStruct();
+            context.AddItem( depth, "depth" );
+            context.AddItem( "VRuler", "label" );
+            context.StartField("box");
+            context.StartArray();
+            context.AddItem( R.GetLeft() );
+            context.AddItem( R.GetTop() );
+            context.AddItem( R.GetRight() );
+            context.AddItem( R.GetBottom() );
+            context.EndArray();
+            context.EndField();
+            context.EndStruct();
+         }
       }
    }
 }
@@ -776,3 +782,19 @@ void GetInfoCommand::ExploreWindows( const CommandContext &context,
    }
 }
 
+namespace {
+using namespace MenuTable;
+
+// Register menu items
+
+AttachedItem sAttachment{
+   wxT("Optional/Extra/Part2/Scriptables2"),
+   // Note that the PLUGIN_SYMBOL must have a space between words,
+   // whereas the short-form used here must not.
+   // (So if you did write "Compare Audio" for the PLUGIN_SYMBOL name, then
+   // you would have to use "CompareAudio" here.)
+   Command( wxT("GetInfo"), XXO("Get Info..."),
+      CommandDispatch::OnAudacityCommand, AudioIONotBusyFlag() )
+};
+
+}
