@@ -17,15 +17,18 @@
 
 #include "OpenSaveCommands.h"
 
+#include "CommandDispatch.h"
+#include "CommandManager.h"
+#include "../CommonCommandFlags.h"
 #include "LoadCommands.h"
 #include "AudacityLogger.h"
 #include "Project.h"
-#include "../ProjectFileIO.h"
+#include "ProjectFileIO.h"
 #include "../ProjectFileManager.h"
 #include "../ProjectManager.h"
-#include "../export/Export.h"
-#include "../Shuttle.h"
-#include "../ShuttleGui.h"
+#include "Export.h"
+#include "SettingsVisitor.h"
+#include "ShuttleGui.h"
 #include "CommandContext.h"
 
 
@@ -220,3 +223,23 @@ bool ClearLogCommand::Apply(const CommandContext &context)
    auto logger = AudacityLogger::Get();
    return logger->ClearLog();
 }
+
+namespace {
+using namespace MenuTable;
+
+// Register menu items
+
+AttachedItem sAttachment{
+   wxT("Optional/Extra/Part2/Scriptables2"),
+   Items( wxT(""),
+      // Note that the PLUGIN_SYMBOL must have a space between words,
+      // whereas the short-form used here must not.
+      // (So if you did write "Compare Audio" for the PLUGIN_SYMBOL name, then
+      // you would have to use "CompareAudio" here.)
+      Command( wxT("OpenProject2"), XXO("Open Project..."),
+         CommandDispatch::OnAudacityCommand, AudioIONotBusyFlag() ),
+      Command( wxT("SaveProject2"), XXO("Save Project..."),
+         CommandDispatch::OnAudacityCommand, AudioIONotBusyFlag() )
+   )
+};
+}
