@@ -33,18 +33,20 @@ small calculations of rectangles.
 #include <wx/bitmap.h>
 #include <wx/valgen.h>
 
+#include "ActiveProject.h"
 #include "../AdornedRulerPanel.h"
 #include "../TrackPanel.h"
 #include "../toolbars/ToolManager.h"
 #include "Prefs.h"
 #include "../ProjectWindow.h"
-#include "../Shuttle.h"
-#include "../ShuttleGui.h"
+#include "SettingsVisitor.h"
+#include "ShuttleGui.h"
 #include "Track.h"
 #include "../widgets/VetoDialogHook.h"
 #include "CommandContext.h"
 #include "CommandManager.h"
 #include "CommandDispatch.h"
+#include "../CommonCommandFlags.h"
 
 const ComponentInterfaceSymbol ScreenshotCommand::Symbol
 { XO("Screenshot") };
@@ -52,41 +54,56 @@ const ComponentInterfaceSymbol ScreenshotCommand::Symbol
 namespace{ BuiltinCommandsModule::Registration< ScreenshotCommand > reg; }
 
 
-static const EnumValueSymbol
-kCaptureWhatStrings[ ScreenshotCommand::nCaptureWhats ] =
+EnumValueSymbols ScreenshotCommand::kCaptureWhatStrings()
 {
-   { XO("Window") },
-   { wxT("FullWindow"), XO("Full Window") },
-   { wxT("WindowPlus"), XO("Window Plus") },
-   { XO("Fullscreen") },
-   { XO("Toolbars") },
-   { XO("Effects") },
-   { XO("Scriptables") },
-   { XO("Preferences") },
-   { XO("Selectionbar") },
-   { wxT("SpectralSelection"), XO("Spectral Selection") },
-   { XO("Timer") },
-   { XO("Tools") },
-   { XO("Transport") },
-   { XO("Meter") },
-   { wxT("PlayMeter"), XO("Play Meter") },
-   { wxT("RecordMeter"), XO("Record Meter") },
-   { XO("Edit") },
-   { XO("Device") },
-   { XO("Scrub") },
-   { XO("Play-at-Speed") },
-   { XO("Trackpanel") },
-   { XO("Ruler") },
-   { XO("Tracks") },
-   { wxT("FirstTrack"),       XO("First Track") },
-   { wxT("FirstTwoTracks"),   XO("First Two Tracks") },
-   { wxT("FirstThreeTracks"), XO("First Three Tracks") },
-   { wxT("FirstFourTracks"),  XO("First Four Tracks") },
-   { wxT("SecondTrack"),      XO("Second Track") },
-   { wxT("TracksPlus"),       XO("Tracks Plus") },
-   { wxT("FirstTrackPlus"),   XO("First Track Plus") },
-   { wxT("AllTracks"),        XO("All Tracks") },
-   { wxT("AllTracksPlus"),    XO("All Tracks Plus") },
+   static EnumValueSymbol symbols[]{
+      { XO("Window") },
+      { wxT("FullWindow"), XO("Full Window") },
+      { wxT("WindowPlus"), XO("Window Plus") },
+      { XO("Fullscreen") },
+      { XO("Toolbars") },
+      { XO("Effects") },
+      { XO("Scriptables") },
+      { XO("Preferences") },
+      { XO("Trackpanel") },
+      { XO("Ruler") },
+      { XO("Tracks") },
+      { wxT("FirstTrack"),       XO("First Track") },
+      { wxT("FirstTwoTracks"),   XO("First Two Tracks") },
+      { wxT("FirstThreeTracks"), XO("First Three Tracks") },
+      { wxT("FirstFourTracks"),  XO("First Four Tracks") },
+      { wxT("SecondTrack"),      XO("Second Track") },
+      { wxT("TracksPlus"),       XO("Tracks Plus") },
+      { wxT("FirstTrackPlus"),   XO("First Track Plus") },
+      { wxT("AllTracks"),        XO("All Tracks") },
+      { wxT("AllTracksPlus"),    XO("All Tracks Plus") },
+   };
+
+   if (mSymbols.empty()) {
+      // Compute the table of strings once
+
+      // Some fixed choices
+      copy(std::begin(symbols), std::end(symbols), back_inserter(mSymbols));
+      auto nFixed = mSymbols.size();
+
+      // Discover the set of toolbars -- don't hard-code it here
+      /*
+       There is no context passed in for a project.
+       So this need to use the active project is unfortunate, but the set of
+       toolbars and their identifying strings should not vary among projects
+       */
+      auto pProject = ::GetActiveProject().lock();
+      if ( pProject ) {
+         // Toolbars will be sorted by textual ID.
+         // Macro programmers do use these English ids no matter what their
+         // preferred language is.
+         ToolManager::Get(*pProject).ForEach([&](ToolBar *pBar){
+            mSymbols.emplace_back( pBar->GetSection(), pBar->GetLabel() );
+         });
+      }
+   }
+
+   return mSymbols;
 };
 
 
@@ -111,8 +128,9 @@ ScreenshotCommand::ScreenshotCommand()
 
 template<bool Const>
 bool ScreenshotCommand::VisitSettings( SettingsVisitorBase<Const> & S ){
+   auto strings = kCaptureWhatStrings();
    S.Define(                               mPath,        wxT("Path"),         wxString{});
-   S.DefineEnum(                           mWhat,        wxT("CaptureWhat"),  kwindow,kCaptureWhatStrings, nCaptureWhats );
+   S.DefineEnum(                           mWhat,        wxT("CaptureWhat"),  kwindow, strings.data(), strings.size() );
    S.DefineEnum(                           mBack,        wxT("Background"),   kNone, kBackgroundStrings, nBackgrounds );
    S.Define(                               mbBringToTop, wxT("ToTop"), true );
    return true;
@@ -126,13 +144,14 @@ bool ScreenshotCommand::VisitSettings( ConstSettingsVisitor & S )
 
 void ScreenshotCommand::PopulateOrExchange(ShuttleGui & S)
 {
+   auto strings = kCaptureWhatStrings();
    S.AddSpace(0, 5);
 
    S.StartMultiColumn(2, wxALIGN_CENTER);
    {
       S.TieTextBox(  XXO("Path:"), mPath);
       S.TieChoice(   XXO("Capture What:"),
-         mWhat, Msgids(kCaptureWhatStrings, nCaptureWhats));
+         mWhat, Msgids(strings.data(), strings.size()));
       S.TieChoice(   XXO("Background:"),
          mBack, Msgids(kBackgroundStrings, nBackgrounds));
       S.TieCheckBox( XXO("Bring To Top"), mbBringToTop);
@@ -303,7 +322,7 @@ bool ScreenshotCommand::Capture(
 
 bool ScreenshotCommand::CaptureToolbar(
    const CommandContext & context,
-   ToolManager *man, int type, const wxString &name)
+   ToolManager *man, Identifier type, const wxString &name)
 {
    bool visible = man->IsVisible(type);
    if (!visible) {
@@ -312,6 +331,9 @@ bool ScreenshotCommand::CaptureToolbar(
    }
 
    wxWindow *w = man->GetToolBar(type);
+   if (!w)
+      return false;
+
    int x = 0, y = 0;
    int width, height;
 
@@ -414,7 +436,7 @@ void ScreenshotCommand::CapturePreferences(
       gPrefs->Flush();
       CommandID Command{ wxT("Preferences") };
       const CommandContext projectContext( *pProject );
-      if( !::HandleTextualCommand( commandManager,
+      if( !CommandDispatch::HandleTextualCommand( commandManager,
          Command, projectContext, AlwaysEnabledFlag, true ) )
       {
          // using GET in a log message for devs' eyes only
@@ -619,7 +641,7 @@ void ScreenshotCommand::GetDerivedParams()
 
    // Build a suitable filename
    mFileName = MakeFileName(mFilePath,
-      kCaptureWhatStrings[ mCaptureMode ].Translation() );
+      kCaptureWhatStrings()[ mCaptureMode ].Translation() );
 
    if (mBack == kBlue)
    {
@@ -729,7 +751,7 @@ wxRect ScreenshotCommand::GetTrackRect( AudacityProject * pProj, TrackPanel * pa
    };
 
    int count = 0;
-   for (auto t : TrackList::Get( *pProj ).Leaders()) {
+   for (auto t : TrackList::Get( *pProj )) {
       count +=  1;
       if( count > n )
       {
@@ -743,7 +765,7 @@ wxRect ScreenshotCommand::GetTrackRect( AudacityProject * pProj, TrackPanel * pa
 wxString ScreenshotCommand::WindowFileName(AudacityProject * proj, wxTopLevelWindow *w){
    if (w != ProjectWindow::Find( proj ) && !w->GetTitle().empty()) {
       mFileName = MakeFileName(mFilePath,
-         kCaptureWhatStrings[ mCaptureMode ].Translation() +
+         kCaptureWhatStrings()[ mCaptureMode ].Translation() +
             (wxT("-") + w->GetTitle() + wxT("-")));
    }
    return mFileName;
@@ -763,7 +785,7 @@ bool ScreenshotCommand::Apply(const CommandContext & context)
    TrackPanel *panel = &TrackPanel::Get( context.project );
    AdornedRulerPanel *ruler = panel->GetRuler();
 
-   int nTracks = TrackList::Get( context.project ).size();
+   int nTracks = TrackList::Get(context.project).Size();
 
    int x1,y1,x2,y2;
    w->ClientToScreen(&x1, &y1);
@@ -773,101 +795,100 @@ bool ScreenshotCommand::Apply(const CommandContext & context)
 
    auto &toolManager = ToolManager::Get( context.project );
 
-   switch (mCaptureMode) {
-   case kwindow:
-      return Capture(context,  WindowFileName( &context.project, w ) , w, GetWindowRect(w));
-   case kfullwindow:
-   case kwindowplus:
-      return Capture(context,  WindowFileName( &context.project, w ) , w, GetFullWindowRect(w));
-   case kfullscreen:
-      return Capture(context, mFileName, w,GetScreenRect());
-   case ktoolbars:
-      return CaptureDock(context, toolManager.GetTopDock(), mFileName);
-   case kscriptables:
-      CaptureScriptables(context, &context.project, mFileName);
-      break;
-   case keffects:
-      CaptureEffects(context, &context.project, mFileName);
-      break;
-   case kpreferences:
-      CapturePreferences(context, &context.project, mFileName);
-      break;
-   case kselectionbar:
-      return CaptureToolbar(context, &toolManager, SelectionBarID, mFileName);
-   case kspectralselection:
-      return CaptureToolbar(context, &toolManager, SpectralSelectionBarID, mFileName);
-   case ktimer:
-      return CaptureToolbar(context, &toolManager, TimeBarID, mFileName);
-   case ktools:
-      return CaptureToolbar(context, &toolManager, ToolsBarID, mFileName);
-   case ktransport:
-      return CaptureToolbar(context, &toolManager, TransportBarID, mFileName);
-   case kmeter:
-      return CaptureToolbar(context, &toolManager, MeterBarID, mFileName);
-   case krecordmeter:
-      return CaptureToolbar(context, &toolManager, RecordMeterBarID, mFileName);
-   case kplaymeter:
-      return CaptureToolbar(context, &toolManager, PlayMeterBarID, mFileName);
-   case kedit:
-      return CaptureToolbar(context, &toolManager, EditBarID, mFileName);
-   case kdevice:
-      return CaptureToolbar(context, &toolManager, DeviceBarID, mFileName);
-   case ktranscription:
-      return CaptureToolbar(context, &toolManager, TranscriptionBarID, mFileName);
-   case kscrub:
-      return CaptureToolbar(context, &toolManager, ScrubbingBarID, mFileName);
-   case ktrackpanel:
-      return Capture(context, mFileName, panel, GetPanelRect(panel));
-   case kruler:
-      return Capture(context, mFileName, ruler, GetRulerRect(ruler) );
-   case ktracks:
-      return Capture(context, mFileName, panel, GetTracksRect(panel));
-   case kfirsttrack:
-      return Capture(context, mFileName, panel, GetTrackRect( &context.project, panel, 0 ) );
-   case ksecondtrack:
-      return Capture(context, mFileName, panel, GetTrackRect( &context.project, panel, 1 ) );
-   case ktracksplus:
-   {  wxRect r = GetTracksRect(panel);
-      r.SetTop( r.GetTop() - ruler->GetRulerHeight() );
-      r.SetHeight( r.GetHeight() + ruler->GetRulerHeight() );
-      return Capture(context, mFileName, panel, r);
+   if (mCaptureMode < nCaptureWhats) {
+      switch (mCaptureMode) {
+      case kwindow:
+         return Capture(context,  WindowFileName( &context.project, w ) , w, GetWindowRect(w));
+      case kfullwindow:
+      case kwindowplus:
+         return Capture(context,  WindowFileName( &context.project, w ) , w, GetFullWindowRect(w));
+      case kfullscreen:
+         return Capture(context, mFileName, w,GetScreenRect());
+      case ktoolbars:
+         return CaptureDock(context, toolManager.GetTopDock(), mFileName);
+      case kscriptables:
+         CaptureScriptables(context, &context.project, mFileName);
+         return true;
+      case keffects:
+         CaptureEffects(context, &context.project, mFileName);
+         return true;
+      case kpreferences:
+         CapturePreferences(context, &context.project, mFileName);
+         return true;
+      case ktrackpanel:
+         return Capture(context, mFileName, panel, GetPanelRect(panel));
+      case kruler:
+         return Capture(context, mFileName, ruler, GetRulerRect(ruler) );
+      case ktracks:
+         return Capture(context, mFileName, panel, GetTracksRect(panel));
+      case kfirsttrack:
+         return Capture(context, mFileName, panel, GetTrackRect( &context.project, panel, 0 ) );
+      case ksecondtrack:
+         return Capture(context, mFileName, panel, GetTrackRect( &context.project, panel, 1 ) );
+      case ktracksplus:
+      {  wxRect r = GetTracksRect(panel);
+         r.SetTop( r.GetTop() - ruler->GetRulerHeight() );
+         r.SetHeight( r.GetHeight() + ruler->GetRulerHeight() );
+         return Capture(context, mFileName, panel, r);
+      }
+      case kfirsttrackplus:
+      {  wxRect r = GetTrackRect(&context.project, panel, 0 );
+         r.SetTop( r.GetTop() - ruler->GetRulerHeight() );
+         r.SetHeight( r.GetHeight() + ruler->GetRulerHeight() );
+         return Capture(context, mFileName, panel, r );
+      }
+      case kfirsttwotracks:
+      {  wxRect r = GetTrackRect( &context.project, panel, 0 );
+         r = r.Union( GetTrackRect( &context.project, panel, 1 ));
+         return Capture(context, mFileName, panel, r );
+      }
+      case kfirstthreetracks:
+      {  wxRect r = GetTrackRect( &context.project, panel, 0 );
+         r = r.Union( GetTrackRect( &context.project, panel, 2 ));
+         return Capture(context, mFileName, panel, r );
+      }
+      case kfirstfourtracks:
+      {  wxRect r = GetTrackRect( &context.project, panel, 0 );
+         r = r.Union( GetTrackRect( &context.project, panel, 3 ));
+         return Capture(context, mFileName, panel, r );
+      }
+      case kalltracks:
+      {  wxRect r = GetTrackRect( &context.project, panel, 0 );
+         r = r.Union( GetTrackRect( &context.project, panel, nTracks-1 ));
+         return Capture(context, mFileName, panel, r );
+      }
+      case kalltracksplus:
+      {  wxRect r = GetTrackRect( &context.project, panel, 0 );
+         r.SetTop( r.GetTop() - ruler->GetRulerHeight() );
+         r.SetHeight( r.GetHeight() + ruler->GetRulerHeight() );
+         r = r.Union( GetTrackRect( &context.project, panel, nTracks-1 ));
+         return Capture(context, mFileName, panel, r );
+      }
+      default:
+         return false;
+      }
    }
-   case kfirsttrackplus:
-   {  wxRect r = GetTrackRect(&context.project, panel, 0 );
-      r.SetTop( r.GetTop() - ruler->GetRulerHeight() );
-      r.SetHeight( r.GetHeight() + ruler->GetRulerHeight() );
-      return Capture(context, mFileName, panel, r );
+   else if (mCaptureMode < nCaptureWhats + toolManager.CountBars()) {
+      auto id = kCaptureWhatStrings()[mCaptureMode].Internal();
+      return CaptureToolbar(context, &toolManager, id, mFileName);
    }
-   case kfirsttwotracks:
-   {  wxRect r = GetTrackRect( &context.project, panel, 0 );
-      r = r.Union( GetTrackRect( &context.project, panel, 1 ));
-      return Capture(context, mFileName, panel, r );
-   }
-   case kfirstthreetracks:
-   {  wxRect r = GetTrackRect( &context.project, panel, 0 );
-      r = r.Union( GetTrackRect( &context.project, panel, 2 ));
-      return Capture(context, mFileName, panel, r );
-   }
-   case kfirstfourtracks:
-   {  wxRect r = GetTrackRect( &context.project, panel, 0 );
-      r = r.Union( GetTrackRect( &context.project, panel, 3 ));
-      return Capture(context, mFileName, panel, r );
-   }
-   case kalltracks:
-   {  wxRect r = GetTrackRect( &context.project, panel, 0 );
-      r = r.Union( GetTrackRect( &context.project, panel, nTracks-1 ));
-      return Capture(context, mFileName, panel, r );
-   }
-   case kalltracksplus:
-   {  wxRect r = GetTrackRect( &context.project, panel, 0 );
-      r.SetTop( r.GetTop() - ruler->GetRulerHeight() );
-      r.SetHeight( r.GetHeight() + ruler->GetRulerHeight() );
-      r = r.Union( GetTrackRect( &context.project, panel, nTracks-1 ));
-      return Capture(context, mFileName, panel, r );
-   }
-   default:
+   else
       return false;
-   }
+}
 
-   return true;
+namespace {
+using namespace MenuTable;
+
+// Register menu items
+
+AttachedItem sAttachment{
+   wxT("Optional/Extra/Part2/Scriptables2"),
+   // Note that the PLUGIN_SYMBOL must have a space between words,
+   // whereas the short-form used here must not.
+   // (So if you did write "Compare Audio" for the PLUGIN_SYMBOL name, then
+   // you would have to use "CompareAudio" here.)
+   // i18n-hint: Screenshot in the help menu has a much bigger dialog.
+   Command( wxT("Screenshot"), XXO("Screenshot (short format)..."),
+      CommandDispatch::OnAudacityCommand, AudioIONotBusyFlag() )
+};
 }
