@@ -20,6 +20,7 @@
 
 
 #include "Amplify.h"
+#include "EffectEditor.h"
 #include "LoadEffects.h"
 
 #include <math.h>
@@ -27,7 +28,6 @@
 
 #include <wx/button.h>
 #include <wx/checkbox.h>
-#include <wx/intl.h>
 #include <wx/sizer.h>
 #include <wx/slider.h>
 #include <wx/stattext.h>
@@ -35,8 +35,10 @@
 #include <wx/valtext.h>
 #include <wx/log.h>
 
-#include "../ShuttleGui.h"
-#include "../WaveTrack.h"
+#include "EffectOutputTracks.h"
+#include "ShuttleGui.h"
+#include "WaveTrack.h"
+#include "WaveTrackUtilities.h"
 #include "../widgets/valnum.h"
 
 
@@ -89,6 +91,12 @@ BEGIN_EVENT_TABLE(EffectAmplify, wxEvtHandler)
    EVT_CHECKBOX(ID_Clip, EffectAmplify::OnClipCheckBox)
 END_EVENT_TABLE()
 
+EffectAmplify::Instance::~Instance()
+{
+   // In case the dialog is cancelled before effect processing
+   static_cast<EffectAmplify&>(GetEffect()).DestroyOutputTracks();
+}
+
 EffectAmplify::EffectAmplify()
 {
    mAmp = Amp.def;
@@ -181,35 +189,37 @@ OptionalMessage EffectAmplify::DoLoadFactoryDefaults(EffectSettings &settings)
 
 bool EffectAmplify::Init()
 {
+   auto range = inputTracks()->Selected<const WaveTrack>();
+   bool hasStretch = any_of(begin(range), end(range),
+      [this](auto *pTrack){
+         return WaveTrackUtilities::HasStretch(*pTrack, mT0, mT1); });
+   if (hasStretch)
+      range = MakeOutputTracks()->Get().Selected<const WaveTrack>();
    mPeak = 0.0;
-
-   for (auto t : inputTracks()->Selected< const WaveTrack >())
-   {
-      auto pair = t->GetMinMax(mT0, mT1); // may throw
-      const float min = pair.first, max = pair.second;
-      float newpeak = (fabs(min) > fabs(max) ? fabs(min) : fabs(max));
-
-      if (newpeak > mPeak)
-      {
-         mPeak = newpeak;
+   for (auto t : range) {
+      for (const auto pChannel : t->Channels()) {
+         auto pair = pChannel->GetMinMax(mT0, mT1); // may throw
+         const float min = pair.first, max = pair.second;
+         const float newpeak = std::max(fabs(min), fabs(max));
+         mPeak = std::max<double>(mPeak, newpeak);
       }
    }
-
    return true;
 }
 
-void EffectAmplify::Preview(EffectSettingsAccess &access, bool dryOnly)
+std::any EffectAmplify::BeginPreview(const EffectSettings &settings)
 {
-   auto cleanup1 = valueRestorer( mRatio );
-   auto cleanup2 = valueRestorer( mPeak );
-
-   Effect::Preview(access, dryOnly);
+   return { std::pair{
+      CopyableValueRestorer(mRatio), CopyableValueRestorer(mPeak)
+   } };
 }
 
-std::unique_ptr<EffectUIValidator> EffectAmplify::PopulateOrExchange(
+std::unique_ptr<EffectEditor> EffectAmplify::PopulateOrExchange(
    ShuttleGui & S, EffectInstance &, EffectSettingsAccess &,
    const EffectOutputs *)
 {
+   mUIParent = S.GetParent();
+
    enum{ precision = 3 }; // allow (a generous) 3 decimal  places for Amplification (dB)
 
    bool batch = IsBatchProcessing();
@@ -336,18 +346,26 @@ bool EffectAmplify::TransferDataFromWindow(EffectSettings &)
    return true;
 }
 
+std::shared_ptr<EffectInstance> EffectAmplify::MakeInstance() const
+{
+   // Cheat with const_cast to return an object that calls through to
+   // non-const methods of a stateful effect.
+   return std::make_shared<Instance>(const_cast<EffectAmplify&>(*this));
+}
+
 // EffectAmplify implementation
 
 void EffectAmplify::CheckClip()
 {
-   EnableApply(mClip->GetValue() || (mPeak > 0.0 && mRatio <= mRatioClip));
+   EffectEditor::EnableApply(mUIParent,
+      mClip->GetValue() || (mPeak > 0.0 && mRatio <= mRatioClip));
 }
 
 void EffectAmplify::OnAmpText(wxCommandEvent & WXUNUSED(evt))
 {
    if (!mAmpT->GetValidator()->TransferFromWindow())
    {
-      EnableApply(false);
+      EffectEditor::EnableApply(mUIParent, false);
       return;
    }
 
@@ -365,7 +383,7 @@ void EffectAmplify::OnPeakText(wxCommandEvent & WXUNUSED(evt))
 {
    if (!mNewPeakT->GetValidator()->TransferFromWindow())
    {
-      EnableApply(false);
+      EffectEditor::EnableApply(mUIParent, false);
       return;
    }
 
