@@ -13,14 +13,17 @@
 #ifndef __AUDACITY_EFFECT_CHANGESPEED__
 #define __AUDACITY_EFFECT_CHANGESPEED__
 
-#include "Effect.h"
-#include "../ShuttleAutomation.h"
+#include "StatefulEffect.h"
+#include "ShuttleAutomation.h"
+#include <wx/weakref.h>
 
 class wxSlider;
 class wxChoice;
 class wxTextCtrl;
+class LabelTrack;
 class NumericTextCtrl;
 class ShuttleGui;
+class WaveChannel;
 
 class EffectChangeSpeed final : public StatefulEffect
 {
@@ -50,7 +53,7 @@ public:
       const EffectSettings &settings, double previewLength) const override;
    bool Init() override;
    bool Process(EffectInstance &instance, EffectSettings &settings) override;
-   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+   std::unique_ptr<EffectEditor> PopulateOrExchange(
       ShuttleGui & S, EffectInstance &instance,
       EffectSettingsAccess &access, const EffectOutputs *pOutputs) override;
    bool TransferDataToWindow(const EffectSettings &settings) override;
@@ -59,7 +62,13 @@ public:
 private:
    // EffectChangeSpeed implementation
 
-   bool ProcessOne(WaveTrack *t, sampleCount start, sampleCount end);
+   using Gap = std::pair<double, double>;
+   using Gaps = std::vector<Gap>;
+   Gaps FindGaps(
+      const WaveTrack &track, const double curT0, const double curT1);
+
+   bool ProcessOne(const WaveChannel &track, WaveChannel &outputTrack,
+      sampleCount start, sampleCount end);
    bool ProcessLabelTrack(LabelTrack *t);
 
    // handlers
@@ -79,9 +88,10 @@ private:
    void UpdateUI();                    // Enable / disable OK / preview.
 
 private:
+   wxWeakRef<wxWindow> mUIParent{};
+
    // track related
    int    mCurTrackNum;
-   double mMaxNewLength;
    double mCurT0;
    double mCurT1;
 
