@@ -12,9 +12,9 @@
 #define __AUDACITY_EFFECT_COMPRESSOR__
 
 #include "TwoPassSimpleMono.h"
-#include "../ShuttleAutomation.h"
+#include "ShuttleAutomation.h"
 #include "MemoryX.h"
-#include "../widgets/wxPanelWrapper.h"
+#include "wxPanelWrapper.h"
 
 class wxCheckBox;
 class wxSlider;
@@ -48,7 +48,7 @@ public:
 
    // Effect implementation
 
-   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+   std::unique_ptr<EffectEditor> PopulateOrExchange(
       ShuttleGui & S, EffectInstance &instance,
       EffectSettingsAccess &access, const EffectOutputs *pOutputs) override;
    bool DoTransferDataFromWindow();
@@ -77,6 +77,8 @@ private:
    void UpdateUI();
 
 private:
+   wxWeakRef<wxWindow> mUIParent{};
+
    double    mRMSSum;
    size_t    mCircleSize;
    size_t    mCirclePos;
