@@ -15,9 +15,11 @@
 class wxString;
 
 class LabelTrack;
+class WaveChannel;
 
-#include "Effect.h"
-#include "../ShuttleAutomation.h"
+#include "StatefulEffect.h"
+#include "ShuttleAutomation.h"
+#include <wx/weakref.h>
 
 class EffectFindClipping final : public StatefulEffect
 {
@@ -42,7 +44,7 @@ public:
    // Effect implementation
 
    bool Process(EffectInstance &instance, EffectSettings &settings) override;
-   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+   std::unique_ptr<EffectEditor> PopulateOrExchange(
       ShuttleGui & S, EffectInstance &instance,
       EffectSettingsAccess &access, const EffectOutputs *pOutputs) override;
    void DoPopulateOrExchange(
@@ -53,8 +55,10 @@ public:
 private:
    // EffectFindCliping implementation
 
-   bool ProcessOne(LabelTrack *lt, int count, const WaveTrack * wt,
-                   sampleCount start, sampleCount len);
+   bool ProcessOne(LabelTrack &lt, int count, const WaveChannel &wt,
+      sampleCount start, sampleCount len);
+
+   wxWeakRef<wxWindow> mUIParent{};
 
    int mStart;   ///< Using int rather than sampleCount because values are only ever small numbers
    int mStop;    ///< Using int rather than sampleCount because values are only ever small numbers
