@@ -13,20 +13,21 @@
 
 #include <wx/checkbox.h>
 #include <wx/choice.h>
-#include <wx/event.h>
 #include <wx/stattext.h>
-#include <wx/string.h>
 #include <wx/textctrl.h>
+#include <wx/weakref.h>
 
-#include "Effect.h"
+#include "StatefulEffect.h"
 #include "Biquad.h"
-#include "EBUR128.h"
-#include "../ShuttleAutomation.h"
+#include "ShuttleAutomation.h"
 #include "Track.h"
 
 class wxChoice;
 class wxSimplebook;
+class EBUR128;
 class ShuttleGui;
+class WaveChannel;
+using Floats = ArrayOf<float>;
 
 class EffectLoudness final : public StatefulEffect
 {
@@ -58,7 +59,7 @@ public:
    // Effect implementation
 
    bool Process(EffectInstance &instance, EffectSettings &settings) override;
-   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+   std::unique_ptr<EffectEditor> PopulateOrExchange(
       ShuttleGui & S, EffectInstance &instance,
       EffectSettingsAccess &access, const EffectOutputs *pOutputs) override;
    bool TransferDataToWindow(const EffectSettings &settings) override;
@@ -67,16 +68,18 @@ public:
 private:
    // EffectLoudness implementation
 
-   void AllocBuffers();
+   void AllocBuffers(TrackList &outputs);
    void FreeBuffers();
-   bool GetTrackRMS(WaveTrack* track, float& rms);
-   bool ProcessOne(TrackIterRange<WaveTrack> range, bool analyse);
-   void LoadBufferBlock(TrackIterRange<WaveTrack> range,
-                        sampleCount pos, size_t len);
-   bool AnalyseBufferBlock();
-   bool ProcessBufferBlock();
-   void StoreBufferBlock(TrackIterRange<WaveTrack> range,
-                         sampleCount pos, size_t len);
+   static bool GetTrackRMS(WaveChannel &track,
+      double curT0, double curT1, float &rms);
+   [[nodiscard]] bool ProcessOne(WaveChannel &track, size_t nChannels,
+      double curT0, double curT1, float mult, EBUR128 *pLoudnessProcessor);
+   void LoadBufferBlock(WaveChannel &track, size_t nChannels,
+      sampleCount pos, size_t len);
+   bool AnalyseBufferBlock(EBUR128 &loudnessProcessor);
+   bool ProcessBufferBlock(float mult);
+   [[nodiscard]] bool StoreBufferBlock(WaveChannel &track, size_t nChannels,
+      sampleCount pos, size_t len);
 
    bool UpdateProgress();
    void OnChoice(wxCommandEvent & evt);
@@ -84,25 +87,20 @@ private:
    void UpdateUI();
 
 private:
+   wxWeakRef<wxWindow> mUIParent{};
+
    bool   mStereoInd;
    double mLUFSLevel;
    double mRMSLevel;
    bool   mDualMono;
    int    mNormalizeTo;
 
-   double mCurT0;
-   double mCurT1;
    double mProgressVal;
    int    mSteps;
    TranslatableString mProgressMsg;
    double mTrackLen;
    double mCurRate;
 
-   float  mMult;
-   float  mRatio;
-   float  mRMS[2];
-   std::unique_ptr<EBUR128> mLoudnessProcessor;
-
    wxSimplebook *mBook;
    wxChoice *mChoice;
    wxStaticText *mWarning;
