@@ -13,7 +13,7 @@
 #ifndef __AUDACITY_EFFECT_NOISE_REDUCTION__
 #define __AUDACITY_EFFECT_NOISE_REDUCTION__
 
-#include "Effect.h"
+#include "StatefulEffect.h"
 
 class EffectNoiseReduction final : public StatefulEffect {
 public:
@@ -37,7 +37,7 @@ public:
 
 //   using Effect::TrackProgress;
 
-   int ShowHostInterface( wxWindow &parent,
+   int ShowHostInterface(EffectBase &plugin, wxWindow &parent,
       const EffectDialogFactory &factory,
       std::shared_ptr<EffectInstance> &pInstance, EffectSettingsAccess &access,
       bool forceModal = false) override;
@@ -47,9 +47,9 @@ public:
    class Settings;
    class Statistics;
    class Dialog;
+   class Worker;
 
 private:
-   class Worker;
    friend class Dialog;
 
    std::unique_ptr<Settings> mSettings;
