@@ -12,14 +12,17 @@
 #ifndef __AUDACITY_EFFECT_NORMALIZE__
 #define __AUDACITY_EFFECT_NORMALIZE__
 
-#include "Effect.h"
+#include "StatefulEffect.h"
 #include "Biquad.h"
-#include "../ShuttleAutomation.h"
+#include "ShuttleAutomation.h"
+#include <wx/weakref.h>
+#include <functional>
 
 class wxCheckBox;
 class wxStaticText;
 class wxTextCtrl;
 class ShuttleGui;
+class WaveChannel;
 
 class EffectNormalize final : public StatefulEffect
 {
@@ -45,7 +48,7 @@ public:
 
    bool CheckWhetherSkipEffect(const EffectSettings &settings) const override;
    bool Process(EffectInstance &instance, EffectSettings &settings) override;
-   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+   std::unique_ptr<EffectEditor> PopulateOrExchange(
       ShuttleGui & S, EffectInstance &instance,
       EffectSettingsAccess &access, const EffectOutputs *pOutputs) override;
    bool TransferDataToWindow(const EffectSettings &settings) override;
@@ -54,19 +57,25 @@ public:
 private:
    // EffectNormalize implementation
 
-   bool ProcessOne(
-      WaveTrack * t, const TranslatableString &msg, double& progress, float offset);
-   bool AnalyseTrack(const WaveTrack * track, const TranslatableString &msg,
-                     double &progress, float &offset, float &extent);
-   bool AnalyseTrackData(const WaveTrack * track, const TranslatableString &msg, double &progress,
-                     float &offset);
-   void AnalyseDataDC(float *buffer, size_t len);
+   bool ProcessOne(WaveChannel &track,
+      const TranslatableString &msg, double& progress, float offset);
+   using ProgressReport = std::function<bool(double fraction)>;
+   static bool AnalyseTrack(const WaveChannel &track,
+      const ProgressReport &report,
+      bool gain, bool dc, double curT0, double curT1,
+      float &offset, float &extent);
+   static bool AnalyseTrackData(const WaveChannel &track,
+      const ProgressReport &report, double curT0, double curT1,
+      float &offset);
+   static double AnalyseDataDC(float *buffer, size_t len, double sum);
    void ProcessData(float *buffer, size_t len, float offset);
 
    void OnUpdateUI(wxCommandEvent & evt);
    void UpdateUI();
 
 private:
+   wxWeakRef<wxWindow> mUIParent{};
+
    double mPeakLevel;
    bool   mGain;
    bool   mDC;
@@ -75,7 +84,6 @@ private:
    double mCurT0;
    double mCurT1;
    float  mMult;
-   double mSum;
 
    wxCheckBox *mGainCheckBox;
    wxCheckBox *mDCCheckBox;
