@@ -19,20 +19,16 @@ renamed and focused on the smaller subproblem of repairing
 the audio, rather than actually finding the clicks.
 
 *//*******************************************************************/
-
-
-
 #include "Repair.h"
 
 #include <math.h>
 
-#include <wx/intl.h>
-
+#include "AudacityMessageBox.h"
+#include "EffectOutputTracks.h"
 #include "InterpolateAudio.h"
-#include "../WaveTrack.h"
-#include "../widgets/AudacityMessageBox.h"
-
 #include "LoadEffects.h"
+#include "WaveTrack.h"
+#include "WaveTrackUtilities.h"
 
 const ComponentInterfaceSymbol EffectRepair::Symbol
 { XO("Repair") };
@@ -75,27 +71,34 @@ bool EffectRepair::IsInteractive() const
 
 bool EffectRepair::Process(EffectInstance &, EffectSettings &)
 {
-   //v This may be too much copying for EffectRepair. To support Cancel, may be able to copy much less.
-   //  But for now, Cancel isn't supported without this.
-   this->CopyInputTracks(); // Set up mOutputTracks. //v This may be too much copying for EffectRepair.
+   // This may be too much copying for EffectRepair. To support Cancel, may be
+   // able to copy much less.
+   // But for now, Cancel isn't supported without this.
+   // Repair doesn't make sense for stretched clips, so don't pass a stretch
+   // interval.
+   EffectOutputTracks outputs { *mTracks, GetType(), {} };
    bool bGoodResult = true;
 
    int count = 0;
-   for( auto track : mOutputTracks->Selected< WaveTrack >() ) {
-      const
-      double trackStart = track->GetStartTime();
+   for (auto track : outputs.Get().Selected<WaveTrack>()) {
+      const double trackStart = track->GetStartTime();
       const double repair_t0 = std::max(mT0, trackStart);
-      const
-      double trackEnd = track->GetEndTime();
+      const double trackEnd = track->GetEndTime();
       const double repair_t1 = std::min(mT1, trackEnd);
-      const
-      double repair_deltat = repair_t1 - repair_t0;
+      const double repair_deltat = repair_t1 - repair_t0;
       if (repair_deltat > 0) {  // selection is within track audio
          const auto repair0 = track->TimeToLongSamples(repair_t0);
          const auto repair1 = track->TimeToLongSamples(repair_t1);
          const auto repairLen = repair1 - repair0;
+         if (WaveTrackUtilities::HasStretch(*track, repair_t0, repair_t1)) {
+            EffectUIServices::DoMessageBox(*this,
+               XO(
+"The Repair effect cannot be applied within stretched or shrunk clips") );
+            bGoodResult = false;
+            break;
+         }
          if (repairLen > 128) {
-            ::Effect::MessageBox(
+            EffectUIServices::DoMessageBox(*this,
                XO(
 "The Repair effect is intended to be used on very short sections of damaged audio (up to 128 samples).\n\nZoom in and select a tiny fraction of a second to repair.") );
             bGoodResult = false;
@@ -114,7 +117,7 @@ bool EffectRepair::Process(EffectInstance &, EffectSettings &)
          const auto len = s1 - s0;
 
          if (s0 == repair0 && s1 == repair1) {
-            ::Effect::MessageBox(
+            EffectUIServices::DoMessageBox(*this,
                XO(
 "Repair works by using audio data outside the selection region.\n\nPlease select a region that has audio touching at least one side of it.\n\nThe more surrounding audio, the better it performs.") );
    ///            The Repair effect needs some data to go on.\n\nPlease select an area to repair with some audio on at least one side (the more the better).") );
@@ -122,33 +125,43 @@ bool EffectRepair::Process(EffectInstance &, EffectSettings &)
             break;
          }
 
-         if (!ProcessOne(count, track, s0,
-                         // len is at most 5 * 128.
-                         len.as_size_t(),
-                         repairStart,
-                         // repairLen is at most 128.
-                         repairLen.as_size_t() )) {
-            bGoodResult = false;
-            break;
-         }
+         for (const auto pChannel : track->Channels())
+            if (!ProcessOne(count++, *pChannel, s0,
+               // len is at most 5 * 128.
+               len.as_size_t(),
+               repairStart,
+               // repairLen is at most 128.
+               repairLen.as_size_t())
+            ) {
+               bGoodResult = false;
+               goto done;
+            }
       }
-
-      count++;
    }
+done:
+
+   if (bGoodResult)
+      outputs.Commit();
 
-   this->ReplaceProcessedTracks(bGoodResult);
    return bGoodResult;
 }
 
-bool EffectRepair::ProcessOne(int count, WaveTrack * track,
-                              sampleCount start,
-                              size_t len,
-                              size_t repairStart, size_t repairLen)
+bool EffectRepair::ProcessOne(int count, WaveChannel &track,
+   sampleCount start, size_t len, size_t repairStart, size_t repairLen)
 {
    Floats buffer{ len };
-   track->GetFloats(buffer.get(), start, len);
+   track.GetFloats(buffer.get(), start, len);
    InterpolateAudio(buffer.get(), len, repairStart, repairLen);
-   track->Set((samplePtr)&buffer[repairStart], floatSample,
-              start + repairStart, repairLen);
+   if (!track.Set((samplePtr)&buffer[repairStart], floatSample,
+      start + repairStart, repairLen,
+      // little repairs shouldn't force dither on rendering:
+      narrowestSampleFormat
+   ))
+      return false;
    return !TrackProgress(count, 1.0); // TrackProgress returns true on Cancel.
 }
+
+bool EffectRepair::NeedsDither() const
+{
+   return false;
+}
