@@ -18,21 +18,18 @@
 \brief Dialog used with EffectRepeat
 
 *//*******************************************************************/
-
-
-
 #include "Repeat.h"
-
+#include "EffectEditor.h"
+#include "EffectOutputTracks.h"
 
 #include <math.h>
 
-#include <wx/intl.h>
 #include <wx/stattext.h>
 
 #include "../LabelTrack.h"
-#include "../ShuttleGui.h"
-#include "../SyncLock.h"
-#include "../WaveTrack.h"
+#include "ShuttleGui.h"
+#include "SyncLock.h"
+#include "WaveTrack.h"
 #include "../widgets/NumericTextCtrl.h"
 #include "../widgets/valnum.h"
 
@@ -95,53 +92,90 @@ bool EffectRepeat::Process(EffectInstance &, EffectSettings &)
 {
    // Set up mOutputTracks.
    // This effect needs all for sync-lock grouping.
-   CopyInputTracks(true);
+   EffectOutputTracks outputs { *mTracks, GetType(), { { mT0, mT1 } }, true };
 
    int nTrack = 0;
    bool bGoodResult = true;
    double maxDestLen = 0.0; // used to change selection to generated bit
 
-   mOutputTracks->Any().VisitWhile( bGoodResult,
-      [&](LabelTrack *track)
-      {
-         if (SyncLock::IsSelectedOrSyncLockSelected(track))
+   outputs.Get().Any().VisitWhile(bGoodResult,
+      [&](LabelTrack &track) {
+         if (SyncLock::IsSelectedOrSyncLockSelected(&track))
          {
-            if (!track->Repeat(mT0, mT1, repeatCount))
+            if (!track.Repeat(mT0, mT1, repeatCount))
                bGoodResult = false;
          }
       },
-      [&](WaveTrack *track, const Track::Fallthrough &fallthrough)
-      {
-         if (!track->GetSelected())
+      [&](auto &&fallthrough){ return [&](WaveTrack &track) {
+         if (!track.GetSelected())
             return fallthrough(); // Fall through to next lambda
-         auto start = track->TimeToLongSamples(mT0);
-         auto end = track->TimeToLongSamples(mT1);
+         auto start = track.TimeToLongSamples(mT0);
+         auto end = track.TimeToLongSamples(mT1);
          auto len = end - start;
-         double tLen = track->LongSamplesToTime(len);
-         double tc = mT0 + tLen;
+         const double tLen = track.LongSamplesToTime(len);
+         const double tc = mT0 + tLen;
 
          if (len <= 0)
             return;
 
-         auto dest = track->Copy(mT0, mT1);
-         for(int j=0; j<repeatCount; j++)
-         {
-            if (TrackProgress(nTrack, j / repeatCount)) // TrackProgress returns true on Cancel.
-            {
+         auto tempList = track.Copy(mT0, mT1);
+         const auto firstTemp = *tempList->Any<const WaveTrack>().begin();
+
+
+
+         auto t0 = tc;
+         for (size_t j = 0; j < repeatCount; ++j) {
+            if (TrackProgress(nTrack, j / repeatCount)) {
+               // TrackProgress returns true on Cancel.
                bGoodResult = false;
                return;
             }
-            track->Paste(tc, dest.get());
-            tc += tLen;
+            track.Paste(t0, *firstTemp);
+            t0 += tLen;
+         }
+         if (t0 > maxDestLen)
+            maxDestLen = t0;
+
+         const auto compareIntervals = [](const auto& a, const auto& b) {
+            return a->Start() < b->Start();
+         };
+
+         const auto eps = 0.5 / track.GetRate();
+         auto sortedIntervals = std::vector(
+            track.Intervals().begin(),
+            track.Intervals().end()
+         );
+         auto sourceIntervals = std::vector(
+            firstTemp->Intervals().begin(),
+            firstTemp->Intervals().end()
+         );
+         std::sort(sortedIntervals.begin(), sortedIntervals.end(), compareIntervals);
+         std::sort(sourceIntervals.begin(), sourceIntervals.end(), compareIntervals);
+         for (auto it = sortedIntervals.begin(); it != sortedIntervals.end(); ++it)
+         {
+            const auto& interval = *it;
+            //Find first pasted interval
+            if (std::abs((*it)->GetPlayStartTime() - tc) > eps)
+               continue;
+
+            //Fix pasted clips names
+            for(int j = 0; j < repeatCount; ++j)
+            {
+               for (const auto& src : sourceIntervals)
+               {
+                  if(it == sortedIntervals.end())
+                     break;
+                  (*it++)->SetName(src->GetName());
+               }
+            }
+            break;
          }
-         if (tc > maxDestLen)
-            maxDestLen = tc;
          nTrack++;
-      },
-      [&](Track *t)
+      }; },
+      [&](Track &t)
       {
-         if( SyncLock::IsSyncLockSelected(t) )
-            t->SyncLockAdjust(mT1, mT1 + (mT1 - mT0) * repeatCount);
+         if (SyncLock::IsSyncLockSelected(&t))
+            t.SyncLockAdjust(mT1, mT1 + (mT1 - mT0) * repeatCount);
       }
    );
 
@@ -151,14 +185,16 @@ bool EffectRepeat::Process(EffectInstance &, EffectSettings &)
       mT1 = maxDestLen;
    }
 
-   ReplaceProcessedTracks(bGoodResult);
+   if (bGoodResult)
+      outputs.Commit();
    return bGoodResult;
 }
 
-std::unique_ptr<EffectUIValidator> EffectRepeat::PopulateOrExchange(
+std::unique_ptr<EffectEditor> EffectRepeat::PopulateOrExchange(
    ShuttleGui & S, EffectInstance &, EffectSettingsAccess &,
    const EffectOutputs *)
 {
+   mUIParent = S.GetParent();
    S.StartHorizontalLay(wxCENTER, false);
    {
       mRepeatCount = S.Validator<IntegerValidator<int>>(
@@ -209,10 +245,10 @@ void EffectRepeat::DisplayNewTime()
    wxString str;
    mRepeatCount->GetValue().ToLong(&l);
 
-   NumericConverter nc(NumericConverter::TIME,
+   NumericConverter nc(FormatterContext::SampleRateContext(mProjectRate),
+                       NumericConverterType_TIME(),
                        GetSelectionFormat(),
-                       mT1 - mT0,
-                       mProjectRate);
+                       mT1 - mT0);
 
    str = wxString::Format( _("Current selection length: %s"), nc.GetString() );
 
@@ -220,7 +256,7 @@ void EffectRepeat::DisplayNewTime()
    mCurrentTime->SetName(str); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
 
    if (l > 0) {
-      EnableApply(true);
+      EffectEditor::EnableApply(mUIParent, true);
       repeatCount = l;
 
       nc.SetValue((mT1 - mT0) * (repeatCount + 1));
@@ -228,7 +264,7 @@ void EffectRepeat::DisplayNewTime()
    }
    else {
       str = _("Warning: No repeats.");
-      EnableApply(false);
+      EffectEditor::EnableApply(mUIParent, false);
    }
    mTotalTime->SetLabel(str);
    mTotalTime->SetName(str); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
@@ -238,3 +274,8 @@ void EffectRepeat::OnRepeatTextChange(wxCommandEvent & WXUNUSED(evt))
 {
    DisplayNewTime();
 }
+
+bool EffectRepeat::NeedsDither() const
+{
+   return false;
+}
