@@ -31,9 +31,8 @@
 a graph for EffectScienFilter.
 
 *//*******************************************************************/
-
-
 #include "ScienFilter.h"
+#include "EffectEditor.h"
 #include "LoadEffects.h"
 
 #include <math.h>
@@ -44,7 +43,6 @@ a graph for EffectScienFilter.
 #include <wx/choice.h>
 #include <wx/dcclient.h>
 #include <wx/dcmemory.h>
-#include <wx/intl.h>
 #include <wx/settings.h>
 #include <wx/slider.h>
 #include <wx/stattext.h>
@@ -56,13 +54,15 @@ a graph for EffectScienFilter.
 #include "PlatformCompatibility.h"
 #include "Prefs.h"
 #include "Project.h"
-#include "../ShuttleGui.h"
+#include "ShuttleGui.h"
 #include "Theme.h"
-#include "../WaveTrack.h"
+#include "WaveTrack.h"
 #include "../widgets/valnum.h"
-#include "../widgets/AudacityMessageBox.h"
-#include "../widgets/Ruler.h"
-#include "../widgets/WindowAccessible.h"
+#include "AudacityMessageBox.h"
+#include "../widgets/RulerPanel.h"
+#include "../widgets/IntFormat.h"
+#include "../widgets/LinearDBFormat.h"
+#include "WindowAccessible.h"
 
 #if !defined(M_PI)
 #define PI = 3.1415926535897932384626433832795
@@ -225,7 +225,7 @@ bool EffectScienFilter::Init()
    int selcount = 0;
    double rate = 0.0;
 
-   auto trackRange = inputTracks()->Selected< const WaveTrack >();
+   auto trackRange = inputTracks()->Selected<const WaveTrack>();
 
    {
       auto t = *trackRange.begin();
@@ -236,32 +236,28 @@ bool EffectScienFilter::Init()
          / 2.0;
    }
 
-   for (auto t : trackRange)
-   {
+   for (auto t : trackRange) {
       if (selcount == 0)
-      {
          rate = t->GetRate();
-      }
-      else
-      {
-         if (t->GetRate() != rate)
-         {
-            Effect::MessageBox(
+      else {
+         if (t->GetRate() != rate) {
+            EffectUIServices::DoMessageBox(*this,
                XO(
 "To apply a filter, all selected tracks must have the same sample rate.") );
             return false;
          }
       }
-      selcount++;
+      ++selcount;
    }
 
    return true;
 }
 
-std::unique_ptr<EffectUIValidator> EffectScienFilter::PopulateOrExchange(
+std::unique_ptr<EffectEditor> EffectScienFilter::PopulateOrExchange(
    ShuttleGui & S, EffectInstance &, EffectSettingsAccess &,
    const EffectOutputs *)
 {
+   mUIParent = S.GetParent();
    S.AddSpace(5);
    S.SetSizerProportion(1);
    S.StartMultiColumn(3, wxEXPAND);
@@ -279,7 +275,7 @@ std::unique_ptr<EffectUIValidator> EffectScienFilter::PopulateOrExchange(
             S.GetParent(), wxID_ANY, wxVERTICAL,
             wxSize{ 100, 100 }, // Ruler can't handle small sizes
             RulerPanel::Range{ 30.0, -120.0 },
-            Ruler::LinearDBFormat,
+            LinearDBFormat::Instance(),
             XO("dB"),
             RulerPanel::Options{}
                .LabelEdges(true)
@@ -337,7 +333,7 @@ std::unique_ptr<EffectUIValidator> EffectScienFilter::PopulateOrExchange(
          S.GetParent(), wxID_ANY, wxHORIZONTAL,
          wxSize{ 100, 100 }, // Ruler can't handle small sizes
          RulerPanel::Range{ mLoFreq, mNyquist },
-         Ruler::IntFormat,
+         IntFormat::Instance(),
          {},
          RulerPanel::Options{}
             .Log(true)
@@ -626,7 +622,8 @@ void EffectScienFilter::OnFilterSubtype(wxCommandEvent & WXUNUSED(evt))
 
 void EffectScienFilter::OnCutoff(wxCommandEvent & WXUNUSED(evt))
 {
-   if (!EnableApply(mUIParent->TransferDataFromWindow()))
+   if (!EffectEditor::EnableApply(
+      mUIParent, mUIParent->TransferDataFromWindow()))
    {
       return;
    }
@@ -636,7 +633,8 @@ void EffectScienFilter::OnCutoff(wxCommandEvent & WXUNUSED(evt))
 
 void EffectScienFilter::OnRipple(wxCommandEvent & WXUNUSED(evt))
 {
-   if (!EnableApply(mUIParent->TransferDataFromWindow()))
+   if (!EffectEditor::EnableApply(
+      mUIParent, mUIParent->TransferDataFromWindow()))
    {
       return;
    }
@@ -646,7 +644,8 @@ void EffectScienFilter::OnRipple(wxCommandEvent & WXUNUSED(evt))
 
 void EffectScienFilter::OnStopbandRipple(wxCommandEvent & WXUNUSED(evt))
 {
-   if (!EnableApply(mUIParent->TransferDataFromWindow()))
+   if (!EffectEditor::EnableApply(
+      mUIParent, mUIParent->TransferDataFromWindow()))
    {
       return;
    }
