@@ -12,15 +12,12 @@
 \brief An effect to add silence.
 
 *//*******************************************************************/
-
-
 #include "Silence.h"
+#include "EffectEditor.h"
 #include "LoadEffects.h"
 
-#include <wx/intl.h>
-
-#include "../ShuttleGui.h"
-#include "../WaveTrack.h"
+#include "ShuttleGui.h"
+#include "WaveTrack.h"
 #include "../widgets/NumericTextCtrl.h"
 
 const ComponentInterfaceSymbol EffectSilence::Symbol
@@ -65,7 +62,7 @@ EffectType EffectSilence::GetType() const
 
 // Effect implementation
 
-std::unique_ptr<EffectUIValidator> EffectSilence::PopulateOrExchange(
+std::unique_ptr<EffectEditor> EffectSilence::PopulateOrExchange(
    ShuttleGui & S, EffectInstance &, EffectSettingsAccess &access,
    const EffectOutputs *)
 {
@@ -76,11 +73,11 @@ std::unique_ptr<EffectUIValidator> EffectSilence::PopulateOrExchange(
          S.AddPrompt(XXO("&Duration:"));
          auto &extra = access.Get().extra;
          mDurationT = safenew
-            NumericTextCtrl(S.GetParent(), wxID_ANY,
-                              NumericConverter::TIME,
+            NumericTextCtrl(FormatterContext::SampleRateContext(mProjectRate),
+                              S.GetParent(), wxID_ANY,
+                              NumericConverterType_TIME(),
                               extra.GetDurationFormat(),
                               extra.GetDuration(),
-                               mProjectRate,
                                NumericTextCtrl::Options{}
                                   .AutoPos(true));
          S.Name(XO("Duration"))
@@ -108,9 +105,10 @@ bool EffectSilence::TransferDataFromWindow(EffectSettings &settings)
    return true;
 }
 
-bool EffectSilence::GenerateTrack(EffectSettings &settings,
-   WaveTrack *tmp, const WaveTrack &, int)
+bool EffectSilence::GenerateTrack(
+   const EffectSettings &settings, TrackList &tmp)
 {
-   tmp->InsertSilence(0.0, settings.extra.GetDuration());
+   (*tmp.Any<WaveTrack>().begin())
+      ->InsertSilence(0.0, settings.extra.GetDuration());
    return true;
 }
