@@ -37,7 +37,7 @@ public:
 
    // Effect implementation
 
-   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+   std::unique_ptr<EffectEditor> PopulateOrExchange(
       ShuttleGui & S, EffectInstance &instance,
       EffectSettingsAccess &access, const EffectOutputs *pOutputs) override;
    bool TransferDataToWindow(const EffectSettings &settings) override;
@@ -46,8 +46,7 @@ public:
 protected:
    // Generator implementation
 
-   bool GenerateTrack(EffectSettings &settings,
-      WaveTrack *tmp, const WaveTrack &track, int ntrack) override;
+   bool GenerateTrack(const EffectSettings &settings, TrackList &tmp) override;
 
 private:
    NumericTextCtrl *mDurationT;
