@@ -14,9 +14,10 @@
        where the volume is below a set threshold level.
 
 *//*******************************************************************/
-
-
 #include "TruncSilence.h"
+#include "BasicUI.h"
+#include "EffectEditor.h"
+#include "EffectOutputTracks.h"
 #include "LoadEffects.h"
 
 #include <algorithm>
@@ -30,12 +31,11 @@
 
 #include "Prefs.h"
 #include "Project.h"
-#include "../ProjectSettings.h"
-#include "../ShuttleGui.h"
-#include "../SyncLock.h"
-#include "../WaveTrack.h"
+#include "ShuttleGui.h"
+#include "SyncLock.h"
+#include "WaveTrack.h"
 #include "../widgets/valnum.h"
-#include "../widgets/AudacityMessageBox.h"
+#include "AudacityMessageBox.h"
 
 class Enums {
 public:
@@ -98,7 +98,7 @@ static const size_t DEF_BlendFrameCount = 100;
 
 // Lower bound on the amount of silence to find at a time -- this avoids
 // detecting silence repeatedly in low-frequency sounds.
-static const double DEF_MinTruncMs = 0.001; 
+static const double DEF_MinTruncMs = 0.001;
 
 // Typical fraction of total time taken by detection (better to guess low)
 const double detectFrac = 0.4;
@@ -190,7 +190,7 @@ bool EffectTruncSilence::LoadSettings(
       if (!parms.ReadAndVerify( ActIndex.key, &temp, ActIndex.def,
          kActionStrings, nActions, kObsoleteActions, nObsoleteActions))
          return false;
-   
+
       // TODO:  fix this when settings are really externalized
       const_cast<int&>(mActionIndex) = temp;
    }
@@ -215,15 +215,16 @@ double EffectTruncSilence::CalcPreviewInputLength(
 
    int whichTrack = 0;
 
-   for (auto wt : inputTracks()->Selected< const WaveTrack >()) {
+   for (auto wt : inputTracks()->Selected<const WaveTrack>()) {
       RegionList trackSilences;
 
       auto index = wt->TimeToLongSamples(mT0);
       sampleCount silentFrame = 0; // length of the current silence
 
-      Analyze(silences, trackSilences, wt, &silentFrame, &index, whichTrack, &inputLength, &minInputLength);
+      Analyze(silences, trackSilences, *wt, &silentFrame, &index, whichTrack,
+         &inputLength, &minInputLength);
 
-      whichTrack++;
+      whichTrack += wt->NChannels();
    }
    return inputLength;
 }
@@ -236,9 +237,6 @@ bool EffectTruncSilence::Process(EffectInstance &, EffectSettings &)
       ? ProcessIndependently()
       : ProcessAll();
 
-   if (success)
-      ReplaceProcessedTracks(true);
-
    return success;
 }
 
@@ -246,21 +244,18 @@ bool EffectTruncSilence::ProcessIndependently()
 {
    unsigned nGroups = 0;
 
-   const auto &settings = ProjectSettings::Get( *FindProject() );
-   const bool syncLock = settings.IsSyncLocked();
+   const bool syncLock = SyncLockState::Get(*FindProject()).IsSyncLocked();
 
    // Check if it's permissible
    {
-      for (auto track : inputTracks()->SelectedLeaders< const WaveTrack >() ) {
+      for (auto track : inputTracks()->Selected<const WaveTrack>()) {
          if (syncLock) {
-            auto channels = TrackList::Channels(track);
             auto otherTracks =
                SyncLock::Group(track).Filter<const WaveTrack>()
                   + &Track::IsSelected
-                  - [&](const Track *pTrack){
-                        return channels.contains(pTrack); };
+                  - [&](const Track *pTrack){ return pTrack == track; };
             if (otherTracks) {
-               ::Effect::MessageBox(
+               EffectUIServices::DoMessageBox(*this,
                   XO(
 "When truncating independently, there may only be one selected audio track in each Sync-Locked Track Group.") );
                return false;
@@ -278,31 +273,25 @@ bool EffectTruncSilence::ProcessIndependently()
    // Now do the work
 
    // Copy tracks
-   CopyInputTracks(true);
+   EffectOutputTracks outputs {
+      *mTracks, GetType(), { { mT0, mT1 } }, true, true
+   };
    double newT1 = 0.0;
 
    {
       unsigned iGroup = 0;
-      for (auto track : mOutputTracks->SelectedLeaders< WaveTrack >() ) {
-         Track *const last = *TrackList::Channels(track).rbegin();
-
+      for (auto track : outputs.Get().Selected<WaveTrack>()) {
          RegionList silences;
-
-         if (!FindSilences(silences, mOutputTracks.get(), track, last))
+         if (!FindSilences(silences,
+            TrackList::SingletonRange(&as_const(*track))))
             return false;
          // Treat tracks in the sync lock group only
          Track *groupFirst, *groupLast;
-         if (syncLock) {
-            auto trackRange = SyncLock::Group(track);
-            groupFirst = *trackRange.begin();
-            groupLast = *trackRange.rbegin();
-         }
-         else {
-            groupFirst = track;
-            groupLast = last;
-         }
+         auto range = syncLock
+            ? SyncLock::Group(track)
+            : TrackList::SingletonRange<Track>(track);
          double totalCutLen = 0.0;
-         if (!DoRemoval(silences, iGroup, nGroups, groupFirst, groupLast, totalCutLen))
+         if (!DoRemoval(silences, range, iGroup, nGroups, totalCutLen))
             return false;
          newT1 = std::max(newT1, mT1 - totalCutLen);
 
@@ -312,26 +301,29 @@ bool EffectTruncSilence::ProcessIndependently()
 
    mT1 = newT1;
 
+   outputs.Commit();
+
    return true;
 }
 
 bool EffectTruncSilence::ProcessAll()
 {
    // Copy tracks
-   CopyInputTracks(true);
+   EffectOutputTracks outputs {
+      *mTracks, GetType(), { { mT0, mT1 } }, true, true
+   };
 
    // Master list of silent regions.
    // This list should always be kept in order.
    RegionList silences;
 
-   auto trackRange0 = inputTracks()->Selected< const WaveTrack >();
-   if (FindSilences(
-         silences, inputTracks(), *trackRange0.begin(), *trackRange0.rbegin())) {
-      auto trackRange = mOutputTracks->Any();
+   if (FindSilences(silences, outputs.Get().Selected<const WaveTrack>()))
+   {
+      auto trackRange = outputs.Get().Any();
       double totalCutLen = 0.0;
-      if (DoRemoval(silences, 0, 1,
-         *trackRange.begin(), *trackRange.rbegin(), totalCutLen)) {
+      if (DoRemoval(silences, trackRange, 0, 1, totalCutLen)) {
          mT1 -= totalCutLen;
+         outputs.Commit();
          return true;
       }
    }
@@ -339,22 +331,20 @@ bool EffectTruncSilence::ProcessAll()
    return false;
 }
 
-bool EffectTruncSilence::FindSilences
-   (RegionList &silences, const TrackList *list,
-    const Track *firstTrack, const Track *lastTrack)
+bool EffectTruncSilence::FindSilences(RegionList &silences,
+   const TrackIterRange<const WaveTrack> &range)
 {
    // Start with the whole selection silent
    silences.push_back(Region(mT0, mT1));
 
    // Remove non-silent regions in each track
    int whichTrack = 0;
-   for (auto wt :
-           list->Selected< const WaveTrack >()
-               .StartingWith( firstTrack ).EndingAfter( lastTrack ) )
-   {
+   for (auto wt : range) {
+      assert(wt->IsLeader());
       // Smallest silent region to detect in frames
       auto minSilenceFrames =
-         sampleCount(std::max(mInitialAllowedSilence, DEF_MinTruncMs) * wt->GetRate());
+         sampleCount(std::max(mInitialAllowedSilence, DEF_MinTruncMs)
+            * wt->GetRate());
 
       //
       // Scan the track for silences
@@ -365,14 +355,12 @@ bool EffectTruncSilence::FindSilences
       sampleCount silentFrame = 0;
 
       // Detect silences
-      bool cancelled = !(Analyze(silences, trackSilences, wt, &silentFrame, &index, whichTrack));
+      bool cancelled = !(Analyze(
+         silences, trackSilences, *wt, &silentFrame, &index, whichTrack));
 
       // Buffer has been freed, so we're OK to return if cancelled
       if (cancelled)
-      {
-         ReplaceProcessedTracks(false);
          return false;
-      }
 
       if (silentFrame >= minSilenceFrames)
       {
@@ -391,9 +379,10 @@ bool EffectTruncSilence::FindSilences
    return true;
 }
 
-bool EffectTruncSilence::DoRemoval
-(const RegionList &silences, unsigned iGroup, unsigned nGroups, Track *firstTrack, Track *lastTrack,
- double &totalCutLen)
+bool EffectTruncSilence::DoRemoval(const RegionList &silences,
+   const TrackIterRange<Track> &range,
+   unsigned iGroup, unsigned nGroups,
+   double &totalCutLen)
 {
    //
    // Now remove the silent regions from all selected / sync-lock selected tracks.
@@ -412,10 +401,7 @@ bool EffectTruncSilence::DoRemoval
       const double frac = detectFrac +
          (1 - detectFrac) * (iGroup + whichReg / double(silences.size())) / nGroups;
       if (TotalProgress(frac))
-      {
-         ReplaceProcessedTracks(false);
          return false;
-      }
 
       // Intersection may create regions smaller than allowed; ignore them.
       // Allow one nanosecond extra for consistent results with exact milliseconds of allowed silence.
@@ -445,60 +431,84 @@ bool EffectTruncSilence::DoRemoval
       // Don't waste time cutting nothing.
       if( cutLen == 0.0 )
          continue;
-      
+
       totalCutLen += cutLen;
 
+      bool success = true;
       double cutStart = (r->start + r->end - cutLen) / 2;
       double cutEnd = cutStart + cutLen;
-      (mOutputTracks->Any()
-         .StartingWith(firstTrack).EndingAfter(lastTrack)
+      (range
          + &SyncLock::IsSelectedOrSyncLockSelected
          - [&](const Track *pTrack) { return
            // Don't waste time past the end of a track
            pTrack->GetEndTime() < r->start;
          }
-      ).Visit(
-         [&](WaveTrack *wt) {
-
+      ).VisitWhile(success,
+         [&](WaveTrack &wt) {
+            assert(wt.IsLeader());
             // In WaveTracks, clear with a cross-fade
             auto blendFrames = mBlendFrameCount;
             // Round start/end times to frame boundaries
-            cutStart = wt->LongSamplesToTime(wt->TimeToLongSamples(cutStart));
-            cutEnd = wt->LongSamplesToTime(wt->TimeToLongSamples(cutEnd));
+            cutStart = wt.SnapToSample(cutStart);
+            cutEnd = wt.SnapToSample(cutEnd);
 
             // Make sure the cross-fade does not affect non-silent frames
-            if (wt->LongSamplesToTime(blendFrames) > inLength)
-            {
+            if (wt.LongSamplesToTime(blendFrames) > inLength) {
                // Result is not more than blendFrames:
-               blendFrames = wt->TimeToLongSamples(inLength).as_size_t();
+               blendFrames = wt.TimeToLongSamples(inLength).as_size_t();
             }
 
             // Perform cross-fade in memory
-            Floats buf1{ blendFrames };
-            Floats buf2{ blendFrames };
-            auto t1 = wt->TimeToLongSamples(cutStart) - blendFrames / 2;
-            auto t2 = wt->TimeToLongSamples(cutEnd) - blendFrames / 2;
+            struct Buffers {
+               Buffers(size_t size)
+                  : buf1{ size }, buf2 { size }
+               {}
+               Floats buf1, buf2;
+            };
+            Buffers buffers[2]{ blendFrames, blendFrames };
+            auto t1 = wt.TimeToLongSamples(cutStart) - blendFrames / 2;
+            auto t2 = wt.TimeToLongSamples(cutEnd) - blendFrames / 2;
 
-            wt->GetFloats(buf1.get(), t1, blendFrames);
-            wt->GetFloats(buf2.get(), t2, blendFrames);
+            size_t iChannel = 0;
+            for (const auto pChannel : wt.Channels()) {
+               auto &buffer = buffers[iChannel];
+               pChannel->GetFloats(buffer.buf1.get(), t1, blendFrames);
+               pChannel->GetFloats(buffer.buf2.get(), t2, blendFrames);
 
-            for (decltype(blendFrames) i = 0; i < blendFrames; ++i)
-            {
-               buf1[i] = ((blendFrames-i) * buf1[i] + i * buf2[i]) /
-                         (double)blendFrames;
+               for (decltype(blendFrames) i = 0; i < blendFrames; ++i) {
+                  buffer.buf1[i] =
+                     ((blendFrames - i) * buffer.buf1[i] + i * buffer.buf2[i]) /
+                        (double)blendFrames;
+               }
+               ++iChannel;
             }
 
-            // Perform the cut
-            wt->Clear(cutStart, cutEnd);
+            assert(wt.IsLeader()); // given range visits leaders only
+            wt.Clear(cutStart, cutEnd);
 
-            // Write cross-faded data
-            wt->Set((samplePtr)buf1.get(), floatSample, t1, blendFrames);
+            iChannel = 0;
+            for (const auto pChannel : wt.Channels()) {
+               // Write cross-faded data
+               auto &buffer = buffers[iChannel];
+               success = success &&
+               pChannel->Set((samplePtr)buffer.buf1.get(), floatSample, t1,
+                  blendFrames,
+                  // This effect mostly shifts samples to remove silences, and
+                  // does only a little bit of floating point calculations to
+                  // cross-fade the splices, over a 100 sample interval by default.
+                  // Don't dither.
+                  narrowestSampleFormat);
+               ++iChannel;
+            }
          },
-         [&](Track *t) {
+         [&](Track &t) {
+            assert(t.IsLeader());
             // Non-wave tracks: just do a sync-lock adjust
-            t->SyncLockAdjust(cutEnd, cutStart);
+            t.SyncLockAdjust(cutEnd, cutStart);
          }
       );
+      if (!success)
+         return false;
       ++whichReg;
    }
 
@@ -506,37 +516,42 @@ bool EffectTruncSilence::DoRemoval
 }
 
 bool EffectTruncSilence::Analyze(RegionList& silenceList,
-                                 RegionList& trackSilences,
-                                 const WaveTrack *wt,
-                                 sampleCount* silentFrame,
-                                 sampleCount* index,
-                                 int whichTrack,
-                                 double* inputLength /*= NULL*/,
-                                 double* minInputLength /*= NULL*/) const
+   RegionList& trackSilences, const WaveTrack &wt, sampleCount* silentFrame,
+   sampleCount* index, int whichTrack, double* inputLength,
+   double* minInputLength) const
 {
-   // Smallest silent region to detect in frames
-   auto minSilenceFrames = sampleCount(std::max( mInitialAllowedSilence, DEF_MinTruncMs) * wt->GetRate());
+   assert(wt.IsLeader());
+   const auto rate = wt.GetRate();
 
-   double truncDbSilenceThreshold = DB_TO_LINEAR( mThresholdDB );
-   auto blockLen = wt->GetMaxBlockSize();
-   auto start = wt->TimeToLongSamples(mT0);
-   auto end = wt->TimeToLongSamples(mT1);
+   // Smallest silent region to detect in frames
+   auto minSilenceFrames =
+      sampleCount(std::max(mInitialAllowedSilence, DEF_MinTruncMs) * rate);
+
+   double truncDbSilenceThreshold = DB_TO_LINEAR(mThresholdDB);
+   auto blockLen = wt.GetMaxBlockSize();
+   auto start = wt.TimeToLongSamples(mT0);
+   auto end = wt.TimeToLongSamples(mT1);
    sampleCount outLength = 0;
 
    double previewLength;
    gPrefs->Read(wxT("/AudioIO/EffectsPreviewLen"), &previewLength, 6.0);
    // Minimum required length in samples.
-   const sampleCount previewLen( previewLength * wt->GetRate() );
+   const sampleCount previewLen(previewLength * rate);
 
    // Keep position in overall silences list for optimization
    RegionList::iterator rit(silenceList.begin());
 
-   // Allocate buffer
-   Floats buffer{ blockLen };
+   // Allocate buffers
+   Floats buffers[] {
+      Floats{ blockLen },
+      Floats{ blockLen }
+   };
 
    // Loop through current track
    while (*index < end) {
-      if (inputLength && ((outLength >= previewLen) || (*index - start > wt->TimeToLongSamples(*minInputLength)))) {
+      if (inputLength && ((outLength >= previewLen) ||
+         (*index - start > wt.TimeToLongSamples(*minInputLength)))
+      ) {
          *inputLength = std::min<double>(*inputLength, *minInputLength);
          if (outLength >= previewLen) {
             *minInputLength = *inputLength;
@@ -557,7 +572,7 @@ bool EffectTruncSilence::Analyze(RegionList& silenceList,
 
       // Optimization: if not in a silent region skip ahead to the next one
 
-      double curTime = wt->LongSamplesToTime(*index);
+      double curTime = wt.LongSamplesToTime(*index);
       for ( ; rit != silenceList.end(); ++rit) {
          // Find the first silent region ending after current time
          if (rit->end >= curTime) {
@@ -569,9 +584,12 @@ bool EffectTruncSilence::Analyze(RegionList& silenceList,
          // No more regions -- no need to process the rest of the track
          if (inputLength) {
             // Add available samples up to previewLength.
-            auto remainingTrackSamples = wt->TimeToLongSamples(wt->GetEndTime()) - *index;
+            auto remainingTrackSamples =
+               wt.TimeToLongSamples(wt.GetEndTime()) - *index;
             auto requiredTrackSamples = previewLen - outLength;
-            outLength += (remainingTrackSamples > requiredTrackSamples)? requiredTrackSamples : remainingTrackSamples;
+            outLength += (remainingTrackSamples > requiredTrackSamples)
+               ? requiredTrackSamples
+               : remainingTrackSamples;
          }
 
          break;
@@ -580,16 +598,18 @@ bool EffectTruncSilence::Analyze(RegionList& silenceList,
          // End current silent region, skip ahead
          if (*silentFrame >= minSilenceFrames)  {
             trackSilences.push_back(Region(
-               wt->LongSamplesToTime(*index - *silentFrame),
-               wt->LongSamplesToTime(*index)
+               wt.LongSamplesToTime(*index - *silentFrame),
+               wt.LongSamplesToTime(*index)
             ));
          }
          *silentFrame = 0;
-         auto newIndex = wt->TimeToLongSamples(rit->start);
+         auto newIndex = wt.TimeToLongSamples(rit->start);
          if (inputLength) {
             auto requiredTrackSamples = previewLen - outLength;
             // Add non-silent sample to outLength
-            outLength += ((newIndex - *index) > requiredTrackSamples)? requiredTrackSamples : newIndex - *index;
+            outLength += ((newIndex - *index) > requiredTrackSamples)
+               ? requiredTrackSamples
+               : newIndex - *index;
          }
 
          *index = newIndex;
@@ -599,29 +619,39 @@ bool EffectTruncSilence::Analyze(RegionList& silenceList,
       // Limit size of current block if we've reached the end
       auto count = limitSampleBufferSize( blockLen, end - *index );
 
-      // Fill buffer
-      wt->GetFloats((buffer.get()), *index, count);
+      // Fill buffers
+      size_t iChannel = 0;
+      for (const auto pChannel : wt.Channels())
+         pChannel->GetFloats(buffers[iChannel++].get(), *index, count);
 
       // Look for silenceList in current block
       for (decltype(count) i = 0; i < count; ++i) {
-         if (inputLength && ((outLength >= previewLen) || (outLength > wt->TimeToLongSamples(*minInputLength)))) {
-            *inputLength = wt->LongSamplesToTime(*index + i) - wt->LongSamplesToTime(start);
+         if (inputLength && ((outLength >= previewLen) ||
+            (outLength > wt.TimeToLongSamples(*minInputLength)))
+         ) {
+            *inputLength =
+               wt.LongSamplesToTime(*index + i) - wt.LongSamplesToTime(start);
             break;
          }
 
-         if (fabs(buffer[i]) < truncDbSilenceThreshold) {
+         const bool silent = std::all_of(buffers, buffers + iChannel,
+         [&](const Floats &buffer){
+            return fabs(buffer[i]) < truncDbSilenceThreshold;
+         });
+         if (silent)
             (*silentFrame)++;
-         }
          else {
             sampleCount allowed = 0;
             if (*silentFrame >= minSilenceFrames) {
                if (inputLength) {
                   switch (mActionIndex) {
                      case kTruncate:
-                        outLength += wt->TimeToLongSamples(mTruncLongestAllowedSilence);
+                        outLength +=
+                           wt.TimeToLongSamples(mTruncLongestAllowedSilence);
                         break;
                      case kCompress:
-                        allowed = wt->TimeToLongSamples(mInitialAllowedSilence);
+                        allowed =
+                           wt.TimeToLongSamples(mInitialAllowedSilence);
                         outLength += sampleCount(
                            allowed.as_double() +
                               (*silentFrame - allowed).as_double()
@@ -634,8 +664,8 @@ bool EffectTruncSilence::Analyze(RegionList& silenceList,
 
                // Record the silent region
                trackSilences.push_back(Region(
-                  wt->LongSamplesToTime(*index + i - *silentFrame),
-                  wt->LongSamplesToTime(*index + i)
+                  wt.LongSamplesToTime(*index + i - *silentFrame),
+                  wt.LongSamplesToTime(*index + i)
                ));
             }
             else if (inputLength) {   // included as part of non-silence
@@ -662,9 +692,11 @@ bool EffectTruncSilence::Analyze(RegionList& silenceList,
 }
 
 
-std::unique_ptr<EffectUIValidator> EffectTruncSilence::PopulateOrExchange(
-   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &)
+std::unique_ptr<EffectEditor> EffectTruncSilence::PopulateOrExchange(
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &,
+   const EffectOutputs *)
 {
+   mUIParent = S.GetParent();
    wxASSERT(nActions == WXSIZEOF(kActionStrings));
 
    S.AddSpace(0, 5);
@@ -934,8 +966,14 @@ void EffectTruncSilence::OnControlChange(wxCommandEvent & WXUNUSED(evt))
 
    UpdateUI();
 
-   if (!EnableApply(mUIParent->TransferDataFromWindow()))
+   if (!EffectEditor::EnableApply(
+      mUIParent, mUIParent->TransferDataFromWindow()))
    {
       return;
    }
 }
+
+bool EffectTruncSilence::NeedsDither() const
+{
+   return false;
+}
