@@ -17,8 +17,10 @@
 #ifndef __AUDACITY_EFFECT_TRUNC_SILENCE__
 #define __AUDACITY_EFFECT_TRUNC_SILENCE__
 
-#include "Effect.h"
-#include "../ShuttleAutomation.h"
+#include "StatefulEffect.h"
+#include "ShuttleAutomation.h"
+#include "Track.h"
+#include <wx/weakref.h>
 
 class ShuttleGui;
 class wxChoice;
@@ -57,22 +59,23 @@ public:
    // Analyze a single track to find silences
    // If inputLength is not NULL we are calculating the minimum
    // amount of input for previewing.
-   bool Analyze(RegionList &silenceList,
-                        RegionList &trackSilences,
-                        const WaveTrack *wt,
-                        sampleCount* silentFrame,
-                        sampleCount* index,
-                        int whichTrack,
-                        double* inputLength = NULL,
-                        double* minInputLength = NULL) const;
+   /*!
+    @pre `wt.IsLeader()`
+    */
+   bool Analyze(RegionList &silenceList, RegionList &trackSilences,
+      const WaveTrack &wt, sampleCount* silentFrame, sampleCount* index,
+      int whichTrack, double* inputLength = nullptr,
+      double* minInputLength = nullptr) const;
 
    bool Process(EffectInstance &instance, EffectSettings &settings) override;
-   std::unique_ptr<EffectUIValidator> PopulateOrExchange(
+   std::unique_ptr<EffectEditor> PopulateOrExchange(
       ShuttleGui & S, EffectInstance &instance,
       EffectSettingsAccess &access, const EffectOutputs *pOutputs) override;
    bool TransferDataToWindow(const EffectSettings &settings) override;
    bool TransferDataFromWindow(EffectSettings &settings) override;
 
+   bool NeedsDither() const override;
+
 private:
    // EffectTruncSilence implementation
 
@@ -85,12 +88,20 @@ private:
 
    bool ProcessIndependently();
    bool ProcessAll();
-   bool FindSilences
-      (RegionList &silences, const TrackList *list,
-       const Track *firstTrack, const Track *lastTrack);
-   bool DoRemoval
-      (const RegionList &silences, unsigned iGroup, unsigned nGroups, Track *firstTrack, Track *lastTrack,
-       double &totalCutLen);
+   /*!
+    @pre range visits leaders only
+    */
+   bool FindSilences(RegionList &silences,
+      const TrackIterRange<const WaveTrack> &range);
+   /*!
+    @pre range visits leaders only
+    */
+   bool DoRemoval(const RegionList &silences,
+      const TrackIterRange<Track> &range,
+      unsigned iGroup, unsigned nGroups,
+      double &totalCutLen);
+
+   wxWeakRef<wxWindow> mUIParent{};
 
    double mThresholdDB {} ;
    int mActionIndex;
