@@ -20,14 +20,15 @@
 
 
 #include "Wahwah.h"
+#include "EffectEditor.h"
 #include "LoadEffects.h"
 
 #include <math.h>
 
-#include <wx/intl.h>
 #include <wx/slider.h>
+#include <wx/weakref.h>
 
-#include "../ShuttleGui.h"
+#include "ShuttleGui.h"
 #include "../widgets/valnum.h"
 
 const EffectParameterMethods& EffectWahwah::Parameters() const
@@ -50,17 +51,15 @@ const ComponentInterfaceSymbol EffectWahwah::Symbol
 
 namespace{ BuiltinEffectsModule::Registration< EffectWahwah > reg; }
 
-struct EffectWahwah::Validator
-   : EffectUIValidator
+struct EffectWahwah::Editor
+   : EffectEditor
 {
-   Validator(EffectUIClientInterface& effect,
-      EffectSettingsAccess& access, const EffectWahwahSettings& settings)
-      : EffectUIValidator{ effect, access }
+   Editor(const EffectUIServices& services,
+      EffectSettingsAccess& access, const EffectWahwahSettings& settings
+   )  : EffectEditor{ services, access }
       , mSettings{ settings }
    {}
-   virtual ~Validator() = default;
-
-   Effect& GetEffect() const { return static_cast<Effect&>(mEffect); }
+   virtual ~Editor() = default;
 
    bool ValidateUI() override;
    bool UpdateUI() override;
@@ -96,23 +95,22 @@ struct EffectWahwah::Validator
    wxSlider* mOutGainS;
 
 
+   wxWeakRef<wxWindow> mUIParent;
    EffectWahwahSettings mSettings;
 
    void EnableApplyFromValidate()
    {
-      Effect& actualEffect = static_cast<Effect&>(mEffect);
-      actualEffect.EnableApply(actualEffect.GetUIParent()->Validate());
+      EnableApply(mUIParent, mUIParent->Validate());
    }
 
    bool EnableApplyFromTransferDataToWindow()
    {
-      Effect& actualEffect = static_cast<Effect&>(mEffect);
-      return actualEffect.EnableApply(actualEffect.GetUIParent()->TransferDataFromWindow());
+      return EnableApply(mUIParent, mUIParent->TransferDataFromWindow());
    }
 };
 
 
-bool EffectWahwah::Validator::ValidateUI()
+bool EffectWahwah::Editor::ValidateUI()
 {
    mAccess.ModifySettings
    (
@@ -164,8 +162,8 @@ struct EffectWahwah::Instance
    unsigned GetAudioInCount() const override;
    unsigned GetAudioOutCount() const override;
 
-   EffectWahwahState mMaster;
-   std::vector<EffectWahwahState> mSlaves;
+   EffectWahwahState mState;
+   std::vector<EffectWahwah::Instance> mSlaves;
 };
 
 
@@ -209,22 +207,22 @@ EffectType EffectWahwah::GetType() const
 
 auto EffectWahwah::RealtimeSupport() const -> RealtimeSince
 {
-   return RealtimeSince::Never;
+   return RealtimeSince::After_3_1;
 }
 
 bool EffectWahwah::Instance::ProcessInitialize(EffectSettings & settings,
    double sampleRate, ChannelNames chanMap)
 {
-   InstanceInit(settings, mMaster, sampleRate);
+   InstanceInit(settings, mState, sampleRate);
    if (chanMap[0] == ChannelNameFrontRight)
-      mMaster.phase += M_PI;
+      mState.phase += M_PI;
    return true;
 }
 
 size_t EffectWahwah::Instance::ProcessBlock(EffectSettings &settings,
    const float *const *inBlock, float *const *outBlock, size_t blockLen)
 {
-   return InstanceProcess(settings, mMaster, inBlock, outBlock, blockLen);
+   return InstanceProcess(settings, mState, inBlock, outBlock, blockLen);
 }
 
 bool EffectWahwah::Instance::RealtimeInitialize(EffectSettings &, double)
@@ -237,9 +235,9 @@ bool EffectWahwah::Instance::RealtimeInitialize(EffectSettings &, double)
 bool EffectWahwah::Instance::RealtimeAddProcessor(
    EffectSettings &settings, EffectOutputs *, unsigned, float sampleRate)
 {
-   EffectWahwahState slave;
+   EffectWahwah::Instance slave(mProcessor);
 
-   InstanceInit(settings, slave, sampleRate);
+   InstanceInit(settings, slave.mState, sampleRate);
 
    mSlaves.push_back(slave);
 
@@ -258,24 +256,25 @@ size_t EffectWahwah::Instance::RealtimeProcess(size_t group, EffectSettings &set
 {
    if (group >= mSlaves.size())
       return 0;
-   return InstanceProcess(settings, mSlaves[group], inbuf, outbuf, numSamples);
+   return InstanceProcess(settings, mSlaves[group].mState, inbuf, outbuf, numSamples);
 }
 
 // Effect implementation
 
-std::unique_ptr<EffectUIValidator> EffectWahwah::PopulateOrExchange(
+std::unique_ptr<EffectEditor> EffectWahwah::MakeEditor(
    ShuttleGui & S, EffectInstance &, EffectSettingsAccess &access,
-   const EffectOutputs *)
+   const EffectOutputs *) const
 {
    auto& settings = access.Get();
    auto& myEffSettings = GetSettings(settings);
-   auto result = std::make_unique<Validator>(*this, access, myEffSettings);
+   auto result = std::make_unique<Editor>(*this, access, myEffSettings);
    result->PopulateOrExchange(S);
    return result;
 }
 
-void EffectWahwah::Validator::PopulateOrExchange(ShuttleGui & S)
+void EffectWahwah::Editor::PopulateOrExchange(ShuttleGui & S)
 {
+   mUIParent = S.GetParent();
    auto& ms = mSettings;
 
    S.SetBorder(5);
@@ -289,20 +288,20 @@ void EffectWahwah::Validator::PopulateOrExchange(ShuttleGui & S)
          .Validator<FloatingPointValidator<double>>(
             5, &ms.mFreq, NumValidatorStyle::ONE_TRAILING_ZERO, Freq.min, Freq.max)
          .AddTextBox(XXO("LFO Freq&uency (Hz):"), L"", 12);
-      BindTo(*mFreqT, wxEVT_TEXT, &Validator::OnFreqText);
+      BindTo(*mFreqT, wxEVT_TEXT, &Editor::OnFreqText);
 
       mFreqS = S
          .Name(XO("LFO frequency in hertz"))
          .Style(wxSL_HORIZONTAL)
          .MinSize( { 100, -1 } )
          .AddSlider( {}, Freq.def * Freq.scale, Freq.max * Freq.scale, Freq.min * Freq.scale);
-      BindTo(*mFreqS, wxEVT_SLIDER, &Validator::OnFreqSlider);
+      BindTo(*mFreqS, wxEVT_SLIDER, &Editor::OnFreqSlider);
 
       mPhaseT = S
          .Validator<FloatingPointValidator<double>>(
             1, &ms.mPhase, NumValidatorStyle::DEFAULT, Phase.min, Phase.max)
          .AddTextBox(XXO("LFO Sta&rt Phase (deg.):"), L"", 12);
-      BindTo(*mPhaseT, wxEVT_TEXT, &Validator::OnPhaseText);
+      BindTo(*mPhaseT, wxEVT_TEXT, &Editor::OnPhaseText);
 
       mPhaseS = S
          .Name(XO("LFO start phase in degrees"))
@@ -310,64 +309,64 @@ void EffectWahwah::Validator::PopulateOrExchange(ShuttleGui & S)
          .MinSize( { 100, -1 } )
          .AddSlider( {}, Phase.def * Phase.scale, Phase.max * Phase.scale, Phase.min * Phase.scale);
       mPhaseS->SetLineSize(10);
-      BindTo(*mPhaseS, wxEVT_SLIDER, &Validator::OnPhaseSlider);
+      BindTo(*mPhaseS, wxEVT_SLIDER, &Editor::OnPhaseSlider);
 
       mDepthT = S
          .Validator<IntegerValidator<int>>(
             &ms.mDepth, NumValidatorStyle::DEFAULT, Depth.min, Depth.max)
          .AddTextBox(XXO("Dept&h (%):"), L"", 12);
-      BindTo(*mDepthT, wxEVT_TEXT, &Validator::OnDepthText);
+      BindTo(*mDepthT, wxEVT_TEXT, &Editor::OnDepthText);
 
       mDepthS = S
          .Name(XO("Depth in percent"))
          .Style(wxSL_HORIZONTAL)
          .MinSize( { 100, -1 } )
          .AddSlider( {}, Depth.def * Depth.scale, Depth.max * Depth.scale, Depth.min * Depth.scale);
-      BindTo(*mDepthS, wxEVT_SLIDER, &Validator::OnDepthSlider);
+      BindTo(*mDepthS, wxEVT_SLIDER, &Editor::OnDepthSlider);
 
       mResT = S
          .Validator<FloatingPointValidator<double>>(
             1, &ms.mRes, NumValidatorStyle::DEFAULT, Res.min, Res.max)
          .AddTextBox(XXO("Reso&nance:"), L"", 12);
-      BindTo(*mResT, wxEVT_TEXT, &Validator::OnResonanceText);
+      BindTo(*mResT, wxEVT_TEXT, &Editor::OnResonanceText);
 
       mResS = S
          .Name(XO("Resonance"))
          .Style(wxSL_HORIZONTAL)
          .MinSize( { 100, -1 } )
          .AddSlider( {}, Res.def * Res.scale, Res.max * Res.scale, Res.min * Res.scale);
-      BindTo(*mResS, wxEVT_SLIDER, &Validator::OnResonanceSlider);
+      BindTo(*mResS, wxEVT_SLIDER, &Editor::OnResonanceSlider);
 
       mFreqOfsT = S
          .Validator<IntegerValidator<int>>(
             &ms.mFreqOfs, NumValidatorStyle::DEFAULT, FreqOfs.min, FreqOfs.max)
          .AddTextBox(XXO("Wah Frequency Offse&t (%):"), L"", 12);
-      BindTo(*mFreqOfsT, wxEVT_TEXT, &Validator::OnFreqOffText);
+      BindTo(*mFreqOfsT, wxEVT_TEXT, &Editor::OnFreqOffText);
 
       mFreqOfsS = S
          .Name(XO("Wah frequency offset in percent"))
          .Style(wxSL_HORIZONTAL)
          .MinSize( { 100, -1 } )
          .AddSlider( {}, FreqOfs.def * FreqOfs.scale, FreqOfs.max * FreqOfs.scale, FreqOfs.min * FreqOfs.scale);
-      BindTo(*mFreqOfsS, wxEVT_SLIDER, &Validator::OnFreqOffSlider);
+      BindTo(*mFreqOfsS, wxEVT_SLIDER, &Editor::OnFreqOffSlider);
 
       mOutGainT = S
          .Validator<FloatingPointValidator<double>>(
             1, &ms.mOutGain, NumValidatorStyle::DEFAULT, OutGain.min, OutGain.max)
          .AddTextBox(XXO("&Output gain (dB):"), L"", 12);
-      BindTo(*mOutGainT, wxEVT_TEXT, &Validator::OnGainText);
+      BindTo(*mOutGainT, wxEVT_TEXT, &Editor::OnGainText);
 
       mOutGainS = S
          .Name(XO("Output gain (dB)"))
          .Style(wxSL_HORIZONTAL)
          .MinSize( { 100, -1 } )
          .AddSlider( {}, OutGain.def * OutGain.scale, OutGain.max * OutGain.scale, OutGain.min * OutGain.scale);
-      BindTo(*mOutGainS, wxEVT_SLIDER, &Validator::OnGainSlider);
+      BindTo(*mOutGainS, wxEVT_SLIDER, &Editor::OnGainSlider);
    }
    S.EndMultiColumn();
 }
 
-bool EffectWahwah::Validator::UpdateUI()
+bool EffectWahwah::Editor::UpdateUI()
 {
    // get the settings from the MessageBuffer and write them to our local copy
    const auto& settings = mAccess.Get();
@@ -473,7 +472,7 @@ unsigned EffectWahwah::Instance::GetAudioInCount() const
    return 1;
 }
 
-void EffectWahwah::Validator::OnFreqSlider(wxCommandEvent& evt)
+void EffectWahwah::Editor::OnFreqSlider(wxCommandEvent& evt)
 {
    auto& ms = mSettings;
 
@@ -482,9 +481,10 @@ void EffectWahwah::Validator::OnFreqSlider(wxCommandEvent& evt)
 
    EnableApplyFromValidate();
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnPhaseSlider(wxCommandEvent& evt)
+void EffectWahwah::Editor::OnPhaseSlider(wxCommandEvent& evt)
 {
    auto& ms = mSettings;
 
@@ -496,9 +496,10 @@ void EffectWahwah::Validator::OnPhaseSlider(wxCommandEvent& evt)
 
    EnableApplyFromValidate();
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnDepthSlider(wxCommandEvent& evt)
+void EffectWahwah::Editor::OnDepthSlider(wxCommandEvent& evt)
 {
    auto& ms = mSettings;
 
@@ -507,9 +508,10 @@ void EffectWahwah::Validator::OnDepthSlider(wxCommandEvent& evt)
 
    EnableApplyFromValidate();
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnResonanceSlider(wxCommandEvent& evt)
+void EffectWahwah::Editor::OnResonanceSlider(wxCommandEvent& evt)
 {
    auto& ms = mSettings;
 
@@ -518,9 +520,10 @@ void EffectWahwah::Validator::OnResonanceSlider(wxCommandEvent& evt)
 
    EnableApplyFromValidate();
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnFreqOffSlider(wxCommandEvent& evt)
+void EffectWahwah::Editor::OnFreqOffSlider(wxCommandEvent& evt)
 {
    auto& ms = mSettings;
 
@@ -529,9 +532,10 @@ void EffectWahwah::Validator::OnFreqOffSlider(wxCommandEvent& evt)
 
    EnableApplyFromValidate();
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnGainSlider(wxCommandEvent& evt)
+void EffectWahwah::Editor::OnGainSlider(wxCommandEvent& evt)
 {
    auto& ms = mSettings;
 
@@ -540,9 +544,10 @@ void EffectWahwah::Validator::OnGainSlider(wxCommandEvent& evt)
 
    EnableApplyFromValidate();
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnFreqText(wxCommandEvent& WXUNUSED(evt))
+void EffectWahwah::Editor::OnFreqText(wxCommandEvent& WXUNUSED(evt))
 {
    auto& ms = mSettings;
 
@@ -553,9 +558,10 @@ void EffectWahwah::Validator::OnFreqText(wxCommandEvent& WXUNUSED(evt))
 
    mFreqS->SetValue((int)(ms.mFreq * Freq.scale));
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnPhaseText(wxCommandEvent& WXUNUSED(evt))
+void EffectWahwah::Editor::OnPhaseText(wxCommandEvent& WXUNUSED(evt))
 {
    auto& ms = mSettings;
 
@@ -566,9 +572,10 @@ void EffectWahwah::Validator::OnPhaseText(wxCommandEvent& WXUNUSED(evt))
 
    mPhaseS->SetValue((int)(ms.mPhase * Phase.scale));
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnDepthText(wxCommandEvent& WXUNUSED(evt))
+void EffectWahwah::Editor::OnDepthText(wxCommandEvent& WXUNUSED(evt))
 {
    auto& ms = mSettings;
 
@@ -579,9 +586,10 @@ void EffectWahwah::Validator::OnDepthText(wxCommandEvent& WXUNUSED(evt))
 
    mDepthS->SetValue((int)(ms.mDepth * Depth.scale));
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnResonanceText(wxCommandEvent& WXUNUSED(evt))
+void EffectWahwah::Editor::OnResonanceText(wxCommandEvent& WXUNUSED(evt))
 {
    auto& ms = mSettings;
 
@@ -592,9 +600,10 @@ void EffectWahwah::Validator::OnResonanceText(wxCommandEvent& WXUNUSED(evt))
 
    mResS->SetValue((int)(ms.mRes * Res.scale));
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnFreqOffText(wxCommandEvent& WXUNUSED(evt))
+void EffectWahwah::Editor::OnFreqOffText(wxCommandEvent& WXUNUSED(evt))
 {
    auto& ms = mSettings;
 
@@ -605,9 +614,10 @@ void EffectWahwah::Validator::OnFreqOffText(wxCommandEvent& WXUNUSED(evt))
 
    mFreqOfsS->SetValue((int)(ms.mFreqOfs * FreqOfs.scale));
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
 
-void EffectWahwah::Validator::OnGainText(wxCommandEvent& WXUNUSED(evt))
+void EffectWahwah::Editor::OnGainText(wxCommandEvent& WXUNUSED(evt))
 {
    auto& ms = mSettings;
 
@@ -618,4 +628,5 @@ void EffectWahwah::Validator::OnGainText(wxCommandEvent& WXUNUSED(evt))
 
    mOutGainS->SetValue((int)(ms.mOutGain * OutGain.scale));
    ValidateUI();
+   Publish(EffectSettingChanged{});
 }
