@@ -10,38 +10,33 @@
   http://www.vamp-plugins.org/
 
 **********************************************************************/
-
-
-
 #if defined(USE_VAMP)
 #include "VampEffect.h"
+#include "../AnalysisTracks.h"
+#include "../EffectEditor.h"
 
 #include <vamp-hostsdk/Plugin.h>
 #include <vamp-hostsdk/PluginChannelAdapter.h>
 #include <vamp-hostsdk/PluginInputDomainAdapter.h>
 
 #include <wx/wxprec.h>
-#include <wx/button.h>
 #include <wx/checkbox.h>
 #include <wx/choice.h>
 #include <wx/combobox.h>
-#include <wx/sizer.h>
 #include <wx/slider.h>
 #include <wx/statbox.h>
 #include <wx/stattext.h>
 #include <wx/textctrl.h>
-#include <wx/tokenzr.h>
-#include <wx/intl.h>
 #include <wx/scrolwin.h>
 #include <wx/version.h>
 
 
-#include "../../ShuttleGui.h"
+#include "ShuttleGui.h"
 #include "../../widgets/valnum.h"
-#include "../../widgets/AudacityMessageBox.h"
+#include "AudacityMessageBox.h"
 
 #include "../../LabelTrack.h"
-#include "../../WaveTrack.h"
+#include "WaveTrack.h"
 
 enum
 {
@@ -299,33 +294,13 @@ bool VampEffect::Init()
 {
    mRate = 0.0;
 
-   // PRL: this loop checked that channels of a track have the same rate,
-   // but there was no check that all tracks have one rate, and only the first
+   // PRL: There is no check that all tracks have one rate, and only the first
    // is remembered in mRate.  Is that correct?
 
-   for (auto leader : inputTracks()->Leaders<const WaveTrack>()) {
-      auto channelGroup = TrackList::Channels( leader );
-      auto rate = (*channelGroup.first++) -> GetRate();
-      for(auto channel : channelGroup) {
-         if (rate != channel->GetRate())
-         // PRL:  Track rate might not match individual clip rates.
-         // So is this check not adequate?
-          {
-             // TODO: more-than-two-channels-message
-             Effect::MessageBox(
-                XO(
-"Sorry, Vamp Plug-ins cannot be run on stereo tracks where the individual channels of the track do not match.") );
-             return false;
-         }
-      }
-      if (mRate == 0.0)
-         mRate = rate;
-   }
-
-   if (mRate <= 0.0)
-   {
+   if (inputTracks()->empty())
       mRate = mProjectRate;
-   }
+   else
+      mRate = (*inputTracks()->Any<const WaveTrack>().begin())->GetRate();
 
    // The plugin must be reloaded to allow changing parameters
 
@@ -333,7 +308,8 @@ bool VampEffect::Init()
    mPlugin.reset(loader->loadPlugin(mKey, mRate, Vamp::HostExt::PluginLoader::ADAPT_ALL));
    if (!mPlugin)
    {
-      Effect::MessageBox( XO("Sorry, failed to load Vamp Plug-in.") );
+      EffectUIServices::DoMessageBox(*this,
+         XO("Sorry, failed to load Vamp Plug-in."));
       return false;
    }
 
@@ -361,24 +337,24 @@ bool VampEffect::Process(EffectInstance &, EffectSettings &)
       multiple = true;
    }
 
-   std::vector<std::shared_ptr<Effect::AddedAnalysisTrack>> addedTracks;
+   std::vector<std::shared_ptr<AddedAnalysisTrack>> addedTracks;
 
-   for (auto leader : inputTracks()->Leaders<const WaveTrack>())
+   for (auto leader : inputTracks()->Any<const WaveTrack>())
    {
-      auto channelGroup = TrackList::Channels(leader);
+      auto channelGroup = leader->Channels();
       auto left = *channelGroup.first++;
 
       unsigned channels = 1;
 
       // channelGroup now contains all but the first channel
-      const WaveTrack *right =
+      const auto right =
          channelGroup.size() ? *channelGroup.first++ : nullptr;
       if (right)
          channels = 2;
 
       sampleCount start = 0;
       sampleCount len = 0;
-      GetBounds(*left, right, &start, &len);
+      GetBounds(*leader, &start, &len);
 
       // TODO: more-than-two-channels
 
@@ -426,16 +402,16 @@ bool VampEffect::Process(EffectInstance &, EffectSettings &)
       {
          if (!mPlugin->initialise(channels, step, block))
          {
-            Effect::MessageBox(
-               XO("Sorry, Vamp Plug-in failed to initialize.") );
+            EffectUIServices::DoMessageBox(*this,
+               XO("Sorry, Vamp Plug-in failed to initialize."));
             return false;
          }
       }
 
       const auto effectName = GetSymbol().Translation();
-      addedTracks.push_back(AddAnalysisTrack(
+      addedTracks.push_back(AddAnalysisTrack(*this,
          multiple
-         ? wxString::Format( _("%s: %s"), left->GetName(), effectName )
+         ? wxString::Format( _("%s: %s"), leader->GetName(), effectName )
          : effectName
       ));
       LabelTrack *ltrack = addedTracks.back()->get();
@@ -451,14 +427,10 @@ bool VampEffect::Process(EffectInstance &, EffectSettings &)
          const auto request = limitSampleBufferSize( block, len );
 
          if (left)
-         {
             left->GetFloats(data[0].get(), pos, request);
-         }
 
          if (right)
-         {
             right->GetFloats(data[1].get(), pos, request);
-         }
 
          if (request < block)
          {
@@ -526,10 +498,11 @@ bool VampEffect::Process(EffectInstance &, EffectSettings &)
    return true;
 }
 
-std::unique_ptr<EffectUIValidator> VampEffect::PopulateOrExchange(
+std::unique_ptr<EffectEditor> VampEffect::PopulateOrExchange(
    ShuttleGui & S, EffectInstance &, EffectSettingsAccess &,
    const EffectOutputs *)
 {
+   mUIParent = S.GetParent();
    Vamp::Plugin::ProgramList programs = mPlugin->getPrograms();
 
    mParameters = mPlugin->getParameterDescriptors();
