@@ -25,19 +25,19 @@ and sample size to help you importing data of an unknown format.
 #include "ImportRaw.h"
 
 #include "ImportPlugin.h"
+#include "ImportUtils.h"
 
 #include "AudioIOBase.h"
-#include "../FileFormats.h"
+#include "FileFormats.h"
 #include "Prefs.h"
 #include "ProjectRate.h"
-#include "../SelectFile.h"
-#include "../ShuttleGui.h"
+#include "SelectFile.h"
+#include "ShuttleGui.h"
 #include "UserException.h"
-#include "../WaveTrack.h"
-#include "../widgets/ProgressDialog.h"
+#include "WaveTrack.h"
+#include "ProgressDialog.h"
 
 #include <cmath>
-#include <cstdio>
 #include <stdint.h>
 #include <vector>
 
@@ -46,13 +46,9 @@ and sample size to help you importing data of an unknown format.
 #include <wx/button.h>
 #include <wx/choice.h>
 #include <wx/combobox.h>
-#include <wx/filename.h>
-#include <wx/intl.h>
 #include <wx/panel.h>
-#include <wx/sizer.h>
 #include <wx/stattext.h>
 #include <wx/textctrl.h>
-#include <wx/timer.h>
 
 // #include "RawAudioGuess.h"
 #include "FormatClassifier.h"
@@ -112,7 +108,7 @@ void ImportRaw(const AudacityProject &project, wxWindow *parent, const wxString
 {
    outTracks.clear();
 
-   TrackHolders results;
+   TrackListHolder trackList;
    auto updateResult = ProgressResult::Success;
 
    {
@@ -125,7 +121,7 @@ void ImportRaw(const AudacityProject &project, wxWindow *parent, const wxString
       if (!dlog.GetReturnCode())
          return;
 
-      int encoding = dlog.mEncoding;
+      const int encoding = dlog.mEncoding;
       unsigned numChannels = dlog.mChannels;
       double rate = dlog.mRate;
       sf_count_t offset = (sf_count_t)dlog.mOffset;
@@ -179,21 +175,12 @@ void ImportRaw(const AudacityProject &project, wxWindow *parent, const wxString
       // the quality of the original file.
       //
 
-      auto format = ImportFileHandle::ChooseFormat(
+      const auto format = ImportUtils::ChooseFormat(
          sf_subtype_to_effective_format(encoding));
 
-      results.resize(1);
-      auto &channels = results[0];
-      channels.resize(numChannels);
+      trackList = trackFactory->Create(numChannels, format, rate);
 
-      {
-         // iter not used outside this scope.
-         auto iter = channels.begin();
-         for (decltype(numChannels) c = 0; c < numChannels; ++iter, ++c)
-            *iter = trackFactory->Create(format, rate);
-      }
-      const auto firstChannel = channels.begin()->get();
-      auto maxBlockSize = firstChannel->GetMaxBlockSize();
+      const auto maxBlockSize = (*trackList->Any<WaveTrack>().begin())->GetMaxBlockSize();
 
       SampleBuffer srcbuffer(maxBlockSize * numChannels, format);
       SampleBuffer buffer(maxBlockSize, format);
@@ -230,21 +217,25 @@ void ImportRaw(const AudacityProject &project, wxWindow *parent, const wxString
          }
 
          if (block) {
-            auto iter = channels.begin();
-            for(decltype(numChannels) c = 0; c < numChannels; ++iter, ++c) {
-               if (format==int16Sample) {
-                  for(decltype(block) j=0; j<block; j++)
+            size_t c = 0;
+            ImportUtils::ForEachChannel(*trackList, [&](auto& channel)
+            {
+               if (format == int16Sample) {
+                  for (size_t j = 0; j < block; ++j)
                      ((short *)buffer.ptr())[j] =
-                     ((short *)srcbuffer.ptr())[numChannels*j+c];
+                     ((short *)srcbuffer.ptr())[numChannels * j + c];
                }
                else {
-                  for(decltype(block) j=0; j<block; j++)
+                  for (size_t j = 0; j < block; ++j)
                      ((float *)buffer.ptr())[j] =
-                     ((float *)srcbuffer.ptr())[numChannels*j+c];
+                     ((float *)srcbuffer.ptr())[numChannels * j + c];
                }
 
-               iter->get()->Append(buffer.ptr(), (format == int16Sample)?int16Sample:floatSample, block);
-            }
+               channel.AppendBuffer(buffer.ptr(),
+                  ((format == int16Sample) ? int16Sample : floatSample), block,
+                  1, sf_subtype_to_effective_format(encoding));
+               ++c;
+            });
             framescompleted += block;
          }
 
@@ -261,11 +252,7 @@ void ImportRaw(const AudacityProject &project, wxWindow *parent, const wxString
    if (updateResult == ProgressResult::Failed || updateResult == ProgressResult::Cancelled)
       throw UserException{};
 
-   if (!results.empty() && !results[0].empty()) {
-      for (const auto &channel : results[0])
-         channel->Flush();
-      outTracks.swap(results);
-   }
+   ImportUtils::FinalizeImport(outTracks, trackList);
 }
 
 
