@@ -8,11 +8,13 @@ class PluginDescriptor;
 
 namespace MenuHelper
 {
+using Group = MenuTable::GroupItemBase;
 
 /// The effects come from a plug in list
 /// This code iterates through the list, adding effects into
 /// the menu.
-MenuTable::BaseItemPtrs PopulateEffectsMenu(
+void PopulateEffectsMenu(
+   Group &menuItems,
    EffectType type,
    CommandFlag batchflags,
    const wxString& groupby,
