@@ -21,7 +21,7 @@
 #include <wx/defs.h>
 
 #include "Prefs.h"
-#include "../ShuttleGui.h"
+#include "ShuttleGui.h"
 
 ImportExportPrefs::ImportExportPrefs(wxWindow * parent, wxWindowID winid)
 :   PrefsPanel(parent, winid, XO("Import / Export"))
@@ -60,21 +60,6 @@ void ImportExportPrefs::Populate()
    // ----------------------- End of main section --------------
 }
 
-EnumSetting< bool > ImportExportPrefs::ExportDownMixSetting{
-   wxT("/FileFormats/ExportDownMixChoice"),
-   {
-      EnumValueSymbol{ wxT("MixDown"), XXO("&Mix down to Stereo or Mono") },
-      EnumValueSymbol{ wxT("Custom"), XXO("&Use Advanced Mixing Options") },
-   },
-   0, // true
-
-   // for migrating old preferences:
-   {
-      true, false,
-   },
-   wxT("/FileFormats/ExportDownMix"),
-};
-
 EnumSetting< bool > ImportExportPrefs::LabelStyleSetting{
    wxT("/FileFormats/LabelStyleChoice"),
    {
@@ -92,6 +77,7 @@ EnumSetting< bool > ImportExportPrefs::AllegroStyleSetting{
    wxT("/FileFormats/AllegroStyleChoice"),
    {
       EnumValueSymbol{ wxT("Seconds"), XXO("&Seconds") },
+      /* i18n-hint: The music theory "beat" */
       EnumValueSymbol{ wxT("Beats"), XXO("&Beats") },
    },
    0, // true
@@ -108,31 +94,6 @@ void ImportExportPrefs::PopulateOrExchange(ShuttleGui & S)
    S.SetBorder(2);
    S.StartScroller();
 
-   S.StartStatic(XO("When exporting tracks to an audio file"));
-   {
-      // Bug 2692: Place button group in panel so tabbing will work and,
-      // on the Mac, VoiceOver will announce as radio buttons.
-      S.StartPanel();
-      {
-         S.StartRadioButtonGroup(ImportExportPrefs::ExportDownMixSetting);
-         {
-            S.TieRadioButton();
-            S.TieRadioButton();
-         }
-         S.EndRadioButtonGroup();
-      }
-      S.EndPanel();
-
-      S.TieCheckBox(XXO("S&how Metadata Tags editor before export"),
-                    {wxT("/AudioFiles/ShowId3Dialog"),
-                     true});
-      /* i18n-hint 'blank space' is space on the tracks with no audio in it*/
-      S.TieCheckBox(XXO("&Ignore blank space at the beginning"),
-                    {wxT("/AudioFiles/SkipSilenceAtBeginning"),
-                     false});
-   }
-   S.EndStatic();
-
    S.StartStatic(XO("Exported Label Style:"));
    {
       // Bug 2692: Place button group in panel so tabbing will work and,
@@ -175,7 +136,7 @@ bool ImportExportPrefs::Commit()
 {
    ShuttleGui S(this, eIsSavingToPrefs);
    PopulateOrExchange(S);
-
+   
    return true;
 }
 
