@@ -23,7 +23,7 @@ with names like mod-script-pipe that add NEW features.
 #include <wx/defs.h>
 #include <wx/filename.h>
 
-#include "../ShuttleGui.h"
+#include "ShuttleGui.h"
 
 #include "Prefs.h"
 #include "ModuleSettings.h"
@@ -57,9 +57,6 @@ ManualPageID ModulePrefs::HelpPageName()
 }
 
 void ModulePrefs::GetAllModuleStatuses(){
-   wxString str;
-   long dummy;
-
    // Modules could for example be:
    //    mod-script-pipe
    //    mod-nyq-bench
@@ -75,26 +72,24 @@ void ModulePrefs::GetAllModuleStatuses(){
 
    // Iterate through all Modules listed in prefs.
    // Get their names and values.
-   gPrefs->SetPath( wxT("Module/") );
-   bool bCont = gPrefs->GetFirstEntry(str, dummy);
-   while ( bCont ) {
+   auto moduleGroup = gPrefs->BeginGroup("Module");
+   for(const auto& key : gPrefs->GetChildKeys())
+   {
       int iStatus;
-      gPrefs->Read( str, &iStatus, kModuleDisabled );
+      gPrefs->Read( key, &iStatus, static_cast<decltype(iStatus)>(kModuleDisabled) );
       wxString fname;
-      gPrefs->Read( wxString( wxT("/ModulePath/") ) + str, &fname, wxEmptyString );
+      gPrefs->Read(wxT("/ModulePath/") + key, &fname, {} );
       if( !fname.empty() && wxFileExists( fname ) ){
          if( iStatus > kModuleNew ){
             iStatus = kModuleNew;
-            gPrefs->Write( str, iStatus );
+            gPrefs->Write( key, iStatus );
          }
          //wxLogDebug( wxT("Entry: %s Value: %i"), str, iStatus );
-         mModules.push_back( str );
+         mModules.push_back( key );
          mStatuses.push_back( iStatus );
          mPaths.push_back( fname );
       }
-      bCont = gPrefs->GetNextEntry(str, dummy);
    }
-   gPrefs->SetPath( wxT("") );
 }
 
 void ModulePrefs::Populate()
