@@ -22,14 +22,9 @@
 #include <wx/app.h>
 #include <wx/setup.h> // for wxUSE_* macros
 #include <wx/defs.h>
-#include <wx/button.h>
-#include <wx/dialog.h>
-#include <wx/event.h>
 #include <wx/font.h>
 #include <wx/gdicmn.h>
-#include <wx/intl.h>
 #include <wx/listbox.h>
-#include <wx/sizer.h>
 
 #include <wx/listbook.h>
 
@@ -39,15 +34,15 @@
 #include "AudioIOBase.h"
 #include "Prefs.h"
 #include "ProjectWindows.h"
-#include "../ShuttleGui.h"
+#include "ShuttleGui.h"
 #include "../commands/CommandManager.h"
 
 #include "PrefsPanel.h"
 
-#include "../widgets/HelpSystem.h"
+#include "HelpSystem.h"
 
 #if wxUSE_ACCESSIBILITY
-#include "../widgets/WindowAccessible.h"
+#include "WindowAccessible.h"
 #endif
 
 
@@ -490,12 +485,17 @@ PrefsDialog::PrefsDialog(
                   const auto &node = *it;
                   const auto &factory = node.factory;
                   wxWindow *const w = factory(mCategories, wxID_ANY, pProject);
-                  if (stack.empty())
+                  node.enabled = (w != nullptr);
+                  if (stack.empty()) {
                      // Parameters are: AddPage(page, name, IsSelected, imageId).
-                     mCategories->AddPage(w, w->GetName(), false, 0);
+                     if (w)
+                        mCategories->AddPage(w, w->GetName(), false, 0);
+                  }
                   else {
                      IntPair &top = *stack.rbegin();
-                     mCategories->InsertSubPage(top.first, w, w->GetName(), false, 0);
+                     if (w)
+                        mCategories->InsertSubPage(top.first,
+                           w, w->GetName(), false, 0);
                      if (--top.second == 0) {
                         // Expand all nodes before the layout calculation
                         mCategories->ExpandNode(top.first, true);
@@ -518,18 +518,21 @@ PrefsDialog::PrefsDialog(
          const auto &node = factories[0];
          const auto &factory = node.factory;
          mUniquePage = factory(S.GetParent(), wxID_ANY, pProject);
-         wxWindow * uniquePageWindow = S.Prop(1)
-            .Position(wxEXPAND)
-            .AddWindow(mUniquePage);
-         // We're not in the wxTreebook, so add the accelerator here
-         wxAcceleratorEntry entries[1];
+         node.enabled = (mUniquePage != nullptr);
+         if (mUniquePage) {
+            wxWindow * uniquePageWindow = S.Prop(1)
+               .Position(wxEXPAND)
+               .AddWindow(mUniquePage);
+            // We're not in the wxTreebook, so add the accelerator here
+            wxAcceleratorEntry entries[1];
 #if defined(__WXMAC__)
-         // Is there a standard shortcut on Mac?
+            // Is there a standard shortcut on Mac?
 #else
-         entries[0].Set(wxACCEL_NORMAL, (int) WXK_F1, wxID_HELP);
+            entries[0].Set(wxACCEL_NORMAL, (int) WXK_F1, wxID_HELP);
 #endif
-         wxAcceleratorTable accel(1, entries);
-         uniquePageWindow->SetAcceleratorTable(accel);
+            wxAcceleratorTable accel(1, entries);
+            uniquePageWindow->SetAcceleratorTable(accel);
+         }
       }
    }
    S.EndVerticalLay();
@@ -557,8 +560,10 @@ PrefsDialog::PrefsDialog(
    {
       int iPage = 0;
       for (auto it = factories.begin(), end = factories.end();
-         it != end; ++it, ++iPage)
-         mCategories->ExpandNode(iPage, it->expanded);
+         it != end; ++it) {
+         if (it->enabled)
+            mCategories->ExpandNode(iPage++, it->expanded);
+      }
    }
 
    // This ASSERT was originally used to limit us to 800 x 600.
@@ -610,7 +615,7 @@ int PrefsDialog::ShowModal()
          selected = 0;  // clamp to available range of tabs
       mCategories->SetSelection(selected);
    }
-   else {
+   else if (mUniquePage) {
       auto Temp = mTitlePrefix;
       Temp.Join( Verbatim( mUniquePage->GetLabel() ), wxT(" ") );
       SetTitle(Temp);
@@ -629,7 +634,7 @@ void PrefsDialog::OnCancel(wxCommandEvent & WXUNUSED(event))
          ((PrefsPanel *)mCategories->GetPage(i))->Cancel();
       }
    }
-   else
+   else if (mUniquePage)
       mUniquePage->Cancel();
 
    // Remember modified dialog size, even if cancelling.
@@ -648,21 +653,21 @@ PrefsPanel * PrefsDialog::GetCurrentPanel()
    if( mCategories) 
       return static_cast<PrefsPanel*>(mCategories->GetCurrentPage());
    else
-   {
-      wxASSERT( mUniquePage );
       return mUniquePage;
-   }
 }
 
 void PrefsDialog::OnPreview(wxCommandEvent & WXUNUSED(event))
 {
-   GetCurrentPanel()->Preview();
+   if (const auto pPanel = GetCurrentPanel())
+      pPanel->Preview();
 }
 
 void PrefsDialog::OnHelp(wxCommandEvent & WXUNUSED(event))
 {
-   const auto &page = GetCurrentPanel()->HelpPageName();
-   HelpSystem::ShowHelp(this, page, true);
+   if (const auto pPanel = GetCurrentPanel()) {
+      const auto &page = pPanel->HelpPageName();
+      HelpSystem::ShowHelp(this, page, true);
+   }
 }
 
 void PrefsDialog::ShuttleAll( ShuttleGui & S)
@@ -678,7 +683,8 @@ void PrefsDialog::ShuttleAll( ShuttleGui & S)
    else
    {
       S.ResetId();
-      mUniquePage->PopulateOrExchange( S );
+      if (mUniquePage)
+         mUniquePage->PopulateOrExchange( S );
    }
 }
 
@@ -706,7 +712,7 @@ void PrefsDialog::OnOK(wxCommandEvent & WXUNUSED(event))
          }
       }
    }
-   else {
+   else if (mUniquePage) {
       if (!mUniquePage->Validate())
          return;
    }
@@ -724,7 +730,7 @@ void PrefsDialog::OnOK(wxCommandEvent & WXUNUSED(event))
          panel->Commit();
       }
    }
-   else {
+   else if (mUniquePage) {
       mUniquePage->Preview();
       mUniquePage->Commit();
    }
@@ -840,8 +846,10 @@ void PrefsDialog::RecordExpansionState()
    {
       int iPage = 0;
       for (auto it = mFactories.begin(), end = mFactories.end();
-         it != end; ++it, ++iPage)
-         it->expanded = mCategories->IsNodeExpanded(iPage);
+         it != end; ++it) {
+         if (it->enabled)
+            it->expanded = mCategories->IsNodeExpanded(iPage++);
+      }
    }
    else
       mFactories[0].expanded = true;
