@@ -20,7 +20,7 @@
 
 
 #include "RecordingPrefs.h"
-#include "AudioIOBase.h"
+#include "AudioIO.h"
 
 #include <wx/defs.h>
 #include <wx/textctrl.h>
@@ -28,7 +28,7 @@
 
 #include "Decibels.h"
 #include "Prefs.h"
-#include "../ShuttleGui.h"
+#include "ShuttleGui.h"
 
 using std::min;
 
@@ -87,7 +87,7 @@ void RecordingPrefs::PopulateOrExchange(ShuttleGui & S)
    S.StartStatic(XO("Options"));
    {
       // Start wording of options with a verb, if possible.
-      S.TieCheckBox(XXO("Play &other tracks while recording (overdub)"),
+      S.TieCheckBox(XXO("Hear &other tracks while recording (overdub)"),
                      {wxT("/AudioIO/Duplex"),
 #ifdef EXPERIMENTAL_DA
                       false
@@ -103,7 +103,7 @@ void RecordingPrefs::PopulateOrExchange(ShuttleGui & S)
                     {wxT("/AudioIO/Playthrough"),
                      false});
 #endif
-      S.TieCheckBox(XXO("&Software playthrough of input"),
+      S.TieCheckBox(XXO("Audible input &monitoring"),
                     {wxT("/AudioIO/SWPlaythrough"),
                      false});
 #if !defined(__WXMAC__)
@@ -125,9 +125,7 @@ void RecordingPrefs::PopulateOrExchange(ShuttleGui & S)
 
    S.StartStatic(XO("Sound Activated Recording"));
    {
-      S.TieCheckBox(XXO("&Enable"),
-                    {wxT("/AudioIO/SoundActivatedRecord"),
-                     false});
+      S.TieCheckBox(XXO("&Enable"), SoundActivatedRecord);
 
       S.StartMultiColumn(2, wxEXPAND);
       {
@@ -271,6 +269,8 @@ bool RecordingPrefs::Commit()
    ShuttleGui S(this, eIsSavingToPrefs);
    PopulateOrExchange(S);
 
+   SoundActivatedRecord.Invalidate();
+
    if (AudioIOLatencyDuration.Read() < 0)
       AudioIOLatencyDuration.Reset();
 
