@@ -15,13 +15,13 @@ Paul Licameli
 
 
 #include "WaveformSettings.h"
+#include "WaveTrack.h"
 
 #include "Decibels.h"
 #include "GUIPrefs.h"
 #include "TracksPrefs.h"
 
 #include <algorithm>
-#include <wx/intl.h>
 
 #include "Prefs.h"
 
@@ -46,6 +46,24 @@ WaveformSettings::Globals
    return instance;
 }
 
+static const ChannelGroup::Attachments::RegisteredFactory
+key1{ [](auto &) {
+   return std::make_unique<WaveformSettings>(WaveformSettings::defaults()); } };
+
+WaveformSettings &WaveformSettings::Get(const WaveTrack &track)
+{
+   auto &mutTrack = const_cast<WaveTrack&>(track);
+   return mutTrack.GetGroupData().Attachments
+      ::Get<WaveformSettings>(key1);
+}
+
+void WaveformSettings::Set(
+   WaveTrack &track, std::unique_ptr<WaveformSettings> pSettings)
+{
+   track.GetGroupData().Attachments
+      ::Assign(key1, move(pSettings));
+}
+
 WaveformSettings::WaveformSettings()
 {
    LoadPrefs();
@@ -163,8 +181,9 @@ const EnumValueSymbols &WaveformSettings::GetScaleNames()
 {
    static const EnumValueSymbols result{
       // Keep in correspondence with ScaleTypeValues:
-      XO("Linear"),
-      XO("dB"),
+      { wxT("Linear"), XO("Linear (amp)") },
+      { wxT("dB"), XO("Logarithmic (dB)") },
+      { wxT("LinearDB"), XO("Linear (dB)") },
    };
    return result;
 }
@@ -172,3 +191,31 @@ const EnumValueSymbols &WaveformSettings::GetScaleNames()
 WaveformSettings::~WaveformSettings()
 {
 }
+
+auto WaveformSettings::Clone() const -> PointerType
+{
+   return std::make_unique<WaveformSettings>(*this);
+}
+
+static const ChannelGroup::Attachments::RegisteredFactory
+key2{ [](auto &) { return std::make_unique<WaveformScale>(); } };
+
+WaveformScale &WaveformScale::Get(const WaveTrack &track)
+{
+   auto &mutTrack = const_cast<WaveTrack&>(track);
+   return mutTrack.GetGroupData().Attachments
+      ::Get<WaveformScale>(key2);
+}
+
+WaveformScale::~WaveformScale() = default;
+
+auto WaveformScale::Clone() const -> PointerType
+{
+   return std::make_unique<WaveformScale>(*this);
+}
+
+int WaveformScale::ZeroLevelYCoordinate(wxRect rect) const
+{
+   return rect.GetTop() +
+      (int)((mDisplayMax / (mDisplayMax - mDisplayMin)) * rect.height);
+}
