@@ -27,8 +27,6 @@
 #include <wx/setup.h> // for wxUSE_* macros
 
 #ifndef WX_PRECOMP
-#include <wx/event.h>
-#include <wx/intl.h>
 #include <wx/tooltip.h>
 #endif
 
@@ -37,7 +35,7 @@
 #include "AllThemeResources.h"
 #include "Decibels.h"
 #include "ToolManager.h"
-#include "../ProjectAudioIO.h"
+#include "ProjectAudioIO.h"
 #include "../widgets/MeterPanel.h"
 
 #if wxUSE_ACCESSIBILITY
@@ -131,21 +129,28 @@ BEGIN_EVENT_TABLE( MeterToolBar, ToolBar )
    EVT_SIZE( MeterToolBar::OnSize )
 END_EVENT_TABLE()
 
+Identifier MeterToolBar::ID()
+{
+   return wxT("CombinedMeter");
+}
+
+Identifier MeterToolBar::PlayID()
+{
+   return wxT("PlayMeter");
+}
+
+Identifier MeterToolBar::RecordID()
+{
+   return wxT("RecordMeter");
+}
+
 //Standard constructor
-MeterToolBar::MeterToolBar(AudacityProject &project, int type)
-: ToolBar(project, type, XO("Combined Meter"), wxT("CombinedMeter"), true)
+MeterToolBar::MeterToolBar(AudacityProject &project,
+   unsigned whichMeters,
+   const TranslatableString &label, Identifier ID)
+: ToolBar(project, label, ID, true)
+, mWhichMeters{ whichMeters }
 {
-   if( mType == RecordMeterBarID ){
-      mWhichMeters = kWithRecordMeter;
-      mLabel = XO("Recording Meter");
-      mSection = wxT("RecordMeter");
-   } else if( mType == PlayMeterBarID ){
-      mWhichMeters = kWithPlayMeter;
-      mLabel = XO("Playback Meter");
-      mSection = wxT("PlayMeter");
-   } else {
-      mWhichMeters = kWithPlayMeter | kWithRecordMeter;
-   }
 }
 
 MeterToolBar::~MeterToolBar()
@@ -171,14 +176,19 @@ ConstMeterToolBars MeterToolBar::GetToolBars(const AudacityProject &project)
 MeterToolBar & MeterToolBar::Get(AudacityProject &project, bool forPlayMeterToolBar)
 {
    auto& toolManager = ToolManager::Get(project);
-   auto  toolBarID = forPlayMeterToolBar ? PlayMeterBarID : RecordMeterBarID;
-
+   const auto &toolBarID = forPlayMeterToolBar ? PlayID() : RecordID();
    return *static_cast<MeterToolBar*>(toolManager.GetToolBar(toolBarID));
 }
 
 const MeterToolBar & MeterToolBar::Get(const AudacityProject &project, bool forPlayMeterToolBar)
 {
-   return Get( const_cast<AudacityProject&>( project ), forPlayMeterToolBar );
+   return Get(const_cast<AudacityProject&>(project), forPlayMeterToolBar);
+}
+
+bool MeterToolBar::ShownByDefault() const
+{
+   // The combined meter hides by default
+   return mWhichMeters != (kWithPlayMeter|kWithRecordMeter);
 }
 
 void MeterToolBar::Create(wxWindow * parent)
@@ -489,20 +499,24 @@ void MeterToolBar::AdjustInputGain(int adj)
    mRecordMeter->UpdateSliderControl();
 }
 
-static RegisteredToolbarFactory factory1{ RecordMeterBarID,
+static RegisteredToolbarFactory factory1{
    []( AudacityProject &project ){
       return ToolBar::Holder{
-         safenew MeterToolBar{ project, RecordMeterBarID } }; }
+         safenew MeterToolBar{ project, kWithRecordMeter,
+            XO("Recording Meter"), MeterToolBar::RecordID() } }; }
 };
-static RegisteredToolbarFactory factory2{ PlayMeterBarID,
+static RegisteredToolbarFactory factory2{
    []( AudacityProject &project ){
       return ToolBar::Holder{
-         safenew MeterToolBar{ project, PlayMeterBarID } }; }
+         safenew MeterToolBar{ project, kWithPlayMeter,
+            XO("Playback Meter"), MeterToolBar::PlayID() } }; }
 };
-static RegisteredToolbarFactory factory3{ MeterBarID,
+static RegisteredToolbarFactory factory3{
    []( AudacityProject &project ){
       return ToolBar::Holder{
-         safenew MeterToolBar{ project, MeterBarID } }; }
+         safenew MeterToolBar{ project,
+            (kWithPlayMeter|kWithRecordMeter),
+            XO("Combined Meter"), MeterToolBar::ID() } }; }
 };
 
 #include "ToolManager.h"
@@ -511,20 +525,96 @@ namespace {
 AttachedToolBarMenuItem sAttachment1{
    /* i18n-hint: Clicking this menu item shows the toolbar
       with the recording level meters */
-   RecordMeterBarID, wxT("ShowRecordMeterTB"), XXO("&Recording Meter Toolbar"),
-   {}, { MeterBarID }
+   MeterToolBar::RecordID(),
+   wxT("ShowRecordMeterTB"), XXO("&Recording Meter Toolbar"),
+   {}, { MeterToolBar::ID() }
 };
 AttachedToolBarMenuItem sAttachment2{
    /* i18n-hint: Clicking this menu item shows the toolbar
       with the playback level meter */
-   PlayMeterBarID, wxT("ShowPlayMeterTB"), XXO("&Playback Meter Toolbar"),
-   {}, { MeterBarID }
+   MeterToolBar::PlayID(),
+   wxT("ShowPlayMeterTB"), XXO("&Playback Meter Toolbar"),
+   {}, { MeterToolBar::ID() }
 };
 //AttachedToolBarMenuItem sAttachment3{
 //   /* --i18nhint: Clicking this menu item shows the toolbar
 //      which has sound level meters */
-//   MeterBarID, wxT("ShowMeterTB"), XXO("Co&mbined Meter Toolbar"),
+//   MeterToolBar::ID(), wxT("ShowMeterTB"), XXO("Co&mbined Meter Toolbar"),
 //   { Registry::OrderingHint::After, "ShowPlayMeterTB" },
 //   { PlayMeterBarID, RecordMeterBarID }
 //};
 }
+
+// Now define other related menu items
+#include "../commands/CommandContext.h"
+
+namespace {
+void OnOutputGain(const CommandContext &context)
+{
+   auto &project = context.project;
+   auto &tb = MeterToolBar::Get( project, true );
+   tb.ShowOutputGainDialog();
+}
+
+void OnOutputGainInc(const CommandContext &context)
+{
+   auto &project = context.project;
+   auto &tb = MeterToolBar::Get( project, true );
+   tb.AdjustOutputGain(1);
+}
+
+void OnOutputGainDec(const CommandContext &context)
+{
+   auto &project = context.project;
+   auto &tb = MeterToolBar::Get( project, true );
+   tb.AdjustOutputGain(-1);
+}
+
+void OnInputGain(const CommandContext &context)
+{
+   auto &project = context.project;
+   auto &tb = MeterToolBar::Get( project, false );
+   tb.ShowInputGainDialog();
+}
+
+void OnInputGainInc(const CommandContext &context)
+{
+   auto &project = context.project;
+   auto &tb = MeterToolBar::Get( project, false );
+   tb.AdjustInputGain(1);
+}
+
+void OnInputGainDec(const CommandContext &context)
+{
+   auto &project = context.project;
+   auto &tb = MeterToolBar::Get( project, false );
+   tb.AdjustInputGain(-1);
+}
+   
+using namespace MenuTable;
+BaseItemSharedPtr ExtraMixerMenu()
+{
+   static BaseItemSharedPtr menu{
+   Menu( wxT("Mixer"), XXO("Mi&xer"),
+      Command( wxT("OutputGain"), XXO("Ad&just Playback Volume..."),
+         OnOutputGain, AlwaysEnabledFlag ),
+      Command( wxT("OutputGainInc"), XXO("&Increase Playback Volume"),
+         OnOutputGainInc, AlwaysEnabledFlag ),
+      Command( wxT("OutputGainDec"), XXO("&Decrease Playback Volume"),
+         OnOutputGainDec, AlwaysEnabledFlag ),
+      Command( wxT("InputGain"), XXO("Adj&ust Recording Volume..."),
+         OnInputGain, AlwaysEnabledFlag ),
+      Command( wxT("InputGainInc"), XXO("I&ncrease Recording Volume"),
+         OnInputGainInc, AlwaysEnabledFlag ),
+      Command( wxT("InputGainDec"), XXO("D&ecrease Recording Volume"),
+         OnInputGainDec, AlwaysEnabledFlag )
+   ) };
+   return menu;
+}
+
+AttachedItem sAttachment4{
+   wxT("Optional/Extra/Part1"),
+   Indirect(ExtraMixerMenu())
+};
+
+}
