@@ -23,10 +23,6 @@ class AUDACITY_DLL_API SelectionBarListener /* not final */ {
    SelectionBarListener(){};
    virtual ~SelectionBarListener(){};
 
-   virtual double AS_GetRate() = 0;
-   virtual void AS_SetRate(double rate) = 0;
-   virtual int AS_GetSnapTo() = 0;
-   virtual void AS_SetSnapTo(int snap) = 0;
    virtual const NumericFormatSymbol & AS_GetSelectionFormat() = 0;
    virtual void AS_SetSelectionFormat(const NumericFormatSymbol & format) = 0;
    virtual void AS_ModifySelection(double &start, double &end, bool done) = 0;
