@@ -1,14 +1,14 @@
 /**********************************************************************
- 
+
  Audacity: A Digital Audio Editor
- 
+
  TimeToolBar.cpp
- 
+
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
- 
+
  *//*******************************************************************/
 
 
@@ -19,7 +19,6 @@
 #include <wx/setup.h> // for wxUSE_* macros
 
 #ifndef WX_PRECOMP
-#include <wx/intl.h>
 #include <wx/sizer.h>
 #endif
 
@@ -27,13 +26,16 @@
 #include "SelectionBarListener.h"
 #include "ToolManager.h"
 
-#include "../AudioIO.h"
+#include "AudioIO.h"
 #include "Project.h"
-#include "../ProjectAudioIO.h"
+#include "ProjectAudioIO.h"
+#include "ProjectNumericFormats.h"
 #include "ProjectRate.h"
-#include "../ProjectSettings.h"
+#include "ProjectTimeSignature.h"
 #include "ViewInfo.h"
 
+#include "NumericConverterFormats.h"
+
 IMPLEMENT_CLASS(TimeToolBar, ToolBar);
 
 // Having a fixed ID for the Audio Position is helpful for
@@ -49,23 +51,30 @@ BEGIN_EVENT_TABLE(TimeToolBar, ToolBar)
    EVT_IDLE(TimeToolBar::OnIdle)
 END_EVENT_TABLE()
 
+Identifier TimeToolBar::ID()
+{
+   return wxT("Time");
+}
+
 TimeToolBar::TimeToolBar(AudacityProject &project)
-:  ToolBar(project, TimeBarID, XO("Time"), wxT("Time"), true),
-   mListener(NULL),
-   mAudioTime(NULL)
+:  ToolBar(project, XO("Time"), ID(), true),
+   mListener(NULL), mAudioTime(NULL)
 {
-   mSubscription =
-      ProjectRate::Get(project).Subscribe(*this, &TimeToolBar::OnRateChanged);
 }
 
 TimeToolBar::~TimeToolBar()
 {
 }
 
+ToolBar::DockID TimeToolBar::DefaultDockID() const
+{
+   return BotDockID;
+}
+
 TimeToolBar &TimeToolBar::Get(AudacityProject &project)
 {
    auto &toolManager = ToolManager::Get(project);
-   return *static_cast<TimeToolBar*>(toolManager.GetToolBar(TimeBarID));
+   return *static_cast<TimeToolBar*>(toolManager.GetToolBar(ID()));
 }
 
 const TimeToolBar &TimeToolBar::Get(const AudacityProject &project)
@@ -75,16 +84,13 @@ const TimeToolBar &TimeToolBar::Get(const AudacityProject &project)
 
 void TimeToolBar::Populate()
 {
-   const auto &settings = ProjectSettings::Get(mProject);
-
-   // Get the default sample rate
-   auto rate = ProjectRate::Get(mProject).GetRate();
+   const auto &formats = ProjectNumericFormats::Get(mProject);
 
    // Get the default time format
-   auto format = settings.GetAudioTimeFormat();
+   auto format = formats.GetAudioTimeFormat();
 
    // Create the read-only time control
-   mAudioTime = safenew NumericTextCtrl(this, AudioPositionID, NumericConverter::TIME, format, 0.0, rate);
+   mAudioTime = safenew NumericTextCtrl(FormatterContext::ProjectContext(mProject), this, AudioPositionID, NumericConverterType_TIME(), format, 0.0);
    mAudioTime->SetName(XO("Audio Position"));
    mAudioTime->SetReadOnly(true);
 
@@ -101,7 +107,13 @@ void TimeToolBar::Populate()
    mSettingInitialSize = true;
 
    // Establish initial resizing limits
-//   SetResizingLimits();
+   //   SetResizingLimits();
+   mFormatChangedToFitValueSubscription = mAudioTime->Subscribe(
+      [this](auto)
+      {
+         wxSizeEvent e;
+         OnSize(e);
+      });
 }
 
 void TimeToolBar::UpdatePrefs()
@@ -109,7 +121,7 @@ void TimeToolBar::UpdatePrefs()
    // Since the language may have changed, we need to force an update to accommodate
    // different length text
    wxCommandEvent e;
-   e.SetInt(mAudioTime->GetFormatIndex());
+   e.SetString(mAudioTime->GetFormatName().Internal());
    OnUpdate(e);
 
    // Language may have changed so reset label
@@ -121,12 +133,12 @@ void TimeToolBar::UpdatePrefs()
 
 void TimeToolBar::SetToDefaultSize()
 {
-   // Reset 
+   // Reset
    SetMaxSize(wxDefaultSize);
    SetMinSize(wxDefaultSize);
 
    // Set the default time format
-   SetAudioTimeFormat(NumericConverter::HoursMinsSecondsFormat());
+   SetAudioTimeFormat(NumericConverterFormats::HoursMinsSecondsFormat());
 
    // Set the default size
    SetSize(GetInitialWidth(), 48);
@@ -188,7 +200,7 @@ void TimeToolBar::SetListener(TimeToolBarListener *l)
    // OnUpdate() will not be called and need it to set the initial size.
    if (mSettingInitialSize) {
       wxCommandEvent e;
-      e.SetInt(mAudioTime->GetFormatIndex());
+      e.SetString(mAudioTime->GetFormatName().Internal());
       OnUpdate(e);
    }
 }
@@ -196,10 +208,10 @@ void TimeToolBar::SetListener(TimeToolBarListener *l)
 void TimeToolBar::SetAudioTimeFormat(const NumericFormatSymbol & format)
 {
    // Set the format if it's different from previous
-   if (mAudioTime->SetFormatString(mAudioTime->GetBuiltinFormat(format))) {
+   if (mAudioTime->SetFormatName(format)) {
       // Simulate an update since the format has changed.
       wxCommandEvent e;
-      e.SetInt(mAudioTime->GetFormatIndex());
+      e.SetString(format.Internal());
       OnUpdate(e);
    }
 }
@@ -263,28 +275,23 @@ void TimeToolBar::SetResizingLimits()
    SetMaxSize(maxSize);
 }
 
-// Called when the project rate changes
-void TimeToolBar::OnRateChanged(double rate)
-{
-   if (mAudioTime)
-      mAudioTime->SetSampleRate(rate);
-}
-
 // Called when the format drop downs is changed.
 // This causes recreation of the toolbar contents.
 void TimeToolBar::OnUpdate(wxCommandEvent &evt)
 {
    evt.Skip(false);
 
+   ToolBar::ReCreateButtons();
+
    // Reset to allow resizing to work
    SetMinSize(wxDefaultSize);
    SetMaxSize(wxDefaultSize);
 
    // Save format name before recreating the controls so they resize properly
    if (mListener) {
-      mListener->TT_SetAudioTimeFormat(mAudioTime->GetBuiltinName(evt.GetInt()));
+      mListener->TT_SetAudioTimeFormat(evt.GetString());
    }
-
+   
    // During initialization, the desired size will have already been set at this point
    // and the "best" size" would override it, so we simply send a size event to force
    // the content to fit inside the toolbar.
@@ -370,9 +377,12 @@ void TimeToolBar::OnIdle(wxIdleEvent &evt)
    mAudioTime->SetValue(wxMax(0.0, audioTime));
 }
 
-static RegisteredToolbarFactory factory
+wxSize TimeToolBar::ComputeSizing(int digitH)
 {
-   TimeBarID,
+   return mAudioTime->ComputeSizing(false, digitH * mDigitRatio, digitH);
+}
+
+static RegisteredToolbarFactory factory{
    []( AudacityProject &project )
    {
       return ToolBar::Holder{ safenew TimeToolBar{ project } };
@@ -382,7 +392,7 @@ static RegisteredToolbarFactory factory
 namespace {
 AttachedToolBarMenuItem sAttachment
 {
-   TimeBarID,
+   TimeToolBar::ID(),
    wxT("ShowTimeTB"),
    /* i18n-hint: Clicking this menu item shows the toolbar
       for viewing actual time of the cursor */
