@@ -41,8 +41,6 @@
 
 #ifndef WX_PRECOMP
 #include <wx/defs.h>
-#include <wx/event.h>
-#include <wx/intl.h>
 #include <wx/sizer.h>
 #endif
 #include <wx/tooltip.h>
@@ -71,9 +69,14 @@ BEGIN_EVENT_TABLE(ToolsToolBar, ToolBar)
                      ToolsToolBar::OnTool)
 END_EVENT_TABLE()
 
+Identifier ToolsToolBar::ID()
+{
+   return wxT("Tools");
+}
+
 //Standard constructor
 ToolsToolBar::ToolsToolBar( AudacityProject &project )
-: ToolBar(project, ToolsBarID, XO("Tools"), wxT("Tools"))
+: ToolBar(project, XO("Tools"), ID())
 {
    using namespace ToolCodes;
 
@@ -104,7 +107,7 @@ ToolsToolBar::~ToolsToolBar()
 ToolsToolBar &ToolsToolBar::Get( AudacityProject &project )
 {
    auto &toolManager = ToolManager::Get( project );
-   return *static_cast<ToolsToolBar*>( toolManager.GetToolBar(ToolsBarID) );
+   return *static_cast<ToolsToolBar*>(toolManager.GetToolBar(ID()));
 }
 
 const ToolsToolBar &ToolsToolBar::Get( const AudacityProject &project )
@@ -250,7 +253,7 @@ void ToolsToolBar::Create(wxWindow * parent)
    UpdatePrefs();
 }
 
-static RegisteredToolbarFactory factory{ ToolsBarID,
+static RegisteredToolbarFactory factory{
    []( AudacityProject &project ){
       return ToolBar::Holder{ safenew ToolsToolBar{ project } }; }
 };
@@ -259,7 +262,7 @@ namespace {
 AttachedToolBarMenuItem sAttachment{
    /* i18n-hint: Clicking this menu item shows a toolbar
       that has some tools in it */
-   ToolsBarID, wxT("ShowToolsTB"), XXO("T&ools Toolbar"),
+   ToolsToolBar::ID(), wxT("ShowToolsTB"), XXO("T&ools Toolbar"),
 };
 }
 
@@ -283,14 +286,10 @@ void SetTool(AudacityProject &project, int tool)
 }
 
 /// Namespace for functions for View Toolbar menu
-namespace ToolActions {
-
-// exported helper functions
-// none
+namespace {
 
 // Menu handler functions
 
-struct Handler : CommandHandlerObject {
 /// Handler to set the select tool active
 void OnSelectTool(const CommandContext &context)
 {
@@ -334,43 +333,29 @@ void OnNextTool(const CommandContext &context)
    trackPanel.Refresh(false);
 }
 
-}; // struct Handler
-
-static CommandHandlerObject &findCommandHandler(AudacityProject &) {
-   // Handler is not stateful.  Doesn't need a factory registered with
-   // AudacityProject.
-   static ToolActions::Handler instance;
-   return instance;
-};
-
-#define FN(X) (& ToolActions::Handler :: X)
-
 using namespace MenuTable;
 BaseItemSharedPtr ExtraToolsMenu()
 {
    static BaseItemSharedPtr menu{
-   ( FinderScope{ findCommandHandler },
    Menu( wxT("Tools"), XXO("T&ools"),
-      Command( wxT("SelectTool"), XXO("&Selection Tool"), FN(OnSelectTool),
+      Command( wxT("SelectTool"), XXO("&Selection Tool"), OnSelectTool,
          AlwaysEnabledFlag, wxT("F1") ),
       Command( wxT("EnvelopeTool"), XXO("&Envelope Tool"),
-         FN(OnEnvelopeTool), AlwaysEnabledFlag, wxT("F2") ),
-      Command( wxT("DrawTool"), XXO("&Draw Tool"), FN(OnDrawTool),
+         OnEnvelopeTool, AlwaysEnabledFlag, wxT("F2") ),
+      Command( wxT("DrawTool"), XXO("&Draw Tool"), OnDrawTool,
          AlwaysEnabledFlag, wxT("F3") ),
-      Command( wxT("MultiTool"), XXO("&Multi Tool"), FN(OnMultiTool),
+      Command( wxT("MultiTool"), XXO("&Multi Tool"), OnMultiTool,
          AlwaysEnabledFlag, wxT("F6") ),
-      Command( wxT("PrevTool"), XXO("&Previous Tool"), FN(OnPrevTool),
+      Command( wxT("PrevTool"), XXO("&Previous Tool"), OnPrevTool,
          AlwaysEnabledFlag, wxT("A") ),
-      Command( wxT("NextTool"), XXO("&Next Tool"), FN(OnNextTool),
+      Command( wxT("NextTool"), XXO("&Next Tool"), OnNextTool,
          AlwaysEnabledFlag, wxT("D") )
-   ) ) };
+   ) };
    return menu;
 }
 
-#undef FN
-
 AttachedItem sAttachment2{
    wxT("Optional/Extra/Part1"),
-   Shared( ExtraToolsMenu() )
+   Indirect(ExtraToolsMenu())
 };
 }
