@@ -19,33 +19,32 @@ Paul Licameli split from TrackPanel.cpp
 #include "../../../TrackPanelMouseEvent.h"
 #include "UndoManager.h"
 #include "ViewInfo.h"
-#include "../../../SelectionState.h"
-#include "../../../ProjectAudioIO.h"
+#include "SelectionState.h"
+#include "ProjectAudioIO.h"
 #include "../../../../images/Cursors.h"
-#include "../../../tracks/ui/TimeShiftHandle.h"
 
 #include <wx/cursor.h>
+#include <wx/event.h>
 #include <wx/translation.h>
 
+#include <cassert>
+
 LabelTrackHit::LabelTrackHit( const std::shared_ptr<LabelTrack> &pLT )
    : mpLT{ pLT }
 {
-   pLT->Bind(
-      EVT_LABELTRACK_PERMUTED, &LabelTrackHit::OnLabelPermuted, this );
+   mSubscription = pLT->Subscribe( *this, &LabelTrackHit::OnLabelPermuted );
 }
 
 LabelTrackHit::~LabelTrackHit()
 {
-   // Must do this because this sink isn't wxEvtHandler
-   mpLT->Unbind(
-      EVT_LABELTRACK_PERMUTED, &LabelTrackHit::OnLabelPermuted, this );
 }
 
-void LabelTrackHit::OnLabelPermuted( LabelTrackEvent &e )
+void LabelTrackHit::OnLabelPermuted( const LabelTrackEvent &e )
 {
-   e.Skip();
    if ( e.mpTrack.lock() != mpLT )
       return;
+   if ( e.type != LabelTrackEvent::Permutation )
+      return;
 
    auto former = e.mFormerPosition;
    auto present = e.mPresentPosition;
@@ -327,13 +326,14 @@ bool LabelGlyphHandle::HandleGlyphDragRelease
 
               if (!done) {
                   //otherwise, select all tracks
-                  for (auto t : tracks.Any())
+                  for (auto t : tracks)
                       selectionState.SelectTrack(*t, true, true);
               }
 
               // Do this after, for its effect on TrackPanel's memory of last selected
               // track (which affects shift-click actions)
-              selectionState.SelectTrack(*pTrack.get(), true, true);
+              assert(pTrack->IsLeader()); // It's a label track
+              selectionState.SelectTrack(*pTrack, true, true);
 
               // PRL: bug1659 -- make selection change undo correctly
               updated = !ProjectAudioIO::Get(project).IsAudioActive();
