@@ -12,6 +12,7 @@ Paul Licameli split from TrackPanel.cpp
 #define __AUDACITY_LABEL_GLYPH_HANDLE__
 
 #include "LabelDefaultClickHandle.h"
+#include "Observer.h"
 
 class wxMouseState;
 class LabelTrack;
@@ -43,7 +44,8 @@ struct LabelTrackHit
 
    std::shared_ptr<LabelTrack> mpLT {};
 
-   void OnLabelPermuted( LabelTrackEvent &e );
+   Observer::Subscription mSubscription;
+   void OnLabelPermuted( const LabelTrackEvent &e );
 };
 
 class LabelGlyphHandle final : public LabelDefaultClickHandle
