@@ -23,12 +23,12 @@ Paul Licameli split from TrackPanel.cpp
 #include "../../../HitTestResult.h"
 #include "Project.h"
 #include "ProjectHistory.h"
+#include "ProjectNumericFormats.h"
 #include "ProjectRate.h"
-#include "../../../ProjectSettings.h"
 #include "../../../ProjectWindow.h"
 #include "../../../ProjectWindows.h"
 #include "../../../RefreshCode.h"
-#include "../../../SyncLock.h"
+#include "SyncLock.h"
 #include "Theme.h"
 #include "../../../TrackArt.h"
 #include "../../../TrackArtist.h"
@@ -37,8 +37,8 @@ Paul Licameli split from TrackPanel.cpp
 #include "../../../TrackPanelMouseEvent.h"
 #include "UndoManager.h"
 #include "ViewInfo.h"
-#include "../../../widgets/AudacityTextEntryDialog.h"
-#include "../../../widgets/wxWidgetsWindowPlacement.h"
+#include "AudacityTextEntryDialog.h"
+#include "wxWidgetsWindowPlacement.h"
 
 #include <wx/clipbrd.h>
 #include <wx/dcclient.h>
@@ -97,7 +97,7 @@ void LabelTrackView::Index::SetModified(bool modified)
 }
 
 LabelTrackView::LabelTrackView( const std::shared_ptr<Track> &pTrack )
-   : CommonTrackView{ pTrack }
+   : CommonChannelView{ pTrack, 0 }
 {
    ResetFont();
    CreateCustomGlyphs();
@@ -116,43 +116,35 @@ void LabelTrackView::Reparent( const std::shared_ptr<Track> &parent )
 {
    auto oldParent = FindLabelTrack();
    auto newParent = track_cast<LabelTrack*>(parent.get());
-   if (oldParent.get() != newParent) {
-      UnbindFrom( oldParent.get() );
+   if (oldParent.get() != newParent)
       BindTo( newParent );
-   }
-   CommonTrackView::Reparent( parent );
+   CommonChannelView::Reparent(parent);
 }
 
 void LabelTrackView::BindTo( LabelTrack *pParent )
 {
-   pParent->Bind(
-      EVT_LABELTRACK_ADDITION, &LabelTrackView::OnLabelAdded, this );
-   pParent->Bind(
-      EVT_LABELTRACK_DELETION, &LabelTrackView::OnLabelDeleted, this );
-   pParent->Bind(
-      EVT_LABELTRACK_PERMUTED, &LabelTrackView::OnLabelPermuted, this );
-   pParent->Bind(
-      EVT_LABELTRACK_SELECTION, &LabelTrackView::OnSelectionChange, this );
+   // Destroys any previous subscription to another track
+   mSubscription = pParent->Subscribe([this](const LabelTrackEvent &e){
+      switch (e.type) {
+      case LabelTrackEvent::Addition:
+         return OnLabelAdded(e);
+      case LabelTrackEvent::Deletion:
+         return OnLabelDeleted(e);
+      case LabelTrackEvent::Permutation:
+         return OnLabelPermuted(e);
+      case LabelTrackEvent::Selection:
+         return OnSelectionChange(e);
+      default:
+         return;
+      }
+   });
 }
 
-void LabelTrackView::UnbindFrom( LabelTrack *pParent )
+void LabelTrackView::CopyTo(Track &track) const
 {
-   pParent->Unbind(
-      EVT_LABELTRACK_ADDITION, &LabelTrackView::OnLabelAdded, this );
-   pParent->Unbind(
-      EVT_LABELTRACK_DELETION, &LabelTrackView::OnLabelDeleted, this );
-   pParent->Unbind(
-      EVT_LABELTRACK_PERMUTED, &LabelTrackView::OnLabelPermuted, this );
-   pParent->Unbind(
-      EVT_LABELTRACK_SELECTION, &LabelTrackView::OnSelectionChange, this );
-}
-
-void LabelTrackView::CopyTo( Track &track ) const
-{
-   TrackView::CopyTo( track );
-   auto &other = TrackView::Get( track );
-
-   if ( const auto pOther = dynamic_cast< const LabelTrackView* >( &other ) ) {
+   ChannelView::CopyTo(track);
+   auto &other = ChannelView::Get(*track.GetChannel(0));
+   if (const auto pOther = dynamic_cast<const LabelTrackView*>(&other)) {
       pOther->mNavigationIndex = mNavigationIndex;
       pOther->mInitialCursorPos = mInitialCursorPos;
       pOther->mCurrentCursorPos = mCurrentCursorPos;
@@ -161,14 +153,14 @@ void LabelTrackView::CopyTo( Track &track ) const
    }
 }
 
-LabelTrackView &LabelTrackView::Get( LabelTrack &track )
+LabelTrackView &LabelTrackView::Get(LabelTrack &track)
 {
-   return static_cast< LabelTrackView& >( TrackView::Get( track ) );
+   return static_cast<LabelTrackView&>(ChannelView::Get(track));
 }
 
-const LabelTrackView &LabelTrackView::Get( const LabelTrack &track )
+const LabelTrackView &LabelTrackView::Get(const LabelTrack &track)
 {
-   return static_cast< const LabelTrackView& >( TrackView::Get( track ) );
+   return static_cast<const LabelTrackView&>(ChannelView::Get(track));
 }
 
 std::shared_ptr<LabelTrack> LabelTrackView::FindLabelTrack()
@@ -264,7 +256,7 @@ void LabelTrackView::ResetFont()
 {
    mFontHeight = -1;
    wxString facename = gPrefs->Read(wxT("/GUI/LabelFontFacename"), wxT(""));
-   int size = gPrefs->Read(wxT("/GUI/LabelFontSize"), DefaultFontSize);
+   int size = gPrefs->Read(wxT("/GUI/LabelFontSize"), static_cast<int>(DefaultFontSize));
    msFont = GetFont(facename, size);
 }
 
@@ -933,7 +925,7 @@ void LabelTrackView::Draw(
 {
    if ( iPass == TrackArtist::PassTracks )
       Draw( context, rect );
-   CommonTrackView::Draw( context, rect, iPass );
+   CommonChannelView::Draw(context, rect, iPass);
 }
 
 /// uses GetTextExtent to find the character position
@@ -1691,7 +1683,7 @@ bool LabelTrackView::DoKeyDown(
       case WXK_NUMPAD_ENTER:
       case WXK_TAB:
          if (mRestoreFocus >= 0) {
-            auto track = *TrackList::Get( project ).Any()
+            auto track = *TrackList::Get(project).Any()
                .begin().advance(mRestoreFocus);
             if (track)
                TrackFocus::Get( project ).Set(track);
@@ -2084,7 +2076,8 @@ int LabelTrackView::GetLabelIndex(double t, double t1)
 
 // restoreFocus of -1 is the default, and sets the focus to this label.
 // restoreFocus of -2 or other value leaves the focus unchanged.
-// restoreFocus >= 0 will later cause focus to move to that track.
+// restoreFocus >= 0 will later cause focus to move to that track (counting
+// tracks, not channels)
 int LabelTrackView::AddLabel(const SelectedRegion &selectedRegion,
                          const wxString &title, int restoreFocus)
 {
@@ -2094,9 +2087,8 @@ int LabelTrackView::AddLabel(const SelectedRegion &selectedRegion,
    return pos;
 }
 
-void LabelTrackView::OnLabelAdded( LabelTrackEvent &e )
+void LabelTrackView::OnLabelAdded( const LabelTrackEvent &e )
 {
-   e.Skip();
    if ( e.mpTrack.lock() != FindTrack() )
       return;
 
@@ -2116,9 +2108,8 @@ void LabelTrackView::OnLabelAdded( LabelTrackEvent &e )
       mRestoreFocus = -2;
 }
 
-void LabelTrackView::OnLabelDeleted( LabelTrackEvent &e )
+void LabelTrackView::OnLabelDeleted( const LabelTrackEvent &e )
 {
-   e.Skip();
    if ( e.mpTrack.lock() != FindTrack() )
       return;
 
@@ -2135,9 +2126,8 @@ void LabelTrackView::OnLabelDeleted( LabelTrackEvent &e )
       --mTextEditIndex;//NB: Keep cursor selection region
 }
 
-void LabelTrackView::OnLabelPermuted( LabelTrackEvent &e )
+void LabelTrackView::OnLabelPermuted( const LabelTrackEvent &e )
 {
-   e.Skip();
    if ( e.mpTrack.lock() != FindTrack() )
       return;
 
@@ -2156,9 +2146,8 @@ void LabelTrackView::OnLabelPermuted( LabelTrackEvent &e )
    fix(mTextEditIndex);
 }
 
-void LabelTrackView::OnSelectionChange( LabelTrackEvent &e )
+void LabelTrackView::OnSelectionChange( const LabelTrackEvent &e )
 {
-   e.Skip();
    if ( e.mpTrack.lock() != FindTrack() )
       return;
 
@@ -2282,17 +2271,16 @@ void LabelTrackView::CreateCustomGlyphs()
 void LabelTrackView::DoEditLabels
 (AudacityProject &project, LabelTrack *lt, int index)
 {
-   const auto &settings = ProjectSettings::Get( project );
-   auto format = settings.GetSelectionFormat(),
-      freqFormat = settings.GetFrequencySelectionFormatName();
+   const auto &formats = ProjectNumericFormats::Get( project );
+   auto format = formats.GetSelectionFormat(),
+      freqFormat = formats.GetFrequencySelectionFormatName();
    auto &tracks = TrackList::Get( project );
-   auto rate = ProjectRate::Get( project ).GetRate();
    auto &viewInfo = ViewInfo::Get( project );
    auto &window = ProjectWindow::Get( project );
 
    LabelDialog dlg(&window, project, &tracks,
                    lt, index,
-                   viewInfo, rate,
+                   viewInfo,
                    format, freqFormat);
 #ifdef __WXGTK__
    dlg.Raise();
@@ -2352,14 +2340,14 @@ int LabelTrackView::DialogForLabelName(
    return status;
 }
 
-using DoGetLabelTrackView = DoGetView::Override< LabelTrack >;
+using DoGetLabelTrackView = DoGetView::Override<LabelTrack>;
 DEFINE_ATTACHED_VIRTUAL_OVERRIDE(DoGetLabelTrackView) {
-   return [](LabelTrack &track) {
+   return [](LabelTrack &track, size_t) {
       return std::make_shared<LabelTrackView>( track.SharedPointer() );
    };
 }
 
-std::shared_ptr<TrackVRulerControls> LabelTrackView::DoGetVRulerControls()
+std::shared_ptr<ChannelVRulerControls> LabelTrackView::DoGetVRulerControls()
 {
    return
       std::make_shared<LabelTrackVRulerControls>( shared_from_this() );
