@@ -7,8 +7,6 @@ NoteTrackView.cpp
 Paul Licameli split from TrackPanel.cpp
 
 **********************************************************************/
-
-
 #include "NoteTrackView.h"
 
 #ifdef USE_MIDI
@@ -33,7 +31,7 @@ Paul Licameli split from TrackPanel.cpp
 #include <wx/dc.h>
 
 NoteTrackView::NoteTrackView( const std::shared_ptr<Track> &pTrack )
-   : CommonTrackView{ pTrack }
+   : CommonChannelView{ pTrack, 0 }
 {
 }
 
@@ -60,14 +58,14 @@ std::vector<UIHandlePtr> NoteTrackView::DetailedHitTest
    return results;
 }
 
-using DoGetNoteTrackView = DoGetView::Override< NoteTrack >;
+using DoGetNoteTrackView = DoGetView::Override<NoteTrack>;
 DEFINE_ATTACHED_VIRTUAL_OVERRIDE(DoGetNoteTrackView) {
-   return [](NoteTrack &track) {
+   return [](NoteTrack &track, size_t) {
       return std::make_shared<NoteTrackView>( track.SharedPointer() );
    };
 }
 
-std::shared_ptr<TrackVRulerControls> NoteTrackView::DoGetVRulerControls()
+std::shared_ptr<ChannelVRulerControls> NoteTrackView::DoGetVRulerControls()
 {
    return
       std::make_shared<NoteTrackVRulerControls>( shared_from_this() );
@@ -275,10 +273,10 @@ void DrawNoteBackground(TrackPanelDrawingContext &context,
    dc.DrawRectangle(sel); // fill rectangle with white keys background
 #endif
 
-   int left = TIME_TO_X(track->GetOffset());
+   int left = TIME_TO_X(track->GetStartTime());
    if (left < sel.x) left = sel.x; // clip on left
 
-   int right = TIME_TO_X(track->GetOffset() + track->GetSeq().get_real_dur());
+   int right = TIME_TO_X(track->GetStartTime() + track->GetSeq().get_real_dur());
    if (right > sel.x + sel.width) right = sel.x + sel.width; // clip on right
 
    // need overlap between MIDI data and the background region
@@ -346,7 +344,7 @@ void DrawNoteBackground(TrackPanelDrawingContext &context,
       // map beat to time
       double t = seq->get_time_map()->beat_to_time(next_bar_beat);
       // map time to position
-      int xx = TIME_TO_X(t + track->GetOffset());
+      int xx = TIME_TO_X(t + track->GetStartTime());
       if (xx > right) break;
       AColor::Line(dc, xx, sel.y, xx, sel.y + sel.height);
       next_bar_beat += beats_per_measure;
@@ -478,7 +476,7 @@ void DrawNoteTrack(TrackPanelDrawingContext &context,
          Alg_note_ptr note = (Alg_note_ptr) evt;
          // if the note's channel is visible
          if (track->IsVisibleChan(evt->chan)) {
-            double xx = note->time + track->GetOffset();
+            double xx = note->time + track->GetStartTime();
             double x1 = xx + note->dur;
             if (xx < h1 && x1 > h) { // omit if outside box
                const char *shape = NULL;
@@ -706,14 +704,14 @@ void DrawNoteTrack(TrackPanelDrawingContext &context,
    AColor::Line(dc, rect.x, rect.y + rect.height - marg - 1, // subtract 1 to get
                 rect.x + rect.width, rect.y + rect.height - marg - 1); // top of line
 
-   if (h == 0.0 && track->GetOffset() < 0.0) {
+   if (h == 0.0 && track->GetStartTime() < 0.0) {
       TrackArt::DrawNegativeOffsetTrackArrows( context, rect );
    }
 
    //draw clip edges
    {
-      int left = TIME_TO_X(track->GetOffset());
-      int right = TIME_TO_X(track->GetOffset() + track->GetSeq().get_real_dur());
+      int left = TIME_TO_X(track->GetStartTime());
+      int right = TIME_TO_X(track->GetStartTime() + track->GetSeq().get_real_dur());
 
       TrackArt::DrawClipEdges(dc, wxRect(left, rect.GetTop(), right - left + 1, rect.GetHeight()), selected);
    }
@@ -749,7 +747,7 @@ void NoteTrackView::Draw(
 
       DrawNoteTrack(context, nt.get(), rect, muted, selected);
    }
-   CommonTrackView::Draw( context, rect, iPass );
+   CommonChannelView::Draw(context, rect, iPass);
 }
 
 #include "SyncLock.h"
