@@ -11,9 +11,9 @@ Paul Licameli split from class NoteTrack
 #ifndef __AUDACITY_NOTE_TRACK_VIEW__
 #define __AUDACITY_NOTE_TRACK_VIEW__
 
-#include "../../../ui/CommonTrackView.h"
+#include "../../../ui/CommonChannelView.h"
 
-class NoteTrackView final : public CommonTrackView
+class NoteTrackView final : public CommonChannelView
 {
    NoteTrackView( const NoteTrackView& ) = delete;
    NoteTrackView &operator=( const NoteTrackView& ) = delete;
@@ -24,7 +24,7 @@ public:
    ~NoteTrackView() override;
 
 private:
-   std::shared_ptr<TrackVRulerControls> DoGetVRulerControls() override;
+   std::shared_ptr<ChannelVRulerControls> DoGetVRulerControls() override;
    std::shared_ptr<CommonTrackCell> GetAffordanceControls() override;
 
    std::vector<UIHandlePtr> DetailedHitTest
