@@ -18,16 +18,16 @@ Paul Licameli split from TrackPanel.cpp
 #include "../../../ui/CommonTrackPanelCell.h"
 #include "../../../../HitTestResult.h"
 #include "../../../../NoteTrack.h"
-#include "../../../../ProjectAudioIO.h"
+#include "ProjectAudioIO.h"
 #include "ProjectHistory.h"
-#include "../../../../ProjectSettings.h"
 #include "../../../../RefreshCode.h"
-#include "../../../../SyncLock.h"
+#include "SyncLock.h"
 #include "../../../../TrackPanelMouseEvent.h"
 #include "UndoManager.h"
 #include "ViewInfo.h"
 #include "../../../../../images/Cursors.h"
 
+#include <wx/event.h>
 #include <algorithm>
 
 StretchHandle::StretchHandle
@@ -230,25 +230,23 @@ UIHandle::Result StretchHandle::Release
 
    bool left = mStretchState.mMode == stretchLeft;
    bool right = mStretchState.mMode == stretchRight;
-   const auto &settings = ProjectSettings::Get( *pProject );
    auto &viewInfo = ViewInfo::Get( *pProject );
-   if ( settings.IsSyncLocked() && ( left || right ) ) {
-      for ( auto track :
-           SyncLock::Group( mpTrack.get() ) ) {
-         if ( track != mpTrack.get() ) {
-            if ( left ) {
+   if (SyncLockState::Get(*pProject).IsSyncLocked() && (left || right)) {
+      for (auto track : SyncLock::Group(mpTrack.get())) {
+         if (track != mpTrack.get()) {
+            if (left) {
                auto origT0 = mStretchState.mOrigSel0Quantized;
                auto diff = viewInfo.selectedRegion.t0() - origT0;
-               if ( diff > 0)
-                  track->SyncLockAdjust( origT0 + diff, origT0 );
+               if (diff > 0)
+                  track->SyncLockAdjust(origT0 + diff, origT0);
                else
-                  track->SyncLockAdjust( origT0, origT0 - diff );
-               track->Offset( diff );
+                  track->SyncLockAdjust(origT0, origT0 - diff);
+               track->ShiftBy(diff);
             }
             else {
                auto origT1 = mStretchState.mOrigSel1Quantized;
                auto diff = viewInfo.selectedRegion.t1() - origT1;
-               track->SyncLockAdjust( origT1, origT1 + diff );
+               track->SyncLockAdjust(origT1, origT1 + diff);
             }
          }
       }
@@ -290,7 +288,7 @@ void StretchHandle::Stretch(AudacityProject *pProject, int mouseXCoordinate, int
    if (pTrack == NULL && mpTrack != NULL)
       pTrack = mpTrack.get();
 
-  if (pTrack) pTrack->TypeSwitch( [&](NoteTrack *pNt) {
+  if (pTrack) pTrack->TypeSwitch( [&](NoteTrack &nt) {
       double moveto =
         std::max(0.0, viewInfo.PositionToTime(mouseXCoordinate, trackLeftEdge));
 
@@ -309,9 +307,9 @@ void StretchHandle::Stretch(AudacityProject *pProject, int mouseXCoordinate, int
          dur = t1 - moveto;
          if (dur < mStretchState.mRightBeats * minPeriod)
             return;
-         pNt->StretchRegion
+         nt.StretchRegion
             ( mStretchState.mBeat0, mStretchState.mBeat1, dur );
-         pNt->Offset( moveto - t0 );
+         nt.ShiftBy(moveto - t0);
          mStretchState.mBeat0.first = moveto;
          viewInfo.selectedRegion.setT0(moveto);
          break;
@@ -320,7 +318,7 @@ void StretchHandle::Stretch(AudacityProject *pProject, int mouseXCoordinate, int
          dur = moveto - t0;
          if (dur < mStretchState.mLeftBeats * minPeriod)
             return;
-         pNt->StretchRegion
+         nt.StretchRegion
             ( mStretchState.mBeat0, mStretchState.mBeat1, dur );
          viewInfo.selectedRegion.setT1(moveto);
          mStretchState.mBeat1.first = moveto;
@@ -333,9 +331,9 @@ void StretchHandle::Stretch(AudacityProject *pProject, int mouseXCoordinate, int
          if ( left_dur < mStretchState.mLeftBeats * minPeriod ||
               right_dur < mStretchState.mRightBeats * minPeriod )
             return;
-         pNt->StretchRegion
+         nt.StretchRegion
             ( mStretchState.mBeatCenter, mStretchState.mBeat1, right_dur );
-         pNt->StretchRegion
+         nt.StretchRegion
             ( mStretchState.mBeat0, mStretchState.mBeatCenter, left_dur );
          mStretchState.mBeatCenter.first = moveto;
          break;
