@@ -7,16 +7,14 @@ PlayableTrackButtonHandles.cpp
 Paul Licameli split from TrackPanel.cpp
 
 **********************************************************************/
-
-
 #include "PlayableTrackButtonHandles.h"
-
+#include "PlayableTrack.h"
 #include "PlayableTrackControls.h"
 #include "../../../commands/CommandManager.h"
 #include "Project.h"
-#include "../../../ProjectSettings.h"
 #include "../../../RefreshCode.h"
-#include "Track.h"
+#include "../../../RealtimeEffectPanel.h"
+#include "SampleTrack.h"
 #include "../../../TrackPanelAx.h"
 #include "../../../TrackInfo.h"
 #include "../../../TrackPanelMouseEvent.h"
@@ -24,7 +22,6 @@ Paul Licameli split from TrackPanel.cpp
 
 #include <wx/window.h>
 
-#include "ProjectWindow.h"
 MuteButtonHandle::MuteButtonHandle
 ( const std::shared_ptr<Track> &pTrack, const wxRect &rect )
    : ButtonHandle{ pTrack, rect }
@@ -66,7 +63,7 @@ UIHandlePtr MuteButtonHandle::HitTest
    wxRect buttonRect;
    if ( pTrack )
       PlayableTrackControls::GetMuteSoloRect(rect, buttonRect, false,
-         !ProjectSettings::Get( *pProject ).IsSoloNone(), pTrack.get());
+         (TracksBehaviorsSolo.ReadEnum() != SoloBehaviorNone), pTrack.get());
    if ( TrackInfo::HideTopItem( rect, buttonRect ) )
       return {};
 
@@ -122,7 +119,7 @@ UIHandlePtr SoloButtonHandle::HitTest
    wxRect buttonRect;
    if ( pTrack )
       PlayableTrackControls::GetMuteSoloRect(rect, buttonRect, true,
-         !ProjectSettings::Get( *pProject ).IsSoloNone(), pTrack.get());
+         (TracksBehaviorsSolo.ReadEnum() != SoloBehaviorNone), pTrack.get());
 
    if ( TrackInfo::HideTopItem( rect, buttonRect ) )
       return {};
@@ -150,7 +147,8 @@ EffectsButtonHandle::~EffectsButtonHandle()
 UIHandle::Result EffectsButtonHandle::CommitChanges
 (const wxMouseEvent &event, AudacityProject *pProject, wxWindow *pParent)
 {
-   ProjectWindow::Get(*pProject).ShowEffectsPanel(mpTrack.lock().get(), true);
+   RealtimeEffectPanel::Get(*pProject).ShowPanel(
+      dynamic_cast<SampleTrack *>(mpTrack.lock().get()), true);
    return RefreshCode::RefreshNone;
 }
 
