@@ -22,23 +22,23 @@
 #include "../../../../TrackArt.h"
 #include "../../../../TrackArtist.h"
 #include "../../../../TrackPanelDrawingContext.h"
-#include "../../../../TrackPanelResizeHandle.h"
 #include "ViewInfo.h"
-#include "../../../../WaveTrack.h"
-#include "../../../../WaveClip.h"
+#include "WaveTrack.h"
+#include "WaveClip.h"
+#include "WaveTrackUtilities.h"
 #include "UndoManager.h"
-#include "../../../../ShuttleGui.h"
+#include "ShuttleGui.h"
 #include "../../../../ProjectWindows.h"
 #include "../../../../commands/AudacityCommand.h"
 
 #include "../../../ui/TextEditHelper.h"
 #include "../../../ui/SelectHandle.h"
-#include "WaveTrackView.h"//need only ClipParameters
+#include "WaveChannelView.h"//need only ClipParameters
 #include "WaveTrackAffordanceHandle.h"
 
 #include "ProjectHistory.h"
 #include "../../../../ProjectSettings.h"
-#include "../../../../SelectionState.h"
+#include "SelectionState.h"
 #include "../../../../RefreshCode.h"
 #include "Theme.h"
 #include "../../../../../images/Cursors.h"
@@ -48,8 +48,13 @@
 
 #include "../WaveTrackUtils.h"
 
-#include "WaveClipTrimHandle.h"
+#include "WaveClipAdjustBorderHandle.h"
+#include "WaveClipUtilities.h"
 
+#include "ChangeClipSpeedDialog.h"
+
+#include "BasicUI.h"
+#include "UserException.h"
 
 
 class SetWaveClipNameCommand : public AudacityCommand
@@ -87,7 +92,7 @@ public:
     WaveClipTitleEditHandle(const std::shared_ptr<TextEditHelper>& helper)
         : mHelper(helper)
     { }
-   
+
    ~WaveClipTitleEditHandle()
    {
    }
@@ -135,12 +140,21 @@ public:
 };
 
 WaveTrackAffordanceControls::WaveTrackAffordanceControls(const std::shared_ptr<Track>& pTrack)
-    : CommonTrackCell(pTrack), mClipNameFont(wxFont(wxFontInfo()))
+    : CommonTrackCell{ pTrack, 0 }
+    , mClipNameFont{ wxFontInfo{} }
 {
     if (auto trackList = pTrack->GetOwner())
     {
-        mSubscription = trackList->Subscribe(
-            *this, &WaveTrackAffordanceControls::OnTrackChanged);
+        mTrackListEventSubscription = trackList->Subscribe(
+            *this, &WaveTrackAffordanceControls::OnTrackListEvent);
+        if(auto project = trackList->GetOwner())
+        {
+            auto& viewInfo = ViewInfo::Get(*project);
+            mSelectionChangeSubscription =
+                viewInfo.selectedRegion.Subscribe(
+                    *this,
+                    &WaveTrackAffordanceControls::OnSelectionChange);
+        }
     }
 }
 
@@ -154,10 +168,12 @@ std::vector<UIHandlePtr> WaveTrackAffordanceControls::HitTest(const TrackPanelMo
     const auto rect = state.rect;
 
     auto track = std::static_pointer_cast<WaveTrack>(FindTrack());
+    // Assume only leader channels have affordance areas
+    assert(track->IsLeader());
 
     {
-        auto handle = WaveClipTrimHandle::HitAnywhere(
-            mClipTrimHandle,
+        auto handle = WaveClipAdjustBorderHandle::HitAnywhere(
+            mClipBorderAdjustHandle,
             track,
             pProject,
             state);
@@ -166,22 +182,6 @@ std::vector<UIHandlePtr> WaveTrackAffordanceControls::HitTest(const TrackPanelMo
             results.push_back(handle);
     }
 
-    auto trackList = track->GetOwner();
-    if ((std::abs(rect.GetTop() - py) <= WaveTrackView::kChannelSeparatorThickness / 2) 
-        && trackList
-        && !track->IsLeader())
-    {
-        //given that track is not a leader there always should be
-        //another track before this one
-        auto prev = std::prev(trackList->Find(track.get()));
-        results.push_back(
-            AssignUIHandlePtr(
-                mResizeHandle, 
-                std::make_shared<TrackPanelResizeHandle>((*prev)->shared_from_this(), py)
-            )
-        );
-    }
-
     if (mTextEditHelper && mTextEditHelper->GetBBox().Contains(px, py))
     {
         results.push_back(
@@ -192,23 +192,24 @@ std::vector<UIHandlePtr> WaveTrackAffordanceControls::HitTest(const TrackPanelMo
         );
     }
 
-    auto editClipLock = mEditedClip.lock();
     const auto waveTrack = std::static_pointer_cast<WaveTrack>(track->SubstitutePendingChangedTrack());
     auto& zoomInfo = ViewInfo::Get(*pProject);
-    for (const auto& clip : waveTrack->GetClips())
+    const auto intervals = waveTrack->Intervals();
+    for(auto it = intervals.begin(); it != intervals.end(); ++it)
     {
-        if (clip == editClipLock)
+        if (it == mEditedInterval)
             continue;
 
-        if (WaveTrackView::HitTest(*clip, zoomInfo, state.rect, {px, py}))
+        auto interval = *it;
+        if (WaveChannelView::HitTest(*interval->GetClip(0), zoomInfo, state.rect, {px, py}))
         {
             results.push_back(
                 AssignUIHandlePtr(
                     mAffordanceHandle,
-                    std::make_shared<WaveTrackAffordanceHandle>(track, clip)
+                    std::make_shared<WaveTrackAffordanceHandle>(track, interval->GetClip(0))
                 )
             );
-            mFocusClip = clip;
+            mFocusInterval = it;
             break;
         }
     }
@@ -220,7 +221,7 @@ std::vector<UIHandlePtr> WaveTrackAffordanceControls::HitTest(const TrackPanelMo
         results.push_back(
             SelectHandle::HitTest(
                 mSelectHandle, state, pProject,
-                TrackView::Get(*track).shared_from_this()
+                ChannelView::Get(*track).shared_from_this()
             )
         );
     }
@@ -236,9 +237,10 @@ void WaveTrackAffordanceControls::Draw(TrackPanelDrawingContext& context, const
 
         TrackArt::DrawBackgroundWithSelection(context, rect, track.get(), artist->blankSelectedBrush, artist->blankBrush);
 
+        mVisibleIntervals.clear();
+
         const auto waveTrack = std::static_pointer_cast<WaveTrack>(track->SubstitutePendingChangedTrack());
         const auto& zoomInfo = *artist->pZoomInfo;
-
         {
             wxDCClipper dcClipper(context.dc, rect);
 
@@ -249,28 +251,37 @@ void WaveTrackAffordanceControls::Draw(TrackPanelDrawingContext& context, const
             auto px = context.lastState.m_x;
             auto py = context.lastState.m_y;
 
-            for (const auto& clip : waveTrack->GetClips())
+            const auto intervals = waveTrack->Intervals();
+            for(auto it = intervals.begin(); it != intervals.end(); ++it)
             {
+                auto interval = *it;
                 auto affordanceRect
-                   = ClipParameters::GetClipRect(*clip.get(), zoomInfo, rect);
+                   = ClipParameters::GetClipRect(*interval->GetClip(0), zoomInfo, rect);
 
-                if(!WaveTrackView::ClipDetailsVisible(*clip, zoomInfo, rect))
+                if(!WaveChannelView::ClipDetailsVisible(*interval->GetClip(0), zoomInfo, rect))
                 {
                    TrackArt::DrawClipFolded(context.dc, affordanceRect);
                    continue;
                 }
 
-                auto selected = GetSelectedClip().lock() == clip;
-                auto highlight = selected || affordanceRect.Contains(px, py);
-                if (mTextEditHelper && mEditedClip.lock() == clip)
+                const auto selected = GetSelectedInterval() == it;
+                const auto highlight = selected || affordanceRect.Contains(px, py);
+                const auto titleRect = TrackArt::DrawClipAffordance(context.dc, affordanceRect, highlight, selected);
+                if (mTextEditHelper && mEditedInterval == it)
                 {
-                    auto titleRect = TrackArt::DrawClipAffordance(context.dc, affordanceRect, wxEmptyString, highlight, selected);
-                    mTextEditHelper->Draw(context.dc, titleRect);
+                    if(!mTextEditHelper->Draw(context.dc, titleRect))
+                    {
+                        mTextEditHelper->Cancel(nullptr);
+                        TrackArt::DrawAudioClipTitle(
+                           context.dc, titleRect, interval->GetName(),
+                           interval->GetStretchRatio());
+                    }
                 }
-                else
+                else if (TrackArt::DrawAudioClipTitle(
+                            context.dc, titleRect, interval->GetName(),
+                            interval->GetStretchRatio()))
                 {
-                    auto titleRect = TrackArt::DrawClipAffordance(context.dc, affordanceRect, clip->GetName(), highlight, selected);
-                    mClipNameVisible = !titleRect.IsEmpty();
+                   mVisibleIntervals.push_back(it);
                 }
             }
         }
@@ -278,50 +289,55 @@ void WaveTrackAffordanceControls::Draw(TrackPanelDrawingContext& context, const
     }
 }
 
-bool WaveTrackAffordanceControls::StartEditClipName(AudacityProject* project)
+bool WaveTrackAffordanceControls::IsIntervalVisible(const IntervalIterator& it) const noexcept
 {
-    if (auto lock = mFocusClip.lock())
-    {
-        auto clip = lock.get();
-
-        bool useDialog{ false };
-        gPrefs->Read(wxT("/GUI/DialogForNameNewLabel"), &useDialog, false);
-
-        if (useDialog)
-        {
-            SetWaveClipNameCommand Command;
-            auto oldName = clip->GetName();
-            Command.mName = oldName;
-            auto result = Command.PromptUser(&GetProjectFrame(*project));
-            if (result && Command.mName != oldName)
-            {
-                clip->SetName(Command.mName);
-                ProjectHistory::Get(*project).PushState(XO("Modified Clip Name"),
-                    XO("Clip Name Edit"));
-            }
-        }
-        else
-        {
-            if (mTextEditHelper)
-                mTextEditHelper->Finish(project);
-
-            if(!mClipNameVisible)
-                //Clip name isn't visible, there is no point in showing editor then
-                return false;
-
-            mEditedClip = lock;
-            mTextEditHelper = MakeTextEditHelper(clip->GetName());
-        }
-        return true;
-    }
-    return false;
+    return std::find(mVisibleIntervals.begin(),
+                     mVisibleIntervals.end(),
+                     it) != mVisibleIntervals.end();
 }
 
-std::weak_ptr<WaveClip> WaveTrackAffordanceControls::GetSelectedClip() const
+bool WaveTrackAffordanceControls::StartEditClipName(AudacityProject& project, IntervalIterator it)
+{
+    bool useDialog{ false };
+    gPrefs->Read(wxT("/GUI/DialogForNameNewLabel"), &useDialog, false);
+
+    auto interval = *it;
+    if(!interval)
+        return false;
+
+    if (useDialog)
+    {
+        SetWaveClipNameCommand Command;
+        auto oldName = interval->GetName();
+        Command.mName = oldName;
+        auto result = Command.PromptUser(&GetProjectFrame(project));
+        if (result && Command.mName != oldName)
+        {
+            interval->SetName(Command.mName);
+            ProjectHistory::Get(project).PushState(XO("Modified Clip Name"),
+                XO("Clip Name Edit"));
+        }
+    }
+    else if(it != mEditedInterval)
+    {
+        if(!IsIntervalVisible(it))
+           return false;
+
+        if (mTextEditHelper)
+            mTextEditHelper->Finish(&project);
+
+        mEditedInterval = it;
+        mTextEditHelper = MakeTextEditHelper(interval->GetName());
+    }
+
+    return true;
+}
+
+auto WaveTrackAffordanceControls::GetSelectedInterval() const -> IntervalIterator
 {
     if (auto handle = mAffordanceHandle.lock())
     {
-        return handle->Clicked() ? mFocusClip : std::weak_ptr<WaveClip>();
+        return handle->Clicked() ? mFocusInterval : IntervalIterator{};
     }
     return {};
 }
@@ -330,31 +346,34 @@ namespace {
 
 auto FindAffordance(WaveTrack &track)
 {
-   auto &view = TrackView::Get( track );
+   auto &view = ChannelView::Get(track);
    auto pAffordance = view.GetAffordanceControls();
    return std::dynamic_pointer_cast<WaveTrackAffordanceControls>(
       pAffordance );
 }
 
-std::pair<WaveTrack *, WaveClip *>
-SelectedClipOfFocusedTrack(AudacityProject &project)
+std::pair<WaveTrack*, ChannelGroup::IntervalIterator<WaveTrack::Interval>>
+SelectedIntervalOfFocusedTrack(AudacityProject &project)
 {
    // Note that TrackFocus may change its state as a side effect, defining
    // a track focus if there was none
    if (auto pWaveTrack =
       dynamic_cast<WaveTrack *>(TrackFocus::Get(project).Get())) {
-      for (auto pChannel : TrackList::Channels(pWaveTrack)) {
-         if (FindAffordance(*pChannel)) {
-            auto &viewInfo = ViewInfo::Get(project);
-            auto &clips = pChannel->GetClips();
-            auto begin = clips.begin(), end = clips.end(),
-               iter = WaveTrackUtils::SelectedClip(viewInfo, begin, end);
-            if (iter != end)
-               return { pChannel, iter->get() };
-         }
+      if (FindAffordance(*pWaveTrack)) {
+         auto &viewInfo = ViewInfo::Get(project);
+         auto intervals = pWaveTrack->Intervals();
+
+         auto it = std::find_if(intervals.begin(), intervals.end(), [&](const auto& interval)
+         {
+            return interval->Start() == viewInfo.selectedRegion.t0() &&
+               interval->End() == viewInfo.selectedRegion.t1();
+         });
+
+         if(it != intervals.end())
+            return { pWaveTrack, it };
       }
    }
-   return { nullptr, nullptr };
+   return {};
 }
 
 // condition for enabling the command
@@ -362,10 +381,23 @@ const ReservedCommandFlag &SomeClipIsSelectedFlag()
 {
    static ReservedCommandFlag flag{
       [](const AudacityProject &project){
-         return nullptr !=
-            // const_cast isn't pretty but not harmful in this case
-            SelectedClipOfFocusedTrack(const_cast<AudacityProject&>(project))
-               .second;
+         // const_cast isn't pretty but not harmful in this case
+         return SelectedIntervalOfFocusedTrack(const_cast<AudacityProject&>(project)).first != nullptr;
+      }
+   };
+   return flag;
+}
+
+const ReservedCommandFlag &StretchedClipIsSelectedFlag()
+{
+   static ReservedCommandFlag flag{
+      [](const AudacityProject &project){
+         // const_cast isn't pretty but not harmful in this case
+         auto result = SelectedIntervalOfFocusedTrack(
+            const_cast<AudacityProject&>(project));
+
+         auto interval = *result.second;
+         return interval != nullptr && !interval->StretchRatioEquals(1.0);
       }
    };
    return flag;
@@ -375,7 +407,7 @@ const ReservedCommandFlag &SomeClipIsSelectedFlag()
 
 unsigned WaveTrackAffordanceControls::CaptureKey(wxKeyEvent& event, ViewInfo& viewInfo, wxWindow* pParent, AudacityProject* project)
 {
-    if (!mTextEditHelper 
+    if (!mTextEditHelper
        || !mTextEditHelper->CaptureKey(event.GetKeyCode(), event.GetModifiers()))
        // Handle the event if it can be processed by the text editor (if any)
        event.Skip();
@@ -386,10 +418,10 @@ unsigned WaveTrackAffordanceControls::CaptureKey(wxKeyEvent& event, ViewInfo& vi
 unsigned WaveTrackAffordanceControls::KeyDown(wxKeyEvent& event, ViewInfo& viewInfo, wxWindow*, AudacityProject* project)
 {
     auto keyCode = event.GetKeyCode();
-    
+
     if (mTextEditHelper)
     {
-       if (!mTextEditHelper->OnKeyDown(keyCode, event.GetModifiers(), project) 
+       if (!mTextEditHelper->OnKeyDown(keyCode, event.GetModifiers(), project)
           && !TextEditHelper::IsGoodEditKeyCode(keyCode))
           event.Skip();
 
@@ -412,10 +444,10 @@ unsigned WaveTrackAffordanceControls::LoseFocus(AudacityProject *)
 
 void WaveTrackAffordanceControls::OnTextEditFinished(AudacityProject* project, const wxString& text)
 {
-    if (auto lock = mEditedClip.lock())
+    if (auto interval = *mEditedInterval)
     {
-        if (text != lock->GetName()) {
-            lock->SetName(text);
+        if (text != interval->GetName()) {
+            interval->SetName(text);
 
             ProjectHistory::Get(*project).PushState(XO("Modified Clip Name"),
                 XO("Clip Name Edit"));
@@ -441,15 +473,20 @@ void WaveTrackAffordanceControls::OnTextContextMenu(AudacityProject* project, co
 void WaveTrackAffordanceControls::ResetClipNameEdit()
 {
     mTextEditHelper.reset();
-    mEditedClip.reset();
+    mEditedInterval = {};
 }
 
-void WaveTrackAffordanceControls::OnTrackChanged(const TrackListEvent& evt)
+void WaveTrackAffordanceControls::OnTrackListEvent(const TrackListEvent& evt)
 {
     if (evt.mType == TrackListEvent::SELECTION_CHANGE)
        ExitTextEditing();
 }
 
+void WaveTrackAffordanceControls::OnSelectionChange(NotifyingSelectedRegionMessage)
+{
+    ExitTextEditing();
+}
+
 unsigned WaveTrackAffordanceControls::ExitTextEditing()
 {
     using namespace RefreshCode;
@@ -465,26 +502,6 @@ unsigned WaveTrackAffordanceControls::ExitTextEditing()
     return RefreshNone;
 }
 
-
-bool WaveTrackAffordanceControls::StartEditNameOfMatchingClip(
-    AudacityProject &project, std::function<bool(WaveClip&)> test )
-{
-    //Attempts to invoke name editing if there is a selected clip
-    auto waveTrack = std::dynamic_pointer_cast<WaveTrack>(FindTrack());
-    if (!waveTrack)
-        return false;
-    auto clips = waveTrack->GetClips();
-
-    auto it = std::find_if(clips.begin(), clips.end(),
-      [&](auto pClip){ return pClip && test && test(*pClip); });
-    if (it != clips.end())
-    {
-        mFocusClip = *it;
-        return StartEditClipName(&project);
-    }
-    return false;
-}
-
 bool WaveTrackAffordanceControls::OnTextCopy(AudacityProject& project)
 {
    if (mTextEditHelper)
@@ -530,20 +547,20 @@ unsigned WaveTrackAffordanceControls::OnAffordanceClick(const TrackPanelMouseEve
     auto& viewInfo = ViewInfo::Get(*project);
     if (mTextEditHelper)
     {
-        if (auto lock = mEditedClip.lock())
+        if (auto interval = *mEditedInterval)
         {
-            auto affordanceRect = ClipParameters::GetClipRect(*lock.get(), viewInfo, event.rect);
+            auto affordanceRect = ClipParameters::GetClipRect(*interval->GetClip(0), viewInfo, event.rect);
             if (!affordanceRect.Contains(event.event.GetPosition()))
                return ExitTextEditing();
         }
     }
-    else if (auto lock = mFocusClip.lock())
+    else if (auto interval = *mFocusInterval)
     {
         if (event.event.LeftDClick())
         {
-            auto affordanceRect = ClipParameters::GetClipRect(*lock.get(), viewInfo, event.rect);
+            auto affordanceRect = ClipParameters::GetClipRect(*interval->GetClip(0), viewInfo, event.rect);
             if (affordanceRect.Contains(event.event.GetPosition()) &&
-                StartEditClipName(project))
+                StartEditClipName(*project, mFocusInterval))
             {
                 event.event.Skip(false);
                 return RefreshCode::RefreshCell;
@@ -553,12 +570,86 @@ unsigned WaveTrackAffordanceControls::OnAffordanceClick(const TrackPanelMouseEve
     return RefreshCode::RefreshNone;
 }
 
+void WaveTrackAffordanceControls::StartEditSelectedClipName(AudacityProject& project)
+{
+   auto [track, it] = SelectedIntervalOfFocusedTrack(project);
+   if(track != FindTrack().get())
+      return;
+   StartEditClipName(project, it);
+}
+
+namespace
+{
+void SelectInterval(AudacityProject& project, const WaveTrack::Interval& interval)
+{
+   auto& viewInfo = ViewInfo::Get(project);
+   viewInfo.selectedRegion.setTimes(interval.GetPlayStartTime(), interval.GetPlayEndTime());
+   ProjectHistory::Get(project).ModifyState(false);
+}
+}
+
+void WaveTrackAffordanceControls::StartEditSelectedClipSpeed(
+   AudacityProject& project)
+{
+   auto [track, it] = SelectedIntervalOfFocusedTrack(project);
+
+   if (track != FindTrack().get())
+      return;
+
+   auto interval = *it;
+
+   if (!interval)
+      return;
+
+   ChangeClipSpeedDialog dlg(*track, *interval, &GetProjectFrame(project));
+
+   if (wxID_OK == dlg.ShowModal())
+   {
+      PushClipSpeedChangedUndoState(project, 100.0 / interval->GetStretchRatio());
+      SelectInterval(project, *interval);
+   }
+}
+
+void WaveTrackAffordanceControls::OnRenderClipStretching(
+   AudacityProject& project)
+{
+   const auto [track, it] = SelectedIntervalOfFocusedTrack(project);
+
+   if (track != FindTrack().get())
+      return;
+
+   auto interval = *it;
+
+   if (!interval || interval->StretchRatioEquals(1.0))
+      return;
+
+   WaveTrackUtilities::WithStretchRenderingProgress(
+      [track = track, interval = interval](const ProgressReporter& progress) {
+         track->ApplyStretchRatio(
+            { { interval->GetPlayStartTime(), interval->GetPlayEndTime() } },
+            progress);
+      },
+      XO("Applying..."));
+
+   ProjectHistory::Get(project).PushState(
+      XO("Rendered time-stretched audio"), XO("Render"));
+
+   SelectInterval(project, *interval);
+}
+
 std::shared_ptr<TextEditHelper> WaveTrackAffordanceControls::MakeTextEditHelper(const wxString& text)
 {
     auto helper = std::make_shared<TextEditHelper>(shared_from_this(), text, mClipNameFont);
     helper->SetTextColor(theTheme.Colour(clrClipNameText));
     helper->SetTextSelectionColor(theTheme.Colour(clrClipNameTextSelection));
-    return helper; 
+    return helper;
+}
+
+auto WaveTrackAffordanceControls::GetMenuItems(
+   const wxRect &rect, const wxPoint *pPosition, AudacityProject *pProject)
+      -> std::vector<MenuItem>
+{
+   return GetWaveClipMenuItems();
 }
 
 // Register a menu item
@@ -567,44 +658,63 @@ namespace {
 
 // Menu handler functions
 
-struct Handler : CommandHandlerObject {
-
 void OnEditClipName(const CommandContext &context)
 {
    auto &project = context.project;
-   const auto [pTrack, pClip] = SelectedClipOfFocusedTrack(project);
-   if (pTrack && pClip) {
-      if (auto pAffordance = FindAffordance(*pTrack)) {
-         pAffordance->StartEditNameOfMatchingClip(project,
-            [pClip = pClip](auto &clip){ return &clip == pClip; });
+
+   if(auto pWaveTrack = dynamic_cast<WaveTrack *>(TrackFocus::Get(project).Get()))
+   {
+      if(auto pAffordance = FindAffordance(*pWaveTrack))
+      {
+         pAffordance->StartEditSelectedClipName(project);
          // Refresh so the cursor appears
-         TrackPanel::Get(project).RefreshTrack(pTrack);
+         TrackPanel::Get(project).RefreshTrack(pWaveTrack);
       }
    }
 }
 
-};
+void OnChangeClipSpeed(const CommandContext& context)
+{
+   auto& project = context.project;
 
-#define FN(X) (& Handler :: X)
+   if (
+      auto pWaveTrack =
+         dynamic_cast<WaveTrack*>(TrackFocus::Get(project).Get()))
+   {
+      if (auto pAffordance = FindAffordance(*pWaveTrack))
+         pAffordance->StartEditSelectedClipSpeed(project);
+   }
+}
 
-CommandHandlerObject &findCommandHandler(AudacityProject &) {
-   // Handler is not stateful.  Doesn't need a factory registered with
-   // AudacityProject.
-   static Handler instance;
-   return instance;
-};
+void OnRenderClipStretching(const CommandContext& context)
+{
+   auto& project = context.project;
+
+   if (
+      auto pWaveTrack =
+         dynamic_cast<WaveTrack*>(TrackFocus::Get(project).Get()))
+   {
+      if (auto pAffordance = FindAffordance(*pWaveTrack))
+         pAffordance->OnRenderClipStretching(project);
+   }
+}
 
 using namespace MenuTable;
 
 // Register menu items
 
-AttachedItem sAttachment{ wxT("Edit/Other"),
-   ( FinderScope{ findCommandHandler },
-      Command( L"RenameClip", XXO("Rename Clip..."),
-         &Handler::OnEditClipName, SomeClipIsSelectedFlag(),
-         wxT("Ctrl+F2") ) )
+AttachedItem sAttachment{ wxT("Edit/Other/Clip"),
+   Command( L"RenameClip", XXO("&Rename Clip..."),
+      OnEditClipName, SomeClipIsSelectedFlag(), wxT("Ctrl+F2") )
 };
 
-}
+AttachedItem sAttachment2{ wxT("Edit/Other/Clip"),
+   Command( L"ChangeClipSpeed", XXO("Change &Speed..."),
+      OnChangeClipSpeed, SomeClipIsSelectedFlag() )
+};
 
-#undef FN
+AttachedItem sAttachment3{ wxT("Edit/Other/Clip"),
+   Command( L"RenderClipStretching", XXO("Render Clip S&tretching"),
+      OnRenderClipStretching, StretchedClipIsSelectedFlag())
+};
+}
