@@ -4,26 +4,87 @@ Audacity: A Digital Audio Editor
 
 WaveformVRulerControls.cpp
 
-Paul Licameli split from WaveTrackVRulerControls.cpp
+Paul Licameli split from WaveChannelVRulerControls.cpp
 
 **********************************************************************/
 
 #include "WaveformVRulerControls.h"
 
 #include "WaveformVZoomHandle.h"
-#include "WaveTrackVRulerControls.h"
+#include "WaveChannelVRulerControls.h"
 
+#include "../../../ui/ChannelView.h"
 #include "NumberScale.h"
 #include "ProjectHistory.h"
 #include "../../../../RefreshCode.h"
 #include "../../../../TrackPanelMouseEvent.h"
 #include "../../../../UIHandle.h"
-#include "../../../../WaveTrack.h"
+#include "WaveTrack.h"
 #include "../../../../prefs/WaveformSettings.h"
 #include "../../../../widgets/Ruler.h"
+#include "../../../../widgets/LinearUpdater.h"
+#include "../../../../widgets/RealFormat.h"
+#include "../../../../widgets/CustomUpdaterValue.h"
 
 WaveformVRulerControls::~WaveformVRulerControls() = default;
 
+// These are doubles beacuse of the type of value in Label,
+// but for the purpose of labelling the linear dB waveform ruler,
+// these should always be integer numbers.
+using LinearDBValues = std::vector<double>;
+
+static LinearDBValues majorValues{}, minorValues{}, minorMinorValues{};
+
+void RegenerateLinearDBValues(int dBRange, float min, float max, int height)
+{
+   majorValues.clear();
+   minorValues.clear();
+   minorMinorValues.clear();
+
+   majorValues.push_back(0);
+   majorValues.push_back(-dBRange);
+   majorValues.push_back(2 * -dBRange);
+
+   const double EPSILON = .1e-5;
+
+   // No marks allowed within CENTER_SPACING pixels from the center
+   // Calculate the closest allowed major / minor and remove everything around those
+   const double CENTER = height / 2;
+   const int CENTER_SPACING = 30;
+   const double SIZE_SCALE = (max - min) / 2;
+
+   double centerSpacingMark = 0;
+
+   for (double major = 0.1; major < .95; major += .1) {
+      double val = std::round(major * 10) / 10;
+      double mappedVal = std::trunc(-dBRange * val);
+      double pixDist = CENTER - ((1 - DB_TO_LINEAR(mappedVal)) * CENTER) / SIZE_SCALE;
+      if (pixDist > CENTER_SPACING) {
+         centerSpacingMark = major;
+         majorValues.push_back(mappedVal);
+         majorValues.push_back(-2 * dBRange - mappedVal);
+      }
+   }
+   for (double minor = 0.05; minor < 1 ; minor += .1) {
+      double val = std::round(minor * 100) / 100;
+      double mappedVal = std::trunc(-dBRange * val);
+      if (minor < centerSpacingMark) {
+         minorValues.push_back(mappedVal);
+         minorValues.push_back(-2 * dBRange - mappedVal);
+      }
+   }
+   for (int minorMinor = 1; minorMinor < dBRange - EPSILON; minorMinor++) {
+      double absDist = fabs(fabs(dBRange - minorMinor) - dBRange) / dBRange;
+      if (absDist < centerSpacingMark) {
+         if ((minorMinor % (int)std::round(dBRange / 20)) != 0) {
+            minorMinorValues.push_back(-minorMinor);
+            minorMinorValues.push_back(-2 * dBRange + minorMinor);
+         }
+      }
+   }
+
+}
+
 std::vector<UIHandlePtr> WaveformVRulerControls::HitTest(
    const TrackPanelMouseState &st,
    const AudacityProject *pProject)
@@ -40,7 +101,7 @@ std::vector<UIHandlePtr> WaveformVRulerControls::HitTest(
       }
    }
 
-   auto more = TrackVRulerControls::HitTest(st, pProject);
+   auto more = ChannelVRulerControls::HitTest(st, pProject);
    std::copy(more.begin(), more.end(), std::back_inserter(results));
 
    return results;
@@ -57,6 +118,20 @@ unsigned WaveformVRulerControls::HandleWheelRotation(
    return DoHandleWheelRotation( evt, pProject, wt );
 }
 
+namespace {
+void SetLastdBRange(
+   WaveformScale &cache, const WaveTrack &track)
+{
+   cache.SetLastDBRange(WaveformSettings::Get(track).dBRange);
+}
+
+void SetLastScaleType(
+   WaveformScale &cache, const WaveTrack &track)
+{
+   cache.SetLastScaleType(WaveformSettings::Get(track).scaleType);
+}
+}
+
 unsigned WaveformVRulerControls::DoHandleWheelRotation(
    const TrackPanelMouseEvent &evt, AudacityProject *pProject, WaveTrack *wt)
 {
@@ -72,36 +147,32 @@ unsigned WaveformVRulerControls::DoHandleWheelRotation(
 
    auto steps = evt.steps;
 
-   using namespace WaveTrackViewConstants;
-   const bool isDB =
-      wt->GetWaveformSettings().scaleType == WaveformSettings::stLogarithmic;
-   // Special cases for Waveform dB only.
+   using namespace WaveChannelViewConstants;
+   auto &settings = WaveformSettings::Get(*wt);
+   auto &cache = WaveformScale::Get(*wt);
+   const bool isDB = !settings.isLinear();
+   // Special cases for Waveform (logarithmic) dB only.
    // Set the bottom of the dB scale but only if it's visible
    if (isDB && event.ShiftDown() && event.CmdDown()) {
       float min, max;
-      wt->GetDisplayBounds(&min, &max);
+      cache.GetDisplayBounds(min, max);
       if (!(min < 0.0 && max > 0.0))
          return RefreshNone;
 
-      WaveformSettings &settings =
-         wt->GetWaveformSettings();
       float olddBRange = settings.dBRange;
-      for (auto channel : TrackList::Channels(wt)) {
-         WaveformSettings &channelSettings =
-            channel->GetWaveformSettings();
-         if (steps < 0)
-            // Zoom out
-            channelSettings.NextLowerDBRange();
-         else
-            channelSettings.NextHigherDBRange();
-      }
+      auto &settings = WaveformSettings::Get(*wt);
+      if (steps < 0)
+         // Zoom out
+         settings.NextLowerDBRange();
+      else
+         settings.NextHigherDBRange();
 
       float newdBRange = settings.dBRange;
 
       // Is y coordinate within the rectangle half-height centered about
       // the zero level?
       const auto &rect = evt.rect;
-      const auto zeroLevel = wt->ZeroLevelYCoordinate(rect);
+      const auto zeroLevel = cache.ZeroLevelYCoordinate(rect);
       const bool fixedMagnification =
       (4 * std::abs(event.GetY() - zeroLevel) < rect.GetHeight());
 
@@ -112,10 +183,9 @@ unsigned WaveformVRulerControls::DoHandleWheelRotation(
          const float extreme = (LINEAR_TO_DB(2) + newdBRange) / newdBRange;
          max = std::min(extreme, max * olddBRange / newdBRange);
          min = std::max(-extreme, min * olddBRange / newdBRange);
-         for (auto channel : TrackList::Channels(wt)) {
-            channel->SetLastdBRange();
-            channel->SetDisplayBounds(min, max);
-         }
+         auto &cache = WaveformScale::Get(*wt);
+         SetLastdBRange(cache, *wt);
+         cache.SetDisplayBounds(min, max);
       }
    }
    else if (event.CmdDown() && !event.ShiftDown()) {
@@ -134,19 +204,19 @@ unsigned WaveformVRulerControls::DoHandleWheelRotation(
       {
          float topLimit = 2.0;
          if (isDB) {
-            const float dBRange = wt->GetWaveformSettings().dBRange;
+            const float dBRange = WaveformSettings::Get(*wt).dBRange;
             topLimit = (LINEAR_TO_DB(topLimit) + dBRange) / dBRange;
          }
          const float bottomLimit = -topLimit;
          float top, bottom;
-         wt->GetDisplayBounds(&bottom, &top);
+         auto &cache = WaveformScale::Get(*wt);
+         cache.GetDisplayBounds(bottom, top);
          const float range = top - bottom;
          const float delta = range * steps * movement / height;
          float newTop = std::min(topLimit, top + delta);
          const float newBottom = std::max(bottomLimit, newTop - range);
          newTop = std::min(topLimit, newBottom + range);
-         for (auto channel : TrackList::Channels(wt))
-            channel->SetDisplayBounds(newBottom, newTop);
+         WaveformScale::Get(*wt).SetDisplayBounds(newBottom, newTop);
       }
    }
    else
@@ -161,8 +231,8 @@ void WaveformVRulerControls::Draw(
    TrackPanelDrawingContext &context,
    const wxRect &rect_, unsigned iPass )
 {
-   TrackVRulerControls::Draw( context, rect_, iPass );
-   WaveTrackVRulerControls::DoDraw( *this, context, rect_, iPass );
+   ChannelVRulerControls::Draw(context, rect_, iPass);
+   WaveChannelVRulerControls::DoDraw(*this, context, rect_, iPass);
 }
 
 void WaveformVRulerControls::UpdateRuler( const wxRect &rect )
@@ -173,30 +243,37 @@ void WaveformVRulerControls::UpdateRuler( const wxRect &rect )
    DoUpdateVRuler( rect, wt.get() );
 }
 
+static CustomUpdaterValue updater;
+
 void WaveformVRulerControls::DoUpdateVRuler(
-   const wxRect &rect, const WaveTrack *wt )
+   const wxRect &rect, const WaveTrack *wt)
 {
-   auto vruler = &WaveTrackVRulerControls::ScratchRuler();
+   auto vruler = &WaveChannelVRulerControls::ScratchRuler();
 
    // All waves have a ruler in the info panel
    // The ruler needs a bevelled surround.
-   const float dBRange =
-      wt->GetWaveformSettings().dBRange;
+   auto &settings = WaveformSettings::Get(*wt);
+   const float dBRange = settings.dBRange;
 
-   WaveformSettings::ScaleType scaleType =
-   wt->GetWaveformSettings().scaleType;
+   float min, max;
+   auto &cache = WaveformScale::Get(*wt);
+   cache.GetDisplayBounds(min, max);
+   const float lastdBRange = cache.GetLastDBRange();
+   if (dBRange != lastdBRange)
+      SetLastdBRange(cache, *wt);
+
+   auto scaleType = settings.scaleType;
    
-   if (scaleType == WaveformSettings::stLinear) {
+   if (settings.isLinear()) {
       // Waveform
       
-      float min, max;
-      wt->GetDisplayBounds(&min, &max);
-      if (wt->GetLastScaleType() != scaleType &&
-          wt->GetLastScaleType() != -1)
+      if (cache.GetLastScaleType() != WaveformSettings::stLinearAmp &&
+          cache.GetLastScaleType() != WaveformSettings::stLinearDb &&
+          cache.GetLastScaleType() != -1)
       {
          // do a translation into the linear space
-         wt->SetLastScaleType();
-         wt->SetLastdBRange();
+         SetLastScaleType(cache, *wt);
+         SetLastdBRange(cache, *wt);
          float sign = (min >= 0 ? 1 : -1);
          if (min != 0.) {
             min = DB_TO_LINEAR(fabs(min) * dBRange - dBRange);
@@ -212,34 +289,77 @@ void WaveformVRulerControls::DoUpdateVRuler(
                max = 0.0;
             max *= sign;
          }
-         wt->SetDisplayBounds(min, max);
+         cache.SetDisplayBounds(min, max);
       }
       
-      vruler->SetDbMirrorValue( 0.0 );
-      vruler->SetBounds(rect.x, rect.y, rect.x + rect.width, rect.y + rect.height - 1);
+      vruler->SetDbMirrorValue(0.0);
+      vruler->SetBounds(
+         rect.x, rect.y, rect.x + rect.width, rect.y + rect.height - 1);
       vruler->SetOrientation(wxVERTICAL);
       vruler->SetRange(max, min);
-      vruler->SetFormat(Ruler::RealFormat);
-      vruler->SetUnits({});
-      vruler->SetLabelEdges(false);
-      vruler->SetLog(false);
+      vruler->SetFormat(&RealFormat::LinearInstance());
+      if (scaleType == WaveformSettings::stLinearAmp) {
+         vruler->SetLabelEdges(false);
+         vruler->SetUnits({});
+         vruler->SetUpdater(&LinearUpdater::Instance());
+      }
+      else {
+         RegenerateLinearDBValues(dBRange, min, max, rect.GetHeight());
+         vruler->SetLabelEdges(true);
+         vruler->SetUnits(XO("dB"));
+         vruler->SetUpdater(&updater);
+         RulerUpdater::Labels major, minor, minorMinor;
+         std::vector<LinearDBValues> values =
+            { majorValues, minorValues, minorMinorValues };
+         for (int ii = 0; ii < 3; ii++) {
+            RulerUpdater::Labels labs;
+            int size = (ii == 0) ? majorValues.size() :
+               (ii == 1) ? minorValues.size() : minorMinorValues.size();
+            for (int i = 0; i < size; i++) {
+               double value = (ii == 0) ? majorValues[i] :
+                  (ii == 1) ? minorValues[i] : minorMinorValues[i];
+               RulerUpdater::Label lab;
+
+               if (value == -dBRange)
+                  lab.value = 0;
+               else {
+                  float sign = (value > -dBRange ? 1 : -1);
+                  if (value < -dBRange)
+                     value = -2 * dBRange - value;
+                  lab.value = DB_TO_LINEAR(value) * sign;
+               }
+
+               wxString s = (value == -dBRange) ?
+                  wxString(L"-\u221e") : wxString::FromDouble(value);
+               // \u221e represents the infinity symbol
+               // Should this just be -dBRange so it is consistent?
+               //wxString s = wxString::FromDouble(value);
+               lab.text = Verbatim(s);
+
+               labs.push_back(lab);
+            }
+            if (ii == 0)
+               major = labs;
+            else if (ii == 1)
+               minor = labs;
+            else
+               minorMinor = labs;
+         }
+         updater.SetData(major, minor, minorMinor);
+      }
    }
    else {
-      wxASSERT(scaleType == WaveformSettings::stLogarithmic);
-      scaleType = WaveformSettings::stLogarithmic;
+      vruler->SetUnits(XO("dB"));
       
-      vruler->SetUnits({});
+      auto &cache = WaveformScale::Get(*wt);
       
-      float min, max;
-      wt->GetDisplayBounds(&min, &max);
-      float lastdBRange;
-      
-      if (wt->GetLastScaleType() != scaleType &&
-          wt->GetLastScaleType() != -1)
+      if (cache.GetLastScaleType() != WaveformSettings::stLogarithmicDb &&
+         // When Logarithmic Amp happens, put that here
+          cache.GetLastScaleType() != -1)
       {
          // do a translation into the dB space
-         wt->SetLastScaleType();
-         wt->SetLastdBRange();
+         SetLastScaleType(cache, *wt);
+         SetLastdBRange(cache, *wt);
          float sign = (min >= 0 ? 1 : -1);
          if (min != 0.) {
             min = (LINEAR_TO_DB(fabs(min)) + dBRange) / dBRange;
@@ -255,10 +375,10 @@ void WaveformVRulerControls::DoUpdateVRuler(
                max = 0.0;
             max *= sign;
          }
-         wt->SetDisplayBounds(min, max);
+         cache.SetDisplayBounds(min, max);
       }
-      else if (dBRange != (lastdBRange = wt->GetLastdBRange())) {
-         wt->SetLastdBRange();
+      else if (dBRange != lastdBRange) {
+         SetLastdBRange(cache, *wt);
          // Remap the max of the scale
          float newMax = max;
          
@@ -273,7 +393,8 @@ void WaveformVRulerControls::DoUpdateVRuler(
             
             const float extreme = LINEAR_TO_DB(2);
             // recover dB value of max
-            const float dB = std::min(extreme, (float(fabs(max)) * lastdBRange - lastdBRange));
+            const float dB = std::min(
+               extreme, (float(fabs(max)) * lastdBRange - lastdBRange));
             // find NEW scale position, but old max may get trimmed if the db limit rises
             // Don't trim it to zero though, but leave max and limit distinct
             newMax = sign * std::max(ZOOMLIMIT, (dBRange + dB) / dBRange);
@@ -283,7 +404,7 @@ void WaveformVRulerControls::DoUpdateVRuler(
                min = std::max(-extreme, newMax * min / max);
          }
 #endif
-         wt->SetDisplayBounds(min, newMax);
+         cache.SetDisplayBounds(min, newMax);
       }
       
       // Old code was if ONLY_LABEL_POSITIVE were defined.
@@ -328,9 +449,10 @@ void WaveformVRulerControls::DoUpdateVRuler(
       else
          vruler->SetBounds(0.0, 0.0, 0.0, 0.0); // A.C.H I couldn't find a way to just disable it?
 #endif
-      vruler->SetFormat(Ruler::RealLogFormat);
+      vruler->SetFormat(&RealFormat::LogInstance());
       vruler->SetLabelEdges(true);
-      vruler->SetLog(false);
+      vruler->SetUpdater(&LinearUpdater::Instance());
    }
-   vruler->GetMaxSize( &wt->vrulerSize.first, &wt->vrulerSize.second );
+   auto &size = ChannelView::Get(*wt).vrulerSize;
+   vruler->GetMaxSize(&size.first, &size.second);
 }
