@@ -19,10 +19,12 @@ Paul Licameli split from TrackPanel.cpp
 #include "../../commands/CommandContext.h"
 #include "../../commands/CommandManager.h"
 #include "../../HitTestResult.h"
+#include "../../Menus.h"
 #include "../../RefreshCode.h"
 #include "../../TrackPanelMouseEvent.h"
+#include "Track.h"
 #include "ViewInfo.h"
-#include "../../widgets/wxWidgetsWindowPlacement.h"
+#include "wxWidgetsWindowPlacement.h"
 
 CommonTrackPanelCell::~CommonTrackPanelCell()
 {
@@ -49,6 +51,9 @@ unsigned CommonTrackPanelCell::DoContextMenu( const wxRect &rect,
    if (items.empty())
       return RefreshCode::RefreshNone;
 
+   auto& menuManager = MenuManager::Get(*pProject);
+   menuManager.UpdateMenus();
+
    // Set up command context with extras
    CommandContext context{ *pProject };
    SelectedRegion region;
@@ -60,7 +65,7 @@ unsigned CommonTrackPanelCell::DoContextMenu( const wxRect &rect,
    context.temporarySelection.pTrack = FindTrack().get();
 
    auto &commandManager = CommandManager::Get(*pProject);
-   auto flags = MenuManager::Get( *pProject ).GetUpdateFlags();
+   auto flags = menuManager.GetUpdateFlags();
 
    // Common dispatcher for the menu items
    auto dispatcher = [&]( wxCommandEvent &evt ){
@@ -112,9 +117,19 @@ unsigned CommonTrackPanelCell::HandleWheelRotation
    return hook ? hook( evt, pProject ) : RefreshCode::Cancelled;
 }
 
-CommonTrackCell::CommonTrackCell( const std::shared_ptr< Track > &parent )
-   : mwTrack { parent }
-{}
+CommonTrackCell::CommonTrackCell(
+   const std::shared_ptr<Track> &parent, size_t iChannel
+)  : mwTrack{ parent }
+   , miChannel{ iChannel }
+{
+   // TODO wide wave tracks -- remove assertion
+   assert(iChannel == 0);
+}
+
+CommonTrackCell::CommonTrackCell(ChannelGroup &group, size_t iChannel)
+   : CommonTrackCell{ static_cast<Track&>(group).shared_from_this(), iChannel }
+{
+}
 
 CommonTrackCell::~CommonTrackCell() = default;
 
@@ -127,3 +142,15 @@ std::shared_ptr<Track> CommonTrackCell::DoFindTrack()
 {
    return mwTrack.lock();
 }
+
+std::shared_ptr<Channel> CommonTrackCell::FindChannel()
+{
+   if (const auto pTrack = FindTrack())
+      return pTrack->GetChannel(miChannel);
+   return {};
+}
+
+std::shared_ptr<const Channel> CommonTrackCell::FindChannel() const
+{
+   return const_cast<CommonTrackCell*>(this)->FindChannel();
+}
