@@ -22,6 +22,8 @@ Paul Licameli split from TrackPanel.cpp
 
 #include "XMLTagHandler.h"
 
+class Channel;
+class ChannelGroup;
 class CommandContext;
 class Track;
 class XMLWriter;
@@ -99,7 +101,14 @@ class AUDACITY_DLL_API CommonTrackCell /* not final */
    : public CommonTrackPanelCell, public TrackAttachment
 {
 public:
-   explicit CommonTrackCell( const std::shared_ptr<Track> &pTrack );
+   //! Construct from a track and a channel index
+   CommonTrackCell(const std::shared_ptr<Track> &pTrack, size_t iChannel);
+
+   //! Construct from a channel group and a channel index
+   /*!
+    @pre `dynamic_cast<Track&>(&group) != nullptr`
+    */
+   CommonTrackCell(ChannelGroup &group, size_t iChannel);
 
   ~CommonTrackCell();
 
@@ -107,8 +116,17 @@ public:
 
    void Reparent( const std::shared_ptr<Track> &parent ) override;
 
+   size_t GetChannelIndex() const { return miChannel; }
+
+   //! May return null
+   std::shared_ptr<Channel> FindChannel();
+
+   //! May return null
+   std::shared_ptr<const Channel> FindChannel() const;
+
 private:
    std::weak_ptr< Track > mwTrack;
+   const size_t miChannel;
 };
 
 #endif
