@@ -13,10 +13,10 @@ Paul Licameli split from TrackPanel.cpp
 
 #include <functional>
 
-#include "../../AudioIO.h"
+#include "AudioIO.h"
 #include "../../CommonCommandFlags.h"
 #include "Project.h"
-#include "../../ProjectAudioIO.h"
+#include "ProjectAudioIO.h"
 #include "../../ProjectAudioManager.h"
 #include "ProjectHistory.h"
 #include "../../ProjectWindows.h"
@@ -24,7 +24,7 @@ Paul Licameli split from TrackPanel.cpp
 #include "../../ScrubState.h"
 #include "Track.h"
 #include "ViewInfo.h"
-#include "../../WaveTrack.h"
+#include "WaveTrack.h"
 #include "../../prefs/PlaybackPrefs.h"
 #include "../../prefs/TracksPrefs.h"
 #include "../../toolbars/ToolManager.h"
@@ -234,7 +234,7 @@ Scrubber::~Scrubber()
 
 static const auto HasWaveDataPred =
    [](const AudacityProject &project){
-      auto range = TrackList::Get( project ).Any<const WaveTrack>()
+      auto range = TrackList::Get(project).Any<const WaveTrack>()
          + [](const WaveTrack *pTrack){
             return pTrack->GetEndTime() > pTrack->GetStartTime();
          };
@@ -375,9 +375,9 @@ bool Scrubber::MaybeStartScrubbing(wxCoord xx)
 
       wxCoord position = xx;
       if (abs(mScrubStartPosition - position) >= SCRUBBING_PIXEL_TOLERANCE) {
-         auto &viewInfo = ViewInfo::Get( *mProject );
-         auto &projectAudioManager = ProjectAudioManager::Get( *mProject );
-         double maxTime = TrackList::Get( *mProject ).GetEndTime();
+         auto &viewInfo = ViewInfo::Get(*mProject);
+         auto &projectAudioManager = ProjectAudioManager::Get(*mProject);
+         double maxTime = TrackList::Get(*mProject).GetEndTime();
          const int leftOffset = viewInfo.GetLeftOffset();
          double time0 = std::min(maxTime,
             viewInfo.PositionToTime(mScrubStartPosition, leftOffset)
@@ -406,7 +406,7 @@ bool Scrubber::MaybeStartScrubbing(wxCoord xx)
             mSpeedPlaying = false;
             mKeyboardScrubbing = false;
             auto options =
-               DefaultPlayOptions( *mProject );
+               ProjectAudioIO::GetDefaultOptions(*mProject);
 
 #ifndef USE_SCRUB_THREAD
             // Yuck, we either have to poll "by hand" when scrub polling doesn't
@@ -442,7 +442,7 @@ bool Scrubber::MaybeStartScrubbing(wxCoord xx)
 #endif
             mOptions.minTime = 0;
             mOptions.maxTime =
-               std::max(0.0, TrackList::Get( *mProject ).GetEndTime());
+               std::max(0.0, TrackList::Get(*mProject).GetEndTime());
             mOptions.minStutterTime =
 #ifdef DRAG_SCRUB
                mDragging ? PlaybackPolicy::Duration{} :
@@ -970,8 +970,8 @@ void Scrubber::OnToggleScrubRuler(const CommandContext&)
    mShowScrubbing = !mShowScrubbing;
    WriteScrubEnabledPref(mShowScrubbing);
    gPrefs->Flush();
-   const auto toolbar =
-      ToolManager::Get( *mProject ).GetToolBar( ScrubbingBarID );
+   // To do: move this, or eliminate it, use an event instead
+   const auto toolbar = ToolManager::Get(*mProject).GetToolBar(wxT("Scrub"));
    toolbar->EnableDisableButtons();
    CheckMenuItems();
 }
@@ -1136,36 +1136,29 @@ using namespace MenuTable;
 BaseItemSharedPtr ToolbarMenu()
 {
    using Options = CommandManager::Options;
-
-   static BaseItemSharedPtr menu { (
-   FinderScope{ finder },
-   Menu( wxT("Scrubbing"),
-      XXO("Scru&bbing"),
-      []{
-         BaseItemPtrs ptrs;
-         for (const auto &item : menuItems()) {
-            ptrs.push_back( Command( item.name, item.label,
-               item.memFn,
-               item.flags,
-               item.StatusTest
-                  ? // a checkmark item
-                     Options{}.CheckTest( [&item](AudacityProject &project){
-                     return ( Scrubber::Get(project).*(item.StatusTest) )(); } )
-                  : // not a checkmark item
-                     Options{}
-            ) );
-         }
-         return ptrs;
-      }()
-   )
-   ) };
-   
+   static auto menu = []{
+      FinderScope scope{ finder };
+      auto menu = std::shared_ptr{ Menu("Scrubbing", XXO("Scru&bbing")) };
+      for (const auto &item : menuItems()) {
+         menu->push_back(Command(item.name, item.label,
+            item.memFn,
+            item.flags,
+            item.StatusTest
+               ? // a checkmark item
+                  Options{}.CheckTest([&item](AudacityProject &project){
+                  return (Scrubber::Get(project).*(item.StatusTest))(); } )
+               : // not a checkmark item
+                  Options{}
+         ));
+      }
+      return menu;
+   }();
    return menu;
 }
 
 AttachedItem sAttachment{
    wxT("Transport/Basic"),
-   Shared( ToolbarMenu() )
+   Indirect(ToolbarMenu())
 };
 
 BaseItemSharedPtr KeyboardScrubbingItems()
@@ -1189,7 +1182,7 @@ BaseItemSharedPtr KeyboardScrubbingItems()
 
 AttachedItem sAttachment2{
    wxT("Optional/Extra/Part1/Transport"),
-   Shared( KeyboardScrubbingItems() )
+   Indirect(KeyboardScrubbingItems())
 };
 
 }
