@@ -11,7 +11,7 @@ Paul Licameli split from TrackPanel.cpp
 
 #include "PopupMenuTable.h"
 #include "widgets/BasicMenu.h"
-#include "widgets/wxWidgetsWindowPlacement.h"
+#include "wxWidgetsWindowPlacement.h"
 
 PopupMenuTableEntry::~PopupMenuTableEntry()
 {}
@@ -19,15 +19,22 @@ PopupMenuTableEntry::~PopupMenuTableEntry()
 PopupSubMenu::~PopupSubMenu()
 {}
 
-PopupSubMenu::PopupSubMenu( const Identifier &stringId,
-   const TranslatableString &caption_, PopupMenuTable &table_ )
-   : ConcreteGroupItem< false >{ stringId }
-   , WholeMenu{ caption_.empty() }
-   , caption{ caption_ }
-   , table{ table_ }
+PopupSubMenu::PopupSubMenu(const Identifier &stringId,
+   const TranslatableString &caption, PopupMenuTable &table
+)  : GroupItem{ stringId }
+   , WholeMenu{ caption.empty() }
+   , caption{ caption }
+   , table{ table }
 {
 }
 
+PopupMenuVisitor::~PopupMenuVisitor() = default;
+
+void *PopupMenuVisitor::GetComputedItemContext()
+{
+   return this;
+}
+
 PopupMenu::~PopupMenu() = default;
 
 namespace {
@@ -55,8 +62,8 @@ public:
       , mpUserData{ pUserData }
    {}
 
-   void DoBeginGroup( Registry::GroupItem &item, const Path &path ) override;
-   void DoEndGroup( Registry::GroupItem &item, const Path &path ) override;
+   void DoBeginGroup( Registry::GroupItemBase &item, const Path &path ) override;
+   void DoEndGroup( Registry::GroupItemBase &item, const Path &path ) override;
    void DoVisit( Registry::SingleItem &item, const Path &path ) override;
    void DoSeparator() override;
 
@@ -65,7 +72,7 @@ public:
    void *const mpUserData;
 };
 
-void PopupMenuBuilder::DoBeginGroup( Registry::GroupItem &item, const Path &path )
+void PopupMenuBuilder::DoBeginGroup( Registry::GroupItemBase &item, const Path &path )
 {
    if ( auto pItem = dynamic_cast<PopupSubMenu*>(&item) ) {
       if ( !pItem->caption.empty() ) {
@@ -77,7 +84,7 @@ void PopupMenuBuilder::DoBeginGroup( Registry::GroupItem &item, const Path &path
    }
 }
 
-void PopupMenuBuilder::DoEndGroup( Registry::GroupItem &item, const Path &path )
+void PopupMenuBuilder::DoEndGroup( Registry::GroupItemBase &item, const Path &path )
 {
    if ( auto pItem = dynamic_cast<PopupSubMenu*>(&item) ) {
       if ( !pItem->caption.empty() ) {
@@ -159,11 +166,6 @@ void PopupMenuTable::RegisterItem(
    Registry::RegisterItem( *mRegistry, placement, std::move( pItem ) );
 }
 
-void PopupMenuTable::Append( Registry::BaseItemPtr pItem )
-{
-   mStack.back()->items.push_back( std::move( pItem ) );
-}
-
 void PopupMenuTable::Append(
    const Identifier &stringId, PopupMenuTableEntry::Type type, int id,
    const TranslatableString &string, wxCommandEventFunction memFn,
@@ -177,7 +179,7 @@ void PopupMenuTable::BeginSection( const Identifier &name )
 {
    auto uSection = std::make_unique< PopupMenuSection >( name );
    auto section = uSection.get();
-   mStack.back()->items.push_back( std::move( uSection ) );
+   mStack.back()->push_back( std::move( uSection ) );
    mStack.push_back( section );
 }
 
